@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -25,33 +25,35 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
-#include "OgreStableHeaders.h"
-
 #include "OgreRoot.h"
 #include "OgreRenderSystem.h"
 #include "OgreOverlayElement.h"
-#include "OgreMaterialManager.h"
 #include "OgreOverlay.h"
-#include "OgreOverlayContainer.h"
 #include "OgreOverlayManager.h"
 #include "OgreException.h"
-#include "OgreRenderQueue.h"
+#include "OgreMaterialManager.h"
+#include "OgreOverlayContainer.h"
+#include "OgreResourceGroupManager.h"
+#include "OgreOverlayElementCommands.h"
 
 namespace Ogre {
 
 
     //---------------------------------------------------------------------
     // Define static members
-    OverlayElementCommands::CmdLeft OverlayElement::msLeftCmd;
-    OverlayElementCommands::CmdTop OverlayElement::msTopCmd;
-    OverlayElementCommands::CmdWidth OverlayElement::msWidthCmd;
-    OverlayElementCommands::CmdHeight OverlayElement::msHeightCmd;
-    OverlayElementCommands::CmdMaterial OverlayElement::msMaterialCmd;
-    OverlayElementCommands::CmdCaption OverlayElement::msCaptionCmd;
-    OverlayElementCommands::CmdMetricsMode OverlayElement::msMetricsModeCmd;
-    OverlayElementCommands::CmdHorizontalAlign OverlayElement::msHorizontalAlignCmd;
-    OverlayElementCommands::CmdVerticalAlign OverlayElement::msVerticalAlignCmd;
-    OverlayElementCommands::CmdVisible OverlayElement::msVisibleCmd;
+    // Command object for setting / getting parameters
+    static OverlayElementCommands::CmdLeft msLeftCmd;
+    static OverlayElementCommands::CmdTop msTopCmd;
+    static OverlayElementCommands::CmdWidth msWidthCmd;
+    static OverlayElementCommands::CmdHeight msHeightCmd;
+    static OverlayElementCommands::CmdMaterial msMaterialCmd;
+    static OverlayElementCommands::CmdCaption msCaptionCmd;
+    static OverlayElementCommands::CmdMetricsMode msMetricsModeCmd;
+    static OverlayElementCommands::CmdHorizontalAlign msHorizontalAlignCmd;
+    static OverlayElementCommands::CmdVerticalAlign msVerticalAlignCmd;
+    static OverlayElementCommands::CmdVisible msVisibleCmd;
+
+    const String& OverlayElement::DEFAULT_RESOURCE_GROUP = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME;
     //---------------------------------------------------------------------
     OverlayElement::OverlayElement(const String& name)
       : mName(name)
@@ -82,8 +84,8 @@ namespace Ogre {
       , mInitialised(false)
       , mSourceTemplate(0)
     {
-		// default overlays to preserve their own detail level
-		mPolygonModeOverrideable = false;
+        // default overlays to preserve their own detail level
+        mPolygonModeOverrideable = false;
 
         // use identity projection and view matrices
         mUseIdentityProjection = true;
@@ -92,11 +94,11 @@ namespace Ogre {
     //---------------------------------------------------------------------
     OverlayElement::~OverlayElement()
     {
-		if (mParent)
-		{
-			mParent->removeChild(mName);
-			mParent = 0;
-		}
+        if (mParent)
+        {
+            mParent->removeChild(mName);
+            mParent = 0;
+        }
     }
     //---------------------------------------------------------------------
     const String& OverlayElement::getName(void) const
@@ -169,12 +171,12 @@ namespace Ogre {
     {
         if (mMetricsMode != GMM_RELATIVE)
         {
-			return mPixelWidth;
-		}
-		else
-		{
-        	return mWidth;
-		}
+            return mPixelWidth;
+        }
+        else
+        {
+            return mWidth;
+        }
     }
     //---------------------------------------------------------------------
     void OverlayElement::setHeight(Real height)
@@ -195,12 +197,12 @@ namespace Ogre {
     {
         if (mMetricsMode != GMM_RELATIVE)
         {
-			return mPixelHeight;
-		}
-		else
-		{
-			return mHeight;
-		}
+            return mPixelHeight;
+        }
+        else
+        {
+            return mHeight;
+        }
     }
     //---------------------------------------------------------------------
     void OverlayElement::setLeft(Real left)
@@ -221,12 +223,12 @@ namespace Ogre {
     {
         if (mMetricsMode != GMM_RELATIVE)
         {
-			return mPixelLeft;
-		}
-		else
-		{
-        	return mLeft;
-		}
+            return mPixelLeft;
+        }
+        else
+        {
+            return mLeft;
+        }
     }
     //---------------------------------------------------------------------
     void OverlayElement::setTop(Real top)
@@ -248,12 +250,12 @@ namespace Ogre {
     {
         if (mMetricsMode != GMM_RELATIVE)
         {
-			return mPixelTop;
-		}
-		else
-		{
-	        return mTop;
-		}
+            return mPixelTop;
+        }
+        else
+        {
+            return mTop;
+        }
     }
     //---------------------------------------------------------------------
     void OverlayElement::_setLeft(Real left)
@@ -316,27 +318,37 @@ namespace Ogre {
     //---------------------------------------------------------------------
     const String& OverlayElement::getMaterialName(void) const
     {
-        return mMaterialName;
+        return mMaterial ? mMaterial->getName() : BLANKSTRING;
     }
     //---------------------------------------------------------------------
-    void OverlayElement::setMaterialName(const String& matName)
+    void OverlayElement::setMaterial(const MaterialPtr& mat)
     {
-        mMaterialName = matName;
-		if (matName != StringUtil::BLANK)
-		{
-			mMaterial = MaterialManager::getSingleton().getByName(matName);
-			if (mMaterial.isNull())
-				OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material " + matName,
-					"OverlayElement::setMaterialName" );
-			mMaterial->load();
-			// Set some prerequisites to be sure
-			mMaterial->setLightingEnabled(false);
-			mMaterial->setDepthCheckEnabled(false);
-		}
-		else
-		{
-			mMaterial.setNull();
-		}
+        mMaterial = mat;
+
+        if(!mMaterial)
+            return;
+
+        mMaterial->load();
+        // Set some prerequisites to be sure
+        mMaterial->setLightingEnabled(false);
+        mMaterial->setDepthCheckEnabled(false);
+    }
+
+    void OverlayElement::setMaterialName(const String& matName, const String& group)
+    {
+        if (!matName.empty())
+        {
+            mMaterial = MaterialManager::getSingleton().getByName(matName, group);
+            if (!mMaterial)
+                OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material " + matName,
+                    "OverlayElement::setMaterialName" );
+
+            setMaterial(mMaterial);
+        }
+        else
+        {
+            mMaterial.reset();
+        }
     }
     //---------------------------------------------------------------------
     const MaterialPtr& OverlayElement::getMaterial(void) const
@@ -349,10 +361,10 @@ namespace Ogre {
         mOverlay->_getWorldTransforms(xform);
     }
     //---------------------------------------------------------------------
-	void OverlayElement::_positionsOutOfDate(void)
-	{
-		mGeomPositionsOutOfDate = true;
-	}
+    void OverlayElement::_positionsOutOfDate(void)
+    {
+        mGeomPositionsOutOfDate = true;
+    }
     //---------------------------------------------------------------------
     void OverlayElement::_update(void)
     {
@@ -365,7 +377,7 @@ namespace Ogre {
         switch (mMetricsMode)
         {
         case GMM_PIXELS :
-            if (OverlayManager::getSingleton().hasViewportChanged() || mGeomPositionsOutOfDate)
+            if (mGeomPositionsOutOfDate)
             {               
                 mPixelScaleX = 1.0f / vpWidth;
                 mPixelScaleY = 1.0f / vpHeight; 
@@ -373,7 +385,7 @@ namespace Ogre {
             break;
 
         case GMM_RELATIVE_ASPECT_ADJUSTED :
-            if (OverlayManager::getSingleton().hasViewportChanged() || mGeomPositionsOutOfDate)
+            if (mGeomPositionsOutOfDate)
             {
                 mPixelScaleX = 1.0f / (10000.0f * (vpWidth / vpHeight));
                 mPixelScaleY = 1.0f /  10000.0f;
@@ -516,10 +528,10 @@ namespace Ogre {
         mParent = parent;
         mOverlay = overlay;
 
-		if (mOverlay && mOverlay->isInitialised() && !mInitialised)
-		{
-			initialise();
-		}
+        if (mOverlay && mOverlay->isInitialised() && !mInitialised)
+        {
+            initialise();
+        }
 
         mDerivedOutOfDate = true;
     }
@@ -541,16 +553,16 @@ namespace Ogre {
         }
         return mDerivedTop;
     }
-	//---------------------------------------------------------------------
-	Real OverlayElement::_getRelativeWidth(void)
-	{
-		return mWidth;
-	}
-	//---------------------------------------------------------------------
-	Real OverlayElement::_getRelativeHeight(void)
-	{
-		return mHeight;
-	}
+    //---------------------------------------------------------------------
+    Real OverlayElement::_getRelativeWidth(void)
+    {
+        return mWidth;
+    }
+    //---------------------------------------------------------------------
+    Real OverlayElement::_getRelativeHeight(void)
+    {
+        return mHeight;
+    }
     //---------------------------------------------------------------------    
     void OverlayElement::_getClippingRegion(RealRect &clippingRegion)
     {
@@ -564,7 +576,7 @@ namespace Ogre {
     ushort OverlayElement::_notifyZOrder(ushort newZOrder)
     {
         mZOrder = newZOrder;
-		return mZOrder + 1;
+        return mZOrder + 1;
     }
     //---------------------------------------------------------------------
     void OverlayElement::_notifyWorldTransforms(const Matrix4& xform)
@@ -626,12 +638,12 @@ namespace Ogre {
             queue->addRenderable(this, RENDER_QUEUE_OVERLAY, mZOrder);
         }      
     }
-	//---------------------------------------------------------------------
-	void OverlayElement::visitRenderables(Renderable::Visitor* visitor, 
-		bool debugRenderables)
-	{
-		visitor->visit(this, 0, false);
-	}
+    //---------------------------------------------------------------------
+    void OverlayElement::visitRenderables(Renderable::Visitor* visitor, 
+        bool debugRenderables)
+    {
+        visitor->visit(this, 0, false);
+    }
     //-----------------------------------------------------------------------
     void OverlayElement::addBaseParameters(void)    
     {
@@ -685,7 +697,7 @@ namespace Ogre {
         _positionsOutOfDate();
     }
     //-----------------------------------------------------------------------
-	const DisplayString& OverlayElement::getCaption() const
+    const DisplayString& OverlayElement::getCaption() const
     {
         return mCaption;
     }
@@ -711,9 +723,9 @@ namespace Ogre {
                 vpWidth = (Real) (oMgr.getViewportWidth());
                 vpHeight = (Real) (oMgr.getViewportHeight());
 
-				// cope with temporarily zero dimensions, avoid divide by zero
-				vpWidth = vpWidth == 0.0f? 1.0f : vpWidth;
-				vpHeight = vpHeight == 0.0f? 1.0f : vpHeight;
+                // cope with temporarily zero dimensions, avoid divide by zero
+                vpWidth = vpWidth == 0.0f? 1.0f : vpWidth;
+                vpHeight = vpHeight == 0.0f? 1.0f : vpHeight;
 
                 mPixelScaleX = 1.0f / vpWidth;
                 mPixelScaleY = 1.0f / vpHeight;
@@ -796,53 +808,53 @@ namespace Ogre {
         return mVertAlign;
     }
     //-----------------------------------------------------------------------    
-	bool OverlayElement::contains(Real x, Real y) const
-	{
-		return x >= mClippingRegion.left && x <= mClippingRegion.right && y >= mClippingRegion.top && y <= mClippingRegion.bottom;
-	}
+    bool OverlayElement::contains(Real x, Real y) const
+    {
+        return x >= mClippingRegion.left && x <= mClippingRegion.right && y >= mClippingRegion.top && y <= mClippingRegion.bottom;
+    }
     //-----------------------------------------------------------------------
-	OverlayElement* OverlayElement::findElementAt(Real x, Real y) 		// relative to parent
-	{
-		OverlayElement* ret = NULL;
-		if (contains(x , y ))
-		{
-			ret = this;
-		}
-		return ret;
-	}
+    OverlayElement* OverlayElement::findElementAt(Real x, Real y)       // relative to parent
+    {
+        OverlayElement* ret = NULL;
+        if (contains(x , y ))
+        {
+            ret = this;
+        }
+        return ret;
+    }
     //-----------------------------------------------------------------------
-	OverlayContainer* OverlayElement::getParent() 
-	{ 
-		return mParent;		
-	}
+    OverlayContainer* OverlayElement::getParent() 
+    { 
+        return mParent;     
+    }
     //-----------------------------------------------------------------------
     void OverlayElement::copyFromTemplate(OverlayElement* templateOverlay)
-	{
-		templateOverlay->copyParametersTo(this);
+    {
+        templateOverlay->copyParametersTo(this);
         mSourceTemplate = templateOverlay ;
-		return;
-	}
+        return;
+    }
     //-----------------------------------------------------------------------
     OverlayElement* OverlayElement::clone(const String& instanceName)
     {
         OverlayElement* newElement;
 
         newElement = OverlayManager::getSingleton().createOverlayElement(
-			getTypeName(), instanceName + "/" + mName);
+            getTypeName(), instanceName + "/" + mName);
         copyParametersTo(newElement);
 
         return newElement;
     }
     //-----------------------------------------------------------------------
-	bool OverlayElement::isEnabled() const
-	{ 
-		return mEnabled;
-	}
+    bool OverlayElement::isEnabled() const
+    { 
+        return mEnabled;
+    }
     //-----------------------------------------------------------------------
-	void OverlayElement::setEnabled(bool b) 
-	{
-		mEnabled = b;
-	}
+    void OverlayElement::setEnabled(bool b) 
+    {
+        mEnabled = b;
+    }
     //-----------------------------------------------------------------------
 
 }
