@@ -4,7 +4,7 @@ This source file is a part of OGRE
 
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
@@ -34,6 +34,7 @@ THE SOFTWARE
 #include "OgreStringConverter.h"
 #include "OgreFont.h"
 #include "OgreFontManager.h"
+#include "OgreOverlayElement.h"
 
 namespace Ogre {
 
@@ -50,11 +51,11 @@ namespace Ogre {
     //---------------------------------------------------------------------
     #define POS_TEX_BINDING 0
     #define COLOUR_BINDING 1
-	#define UNICODE_NEL 0x0085
-	#define UNICODE_CR 0x000D
-	#define UNICODE_LF 0x000A
-	#define UNICODE_SPACE 0x0020
-	#define UNICODE_ZERO 0x0030
+    #define UNICODE_NEL 0x0085
+    #define UNICODE_CR 0x000D
+    #define UNICODE_LF 0x000A
+    #define UNICODE_SPACE 0x0020
+    #define UNICODE_ZERO 0x0030
     //---------------------------------------------------------------------
     TextAreaOverlayElement::TextAreaOverlayElement(const String& name)
         : OverlayElement(name), mColourBottom(ColourValue::White), mColourTop(ColourValue::White)
@@ -67,10 +68,11 @@ namespace Ogre {
         mAllocSize = 0;
 
         mCharHeight = 0.02;
-		mPixelCharHeight = 12;
-		mSpaceWidth = 0;
-		mPixelSpaceWidth = 0;
-		mViewportAspectCoef = 1;
+        mPixelCharHeight = 12;
+        mSpaceWidthOverridden = false;
+        mSpaceWidth = 0;
+        mPixelSpaceWidth = 0;
+        mViewportAspectCoef = 1;
 
         if (createParamDictionary("TextAreaOverlayElement"))
         {
@@ -80,285 +82,305 @@ namespace Ogre {
 
     void TextAreaOverlayElement::initialise(void)
     {
-		if (!mInitialised)
-		{
-			// Set up the render op
-			// Combine positions and texture coords since they tend to change together
-			// since character sizes are different
-			mRenderOp.vertexData = OGRE_NEW VertexData();
-			VertexDeclaration* decl = mRenderOp.vertexData->vertexDeclaration;
-			size_t offset = 0;
-			// Positions
-			decl->addElement(POS_TEX_BINDING, offset, VET_FLOAT3, VES_POSITION);
-			offset += VertexElement::getTypeSize(VET_FLOAT3);
-			// Texcoords
-			decl->addElement(POS_TEX_BINDING, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, 0);
-			// Colours - store these in a separate buffer because they change less often
-			decl->addElement(COLOUR_BINDING, 0, VET_COLOUR, VES_DIFFUSE);
+        if (!mInitialised)
+        {
+            // Set up the render op
+            // Combine positions and texture coords since they tend to change together
+            // since character sizes are different
+            mRenderOp.vertexData = OGRE_NEW VertexData();
+            VertexDeclaration* decl = mRenderOp.vertexData->vertexDeclaration;
+            size_t offset = 0;
+            // Positions
+            offset += decl->addElement(POS_TEX_BINDING, offset, VET_FLOAT3, VES_POSITION).getSize();
+            // Texcoords
+            decl->addElement(POS_TEX_BINDING, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, 0);
+            // Colours - store these in a separate buffer because they change less often
+            decl->addElement(COLOUR_BINDING, 0, VET_COLOUR, VES_DIFFUSE);
 
-			mRenderOp.operationType = RenderOperation::OT_TRIANGLE_LIST;
-			mRenderOp.useIndexes = false;
-			mRenderOp.vertexData->vertexStart = 0;
+            mRenderOp.operationType = RenderOperation::OT_TRIANGLE_LIST;
+            mRenderOp.useIndexes = false;
+            mRenderOp.vertexData->vertexStart = 0;
             mRenderOp.useGlobalInstancingVertexBufferIsAvailable = false;
-			// Vertex buffer will be created in checkMemoryAllocation
+            // Vertex buffer will be created in checkMemoryAllocation
 
-			checkMemoryAllocation( DEFAULT_INITIAL_CHARS );
+            mInitialised = true;
 
-			mInitialised = true;
-		}
+            checkMemoryAllocation( DEFAULT_INITIAL_CHARS );
+        }
 
     }
-
+    //---------------------------------------------------------------------
     void TextAreaOverlayElement::checkMemoryAllocation( size_t numChars )
     {
         if( mAllocSize < numChars)
         {
+            mAllocSize = numChars;
+
             // Create and bind new buffers
             // Note that old buffers will be deleted automatically through reference counting
-            
-            // 6 verts per char since we're doing tri lists without indexes
-            // Allocate space for positions & texture coords
-            VertexDeclaration* decl = mRenderOp.vertexData->vertexDeclaration;
-            VertexBufferBinding* bind = mRenderOp.vertexData->vertexBufferBinding;
-
-            mRenderOp.vertexData->vertexCount = numChars * 6;
-
-            // Create dynamic since text tends to change a lot
-            // positions & texcoords
-            HardwareVertexBufferSharedPtr vbuf = 
-                HardwareBufferManager::getSingleton().
-                    createVertexBuffer(
-                        decl->getVertexSize(POS_TEX_BINDING), 
-                        mRenderOp.vertexData->vertexCount,
-                        HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY);
-            bind->setBinding(POS_TEX_BINDING, vbuf);
-
-            // colours
-            vbuf = HardwareBufferManager::getSingleton().
-                    createVertexBuffer(
-                        decl->getVertexSize(COLOUR_BINDING), 
-                        mRenderOp.vertexData->vertexCount,
-                        HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY);
-            bind->setBinding(COLOUR_BINDING, vbuf);
-
-            mAllocSize = numChars;
-            mColoursChanged = true; // force colour buffer regeneration
+            _restoreManualHardwareResources();
         }
-
     }
+    //---------------------------------------------------------------------
+    void TextAreaOverlayElement::_restoreManualHardwareResources()
+    {
+        if(!mInitialised)
+            return;
+
+        // 6 verts per char since we're doing tri lists without indexes
+        // Allocate space for positions & texture coords
+        // Note - mRenderOp.vertexData->vertexCount will be less than allocatedVertexCount
+        size_t allocatedVertexCount = mAllocSize * 6;
+        VertexDeclaration* decl = mRenderOp.vertexData->vertexDeclaration;
+        VertexBufferBinding* bind = mRenderOp.vertexData->vertexBufferBinding;
+
+        // Create dynamic since text tends to change a lot
+        // positions & texcoords
+        HardwareVertexBufferSharedPtr vbuf = 
+            HardwareBufferManager::getSingleton().
+                createVertexBuffer(
+                    decl->getVertexSize(POS_TEX_BINDING), 
+                    allocatedVertexCount,
+                    HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY,
+                    true);//Workaround, using shadow buffer to avoid stall due to buffer mapping
+        bind->setBinding(POS_TEX_BINDING, vbuf);
+
+        // colours
+        vbuf = HardwareBufferManager::getSingleton().
+                createVertexBuffer(
+                    decl->getVertexSize(COLOUR_BINDING), 
+                    allocatedVertexCount,
+                    HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY,
+                    true);//Workaround, using shadow buffer to avoid stall due to buffer mapping
+        bind->setBinding(COLOUR_BINDING, vbuf);
+
+        // Buffers are restored, but with trash within
+        mGeomPositionsOutOfDate = true;
+        mGeomUVsOutOfDate = true;
+        mColoursChanged = true;
+    }
+    //---------------------------------------------------------------------
+    void TextAreaOverlayElement::_releaseManualHardwareResources()
+    {
+        if(!mInitialised)
+            return;
+
+        VertexBufferBinding* bind = mRenderOp.vertexData->vertexBufferBinding;
+        bind->unsetAllBindings();
+    }
+    //---------------------------------------------------------------------
 
     void TextAreaOverlayElement::updatePositionGeometry()
     {
-		float *pVert;
+        float *pVert;
 
-		if (mFont.isNull())
-		{
-			// not initialised yet, probably due to the order of creation in a template
-			return;
-		}
+        if (!mFont)
+        {
+            // not initialised yet, probably due to the order of creation in a template
+            return;
+        }
 
-		size_t charlen = mCaption.size();
-		checkMemoryAllocation( charlen );
+        size_t charlen = mCaption.size();
+        checkMemoryAllocation( charlen );
 
-		mRenderOp.vertexData->vertexCount = charlen * 6;
-		// Get position / texcoord buffer
-		const HardwareVertexBufferSharedPtr& vbuf = 
-			mRenderOp.vertexData->vertexBufferBinding->getBuffer(POS_TEX_BINDING);
-		pVert = static_cast<float*>(
-			vbuf->lock(HardwareBuffer::HBL_DISCARD) );
+        mRenderOp.vertexData->vertexCount = charlen * 6;
+        // Get position / texcoord buffer
+        const HardwareVertexBufferSharedPtr& vbuf = 
+            mRenderOp.vertexData->vertexBufferBinding->getBuffer(POS_TEX_BINDING);
+        HardwareBufferLockGuard vbufLock(vbuf, HardwareBuffer::HBL_DISCARD);
+        pVert = static_cast<float*>(vbufLock.pData);
 
-		float largestWidth = 0;
-		float left = _getDerivedLeft() * 2.0f - 1.0f;
-		float top = -( (_getDerivedTop() * 2.0f ) - 1.0f );
+        float largestWidth = 0;
+        float left = _getDerivedLeft() * 2.0f - 1.0f;
+        float top = -( (_getDerivedTop() * 2.0f ) - 1.0f );
 
-		// Derive space with from a number 0
-		if (mSpaceWidth == 0)
-		{
-			mSpaceWidth = mFont->getGlyphAspectRatio(UNICODE_ZERO) * mCharHeight;
-		}
+        // Derive space with from a number 0
+        if(!mSpaceWidthOverridden)
+        {
+            mSpaceWidth = mFont->getGlyphAspectRatio(UNICODE_ZERO) * mCharHeight;
+        }
 
-		// Use iterator
-		DisplayString::iterator i, iend;
-		iend = mCaption.end();
-		bool newLine = true;
-		for( i = mCaption.begin(); i != iend; ++i )
-		{
-			if( newLine )
-			{
-				Real len = 0.0f;
-				for( DisplayString::iterator j = i; j != iend; j++ )
-				{
-					Font::CodePoint character = OGRE_DEREF_DISPLAYSTRING_ITERATOR(j);
-					if (character == UNICODE_CR
-						|| character == UNICODE_NEL
-						|| character == UNICODE_LF) 
-					{
-						break;
-					}
-					else if (character == UNICODE_SPACE) // space
-					{
-						len += mSpaceWidth * 2.0f * mViewportAspectCoef;
-					}
-					else 
-					{
-						len += mFont->getGlyphAspectRatio(character) * mCharHeight * 2.0f * mViewportAspectCoef;
-					}
-				}
+        // Use iterator
+        DisplayString::iterator i, iend;
+        iend = mCaption.end();
+        bool newLine = true;
+        for( i = mCaption.begin(); i != iend; ++i )
+        {
+            if( newLine )
+            {
+                Real len = 0.0f;
+                for( DisplayString::iterator j = i; j != iend; j++ )
+                {
+                    Font::CodePoint character = j.getCharacter();
+                    if (character == UNICODE_CR
+                        || character == UNICODE_NEL
+                        || character == UNICODE_LF) 
+                    {
+                        break;
+                    }
+                    else if (character == UNICODE_SPACE) // space
+                    {
+                        len += mSpaceWidth * 2.0f * mViewportAspectCoef;
+                    }
+                    else 
+                    {
+                        len += mFont->getGlyphAspectRatio(character) * mCharHeight * 2.0f * mViewportAspectCoef;
+                    }
+                }
 
-				if( mAlignment == Right )
-					left -= len;
-				else if( mAlignment == Center )
-					left -= len * 0.5f;
+                if( mAlignment == Right )
+                    left -= len;
+                else if( mAlignment == Center )
+                    left -= len * 0.5f;
 
-				newLine = false;
-			}
+                newLine = false;
+            }
 
-			Font::CodePoint character = OGRE_DEREF_DISPLAYSTRING_ITERATOR(i);
-			if (character == UNICODE_CR
-				|| character == UNICODE_NEL
-				|| character == UNICODE_LF)
-			{
-				left = _getDerivedLeft() * 2.0f - 1.0f;
-				top -= mCharHeight * 2.0f;
-				newLine = true;
-				// Also reduce tri count
-				mRenderOp.vertexData->vertexCount -= 6;
+            Font::CodePoint character = i.getCharacter();
+            if (character == UNICODE_CR
+                || character == UNICODE_NEL
+                || character == UNICODE_LF)
+            {
+                left = _getDerivedLeft() * 2.0f - 1.0f;
+                top -= mCharHeight * 2.0f;
+                newLine = true;
+                // Also reduce tri count
+                mRenderOp.vertexData->vertexCount -= 6;
 
-				// consume CR/LF in one
-				if (character == UNICODE_CR)
-				{
-					DisplayString::iterator peeki = i;
-					peeki++;
-					if (peeki != iend && OGRE_DEREF_DISPLAYSTRING_ITERATOR(peeki) == UNICODE_LF)
-					{
-						i = peeki; // skip both as one newline
-						// Also reduce tri count
-						mRenderOp.vertexData->vertexCount -= 6;
-					}
+                // consume CR/LF in one
+                if (character == UNICODE_CR)
+                {
+                    DisplayString::iterator peeki = i;
+                    peeki++;
+                    if (peeki != iend && peeki.getCharacter() == UNICODE_LF)
+                    {
+                        i = peeki; // skip both as one newline
+                        // Also reduce tri count
+                        mRenderOp.vertexData->vertexCount -= 6;
+                    }
 
-				}
-				continue;
-			}
-			else if (character == UNICODE_SPACE) // space
-			{
-				// Just leave a gap, no tris
-				left += mSpaceWidth * 2.0f * mViewportAspectCoef;
-				// Also reduce tri count
-				mRenderOp.vertexData->vertexCount -= 6;
-				continue;
-			}
+                }
+                continue;
+            }
+            else if (character == UNICODE_SPACE) // space
+            {
+                // Just leave a gap, no tris
+                left += mSpaceWidth * 2.0f * mViewportAspectCoef;
+                // Also reduce tri count
+                mRenderOp.vertexData->vertexCount -= 6;
+                continue;
+            }
 
-			Real horiz_height = mFont->getGlyphAspectRatio(character) * mViewportAspectCoef ;
-			const Font::UVRect& uvRect = mFont->getGlyphTexCoords(character);
+            Real horiz_height = mFont->getGlyphAspectRatio(character) * mViewportAspectCoef ;
+            const Font::UVRect& uvRect = mFont->getGlyphTexCoords(character);
 
-			// each vert is (x, y, z, u, v)
-			//-------------------------------------------------------------------------------------
-			// First tri
-			//
-			// Upper left
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.left;
-			*pVert++ = uvRect.top;
+            // each vert is (x, y, z, u, v)
+            //-------------------------------------------------------------------------------------
+            // First tri
+            //
+            // Upper left
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.left;
+            *pVert++ = uvRect.top;
 
-			top -= mCharHeight * 2.0f;
+            top -= mCharHeight * 2.0f;
 
-			// Bottom left
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.left;
-			*pVert++ = uvRect.bottom;
+            // Bottom left
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.left;
+            *pVert++ = uvRect.bottom;
 
-			top += mCharHeight * 2.0f;
-			left += horiz_height * mCharHeight * 2.0f;
+            top += mCharHeight * 2.0f;
+            left += horiz_height * mCharHeight * 2.0f;
 
-			// Top right
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.right;
-			*pVert++ = uvRect.top;
-			//-------------------------------------------------------------------------------------
+            // Top right
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.right;
+            *pVert++ = uvRect.top;
+            //-------------------------------------------------------------------------------------
 
-			//-------------------------------------------------------------------------------------
-			// Second tri
-			//
-			// Top right (again)
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.right;
-			*pVert++ = uvRect.top;
+            //-------------------------------------------------------------------------------------
+            // Second tri
+            //
+            // Top right (again)
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.right;
+            *pVert++ = uvRect.top;
 
-			top -= mCharHeight * 2.0f;
-			left -= horiz_height  * mCharHeight * 2.0f;
+            top -= mCharHeight * 2.0f;
+            left -= horiz_height  * mCharHeight * 2.0f;
 
-			// Bottom left (again)
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.left;
-			*pVert++ = uvRect.bottom;
+            // Bottom left (again)
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.left;
+            *pVert++ = uvRect.bottom;
 
-			left += horiz_height  * mCharHeight * 2.0f;
+            left += horiz_height  * mCharHeight * 2.0f;
 
-			// Bottom right
-			*pVert++ = left;
-			*pVert++ = top;
-			*pVert++ = -1.0;
-			*pVert++ = uvRect.right;
-			*pVert++ = uvRect.bottom;
-			//-------------------------------------------------------------------------------------
+            // Bottom right
+            *pVert++ = left;
+            *pVert++ = top;
+            *pVert++ = -1.0;
+            *pVert++ = uvRect.right;
+            *pVert++ = uvRect.bottom;
+            //-------------------------------------------------------------------------------------
 
-			// Go back up with top
-			top += mCharHeight * 2.0f;
+            // Go back up with top
+            top += mCharHeight * 2.0f;
 
-			float currentWidth = (left + 1)/2 - _getDerivedLeft();
-			if (currentWidth > largestWidth)
-			{
-				largestWidth = currentWidth;
+            float currentWidth = (left + 1)/2 - _getDerivedLeft();
+            if (currentWidth > largestWidth)
+            {
+                largestWidth = currentWidth;
 
-			}
-		}
-		// Unlock vertex buffer
-		vbuf->unlock();
+            }
+        }
 
-		if (mMetricsMode == GMM_PIXELS)
-		{
-			// Derive parametric version of dimensions
-			Real vpWidth;
-			vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
+        if (mMetricsMode == GMM_PIXELS)
+        {
+            // Derive parametric version of dimensions
+            Real vpWidth;
+            vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
 
-			largestWidth *= vpWidth;
-		};
+            largestWidth *= vpWidth;
+        };
 
-		if (getWidth() < largestWidth)
-			setWidth(largestWidth);
+        if (getWidth() < largestWidth)
+            setWidth(largestWidth);
     }
 
-	void TextAreaOverlayElement::updateTextureGeometry()
-	{
-		// Nothing to do, we combine positions and textures
-	}
+    void TextAreaOverlayElement::updateTextureGeometry()
+    {
+        // Nothing to do, we combine positions and textures
+    }
 
     void TextAreaOverlayElement::setCaption( const DisplayString& caption )
     {
         mCaption = caption;
-		mGeomPositionsOutOfDate = true;
-		mGeomUVsOutOfDate = true;
+        mGeomPositionsOutOfDate = true;
+        mGeomUVsOutOfDate = true;
     }
 
-    void TextAreaOverlayElement::setFontName( const String& font )
+    void TextAreaOverlayElement::setFontName( const String& font, const String& group )
     {
-        mFont = FontManager::getSingleton().getByName( font );
-        if (mFont.isNull())
-			OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find font " + font,
-				"TextAreaOverlayElement::setFontName" );
-		
-		mGeomPositionsOutOfDate = true;
-		mGeomUVsOutOfDate = true;
+        mFont = FontManager::getSingleton().getByName(font, group);
+        if (!mFont)
+            OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find font " + font,
+                "TextAreaOverlayElement::setFontName" );
+        mMaterial.reset();
+
+        mGeomPositionsOutOfDate = true;
+        mGeomUVsOutOfDate = true;
     }
     const String& TextAreaOverlayElement::getFontName() const
     {
@@ -377,20 +399,21 @@ namespace Ogre {
         }
         mGeomPositionsOutOfDate = true;
     }
-	Real TextAreaOverlayElement::getCharHeight() const
-	{
-		if (mMetricsMode == GMM_PIXELS)
-		{
-			return mPixelCharHeight;
-		}
-		else
-		{
-			return mCharHeight;
-		}
-	}
+    Real TextAreaOverlayElement::getCharHeight() const
+    {
+        if (mMetricsMode == GMM_PIXELS)
+        {
+            return mPixelCharHeight;
+        }
+        else
+        {
+            return mCharHeight;
+        }
+    }
 
     void TextAreaOverlayElement::setSpaceWidth( Real width )
     {
+        mSpaceWidthOverridden = true;
         if (mMetricsMode != GMM_RELATIVE)
         {
             mPixelSpaceWidth = static_cast<unsigned short>(width);
@@ -402,17 +425,17 @@ namespace Ogre {
 
         mGeomPositionsOutOfDate = true;
     }
-	Real TextAreaOverlayElement::getSpaceWidth() const
-	{
-		if (mMetricsMode == GMM_PIXELS)
-		{
-			return mPixelSpaceWidth;
-		}
-		else
-		{
-			return mSpaceWidth;
-		}
-	}
+    Real TextAreaOverlayElement::getSpaceWidth() const
+    {
+        if (mMetricsMode == GMM_PIXELS)
+        {
+            return mPixelSpaceWidth;
+        }
+        else
+        {
+            return mSpaceWidth;
+        }
+    }
 
     //---------------------------------------------------------------------
     TextAreaOverlayElement::~TextAreaOverlayElement()
@@ -427,16 +450,16 @@ namespace Ogre {
     //---------------------------------------------------------------------
     const MaterialPtr& TextAreaOverlayElement::getMaterial(void) const
     {
-		// On-demand load
-		// Moved from setFontName to avoid issues with background parsing of scripts
-		if (mMaterial.isNull() && !mFont.isNull())
-		{
-			mFont->load();
-			// Ugly hack, but we need to override for lazy-load
-			*const_cast<MaterialPtr*>(&mMaterial) = mFont->getMaterial();
-			mMaterial->setDepthCheckEnabled(false);
-			mMaterial->setLightingEnabled(false);
-		}
+        // On-demand load
+        // Moved from setFontName to avoid issues with background parsing of scripts
+        if (!mMaterial && mFont)
+        {
+            mFont->load();
+            // Ugly hack, but we need to override for lazy-load
+            *const_cast<MaterialPtr*>(&mMaterial) = mFont->getMaterial();
+            mMaterial->setDepthCheckEnabled(false);
+            mMaterial->setLightingEnabled(false);
+        }
         return mMaterial;
     }
     //---------------------------------------------------------------------
@@ -445,11 +468,6 @@ namespace Ogre {
         op = mRenderOp;
     }
     //---------------------------------------------------------------------
-    void TextAreaOverlayElement::setMaterialName(const String& matName)
-    {
-        OverlayElement::setMaterialName(matName);
-    }
-    //---------------------------------------------------------------------
     void TextAreaOverlayElement::addBaseParameters(void)
     {
         OverlayElement::addBaseParameters();
@@ -535,8 +553,8 @@ namespace Ogre {
         HardwareVertexBufferSharedPtr vbuf = 
             mRenderOp.vertexData->vertexBufferBinding->getBuffer(COLOUR_BINDING);
 
-        RGBA* pDest = static_cast<RGBA*>(
-            vbuf->lock(HardwareBuffer::HBL_DISCARD) );
+        HardwareBufferLockGuard vbufLock(vbuf, HardwareBuffer::HBL_DISCARD);
+        RGBA* pDest = static_cast<RGBA*>(vbufLock.pData);
 
         for (size_t i = 0; i < mAllocSize; ++i)
         {
@@ -549,83 +567,81 @@ namespace Ogre {
             *pDest++ = bottomColour;
             *pDest++ = bottomColour;
         }
-        vbuf->unlock();
+    }
+    //-----------------------------------------------------------------------
+    void TextAreaOverlayElement::setMetricsMode(GuiMetricsMode gmm)
+    {
+        Real vpWidth, vpHeight;
+        vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
+        vpHeight = (Real) (OverlayManager::getSingleton().getViewportHeight());
 
-	}
-	//-----------------------------------------------------------------------
-	void TextAreaOverlayElement::setMetricsMode(GuiMetricsMode gmm)
-	{
-		Real vpWidth, vpHeight;
-		vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
-		vpHeight = (Real) (OverlayManager::getSingleton().getViewportHeight());
+        mViewportAspectCoef = vpHeight/vpWidth;
 
-		mViewportAspectCoef = vpHeight/vpWidth;
+        OverlayElement::setMetricsMode(gmm);
 
-		OverlayElement::setMetricsMode(gmm);
+        switch (mMetricsMode)
+        {
+        case GMM_PIXELS:
+            // set pixel variables based on viewport multipliers
+            mPixelCharHeight = static_cast<unsigned short>(mCharHeight * vpHeight);
+            mPixelSpaceWidth = static_cast<unsigned short>(mSpaceWidth * vpHeight);
+            break;
 
-		switch (mMetricsMode)
-		{
-		case GMM_PIXELS:
-			// set pixel variables based on viewport multipliers
-			mPixelCharHeight = static_cast<unsigned short>(mCharHeight * vpHeight);
-			mPixelSpaceWidth = static_cast<unsigned short>(mSpaceWidth * vpHeight);
-			break;
+        case GMM_RELATIVE_ASPECT_ADJUSTED:
+            // set pixel variables multiplied by the height constant
+            mPixelCharHeight = static_cast<unsigned short>(mCharHeight * 10000.0);
+            mPixelSpaceWidth = static_cast<unsigned short>(mSpaceWidth * 10000.0);
+            break;
 
-		case GMM_RELATIVE_ASPECT_ADJUSTED:
-			// set pixel variables multiplied by the height constant
-			mPixelCharHeight = static_cast<unsigned short>(mCharHeight * 10000.0);
-			mPixelSpaceWidth = static_cast<unsigned short>(mSpaceWidth * 10000.0);
-			break;
+        default:
+            break;
+        }
+    }
+    //-----------------------------------------------------------------------
+    void TextAreaOverlayElement::_update(void)
+    {
+        Real vpWidth, vpHeight;
+        vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
+        vpHeight = (Real) (OverlayManager::getSingleton().getViewportHeight());
 
-		default:
-			break;
-		}
-	}
-	//-----------------------------------------------------------------------
-	void TextAreaOverlayElement::_update(void)
-	{
-		Real vpWidth, vpHeight;
-		vpWidth = (Real) (OverlayManager::getSingleton().getViewportWidth());
-		vpHeight = (Real) (OverlayManager::getSingleton().getViewportHeight());
+        mViewportAspectCoef = vpHeight/vpWidth;
 
-		mViewportAspectCoef = vpHeight/vpWidth;
+        // Check size if pixel-based / relative-aspect-adjusted
+        switch (mMetricsMode)
+        {
+        case GMM_PIXELS:
+            if(mGeomPositionsOutOfDate)
+            {
+                // recalculate character size
+                mCharHeight = (Real) mPixelCharHeight / vpHeight;
+                mSpaceWidth = (Real) mPixelSpaceWidth / vpHeight;
+                mGeomPositionsOutOfDate = true;
+            }
+            break;
 
-		// Check size if pixel-based / relative-aspect-adjusted
-		switch (mMetricsMode)
-		{
-		case GMM_PIXELS:
-			if(OverlayManager::getSingleton().hasViewportChanged() || mGeomPositionsOutOfDate)
-			{
-				// recalculate character size
-				mCharHeight = (Real) mPixelCharHeight / vpHeight;
-				mSpaceWidth = (Real) mPixelSpaceWidth / vpHeight;
-				mGeomPositionsOutOfDate = true;
-			}
-			break;
+        case GMM_RELATIVE_ASPECT_ADJUSTED:
+            if(mGeomPositionsOutOfDate)
+            {
+                // recalculate character size
+                mCharHeight = (Real) mPixelCharHeight / 10000.0f;
+                mSpaceWidth = (Real) mPixelSpaceWidth / 10000.0f;
+                mGeomPositionsOutOfDate = true;
+            }
+            break;
 
-		case GMM_RELATIVE_ASPECT_ADJUSTED:
-			if(OverlayManager::getSingleton().hasViewportChanged() || mGeomPositionsOutOfDate)
-			{
-				// recalculate character size
-				mCharHeight = (Real) mPixelCharHeight / 10000.0f;
-				mSpaceWidth = (Real) mPixelSpaceWidth / 10000.0f;
-				mGeomPositionsOutOfDate = true;
-			}
-			break;
+        default:
+            break;
+        }
 
-		default:
-			break;
-		}
+        OverlayElement::_update();
 
-		OverlayElement::_update();
-
-		if (mColoursChanged && mInitialised)
-		{
-			updateColours();
-			mColoursChanged = false;
-		}
-	}
-	//---------------------------------------------------------------------------------------------
+        if (mColoursChanged && mInitialised)
+        {
+            updateColours();
+            mColoursChanged = false;
+        }
+    }
+    //---------------------------------------------------------------------------------------------
     // Char height command object
     //
     String TextAreaOverlayElement::CmdCharHeight::doGet( const void* target ) const
@@ -658,7 +674,7 @@ namespace Ogre {
     //
     String TextAreaOverlayElement::CmdFontName::doGet( const void* target ) const
     {
-        return static_cast< const TextAreaOverlayElement* >( target )->getFontName();
+        return static_cast< const TextAreaOverlayElement* >( target )->getFont()->getName();
     }
     void TextAreaOverlayElement::CmdFontName::doSet( void* target, const String& val )
     {
