@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
@@ -31,11 +31,10 @@ THE SOFTWARE.
 #include "OgreShaderProgramProcessor.h"
 #include "OgreStringVector.h"
 
-
 namespace Ogre {
 namespace RTShader {
 
-/** \addtogroup Core
+/** \addtogroup Optional
 *  @{
 */
 /** \addtogroup RTShader
@@ -48,41 +47,41 @@ class _OgreRTSSExport GLSLProgramProcessor : public ProgramProcessor
 {
 
 // Interface.
-public:	
+public: 
 
-	/** Class constructor.
-	@param type The type of this program.
-	*/
-	GLSLProgramProcessor			();
+    /** Class constructor.
+    */
+    GLSLProgramProcessor();
 
-	/** Class destructor */
-	virtual ~GLSLProgramProcessor	();
+    /** Class destructor */
+    virtual ~GLSLProgramProcessor();
 
-	/** Return the target language of this processor. */
-	virtual const String&		getTargetLanguage	() const { return TargetLanguage; }
+    /** Return the target language of this processor. */
+    virtual const String& getTargetLanguage() const { return TargetLanguage; }
 
-	/** 
-	@see ProgramProcessor::preCreateGpuPrograms
-	*/
-	virtual bool				preCreateGpuPrograms			(ProgramSet* programSet);
+    /** 
+    @see ProgramProcessor::preCreateGpuPrograms
+    */
+    virtual bool preCreateGpuPrograms(ProgramSet* programSet);
 
-	/** 
-	@see ProgramProcessor::postCreateGpuPrograms
-	*/
-	virtual bool				postCreateGpuPrograms			(ProgramSet* programSet);
+    /** 
+    @see ProgramProcessor::postCreateGpuPrograms
+    */
+    virtual bool postCreateGpuPrograms(ProgramSet* programSet);
 
 
 private:
 
-	/** Bind texture samplers. */
-	void						bindTextureSamplers				(Program* pCpuProgram, GpuProgramPtr pGpuProgram);
+    /** Bind texture samplers. */
+    void bindTextureSamplers(Program* pCpuProgram, GpuProgramPtr pGpuProgram);
 
-	/** Compiles sub shaders (only if the not exists) and attach them to the main gpu program. */
-	void						bindSubShaders					(Program* program, GpuProgramPtr pGpuProgram);
+    /** Compiles sub shaders (only if the not exists) and attach them to the main gpu program. */
+    void bindSubShaders(Program* program, GpuProgramPtr pGpuProgram);
 
-	static String TargetLanguage;
+    static String TargetLanguage;
 
-	StringVector	mLibraryPrograms;		// The list of created library shaders.
+    // The list of created library shaders.
+    StringVector mLibraryPrograms;
 
 };
 
