@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -129,8 +129,6 @@ namespace Ogre {
         /// Default height of each billboard
         Real mDefaultHeight;
 
-        /// Name of the material to use
-        String mMaterialName;
         /// Pointer to the material to use
         MaterialPtr mMaterial;
 
@@ -149,9 +147,9 @@ namespace Ogre {
         bool mAllDefaultRotation;
         bool mWorldSpace;
 
-        typedef list<Billboard*>::type ActiveBillboardList;
-        typedef list<Billboard*>::type FreeBillboardList;
-        typedef vector<Billboard*>::type BillboardPool;
+        typedef std::list<Billboard*> ActiveBillboardList;
+        typedef std::list<Billboard*> FreeBillboardList;
+        typedef std::vector<Billboard*> BillboardPool;
 
         /** Active billboard list.
         @remarks
@@ -179,7 +177,7 @@ namespace Ogre {
         BillboardPool mBillboardPool;
 
         /// The vertex position data for all billboards in this set.
-        VertexData* mVertexData;
+        std::unique_ptr<VertexData> mVertexData;
         /// Shortcut to main buffer (positions, colours, texture coords)
         HardwareVertexBufferSharedPtr mMainBuf;
         /// Locked pointer to buffer
@@ -202,12 +200,12 @@ namespace Ogre {
         Vector3 mCamPos;
 
         /// The vertex index data for all billboards in this set (1 set only)
-        IndexData* mIndexData;
+        std::unique_ptr<IndexData> mIndexData;
 
         /// Flag indicating whether each billboard should be culled separately (default: false)
         bool mCullIndividual;
 
-        typedef vector< Ogre::FloatRect >::type TextureCoordSets;
+        typedef std::vector< Ogre::FloatRect > TextureCoordSets;
         TextureCoordSets mTextureCoords;
 
         /// The type of billboard to render
@@ -526,8 +524,6 @@ namespace Ogre {
         virtual Real getDefaultHeight(void) const;
 
         /** Sets the name of the material to be used for this billboard set.
-        @param name
-            The new name of the material to use for this set.
         */
         virtual void setMaterialName( const String& name, const String& groupName = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME );
 
@@ -536,11 +532,7 @@ namespace Ogre {
         */
         virtual const String& getMaterialName(void) const;
 
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual void _notifyCurrentCamera(Camera* cam);
+        virtual void _notifyCurrentCamera(Camera* cam) override;
 
         /** Begin injection of billboard data; applicable when 
             constructing the BillboardSet for external data use.
@@ -560,28 +552,10 @@ namespace Ogre {
         void setBounds(const AxisAlignedBox& box, Real radius);
 
 
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual const AxisAlignedBox& getBoundingBox(void) const;
-
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual Real getBoundingRadius(void) const;
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual void _updateRenderQueue(RenderQueue* queue);
-
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual const MaterialPtr& getMaterial(void) const;
+        virtual const AxisAlignedBox& getBoundingBox(void) const override;
+        virtual Real getBoundingRadius(void) const override;
+        virtual void _updateRenderQueue(RenderQueue* queue) override;
+        virtual const MaterialPtr& getMaterial(void) const override;
 
         /** Sets the name of the material to be used for this billboard set.
         @param material
@@ -589,17 +563,9 @@ namespace Ogre {
          */
         virtual void setMaterial( const MaterialPtr& material );
 
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual void getRenderOperation(RenderOperation& op);
 
-        /** Overridden from MovableObject
-        @see
-            MovableObject
-        */
-        virtual void getWorldTransforms(Matrix4* xform) const;
+        virtual void getRenderOperation(RenderOperation& op) override;
+        virtual void getWorldTransforms(Matrix4* xform) const override;
 
         /** Internal callback used by Billboards to notify their parent that they have been resized.
         */
@@ -716,11 +682,9 @@ namespace Ogre {
         */
         virtual bool getUseAccurateFacing(void) const { return mAccurateFacing; }
 
-        /** Overridden from MovableObject */
-        virtual const String& getMovableType(void) const;
 
-        /** Overridden, see Renderable */
-        Real getSquaredViewDepth(const Camera* cam) const;
+        virtual const String& getMovableType(void) const override;
+        Real getSquaredViewDepth(const Camera* cam) const override;
 
         /** Update the bounds of the billboardset */
         virtual void _updateBounds(void);
@@ -744,7 +708,11 @@ namespace Ogre {
         */
         virtual void setBillboardsInWorldSpace(bool ws) { mWorldSpace = ws; }
 
-        /** BillboardSet can use custom texture coordinates for various billboards. 
+        /** Gets whether billboards are treated as being in world space.
+         */
+        bool getBillboardsInWorldSpace() { return mWorldSpace; }
+
+        /** BillboardSet can use custom texture coordinates for various billboards.
             This is useful for selecting one of many particle images out of a tiled 
             texture sheet, or doing flipbook animation within a single texture.
         @par
@@ -853,6 +821,9 @@ namespace Ogre {
         */
         void notifyBillboardDataChanged(void) { mBillboardDataChanged = true; }
 
+        /** @copydoc MovableObject::_releaseManualHardwareResources */
+        void _releaseManualHardwareResources() { _destroyBuffers(); }
+
     };
 
     /** Factory object for creating BillboardSet instances */
