@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,12 +31,10 @@ THE SOFTWARE.
 #include "OgrePrerequisites.h"
 #include "OgreCommon.h"
 
-#include "OgreString.h"
 #include "OgreMovableObject.h"
 #include "OgreQuaternion.h"
-#include "OgreVector3.h"
+#include "OgreVector.h"
 #include "OgreHardwareBufferManager.h"
-#include "OgreMesh.h"
 #include "OgreRenderable.h"
 #include "OgreResourceGroupManager.h"
 #include "OgreHeaderPrefix.h"
@@ -49,12 +47,12 @@ namespace Ogre {
     *  @{
     */
     /** Defines an instance of a discrete, movable object based on a Mesh.
-    @remarks
-        Ogre generally divides renderable objects into 2 groups, discrete
+
+        %Ogre generally divides renderable objects into 2 groups, discrete
         (separate) and relatively small objects which move around the world,
         and large, sprawling geometry which makes up generally immovable
         scenery, aka 'level geometry'.
-    @par
+
         The Mesh and SubMesh classes deal with the definition of the geometry
         used by discrete movable objects. Entities are actual instances of
         objects based on this geometry in the world. Therefore there is
@@ -68,14 +66,13 @@ namespace Ogre {
         individual changes is kept in the SubEntity class. There is a 1:1
         relationship between SubEntity instances and the SubMesh instances
         associated with the Mesh the Entity is based on.
-    @par
+
         Entity and SubEntity classes are never created directly. Use the
         createEntity method of the SceneManager (passing a model name) to
         create one.
-    @par
+
         Entities are included in the scene by associating them with a
-        SceneNode, using the attachEntity method. See the SceneNode class
-        for full information.
+        SceneNode, using the @ref SceneNode::attachObject method.
     @note
         No functions were declared virtual to improve performance.
     */
@@ -86,9 +83,9 @@ namespace Ogre {
         friend class SubEntity;
     public:
         
-        typedef set<Entity*>::type EntitySet;
-        typedef map<unsigned short, bool>::type SchemeHardwareAnimMap;
-
+        typedef std::set<Entity*> EntitySet;
+        typedef std::map<unsigned short, bool> SchemeHardwareAnimMap;
+        typedef std::vector<SubEntity*> SubEntityList;
     protected:
 
         /** Private constructor (instances cannot be created directly).
@@ -104,7 +101,6 @@ namespace Ogre {
 
         /** List of SubEntities (point to SubMeshes).
         */
-        typedef vector<SubEntity*>::type SubEntityList;
         SubEntityList mSubEntityList;
 
 
@@ -115,19 +111,34 @@ namespace Ogre {
         /// Temp buffer details for software skeletal anim of shared geometry
         TempBlendedBufferInfo mTempSkelAnimInfo;
         /// Vertex data details for software skeletal anim of shared geometry
-        VertexData* mSkelAnimVertexData;
+        std::unique_ptr<VertexData> mSkelAnimVertexData;
         /// Temp buffer details for software vertex anim of shared geometry
         TempBlendedBufferInfo mTempVertexAnimInfo;
         /// Vertex data details for software vertex anim of shared geometry
-        VertexData* mSoftwareVertexAnimVertexData;
+        std::unique_ptr<VertexData> mSoftwareVertexAnimVertexData;
         /// Vertex data details for hardware vertex anim of shared geometry
         /// - separate since we need to s/w anim for shadows whilst still altering
         ///   the vertex data for hardware morphing (pos2 binding)
-        VertexData* mHardwareVertexAnimVertexData;
+        std::unique_ptr<VertexData> mHardwareVertexAnimVertexData;
+
         /// Have we applied any vertex animation to shared geometry?
-        bool mVertexAnimationAppliedThisFrame;
+        bool mVertexAnimationAppliedThisFrame : 1;
         /// Have the temp buffers already had their geometry prepared for use in rendering shadow volumes?
-        bool mPreparedForShadowVolumes;
+        bool mPreparedForShadowVolumes : 1;
+        /// Flag determines whether or not to display skeleton.
+        bool mDisplaySkeleton : 1;
+        /// Current state of the hardware animation as represented by the entities parameters.
+        bool mCurrentHWAnimationState : 1;
+        /// Flag indicating whether to skip automatic updating of the Skeleton's AnimationState.
+        bool mSkipAnimStateUpdates : 1;
+        /// Flag indicating whether to update the main entity skeleton even when an LOD is displayed.
+        bool mAlwaysUpdateMainSkeleton : 1;
+        /// Flag indicating whether to update the bounding box from the bones of the skeleton.
+        bool mUpdateBoundingBoxFromSkeleton : 1;
+        /// Flag indicating whether we have a vertex program in use on any of our subentities.
+        bool mVertexProgramInUse : 1;
+        /// Has this entity been initialised yet?
+        bool mInitialised : 1;
 
         /** Internal method - given vertex data which could be from the Mesh or
             any submesh, finds the temporary blend copy.
@@ -175,11 +186,13 @@ namespace Ogre {
         */
         void finalisePoseNormals(const VertexData* srcData, VertexData* destData);
 
+        /// Number of hardware poses supported by materials.
+        ushort mHardwarePoseCount;
+        ushort mNumBoneMatrices;
         /// Cached bone matrices, including any world transform.
-        Matrix4 *mBoneWorldMatrices;
+        Affine3 *mBoneWorldMatrices;
         /// Cached bone matrices in skeleton local space, might shares with other entity instances.
-        Matrix4 *mBoneMatrices;
-        unsigned short mNumBoneMatrices;
+        Affine3 *mBoneMatrices;
         /// Records the last frame in which animation was updated.
         unsigned long mFrameAnimationLastUpdated;
 
@@ -203,30 +216,18 @@ namespace Ogre {
         */
         bool cacheBoneMatrices(void);
 
-        /// Flag determines whether or not to display skeleton.
-        bool mDisplaySkeleton;
         /** Flag indicating whether hardware animation is supported by this entities materials
             data is saved per scehme number.
         */
         SchemeHardwareAnimMap mSchemeHardwareAnim;
 
-        /// Current state of the hardware animation as represented by the entities parameters.
-        bool mCurrentHWAnimationState;
-
-        /// Number of hardware poses supported by materials.
-        ushort mHardwarePoseCount;
-        /// Flag indicating whether we have a vertex program in use on any of our subentities.
-        bool mVertexProgramInUse;
         /// Counter indicating number of requests for software animation.
         int mSoftwareAnimationRequests;
         /// Counter indicating number of requests for software blended normals.
         int mSoftwareAnimationNormalsRequests;
-        /// Flag indicating whether to skip automatic updating of the Skeleton's AnimationState.
-        bool mSkipAnimStateUpdates;
-        /// Flag indicating whether to update the main entity skeleton even when an LOD is displayed.
-        bool mAlwaysUpdateMainSkeleton;
 
 
+#if !OGRE_NO_MESHLOD
         /// The LOD number of the mesh to use, calculated by _notifyCurrentCamera.
         ushort mMeshLodIndex;
 
@@ -251,18 +252,24 @@ namespace Ogre {
             same number of SubMeshes, therefore we have to allow a separate Entity list
             with each alternate one.
         */
-        typedef vector<Entity*>::type LODEntityList;
+        typedef std::vector<Entity*> LODEntityList;
         LODEntityList mLodEntityList;
-
+#else
+        const ushort mMeshLodIndex;
+        const Real mMeshLodFactorTransformed;
+        const ushort mMinMeshLodIndex;
+        const ushort mMaxMeshLodIndex;
+        const Real mMaterialLodFactor;
+        const Real mMaterialLodFactorTransformed;
+        const ushort mMinMaterialLodIndex;
+        const ushort mMaxMaterialLodIndex;
+#endif
         /** This Entity's personal copy of the skeleton, if skeletally animated.
         */
         SkeletonInstance* mSkeletonInstance;
 
-        /// Has this entity been initialised yet?
-        bool mInitialised;
-
         /// Last parent transform.
-        Matrix4 mLastParentXform;
+        Affine3 mLastParentXform;
 
         /// Mesh state count, used to detect differences.
         size_t mMeshStateCount;
@@ -301,13 +308,13 @@ namespace Ogre {
 
     public:
         /// Contains the child objects (attached to bones) indexed by name.
-        typedef map<String, MovableObject*>::type ChildObjectList;
+        typedef std::vector<MovableObject*> ChildObjectList;
     protected:
         ChildObjectList mChildObjectList;
 
 
         /// Bounding box that 'contains' all the mesh of each child entity.
-        mutable AxisAlignedBox mFullBoundingBox;
+        mutable AxisAlignedBox mFullBoundingBox;  // note: this exists only so that getBoundingBox() can return an AAB by reference
 
         ShadowRenderableList mShadowRenderables;
 
@@ -322,11 +329,10 @@ namespace Ogre {
             HardwareVertexBufferSharedPtr mWBuffer;
             /// Link to current vertex data used to bind (maybe changes).
             const VertexData* mCurrentVertexData;
-            /// Original position buffer source binding.
-            unsigned short mOriginalPosBufferBinding;
             /// Link to SubEntity, only present if SubEntity has it's own geometry.
             SubEntity* mSubEntity;
-
+            /// Original position buffer source binding.
+            ushort mOriginalPosBufferBinding;
 
         public:
             EntityShadowRenderable(Entity* parent,
@@ -357,8 +363,9 @@ namespace Ogre {
         const MeshPtr& getMesh(void) const;
 
         /** Gets a pointer to a SubEntity, ie a part of an Entity.
+         @deprecated use getSubEntities()
         */
-        SubEntity* getSubEntity(unsigned int index) const;
+        SubEntity* getSubEntity(size_t index) const;
 
         /** Gets a pointer to a SubEntity by name
         @remarks 
@@ -367,8 +374,15 @@ namespace Ogre {
         SubEntity* getSubEntity( const String& name ) const;
 
         /** Retrieves the number of SubEntity objects making up this entity.
+        * @deprecated use getSubEntities()
         */
-        unsigned int getNumSubEntities(void) const;
+        size_t getNumSubEntities(void) const;
+
+        /** Retrieves SubEntity objects making up this entity.
+        */
+        const SubEntityList& getSubEntities() const {
+            return mSubEntityList;
+        }
 
         /** Clones this entity and returns a pointer to the clone.
         @remarks
@@ -402,24 +416,29 @@ namespace Ogre {
         */
         void setMaterial(const MaterialPtr& material);
 
-        /** @copydoc MovableObject::_notifyCurrentCamera.
+        /** @copydoc MovableObject::_releaseManualHardwareResources */
+        void _releaseManualHardwareResources();
+        /** @copydoc MovableObject::_restoreManualHardwareResources */
+        void _restoreManualHardwareResources();
+
+        /** @copydoc MovableObject::_notifyCurrentCamera
         */
         void _notifyCurrentCamera(Camera* cam);
 
-        /// @copydoc MovableObject::setRenderQueueGroup.
+        /// @copydoc MovableObject::setRenderQueueGroup
         void setRenderQueueGroup(uint8 queueID);
 
-        /// @copydoc MovableObject::setRenderQueueGroupAndPriority.
+        /// @copydoc MovableObject::setRenderQueueGroupAndPriority
         void setRenderQueueGroupAndPriority(uint8 queueID, ushort priority);
 
-        /** @copydoc MovableObject::getBoundingBox.
+        /** @copydoc MovableObject::getBoundingBox
         */
         const AxisAlignedBox& getBoundingBox(void) const;
 
         /// Merge all the child object Bounds a return it.
         AxisAlignedBox getChildObjectsBoundingBox(void) const;
 
-        /** @copydoc MovableObject::_updateRenderQueue.
+        /** @copydoc MovableObject::_updateRenderQueue
         */
         void _updateRenderQueue(RenderQueue* queue);
 
@@ -454,14 +473,6 @@ namespace Ogre {
         */
         bool getDisplaySkeleton(void) const;
 
-
-        /** Gets a pointer to the entity representing the numbered manual level of detail.
-        @remarks
-            The zero-based index never includes the original entity, unlike
-            Mesh::getLodLevel.
-        */
-        Entity* getManualLodLevel(size_t index) const;
-
         /** Returns the number of manual levels of detail that this entity supports.
         @remarks
             This number never includes the original entity, it is difference
@@ -473,6 +484,14 @@ namespace Ogre {
         */
         ushort getCurrentLodIndex() { return mMeshLodIndex; }
 
+        /** Gets a pointer to the entity representing the numbered manual level of detail.
+        @remarks
+            The zero-based index never includes the original entity, unlike
+            Mesh::getLodLevel.
+        */
+        Entity* getManualLodLevel(size_t index) const;
+
+#if !OGRE_NO_MESHLOD
         /** Sets a level-of-detail bias for the mesh detail of this entity.
         @remarks
             Level of detail reduction is normally applied automatically based on the Mesh
@@ -534,7 +553,7 @@ namespace Ogre {
             LOD will be limited by the number of LOD indexes used in the Material).
         */
         void setMaterialLodBias(Real factor, ushort maxDetailIndex = 0, ushort minDetailIndex = 99);
-
+#endif
         /** Sets whether the polygon mode of this entire entity may be
             overridden by the camera detail settings.
         */
@@ -581,9 +600,11 @@ namespace Ogre {
         /// Detach all MovableObjects previously attached using attachObjectToBone
         void detachAllObjectsFromBone(void);
 
-        typedef MapIterator<ChildObjectList> ChildObjectListIterator;
+        typedef VectorIterator<ChildObjectList> ChildObjectListIterator;
+        /// @deprecated use getAttachedObjects()
+        OGRE_DEPRECATED ChildObjectListIterator getAttachedObjectIterator(void);
         /** Gets an iterator to the list of objects attached to bones on this entity. */
-        ChildObjectListIterator getAttachedObjectIterator(void);
+        const ChildObjectList& getAttachedObjects() const { return mChildObjectList; }
         /** @copydoc MovableObject::getBoundingRadius */
         Real getBoundingRadius(void) const;
 
@@ -592,18 +613,18 @@ namespace Ogre {
         /** @copydoc MovableObject::getWorldBoundingSphere */
         const Sphere& getWorldBoundingSphere(bool derive = false) const;
 
-        /** @copydoc ShadowCaster::getEdgeList. */
+        /** @copydoc ShadowCaster::getEdgeList */
         EdgeData* getEdgeList(void);
-        /** @copydoc ShadowCaster::hasEdgeList. */
+        /** @copydoc ShadowCaster::hasEdgeList */
         bool hasEdgeList(void);
-        /** @copydoc ShadowCaster::getShadowVolumeRenderableIterator. */
+        /** @copydoc ShadowCaster::getShadowVolumeRenderableIterator */
         ShadowRenderableListIterator getShadowVolumeRenderableIterator(
             ShadowTechnique shadowTechnique, const Light* light,
             HardwareIndexBufferSharedPtr* indexBuffer, size_t* indexBufferUsedSize,
-            bool extrudeVertices, Real extrusionDistance, unsigned long flags = 0 );
+            bool extrudeVertices, Real extrusionDistance, unsigned long flags = 0);
 
         /** Internal method for retrieving bone matrix information. */
-        const Matrix4* _getBoneMatrices(void) const { return mBoneMatrices;}
+        const Affine3* _getBoneMatrices(void) const { return mBoneMatrices;}
         /** Internal method for retrieving bone matrix information. */
         unsigned short _getNumBoneMatrices(void) const { return mNumBoneMatrices; }
         /** Returns whether or not this entity is skeletally animated. */
@@ -814,7 +835,7 @@ namespace Ogre {
         /** Resource::Listener hook to notify Entity that a delay-loaded Mesh is
             complete.
         */
-        void backgroundLoadingComplete(Resource* res);
+        void loadingComplete(Resource* res);
 
         /// @copydoc MovableObject::visitRenderables
         void visitRenderables(Renderable::Visitor* visitor, 
@@ -853,6 +874,24 @@ namespace Ogre {
             return mAlwaysUpdateMainSkeleton;
         }
 
+        /** If true, the skeleton of the entity will be used to update the bounding box for culling.
+            Useful if you have skeletal animations that move the bones away from the root.  Otherwise, the
+            bounding box of the mesh in the binding pose will be used.
+        @remarks
+            When true, the bounding box will be generated to only enclose the bones that are used for skinning.
+            Also the resulting bounding box will be expanded by the amount of GetMesh()->getBoneBoundingRadius().
+            The expansion amount can be changed on the mesh to achieve a better fitting bounding box.
+        */
+        void setUpdateBoundingBoxFromSkeleton(bool update);
+
+        /** If true, the skeleton of the entity will be used to update the bounding box for culling.
+            Useful if you have skeletal animations that move the bones away from the root.  Otherwise, the
+            bounding box of the mesh in the binding pose will be used.
+        */
+        bool getUpdateBoundingBoxFromSkeleton() const {
+            return mUpdateBoundingBoxFromSkeleton;
+        }
+
         
     };
 
