@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,22 +35,23 @@ THE SOFTWARE.
 #include "OgreCommon.h"
 #include "OgreColourValue.h"
 #include "OgreBlendMode.h"
+#include "OgreHeaderPrefix.h"
+#include "OgreSharedPtr.h"
 
 namespace Ogre {
 
-	// Forward declaration
-	class MaterialPtr;
+    // Forward declaration
     class LodStrategy;
 
-	/** \addtogroup Core
-	*  @{
-	*/
-	/** \addtogroup Materials
-	*  @{
-	*/
-	/** Class encapsulates rendering properties of an object.
-    @remarks
-    Ogre's material class encapsulates ALL aspects of the visual appearance,
+    /** \addtogroup Core
+    *  @{
+    */
+    /** \addtogroup Materials
+    *  @{
+    */
+    /** Class encapsulates rendering properties of an object.
+
+    %Ogre's material class encapsulates *all* aspects of the visual appearance,
     of an object. It also includes other flags which 
     might not be traditionally thought of as material properties such as 
     culling modes and depth buffer settings, but these affect the 
@@ -59,7 +60,7 @@ namespace Ogre {
     different to Direct3D which treats a material as just the colour 
     components (diffuse, specular) and not texture maps etc. An Ogre 
     Material can be thought of as equivalent to a 'Shader'.
-    @par
+
     A Material can be rendered in multiple different ways depending on the
     hardware available. You may configure a Material to use high-complexity
     fragment shaders, but these won't work on every card; therefore a Technique
@@ -67,9 +68,9 @@ namespace Ogre {
     to create fallback techniques with lower hardware requirements if you decide to
     use advanced features. In addition, you also might want lower-detail techniques
     for distant geometry.
-    @par
-    Each technique can be made up of multiple passes. A fixed-function pass
-    may combine multiple texture layers using multitexrtuing, but Ogre can 
+
+    Each Technique can be made up of multiple passes. A fixed-function Pass
+    may combine multiple texture layers using multitexturing, but Ogre can 
     break that into multiple passes automatically if the active card cannot
     handle that many simultaneous textures. Programmable passes, however, cannot
     be split down automatically, so if the active graphics card cannot handle the
@@ -77,11 +78,11 @@ namespace Ogre {
     which the card can do. If, at the end of the day, the card cannot handle any of the
     techniques which are listed for the material, the engine will render the 
     geometry plain white, which should alert you to the problem.
-    @par
-    Ogre comes configured with a number of default settings for a newly 
+
+    %Ogre comes configured with a number of default settings for a newly 
     created material. These can be changed if you wish by retrieving the 
     default material settings through 
-    SceneManager::getDefaultMaterialSettings. Any changes you make to the 
+    MaterialManager::getDefaultSettings. Any changes you make to the
     Material returned from this method will apply to any materials created 
     from this point onward.
     */
@@ -92,8 +93,9 @@ namespace Ogre {
 
     public:
         /// distance list used to specify LOD
-		typedef vector<Real>::type LodValueList;
+        typedef std::vector<Real> LodValueList;
         typedef ConstVectorIterator<LodValueList> LodValueIterator;
+        typedef std::vector<Technique*> Techniques;
     protected:
 
 
@@ -101,61 +103,52 @@ namespace Ogre {
         */
         void applyDefaults(void);
 
-        typedef vector<Technique*>::type Techniques;
-		/// All techniques, supported and unsupported
+        /// All techniques, supported and unsupported
         Techniques mTechniques;
-		/// Supported techniques of any sort
+        /// Supported techniques of any sort
         Techniques mSupportedTechniques;
-		typedef map<unsigned short, Technique*>::type LodTechniques;
-        typedef map<unsigned short, LodTechniques*>::type BestTechniquesBySchemeList;
-		/** Map of scheme -> list of LOD techniques. 
-			Current scheme is set on MaterialManager,
-			and can be set per Viewport for auto activation.
-		*/
+        typedef std::map<unsigned short, Technique*> LodTechniques;
+        typedef std::map<unsigned short, LodTechniques*> BestTechniquesBySchemeList;
+        /** Map of scheme -> list of LOD techniques. 
+            Current scheme is set on MaterialManager,
+            and can be set per Viewport for auto activation.
+        */
         BestTechniquesBySchemeList mBestTechniquesBySchemeList;
 
         LodValueList mUserLodValues;
         LodValueList mLodValues;
         const LodStrategy *mLodStrategy;
+        /// Text description of why any techniques are not supported
+        String mUnsupportedReasons;
         bool mReceiveShadows;
-		bool mTransparencyCastsShadows;
+        bool mTransparencyCastsShadows;
         /// Does this material require compilation?
         bool mCompilationRequired;
-		/// Text description of why any techniques are not supported
-		String mUnsupportedReasons;
 
-		/** Insert a supported technique into the local collections. */
-		void insertSupportedTechnique(Technique* t);
+        /** Insert a supported technique into the local collections. */
+        void insertSupportedTechnique(Technique* t);
 
-		/** Clear the best technique list.
-		*/
-		void clearBestTechniqueList(void);
+        /** Clear the best technique list.
+        */
+        void clearBestTechniqueList(void);
 
-		/** Overridden from Resource.
-		*/
-		void prepareImpl(void);
+        void prepareImpl(void) override;
+        void unprepareImpl(void) override;
+        void loadImpl(void) override;
 
-		/** Overridden from Resource.
-		*/
-		void unprepareImpl(void);
-
-		/** Overridden from Resource.
-		*/
-		void loadImpl(void);
-
-		/** Unloads the material, frees resources etc.
-		@see
-		Resource
-		*/
-		void unloadImpl(void);
-		/// @copydoc Resource::calculateSize
-		size_t calculateSize(void) const { return 0; } // TODO 
+        /** Unloads the material, frees resources etc.
+        @see
+        Resource
+        */
+        void unloadImpl(void);
+        /// @copydoc Resource::calculateSize
+        size_t calculateSize(void) const;
     public:
 
         /** Constructor - use resource manager's create method rather than this.
         */
-		Material(ResourceManager* creator, const String& name, ResourceHandle handle,
-			const String& group, bool isManual = false, ManualResourceLoader* loader = 0);
+        Material(ResourceManager* creator, const String& name, ResourceHandle handle,
+            const String& group, bool isManual = false, ManualResourceLoader* loader = 0);
 
         ~Material();
         /** Assignment operator to allow easy copying between materials.
@@ -184,17 +177,17 @@ namespace Ogre {
         /** Returns whether or not objects using this material will receive shadows. */
         bool getReceiveShadows(void) const { return mReceiveShadows; }
 
-		/** Sets whether objects using this material be classified as opaque to	the shadow caster system.
-		@remarks
-		This method allows a material to cast a shadow, even if it is transparent.
-		By default, transparent materials neither cast nor receive shadows. Shadows
-		will not be cast on any objects unless the scene is set up to support shadows 
-		(@see SceneManager::setShadowTechnique), and not all techniques cast
-		shadows on all objects.
-		*/
-		void setTransparencyCastsShadows(bool enabled) { mTransparencyCastsShadows = enabled; }
-		/** Returns whether or not objects using this material be classified as opaque to the shadow caster system. */
-		bool getTransparencyCastsShadows(void) const { return mTransparencyCastsShadows; }
+        /** Sets whether objects using this material be classified as opaque to the shadow caster system.
+        @remarks
+        This method allows a material to cast a shadow, even if it is transparent.
+        By default, transparent materials neither cast nor receive shadows. Shadows
+        will not be cast on any objects unless the scene is set up to support shadows 
+        (@see SceneManager::setShadowTechnique), and not all techniques cast
+        shadows on all objects.
+        */
+        void setTransparencyCastsShadows(bool enabled) { mTransparencyCastsShadows = enabled; }
+        /** Returns whether or not objects using this material be classified as opaque to the shadow caster system. */
+        bool getTransparencyCastsShadows(void) const { return mTransparencyCastsShadows; }
 
         /** Creates a new Technique for this Material.
         @remarks
@@ -211,44 +204,60 @@ namespace Ogre {
             return the first one in the technique list which is supported by the hardware.
         */
         Technique* createTechnique(void);
-        /** Gets the indexed technique. */
-        Technique* getTechnique(unsigned short index);
+        /** Gets the indexed technique.
+         * @deprecated use getTechniques()  */
+        Technique* getTechnique(unsigned short index) const;
         /** searches for the named technique.
             Return 0 if technique with name is not found
         */
-        Technique* getTechnique(const String& name);
-		/** Retrieves the number of techniques. */
+        Technique* getTechnique(const String& name) const;
+        /** Retrieves the number of techniques.
+         * @deprecated use getTechniques()  */
         unsigned short getNumTechniques(void) const;
-        /** Removes the technique at the given index. */		
-        void removeTechnique(unsigned short index);		
+        /** Removes the technique at the given index. */        
+        void removeTechnique(unsigned short index);     
         /** Removes all the techniques in this Material. */
         void removeAllTechniques(void);
         typedef VectorIterator<Techniques> TechniqueIterator;
-        /** Get an iterator over the Techniques in this Material. */
-        TechniqueIterator getTechniqueIterator(void);
-        /** Gets an iterator over all the Techniques which are supported by the current card. 
+        /** Get an iterator over the Techniques in this Material.
+         * @deprecated use getTechniques() */
+        OGRE_DEPRECATED TechniqueIterator getTechniqueIterator(void);
+
+        /** Get the Techniques in this Material. */
+        const Techniques& getTechniques(void) const {
+            return mTechniques;
+        }
+
+        /** Gets all the Techniques which are supported by the current card.
         @remarks
             The supported technique list is only available after this material has been compiled,
             which typically happens on loading the material. Therefore, if this method returns
             an empty list, try calling Material::load.
         */
-        TechniqueIterator getSupportedTechniqueIterator(void);
-		
-		/** Gets the indexed supported technique. */
-        Technique* getSupportedTechnique(unsigned short index);
-		/** Retrieves the number of supported techniques. */
-        unsigned short getNumSupportedTechniques(void) const;
-		/** Gets a string explaining why any techniques are not supported. */
-		const String& getUnsupportedTechniquesExplanation() const { return mUnsupportedReasons; }
+        const Techniques& getSupportedTechniques(void) const {
+            return mSupportedTechniques;
+        }
+
+        /// @deprecated use getSupportedTechniques()
+        OGRE_DEPRECATED TechniqueIterator getSupportedTechniqueIterator(void);
+        
+        /** Gets the indexed supported technique.
+         * @deprecated use getSupportedTechniques() */
+        OGRE_DEPRECATED Technique* getSupportedTechnique(unsigned short index);
+        /** Retrieves the number of supported techniques.
+         * @deprecated use getSupportedTechniques() */
+        OGRE_DEPRECATED unsigned short getNumSupportedTechniques(void) const;
+        /** Gets a string explaining why any techniques are not supported. */
+        const String& getUnsupportedTechniquesExplanation() const { return mUnsupportedReasons; }
 
         /** Gets the number of levels-of-detail this material has in the 
-			given scheme, based on Technique::setLodIndex. 
+            given scheme, based on Technique::setLodIndex. 
         @remarks
             Note that this will not be up to date until the material has been compiled.
         */
         unsigned short getNumLodLevels(unsigned short schemeIndex) const;
         /** Gets the number of levels-of-detail this material has in the 
-			given scheme, based on Technique::setLodIndex. 
+            given scheme, based on Technique::setLodIndex. 
         @remarks
             Note that this will not be up to date until the material has been compiled.
         */
@@ -263,22 +272,22 @@ namespace Ogre {
             The best supported technique is only available after this material has been compiled,
             which typically happens on loading the material. Therefore, if this method returns
             NULL, try calling Material::load.
-		@param lodIndex The material lod index to use
-		@param rend Optional parameter specifying the Renderable that is requesting
-			this technique. Only used if no valid technique for the active material 
-			scheme is found, at which point it is passed to 
-			MaterialManager::Listener::handleSchemeNotFound as information.
+        @param lodIndex The material LOD index to use
+        @param rend Optional parameter specifying the Renderable that is requesting
+            this technique. Only used if no valid technique for the active material 
+            scheme is found, at which point it is passed to 
+            MaterialManager::Listener::handleSchemeNotFound as information.
         */
         Technique* getBestTechnique(unsigned short lodIndex = 0, const Renderable* rend = 0);
 
 
         /** Creates a new copy of this material with the same settings but a new name.
-		@param newName The name for the cloned material
-		@param changeGroup If true, the resource group of the clone is changed
-		@param newGroup Only required if changeGroup is true; the new group to assign
+        @param newName The name for the cloned material
+        @param changeGroup If true, the resource group of the clone is changed
+        @param newGroup Only required if changeGroup is true; the new group to assign
         */
         MaterialPtr clone(const String& newName, bool changeGroup = false, 
-			const String& newGroup = StringUtil::BLANK) const;
+            const String& newGroup = BLANKSTRING) const;
 
         /** Copies the details of this material into another, preserving the target's handle and name
         (unlike operator=) but copying everything else.
@@ -328,17 +337,9 @@ namespace Ogre {
             property there.
         @see Pass::setAmbient
         */
-        void setAmbient(Real red, Real green, Real blue);
+        void setAmbient(float red, float green, float blue);
 
-        /** Sets the ambient colour reflectance properties for every Pass in every Technique.
-        @note
-            This property has been moved to the Pass class, which is accessible via the 
-            Technique. For simplicity, this method allows you to set these properties for 
-            every current Technique, and for every current Pass within those Techniques. If 
-            you need more precision, retrieve the Technique and Pass instances and set the
-            property there.
-        @see Pass::setAmbient
-        */
+        /// @overload
         void setAmbient(const ColourValue& ambient);
 
         /** Sets the diffuse colour reflectance properties of every Pass in every Technique.
@@ -350,17 +351,9 @@ namespace Ogre {
             property there.
         @see Pass::setDiffuse
         */
-        void setDiffuse(Real red, Real green, Real blue, Real alpha);
+        void setDiffuse(float red, float green, float blue, float alpha);
 
-        /** Sets the diffuse colour reflectance properties of every Pass in every Technique.
-        @note
-            This property has been moved to the Pass class, which is accessible via the 
-            Technique. For simplicity, this method allows you to set these properties for 
-            every current Technique, and for every current Pass within those Techniques. If 
-            you need more precision, retrieve the Technique and Pass instances and set the
-            property there.
-        @see Pass::setDiffuse
-        */
+        /// @overload
         void setDiffuse(const ColourValue& diffuse);
 
         /** Sets the specular colour reflectance properties of every Pass in every Technique.
@@ -372,17 +365,9 @@ namespace Ogre {
             property there.
         @see Pass::setSpecular
         */
-        void setSpecular(Real red, Real green, Real blue, Real alpha);
+        void setSpecular(float red, float green, float blue, float alpha);
 
-        /** Sets the specular colour reflectance properties of every Pass in every Technique.
-        @note
-            This property has been moved to the Pass class, which is accessible via the 
-            Technique. For simplicity, this method allows you to set these properties for 
-            every current Technique, and for every current Pass within those Techniques. If 
-            you need more precision, retrieve the Technique and Pass instances and set the
-            property there.
-        @see Pass::setSpecular
-        */
+        /// @overload
         void setSpecular(const ColourValue& specular);
 
         /** Sets the shininess properties of every Pass in every Technique.
@@ -405,20 +390,12 @@ namespace Ogre {
             property there.
         @see Pass::setSelfIllumination
         */
-        void setSelfIllumination(Real red, Real green, Real blue);
+        void setSelfIllumination(float red, float green, float blue);
 
-        /** Sets the amount of self-illumination of every Pass in every Technique.
-        @note
-            This property has been moved to the Pass class, which is accessible via the 
-            Technique. For simplicity, this method allows you to set these properties for 
-            every current Technique, and for every current Pass within those Techniques. If 
-            you need more precision, retrieve the Technique and Pass instances and set the
-            property there.
-        @see Pass::setSelfIllumination
-        */
+        /// @overload
         void setSelfIllumination(const ColourValue& selfIllum);
 
-		/** Sets whether or not each Pass renders with depth-buffer checking on or not.
+        /** Sets whether or not each Pass renders with depth-buffer checking on or not.
         @note
             This property has been moved to the Pass class, which is accessible via the 
             Technique. For simplicity, this method allows you to set these properties for 
@@ -451,7 +428,7 @@ namespace Ogre {
         */
         void setDepthFunction( CompareFunction func );
 
-		/** Sets whether or not colour buffer writing is enabled for each Pass.
+        /** Sets whether or not colour buffer writing is enabled for each Pass.
         @note
             This property has been moved to the Pass class, which is accessible via the 
             Technique. For simplicity, this method allows you to set these properties for 
@@ -459,8 +436,13 @@ namespace Ogre {
             you need more precision, retrieve the Technique and Pass instances and set the
             property there.
         @see Pass::setColourWriteEnabled
-		*/
-		void setColourWriteEnabled(bool enabled);
+        */
+        void setColourWriteEnabled(bool enabled);
+
+        /** Sets which colour buffer channels are enabled for writing for each Pass.
+         @see Pass::setColourWriteEnabled
+         */
+        void setColourWriteEnabled(bool red, bool green, bool blue, bool alpha);
 
         /** Sets the culling mode for each pass  based on the 'vertex winding'.
         @note
@@ -519,7 +501,7 @@ namespace Ogre {
             bool overrideScene,
             FogMode mode = FOG_NONE,
             const ColourValue& colour = ColourValue::White,
-            Real expDensity = 0.001, Real linearStart = 0.0, Real linearEnd = 1.0 );
+            Real expDensity = 0.001f, Real linearStart = 0.0f, Real linearEnd = 1.0f );
 
         /** Sets the depth bias to be used for each Pass.
         @note
@@ -605,33 +587,53 @@ namespace Ogre {
             method to determine the distance at which the lowe levels of detail kick in.
             The decision about what distance is actually used is a combination of this
             and the LOD bias applied to both the current Camera and the current Entity.
-        @param lodValues A vector of Reals which indicate the lod value at which to 
+        @param lodValues A vector of Reals which indicate the LOD value at which to 
             switch to lower details. They are listed in LOD index order, starting at index
             1 (ie the first level down from the highest level 0, which automatically applies
             from a value of 0). These are 'user values', before being potentially 
-			transformed by the strategy, so for the distance strategy this is an
-			unsquared distance for example.
+            transformed by the strategy, so for the distance strategy this is an
+            unsquared distance for example.
         */
         void setLodLevels(const LodValueList& lodValues);
-        /** Gets an iterator over the list of values at which each LOD comes into effect. 
+
+        /** Gets the list of values transformed by the LodStrategy at which each LOD comes into effect.
         @remarks
             Note that the iterator returned from this method is not totally analogous to 
             the one passed in by calling setLodLevels - the list includes a zero
             entry at the start (since the highest LOD starts at value 0). Also, the
-			values returned are after being transformed by LodStrategy::transformUserValue.
+            values returned are after being transformed by LodStrategy::transformUserValue.
         */
-        LodValueIterator getLodValueIterator(void) const;
+        const LodValueList& getLodValues(void) const {
+            return mLodValues;
+        }
+
+        /// @deprecated use getLodValues()
+        OGRE_DEPRECATED LodValueIterator getLodValueIterator(void) const;
+
+        /** Gets the user-defined list of values which are internally transfomed by the LodStrategy.
+        @remarks
+            Note that the iterator returned from this method is not totally analogous to 
+            the one passed in by calling setLodLevels - the list includes a zero
+            entry at the start (since the highest LOD starts at value 0). Also, the
+            values returned are after being transformed by LodStrategy::transformUserValue.
+        */
+        const LodValueList& getUserLodValues(void) const {
+            return mUserLodValues;
+        }
+
+        /// @deprecated use getUserLodValues()
+        OGRE_DEPRECATED LodValueIterator getUserLodValueIterator(void) const;
 
         /** Gets the LOD index to use at the given value. 
-		@note The value passed in is the 'transformed' value. If you are dealing with
-		an original source value (e.g. distance), use LodStrategy::transformUserValue
-		to turn this into a lookup value.
-		*/
+        @note The value passed in is the 'transformed' value. If you are dealing with
+        an original source value (e.g. distance), use LodStrategy::transformUserValue
+        to turn this into a lookup value.
+        */
         ushort getLodIndex(Real value) const;
 
-        /** Get lod strategy used by this material. */
+        /** Get LOD strategy used by this material. */
         const LodStrategy *getLodStrategy() const;
-        /** Set the lod strategy used by this material. */
+        /** Set the LOD strategy used by this material. */
         void setLodStrategy(LodStrategy *lodStrategy);
 
         /** @copydoc Resource::touch
@@ -657,75 +659,21 @@ namespace Ogre {
         */
         bool applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply = true) const;
 
-	    /** Gets the compilation status of the material.
+        /** Gets the compilation status of the material.
         @return True if the material needs recompilation.
         */
-	    bool getCompilationRequired() const
-	    {
-		    return mCompilationRequired;
-	    }
+        bool getCompilationRequired() const
+        {
+            return mCompilationRequired;
+        }
 
 
     };
-
-	/** Specialisation of SharedPtr to allow SharedPtr to be assigned to MaterialPtr 
-	@note Has to be a subclass since we need operator=.
-	We could templatise this instead of repeating per Resource subclass, 
-	except to do so requires a form VC6 does not support i.e.
-	ResourceSubclassPtr<T> : public SharedPtr<T>
-	*/
-	class _OgreExport MaterialPtr : public SharedPtr<Material> 
-	{
-	public:
-		MaterialPtr() : SharedPtr<Material>() {}
-		explicit MaterialPtr(Material* rep) : SharedPtr<Material>(rep) {}
-		MaterialPtr(const MaterialPtr& r) : SharedPtr<Material>(r) {} 
-		MaterialPtr(const ResourcePtr& r) : SharedPtr<Material>()
-		{
-			// lock & copy other mutex pointer
-            OGRE_MUTEX_CONDITIONAL(r.OGRE_AUTO_MUTEX_NAME)
-            {
-			    OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
-			    OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
-			    pRep = static_cast<Material*>(r.getPointer());
-			    pUseCount = r.useCountPointer();
-			    if (pUseCount)
-			    {
-				    ++(*pUseCount);
-			    }
-            }
-		}
-
-		/// Operator used to convert a ResourcePtr to a MaterialPtr
-		MaterialPtr& operator=(const ResourcePtr& r)
-		{
-			if (pRep == static_cast<Material*>(r.getPointer()))
-				return *this;
-			release();
-			// lock & copy other mutex pointer
-            OGRE_MUTEX_CONDITIONAL(r.OGRE_AUTO_MUTEX_NAME)
-            {
-			    OGRE_LOCK_MUTEX(*r.OGRE_AUTO_MUTEX_NAME)
-			    OGRE_COPY_AUTO_SHARED_MUTEX(r.OGRE_AUTO_MUTEX_NAME)
-			    pRep = static_cast<Material*>(r.getPointer());
-			    pUseCount = r.useCountPointer();
-			    if (pUseCount)
-			    {
-				    ++(*pUseCount);
-			    }
-            }
-			else
-			{
-				// RHS must be a null pointer
-				assert(r.isNull() && "RHS must be null if it has no mutex!");
-				setNull();
-			}
-			return *this;
-		}
-	};
-	/** @} */
-	/** @} */
+    /** @} */
+    /** @} */
 
 } //namespace 
 
+#include "OgreHeaderSuffix.h"
+
 #endif
