@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,17 +31,12 @@ THE SOFTWARE.
 #include "OgrePrerequisites.h"
 
 #include "OgreResource.h"
-#include "OgreVertexIndexData.h"
 #include "OgreAxisAlignedBox.h"
 #include "OgreVertexBoneAssignment.h"
-#include "OgreIteratorWrappers.h"
-#include "OgreHardwareVertexBuffer.h"
-#include "OgreSkeleton.h"
 #include "OgreAnimation.h"
 #include "OgreAnimationTrack.h"
-#include "OgrePose.h"
-#include "OgreDataStream.h"
 #include "OgreHeaderPrefix.h"
+#include "OgreSharedPtr.h"
 
 
 namespace Ogre {
@@ -55,7 +50,6 @@ namespace Ogre {
     */
 
     struct MeshLodUsage;
-    struct LodConfig;
     class LodStrategy;
 
     /** Resource holding data about 3D mesh.
@@ -94,18 +88,20 @@ namespace Ogre {
     {
         friend class SubMesh;
         friend class MeshSerializerImpl;
+        friend class MeshSerializerImpl_v1_8;
         friend class MeshSerializerImpl_v1_4;
+        friend class MeshSerializerImpl_v1_3;
         friend class MeshSerializerImpl_v1_2;
         friend class MeshSerializerImpl_v1_1;
 
     public:
-        typedef vector<Real>::type LodValueList;
-        typedef vector<MeshLodUsage>::type MeshLodUsageList;
+        typedef std::vector<Real> LodValueList;
+        typedef std::vector<MeshLodUsage> MeshLodUsageList;
         /// Multimap of vertex bone assignments (orders by vertex index).
-        typedef multimap<size_t, VertexBoneAssignment>::type VertexBoneAssignmentList;
+        typedef std::multimap<size_t, VertexBoneAssignment> VertexBoneAssignmentList;
         typedef MapIterator<VertexBoneAssignmentList> BoneAssignmentIterator;
-        typedef vector<SubMesh*>::type SubMeshList;
-        typedef vector<unsigned short>::type IndexMap;
+        typedef std::vector<SubMesh*> SubMeshList;
+        typedef std::vector<unsigned short> IndexMap;
 
     protected:
         /** A list of submeshes which make up this mesh.
@@ -125,7 +121,7 @@ namespace Ogre {
         /** A hashmap used to store optional SubMesh names.
             Translates a name into SubMesh index.
         */
-        typedef HashMap<String, ushort> SubMeshNameMap ;
+        typedef std::unordered_map<String, ushort> SubMeshNameMap ;
 
         
     protected:
@@ -138,6 +134,8 @@ namespace Ogre {
         AxisAlignedBox mAABB;
         /// Local bounding sphere radius (centered on object).
         Real mBoundRadius;
+        /// Largest bounding radius of any bone in the skeleton (centered on each bone, only considering verts weighted to the bone)
+        Real mBoneBoundingRadius;
 
         /// Optional linked skeleton.
         String mSkeletonName;
@@ -157,12 +155,18 @@ namespace Ogre {
             unsigned short numBlendWeightsPerVertex, 
             IndexMap& blendIndexToBoneIndexMap,
             VertexData* targetVertexData);
-
+#if !OGRE_NO_MESHLOD
         const LodStrategy *mLodStrategy;
-        bool mIsLodManual;
+        bool mHasManualLodLevel;
         ushort mNumLods;
         MeshLodUsageList mMeshLodUsageList;
-
+#else
+        const LodStrategy *mLodStrategy;
+        const bool mHasManualLodLevel;
+        const ushort mNumLods;
+        MeshLodUsageList mMeshLodUsageList;
+#endif
+        HardwareBufferManagerBase* mBufferManager;
         HardwareBuffer::Usage mVertexBufferUsage;
         HardwareBuffer::Usage mIndexBufferUsage;
         bool mVertexBufferShadowBuffer;
@@ -174,7 +178,7 @@ namespace Ogre {
         bool mAutoBuildEdgeLists;
 
         /// Storage of morph animations, lookup by name
-        typedef map<String, Animation*>::type AnimationList;
+        typedef std::map<String, Animation*> AnimationList;
         AnimationList mAnimationsList;
         /// The vertex animation type associated with the shared vertex data
         mutable VertexAnimationType mSharedVertexDataAnimationType;
@@ -206,8 +210,8 @@ namespace Ogre {
         /// @copydoc Resource::calculateSize
         size_t calculateSize(void) const;
 
-		void mergeAdjacentTexcoords( unsigned short finalTexCoordSet,
-									 unsigned short texCoordSetToDestroy, VertexData *vertexData );
+        void mergeAdjacentTexcoords( unsigned short finalTexCoordSet,
+                                     unsigned short texCoordSetToDestroy, VertexData *vertexData );
 
 
     public:
@@ -250,12 +254,18 @@ namespace Ogre {
         ushort _getSubMeshIndex(const String& name) const;
 
         /** Gets the number of sub meshes which comprise this mesh.
+        *  @deprecated use getSubMeshes() instead
         */
-        unsigned short getNumSubMeshes(void) const;
+        size_t getNumSubMeshes(void) const {
+            return mSubMeshList.size();
+        }
 
         /** Gets a pointer to the submesh indicated by the index.
+        *  @deprecated use getSubMeshes() instead
         */
-        SubMesh* getSubMesh(unsigned short index) const;
+        SubMesh* getSubMesh(size_t index) const {
+            return mSubMeshList[index];
+        }
 
         /** Gets a SubMesh by name
         */
@@ -279,9 +289,15 @@ namespace Ogre {
         
         typedef VectorIterator<SubMeshList> SubMeshIterator;
         /// Gets an iterator over the available submeshes
-        SubMeshIterator getSubMeshIterator(void)
+        /// @deprecated use getSubMeshes() instead
+        OGRE_DEPRECATED SubMeshIterator getSubMeshIterator(void)
         { return SubMeshIterator(mSubMeshList.begin(), mSubMeshList.end()); }
       
+        /// Gets the available submeshes
+        const SubMeshList& getSubMeshes() const {
+            return mSubMeshList;
+        }
+
         /** Shared vertex data.
         @remarks
             This vertex data can be shared among multiple submeshes. SubMeshes may not have
@@ -325,7 +341,10 @@ namespace Ogre {
             if you leave this blank, the clone will be assigned to the same
             group as this Mesh.
         */
-        MeshPtr clone(const String& newName, const String& newGroup = StringUtil::BLANK);
+        MeshPtr clone(const String& newName, const String& newGroup = BLANKSTRING);
+
+        /** @copydoc Resource::reload */
+        void reload(LoadingFlags flags = LF_DEFAULT);
 
         /** Get the axis-aligned bounding box for this mesh.
         */
@@ -334,6 +353,9 @@ namespace Ogre {
         /** Gets the radius of the bounding sphere surrounding this mesh. */
         Real getBoundingSphereRadius(void) const;
 
+        /** Gets the radius used to inflate the bounding box around the bones. */
+        Real getBoneBoundingRadius() const;
+
         /** Manually set the bounding box for this Mesh.
         @remarks
             Calling this method is required when building manual meshes now, because OGRE can no longer 
@@ -353,6 +375,35 @@ namespace Ogre {
         */
         void _setBoundingSphereRadius(Real radius);
 
+        /** Manually set the bone bounding radius. 
+        @remarks
+            This value is normally computed automatically, however it can be overriden with this method.
+        */
+        void _setBoneBoundingRadius(Real radius);
+
+        /** Compute the bone bounding radius by looking at the vertices, vertex-bone-assignments, and skeleton bind pose.
+        @remarks
+            This is automatically called by Entity if necessary.  Only does something if the boneBoundingRadius is zero to
+            begin with.  Only works if vertex data is readable (i.e. not WRITE_ONLY).
+        */
+        void _computeBoneBoundingRadius();
+
+        /** Automatically update the bounding radius and bounding box for this Mesh.
+        @remarks
+        Calling this method is required when building manual meshes. However it is recommended to
+        use _setBounds and _setBoundingSphereRadius instead, because the vertex buffer may not have
+        a shadow copy in the memory. Reading back the buffer from video memory is very slow!
+        @param pad If true, a certain padding will be added to the bounding box to separate it from the mesh
+        */
+        void _updateBoundsFromVertexBuffers(bool pad = false);
+
+        /** Calculates 
+        @remarks
+        Calling this method is required when building manual meshes. However it is recommended to
+        use _setBounds and _setBoundingSphereRadius instead, because the vertex buffer may not have
+        a shadow copy in the memory. Reading back the buffer from video memory is very slow!
+        */
+        void _calcBoundsFromVertexBuffer(VertexData* vertexData, AxisAlignedBox& outAABB, Real& outRadius, bool updateOnly = false);
         /** Sets the name of the skeleton this Mesh uses for animation.
         @remarks
             Meshes can optionally be assigned a skeleton which can be used to animate
@@ -430,7 +481,6 @@ namespace Ogre {
         */
         const VertexBoneAssignmentList& getBoneAssignments() const { return mBoneAssignments; }
 
-
         /** Returns the number of levels of detail that this mesh supports. 
         @remarks
             This number includes the original model.
@@ -438,23 +488,22 @@ namespace Ogre {
         ushort getNumLodLevels(void) const;
         /** Gets details of the numbered level of detail entry. */
         const MeshLodUsage& getLodLevel(ushort index) const;
-        /** Adds a new manual level-of-detail entry to this Mesh.
-        @remarks
-            As an alternative to generating lower level of detail versions of a mesh, you can
-            use your own manually modelled meshes as lower level versions. This lets you 
-            have complete control over the LOD, and in addition lets you scale down other
-            aspects of the model which cannot be done using the generated method; for example, 
-            you could use less detailed materials and / or use less bones in the skeleton if
-            this is an animated mesh. Therefore for complex models you are likely to be better off
-            modelling your LODs yourself and using this method, whilst for models with fairly
-            simple materials and no animation you can just use the generateLodLevels method.
-        @param value
-            The value from which this LOD will apply.
-        @param meshName
-            The name of the mesh which will be the lower level detail version.
-        */
-        void createManualLodLevel(Real value, const String& meshName, const String& groupName = Ogre::String());
 
+        /** Retrieves the level of detail index for the given LOD value. 
+        @note
+            The value passed in is the 'transformed' value. If you are dealing with
+            an original source value (e.g. distance), use LodStrategy::transformUserValue
+            to turn this into a lookup value.
+        */
+        ushort getLodIndex(Real value) const;
+
+        /** Returns true if this mesh has a manual LOD level.
+        @remarks
+            A mesh can either use automatically generated LOD, or it can use alternative
+            meshes as provided by an artist.
+        */
+        bool hasManualLodLevel(void) const { return mHasManualLodLevel; }
+#if !OGRE_NO_MESHLOD
         /** Changes the alternate mesh to use as a manual LOD at the given index.
         @remarks
             Note that the index of a LOD may change if you insert other LODs. If in doubt,
@@ -466,32 +515,43 @@ namespace Ogre {
         */
         void updateManualLodLevel(ushort index, const String& meshName);
 
-        /** Retrieves the level of detail index for the given LOD value. 
-        @note
-            The value passed in is the 'transformed' value. If you are dealing with
-            an original source value (e.g. distance), use LodStrategy::transformUserValue
-            to turn this into a lookup value.
-        */
-        ushort getLodIndex(Real value) const;
-
-        /** Returns true if this mesh is using manual LOD.
-        @remarks
-            A mesh can either use automatically generated LOD, or it can use alternative
-            meshes as provided by an artist. A mesh can only use either all manual LODs 
-            or all generated LODs, not a mixture of both.
-        */
-        bool isLodManual(void) const { return mIsLodManual; }
-
         /** Internal methods for loading LOD, do not use. */
-        void _setLodInfo(unsigned short numLevels, bool isManual);
+        void _setLodInfo(unsigned short numLevels);
         /** Internal methods for loading LOD, do not use. */
-        void _setLodUsage(unsigned short level, MeshLodUsage& usage);
+        void _setLodUsage(unsigned short level, const MeshLodUsage& usage);
         /** Internal methods for loading LOD, do not use. */
         void _setSubMeshLodFaceList(unsigned short subIdx, unsigned short level, IndexData* facedata);
+#endif
+        /** Internal methods for loading LOD, do not use. */
+        bool _isManualLodLevel(unsigned short level) const;
+
 
         /** Removes all LOD data from this Mesh. */
         void removeLodLevels(void);
 
+        /** Sets the manager for the vertex and index buffers to be used when loading
+            this Mesh.
+        @remarks
+            By default, when loading the Mesh, static, write-only vertex and index buffers 
+            will be used where possible in order to improve rendering performance. 
+            However, such buffers cannot be manipulated on the fly by CPU code 
+            (although shader code can). If you wish to use the CPU to modify these buffers
+            and will never use it with GPU, you should call this method. Note,
+            however, that it only takes effect after the Mesh has been reloaded. Note that you
+            still have the option of manually repacing the buffers in this mesh with your
+            own if you see fit too, in which case you don't need to call this method since it
+            only affects buffers created by the mesh itself.
+        @par
+            You can define the approach to a Mesh by changing the default parameters to 
+            MeshManager::load if you wish; this means the Mesh is loaded with those options
+            the first time instead of you having to reload the mesh after changing these options.
+        @param bufferManager
+            If set to @c DefaultHardwareBufferManager, the buffers will be created in system memory
+            only, without hardware counterparts. Such mesh could not be rendered, but LODs could be
+            generated for such mesh, it could be cloned, transformed and serialized.
+        */
+        void setHardwareBufferManager(HardwareBufferManagerBase* bufferManager) { mBufferManager = bufferManager; }
+        HardwareBufferManagerBase* getHardwareBufferManager();
         /** Sets the policy for the vertex buffers to be used when loading
             this Mesh.
         @remarks
@@ -586,24 +646,22 @@ namespace Ogre {
         */
         void _updateCompiledBoneAssignments(void);
 
-		/** This method collapses two texcoords into one for all submeshes where this is possible.
+        /** This method collapses two texcoords into one for all submeshes where this is possible.
         @remarks
-			Often a submesh can have two tex. coords. (i.e. TEXCOORD0 & TEXCOORD1), being both
-			composed of two floats. There are many practical reasons why it would be more convenient
-			to merge both of them into one TEXCOORD0 of 4 floats. This function does exactly that
-			The finalTexCoordSet must have enough space for the merge, or else the submesh will be
-			skipped. (i.e. you can't merge a tex. coord with 3 floats with one having 2 floats)
+            Often a submesh can have two tex. coords. (i.e. TEXCOORD0 & TEXCOORD1), being both
+            composed of two floats. There are many practical reasons why it would be more convenient
+            to merge both of them into one TEXCOORD0 of 4 floats. This function does exactly that
+            The finalTexCoordSet must have enough space for the merge, or else the submesh will be
+            skipped. (i.e. you can't merge a tex. coord with 3 floats with one having 2 floats)
 
-			finalTexCoordSet & texCoordSetToDestroy must be in the same buffer source, and must
-			be adjacent.
-		@param finalTexCoordSet The tex. coord index to merge to. Should have enough space to
-			actually work.
+            finalTexCoordSet & texCoordSetToDestroy must be in the same buffer source, and must
+            be adjacent.
+        @param finalTexCoordSet The tex. coord index to merge to. Should have enough space to
+            actually work.
         @param texCoordSetToDestroy The texture coordinate index that will disappear on
-			successful merges.
+            successful merges.
         */
-		void mergeAdjacentTexcoords( unsigned short finalTexCoordSet, unsigned short texCoordSetToDestroy );
-
-		void _configureMeshLodUsage(const LodConfig& lodConfig);
+        void mergeAdjacentTexcoords( unsigned short finalTexCoordSet, unsigned short texCoordSetToDestroy );
 
         /** This method builds a set of tangent vectors for a given mesh into a 3D texture coordinate buffer.
         @remarks
@@ -731,8 +789,8 @@ namespace Ogre {
         @param indexMap
             The index map used to translate blend index to bone index.
         */
-        static void prepareMatricesForVertexBlend(const Matrix4** blendMatrices,
-            const Matrix4* boneMatrices, const IndexMap& indexMap);
+        static void prepareMatricesForVertexBlend(const Affine3** blendMatrices,
+            const Affine3* boneMatrices, const IndexMap& indexMap);
 
         /** Performs a software indexed vertex blend, of the kind used for
             skeletal animation although it can be used for other purposes. 
@@ -759,7 +817,7 @@ namespace Ogre {
         */
         static void softwareVertexBlend(const VertexData* sourceVertexData, 
             const VertexData* targetVertexData,
-            const Matrix4* const* blendMatrices, size_t numMatrices,
+            const Affine3* const* blendMatrices, size_t numMatrices,
             bool blendNormals);
 
         /** Performs a software vertex morph, of the kind used for
@@ -803,8 +861,8 @@ namespace Ogre {
             number in start and end.
         */
         static void softwareVertexPoseBlend(Real weight, 
-            const map<size_t, Vector3>::type& vertexOffsetMap,
-            const map<size_t, Vector3>::type& normalsMap,
+            const std::map<size_t, Vector3>& vertexOffsetMap,
+            const std::map<size_t, Vector3>& normalsMap,
             VertexData* targetVertexData);
         /** Gets a reference to the optional name assignments of the SubMeshes. */
         const SubMeshNameMap& getSubMeshNameMap(void) const { return mSubMeshNameMap; }
@@ -905,11 +963,13 @@ namespace Ogre {
         @return
             A new Pose ready for population.
         */
-        Pose* createPose(ushort target, const String& name = StringUtil::BLANK);
-        /** Get the number of poses.*/
-        size_t getPoseCount(void) const { return mPoseList.size(); }
-        /** Retrieve an existing Pose by index.*/
-        Pose* getPose(ushort index);
+        Pose* createPose(ushort target, const String& name = BLANKSTRING);
+        /** Get the number of poses.
+         * @deprecated use getPoseList() */
+        OGRE_DEPRECATED size_t getPoseCount(void) const { return mPoseList.size(); }
+        /** Retrieve an existing Pose by index.
+         * @deprecated use getPoseList() */
+        OGRE_DEPRECATED Pose* getPose(ushort index);
         /** Retrieve an existing Pose by name.*/
         Pose* getPose(const String& name);
         /** Destroy a pose by index.
@@ -928,24 +988,27 @@ namespace Ogre {
         typedef VectorIterator<PoseList> PoseIterator;
         typedef ConstVectorIterator<PoseList> ConstPoseIterator;
 
-        /** Get an iterator over all the poses defined. */
-        PoseIterator getPoseIterator(void);
-        /** Get an iterator over all the poses defined. */
-        ConstPoseIterator getPoseIterator(void) const;
+        /** Get an iterator over all the poses defined.
+         * @deprecated use getPoseList() */
+        OGRE_DEPRECATED PoseIterator getPoseIterator(void);
+        /** Get an iterator over all the poses defined.
+         * @deprecated use getPoseList()  */
+        OGRE_DEPRECATED ConstPoseIterator getPoseIterator(void) const;
         /** Get pose list. */
         const PoseList& getPoseList(void) const;
 
         /** Get LOD strategy used by this mesh. */
         const LodStrategy *getLodStrategy() const;
-        /** Set the LOD strategy used by this mesh. */
+#if !OGRE_NO_MESHLOD
+        /** Set the lod strategy used by this mesh. */
         void setLodStrategy(LodStrategy *lodStrategy);
-
+#endif
     };
 
     /** A way of recording the way each LODs is recorded this Mesh. */
     struct MeshLodUsage
     {
-        /** User-supplied values used to determine when th is LOD applies.
+        /** User-supplied values used to determine on which distance the lod is applies.
         @remarks
             This is required in case the LOD strategy changes.
         */
@@ -958,10 +1021,9 @@ namespace Ogre {
         */
         Real value;
         
+
         /// Only relevant if mIsLodManual is true, the name of the alternative mesh to use.
         String manualName;
-        /// Only relevant if mIsLodManual is true, the name of the group of the alternative mesh.
-        String manualGroup;
         /// Hard link to mesh to avoid looking up each time.
         mutable MeshPtr manualMesh;
         /// Edge list for this LOD level (may be derived from manual mesh).
