@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -32,31 +32,37 @@ THE SOFTWARE.
 
 #include "OgreResourceManager.h"
 #include "OgreSingleton.h"
-#include "OgreVector3.h"
+#include "OgreVector.h"
 #include "OgreHardwareBuffer.h"
-#include "OgreMesh.h"
-#include "OgrePatchMesh.h"
+#include "OgreHardwareVertexBuffer.h"
+#include "OgrePatchSurface.h"
 #include "OgreHeaderPrefix.h"
+#include "OgrePlane.h"
 
 namespace Ogre {
 
-	class MeshSerializerListener;
+    class MeshSerializerListener;
 
-	/** \addtogroup Core
-	*  @{
-	*/
-	/** \addtogroup Resources
-	*  @{
-	*/
-	/** Handles the management of mesh resources.
+    /** \addtogroup Core
+    *  @{
+    */
+    /** \addtogroup Resources
+    *  @{
+    */
+    /** Handles the management of mesh resources.
         @remarks
             This class deals with the runtime management of
             mesh data; like other resource managers it handles
             the creation of resources (in this case mesh data),
             working within a fixed memory budget.
+        @remarks
+            Ogre loads model files from it's own proprietary
+            format called .mesh. This is because having a single file
+            format is better for runtime performance, and we also have
+            control over pre-processed data (such as
+            collision boxes, LOD reductions etc). 
     */
-    class _OgreExport MeshManager: public ResourceManager, public Singleton<MeshManager>, 
-        public ManualResourceLoader
+    class _OgreExport MeshManager: public ResourceManager, public Singleton<MeshManager>
     {
     public:
         MeshManager();
@@ -65,19 +71,20 @@ namespace Ogre {
         /** Initialises the manager, only to be called by OGRE internally. */
         void _initialise(void);
 
-		/// Get a resource by name
-		/// @see ResourceManager::getResourceByName
-		MeshPtr getByName(const String& name, const String& groupName = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME);
+        /// @copydoc ResourceManager::getResourceByName
+        MeshPtr getByName(const String& name, const String& groupName OGRE_RESOURCE_GROUP_INIT);
 
+        /// Create a new mesh
+        /// @copydetails ResourceManager::createResource
+        MeshPtr create (const String& name, const String& group,
+                            bool isManual = false, ManualResourceLoader* loader = 0,
+                            const NameValuePairList* createParams = 0);
 
-		/// Create a new mesh
-		/// @see ResourceManager::createResource
-		MeshPtr create (const String& name, const String& group,
-							bool isManual = false, ManualResourceLoader* loader = 0,
-							const NameValuePairList* createParams = 0);
+        using ResourceManager::createOrRetrieve;
 
         /** Create a new mesh, or retrieve an existing one with the same
             name if it already exists.
+            @copydetails ResourceManager::createResource
             @param vertexBufferUsage The usage flags with which the vertex buffer(s)
                 will be created
             @param indexBufferUsage The usage flags with which the index buffer(s) created for 
@@ -86,76 +93,51 @@ namespace Ogre {
                 copies for faster read access
             @param indexBufferShadowed If true, the index buffers will be shadowed by system memory 
                 copies for faster read access
-        @see ResourceManager::createOrRetrieve
         */
         ResourceCreateOrRetrieveResult createOrRetrieve(
             const String& name,
             const String& group,
-            bool isManual=false, ManualResourceLoader* loader=0,
-            const NameValuePairList* params=0,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			bool vertexBufferShadowed = true, bool indexBufferShadowed = true);
+            bool isManual, ManualResourceLoader* loader,
+            const NameValuePairList* createParams,
+            HardwareBuffer::Usage vertexBufferUsage,
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            bool vertexBufferShadowed = false, bool indexBufferShadowed = false);
 
         /** Prepares a mesh for loading from a file.  This does the IO in advance of the call to load().
             @note
                 If the model has already been created (prepared or loaded), the existing instance
                 will be returned.
-            @remarks
-                Ogre loads model files from it's own proprietary
-                format called .mesh. This is because having a single file
-                format is better for runtime performance, and we also have
-                control over pre-processed data (such as
-                collision boxes, LOD reductions etc).
-			@param filename The name of the .mesh file
+            @param filename The name of the .mesh file
             @param groupName The name of the resource group to assign the mesh to 
-			@param vertexBufferUsage The usage flags with which the vertex buffer(s)
-				will be created
-			@param indexBufferUsage The usage flags with which the index buffer(s) created for 
-				this mesh will be created with.
-			@param vertexBufferShadowed If true, the vertex buffers will be shadowed by system memory 
+            @param vertexBufferUsage The usage flags with which the vertex buffer(s)
+                will be created
+            @param indexBufferUsage The usage flags with which the index buffer(s) created for 
+                this mesh will be created with.
+            @param vertexBufferShadowed If true, the vertex buffers will be shadowed by system memory 
                 copies for faster read access
-			@param indexBufferShadowed If true, the index buffers will be shadowed by system memory 
+            @param indexBufferShadowed If true, the index buffers will be shadowed by system memory 
                 copies for faster read access
         */
         MeshPtr prepare( const String& filename, const String& groupName,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			bool vertexBufferShadowed = true, bool indexBufferShadowed = true);
+            HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            bool vertexBufferShadowed = false, bool indexBufferShadowed = false);
 
         /** Loads a mesh from a file, making it immediately available for use.
-            @note
-                If the model has already been created (prepared or loaded), the existing instance
-                will be returned.
-            @remarks
-                Ogre loads model files from it's own proprietary
-                format called .mesh. This is because having a single file
-                format is better for runtime performance, and we also have
-                control over pre-processed data (such as
-                collision boxes, LOD reductions etc).
-			@param filename The name of the .mesh file
-            @param groupName The name of the resource group to assign the mesh to 
-			@param vertexBufferUsage The usage flags with which the vertex buffer(s)
-				will be created
-			@param indexBufferUsage The usage flags with which the index buffer(s) created for 
-				this mesh will be created with.
-			@param vertexBufferShadowed If true, the vertex buffers will be shadowed by system memory 
-                copies for faster read access
-			@param indexBufferShadowed If true, the index buffers will be shadowed by system memory 
-                copies for faster read access
+            @copydetails MeshManager::prepare
         */
         MeshPtr load( const String& filename, const String& groupName,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			bool vertexBufferShadowed = true, bool indexBufferShadowed = true);
+            HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            bool vertexBufferShadowed = false, bool indexBufferShadowed = false);
 
 
         /** Creates a new Mesh specifically for manual definition rather
             than loading from an object file. 
-		@remarks
-			Note that once you've defined your mesh, you must call Mesh::_setBounds and
-            Mesh::_setBoundingRadius in order to define the bounds of your mesh. In previous
-            versions of OGRE you could call Mesh::_updateBounds, but OGRE's support of 
+
+            Note that once you've defined your mesh, you must call Mesh::_setBounds
+            in order to define the bounds of your mesh. In previous
+            versions of OGRE could auto-compute that, but OGRE's support of 
             write-only vertex buffers makes this no longer appropriate.
         @param name The name to give the new mesh
         @param groupName The name of the resource group to assign the mesh to 
@@ -192,18 +174,18 @@ namespace Ogre {
                 vTile The number of times the texture should be repeated in the v direction
             @param
                 upVector The 'Up' direction of the plane texture coordinates.
-			@param
-				vertexBufferUsage The usage flag with which the vertex buffer for this plane will be created
-			@param
-				indexBufferUsage The usage flag with which the index buffer for this plane will be created
-			@param
-				vertexShadowBuffer If this flag is set to true, the vertex buffer will be created 
-				with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
-			@param
-				indexShadowBuffer If this flag is set to true, the index buffer will be 
-				created with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
+            @param
+                vertexBufferUsage The usage flag with which the vertex buffer for this plane will be created
+            @param
+                indexBufferUsage The usage flag with which the index buffer for this plane will be created
+            @param
+                vertexShadowBuffer If this flag is set to true, the vertex buffer will be created 
+                with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
+            @param
+                indexShadowBuffer If this flag is set to true, the index buffer will be 
+                created with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
         */
         MeshPtr createPlane(
             const String& name, const String& groupName, const Plane& plane,
@@ -211,13 +193,13 @@ namespace Ogre {
             int xsegments = 1, int ysegments = 1,
             bool normals = true, unsigned short numTexCoordSets = 1,
             Real uTile = 1.0f, Real vTile = 1.0f, const Vector3& upVector = Vector3::UNIT_Y,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
-			bool vertexShadowBuffer = true, bool indexShadowBuffer = true);
+            HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
+            bool vertexShadowBuffer = false, bool indexShadowBuffer = false);
 
         
         /** Creates a plane, which because of it's texture coordinates looks like a curved
-			surface, useful for skies in a skybox. 
+            surface, useful for skies in a skybox. 
             @param name
                 The name to give the resulting mesh
             @param groupName
@@ -229,11 +211,11 @@ namespace Ogre {
             @param height
                 The height of the plane in world coordinates
             @param curvature
-				The curvature of the plane. Good values are
+                The curvature of the plane. Good values are
                 between 2 and 65. Higher values are more curved leading to
                 a smoother effect, lower values are less curved meaning
                 more distortion at the horizons but a better distance effect.
-			@param xsegments
+            @param xsegments
                 The number of segments to the plane in the x direction
             @param ysegments
                 The number of segments to the plane in the y direction
@@ -250,35 +232,35 @@ namespace Ogre {
                 The 'Up' direction of the plane.
             @param orientation
                 The orientation of the overall sphere that's used to create the illusion
-			@param vertexBufferUsage
-				The usage flag with which the vertex buffer for this plane will be created
-			@param indexBufferUsage
-				The usage flag with which the index buffer for this plane will be created
-			@param vertexShadowBuffer
-				If this flag is set to true, the vertex buffer will be created 
-				with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
-			@param indexShadowBuffer
-				If this flag is set to true, the index buffer will be 
-				created with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
+            @param vertexBufferUsage
+                The usage flag with which the vertex buffer for this plane will be created
+            @param indexBufferUsage
+                The usage flag with which the index buffer for this plane will be created
+            @param vertexShadowBuffer
+                If this flag is set to true, the vertex buffer will be created 
+                with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
+            @param indexShadowBuffer
+                If this flag is set to true, the index buffer will be 
+                created with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
             @param ySegmentsToKeep The number of segments from the top of the dome
                 downwards to keep. -1 keeps all of them. This can save fillrate if
                 you cannot see much of the sky lower down.
         */
-		MeshPtr createCurvedIllusionPlane(
+        MeshPtr createCurvedIllusionPlane(
             const String& name, const String& groupName, const Plane& plane,
             Real width, Real height, Real curvature,
             int xsegments = 1, int ysegments = 1,
             bool normals = true, unsigned short numTexCoordSets = 1,
             Real uTile = 1.0f, Real vTile = 1.0f, const Vector3& upVector = Vector3::UNIT_Y,
             const Quaternion& orientation = Quaternion::IDENTITY,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
-			bool vertexShadowBuffer = true, bool indexShadowBuffer = true, 
+            HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
+            bool vertexShadowBuffer = false, bool indexShadowBuffer = false, 
             int ySegmentsToKeep = -1);
 
-		/** Creates a genuinely curved plane, by default majoring on the x/y axes facing positive Z.
+        /** Creates a genuinely curved plane, by default majoring on the x/y axes facing positive Z.
             @param name
                 The name to give the resulting mesh
             @param groupName
@@ -289,8 +271,8 @@ namespace Ogre {
                 The width of the plane in world coordinates
             @param height
                 The height of the plane in world coordinates
-			@param bow
-				The amount of 'bow' in the curved plane.  (Could also be considered the depth.)
+            @param bow
+                The amount of 'bow' in the curved plane.  (Could also be considered the depth.)
             @param xsegments
                 The number of segments to the plane in the x direction
             @param ysegments
@@ -306,28 +288,28 @@ namespace Ogre {
                 The number of times the texture should be repeated in the v direction
             @param upVector
                 The 'Up' direction of the plane.
-			@param vertexBufferUsage
-				The usage flag with which the vertex buffer for this plane will be created
-			@param indexBufferUsage
-				The usage flag with which the index buffer for this plane will be created
-			@param vertexShadowBuffer
-				If this flag is set to true, the vertex buffer will be created 
-				with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
-			@param indexShadowBuffer
-				If this flag is set to true, the index buffer will be 
-				created with a system memory shadow buffer,
-				allowing you to read it back more efficiently than if it is in hardware
+            @param vertexBufferUsage
+                The usage flag with which the vertex buffer for this plane will be created
+            @param indexBufferUsage
+                The usage flag with which the index buffer for this plane will be created
+            @param vertexShadowBuffer
+                If this flag is set to true, the vertex buffer will be created 
+                with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
+            @param indexShadowBuffer
+                If this flag is set to true, the index buffer will be 
+                created with a system memory shadow buffer,
+                allowing you to read it back more efficiently than if it is in hardware
         */
-		MeshPtr createCurvedPlane( 
-			const String& name, const String& groupName, const Plane& plane, 
-			Real width, Real height, Real bow = 0.5f, 
-			int xsegments = 1, int ysegments = 1,
-			bool normals = false, unsigned short numTexCoordSets = 1, 
-			Real uTile = 1.0f, Real vTile = 1.0f, const Vector3& upVector = Vector3::UNIT_Y,
-			HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
-			bool vertexShadowBuffer = true, bool indexShadowBuffer = true);
+        MeshPtr createCurvedPlane( 
+            const String& name, const String& groupName, const Plane& plane, 
+            Real width, Real height, Real bow = 0.5f, 
+            int xsegments = 1, int ysegments = 1,
+            bool normals = false, unsigned short numTexCoordSets = 1, 
+            Real uTile = 1.0f, Real vTile = 1.0f, const Vector3& upVector = Vector3::UNIT_Y,
+            HardwareBuffer::Usage vertexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY, 
+            HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
+            bool vertexShadowBuffer = false, bool indexShadowBuffer = false);
 
         /** Creates a Bezier patch based on an array of control vertices.
             @param name
@@ -382,75 +364,50 @@ namespace Ogre {
         /** Retrieves whether all Meshes should prepare themselves for shadow volumes. */
         bool getPrepareAllMeshesForShadowVolumes(void);
 
-        /** Override standard Singleton retrieval.
-        @remarks
-        Why do we do this? Well, it's because the Singleton
-        implementation is in a .h file, which means it gets compiled
-        into anybody who includes it. This is needed for the
-        Singleton template to work, but we actually only want it
-        compiled into the implementation of the class based on the
-        Singleton, not all of them. If we don't change this, we get
-        link errors when trying to use the Singleton-based class from
-        an outside dll.
-        @par
-        This method just delegates to the template version anyway,
-        but the implementation stays in this single compilation unit,
-        preventing link errors.
-        */
+        /// @copydoc Singleton::getSingleton()
         static MeshManager& getSingleton(void);
-        /** Override standard Singleton retrieval.
-        @remarks
-        Why do we do this? Well, it's because the Singleton
-        implementation is in a .h file, which means it gets compiled
-        into anybody who includes it. This is needed for the
-        Singleton template to work, but we actually only want it
-        compiled into the implementation of the class based on the
-        Singleton, not all of them. If we don't change this, we get
-        link errors when trying to use the Singleton-based class from
-        an outside dll.
-        @par
-        This method just delegates to the template version anyway,
-        but the implementation stays in this single compilation unit,
-        preventing link errors.
-        */
+        /// @copydoc Singleton::getSingleton()
         static MeshManager* getSingletonPtr(void);
 
-	    /** Gets the factor by which the bounding box of an entity is padded.
-	        Default is 0.01
-	    */
+        /** Gets the base element type used for blend weights in vertex buffers.
+        @remarks
+        See the remarks below for SetBlendWeightsBaseElementType().
+        */
+        VertexElementType getBlendWeightsBaseElementType() const;
+
+        /** sets the base element type used for blend weights in vertex buffers.
+        @remarks
+        This takes effect when meshes are loaded.  Default is VET_FLOAT1.
+        Valid values are:
+        VET_UBYTE4_NORM:   8-bit blend weights.  Lowest memory cost but may have precision issues.  Requires SM2.0+ vertex shader.  No software skinning.
+        VET_USHORT2_NORM:  16-bit blend weights.  Requires SM2.0+ vertex shader.  No software skinning.
+        VET_FLOAT1:        32-bit blend weights.  Highest memory cost.  Supports hardware and software skinning.
+        */
+        void setBlendWeightsBaseElementType( VertexElementType vet );
+
+        /** Gets the factor by which the bounding box of an entity is padded.
+            Default is 0.01
+        */
         Real getBoundsPaddingFactor(void);
-	
-	    /** Sets the factor by which the bounding box of an entity is padded
-	    */
+    
+        /** Sets the factor by which the bounding box of an entity is padded
+        */
         void setBoundsPaddingFactor(Real paddingFactor);
 
-		/** Sets the listener used to control mesh loading through the serializer.
-		*/
-		void setListener(MeshSerializerListener *listener);
-		
-		/** Gets the listener used to control mesh loading through the serializer.
-		*/
-		MeshSerializerListener *getListener();
-
-        /** @see ManualResourceLoader::loadResource */
-        void loadResource(Resource* res);
+        /** Sets the listener used to control mesh loading through the serializer.
+        */
+        void setListener(MeshSerializerListener *listener);
+        
+        /** Gets the listener used to control mesh loading through the serializer.
+        */
+        MeshSerializerListener *getListener();
 
     protected:
+
         /// @copydoc ResourceManager::createImpl
         Resource* createImpl(const String& name, ResourceHandle handle, 
             const String& group, bool isManual, ManualResourceLoader* loader, 
             const NameValuePairList* createParams);
-        
-        /** Utility method for tessellating 2D meshes.
-        */
-        void tesselate2DMesh(SubMesh* pSub, unsigned short meshWidth, unsigned short meshHeight, 
-			bool doubleSided = false, 
-			HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
-			bool indexSysMem = false);
-
-        void createPrefabPlane(void);
-		void createPrefabCube(void);
-		void createPrefabSphere(void);
     
         /** Enum identifying the types of manual mesh built by this manager */
         enum MeshBuildType
@@ -481,28 +438,43 @@ namespace Ogre {
             bool indexShadowBuffer;
             int ySegmentsToKeep;
         };
-        /** Map from resource pointer to parameter set */
-        typedef map<Resource*, MeshBuildParams>::type MeshBuildParamsMap;
-        MeshBuildParamsMap mMeshBuildParams;
 
-        /** Utility method for manual loading a plane */
-        void loadManualPlane(Mesh* pMesh, MeshBuildParams& params);
-        /** Utility method for manual loading a curved plane */
-        void loadManualCurvedPlane(Mesh* pMesh, MeshBuildParams& params);
-        /** Utility method for manual loading a curved illusion plane */
-        void loadManualCurvedIllusionPlane(Mesh* pMesh, MeshBuildParams& params);
+        struct PrefabLoader : public ManualResourceLoader
+        {
+            /** Map from resource pointer to parameter set */
+            typedef std::map<Resource*, MeshBuildParams> MeshBuildParamsMap;
+            MeshBuildParamsMap mMeshBuildParams;
+
+            /** Utility method for tessellating 2D meshes.
+            */
+            static void tesselate2DMesh(SubMesh* pSub, unsigned short meshWidth, unsigned short meshHeight,
+                bool doubleSided = false,
+                HardwareBuffer::Usage indexBufferUsage = HardwareBuffer::HBU_STATIC_WRITE_ONLY,
+                bool indexSysMem = false);
+            /** Utility method for manual loading a plane */
+            static void loadManualPlane(Mesh* pMesh, MeshBuildParams& params);
+            /** Utility method for manual loading a curved plane */
+            static void loadManualCurvedPlane(Mesh* pMesh, MeshBuildParams& params);
+            /** Utility method for manual loading a curved illusion plane */
+            static void loadManualCurvedIllusionPlane(Mesh* pMesh, MeshBuildParams& params);
+
+            void loadResource(Resource* res);
+        } mPrefabLoader;
+
+        // element type for blend weights in vertex buffer (VET_UBYTE4, VET_USHORT1, or VET_FLOAT1)
+        VertexElementType mBlendWeightsBaseElementType;
 
         bool mPrepAllMeshesForShadowVolumes;
-	
-		//the factor by which the bounding box of an entity is padded	
-		Real mBoundsPaddingFactor;
+    
+        //the factor by which the bounding box of an entity is padded   
+        Real mBoundsPaddingFactor;
 
-		// The listener to pass to serializers
-		MeshSerializerListener *mListener;
+        // The listener to pass to serializers
+        MeshSerializerListener *mListener;
     };
 
-	/** @} */
-	/** @} */
+    /** @} */
+    /** @} */
 
 } //namespace
 
