@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,23 +29,18 @@ THE SOFTWARE.
 #define __SceneQuery_H__
 
 #include "OgrePrerequisites.h"
-#include "OgreAxisAlignedBox.h"
 #include "OgreSphere.h"
 #include "OgreRay.h"
-#include "OgreRenderOperation.h"
-#include "OgrePlaneBoundedVolume.h"
+#include "OgreHeaderPrefix.h"
 
 namespace Ogre {
-
-    // forward declaration
-    class SceneQueryListener;
-	/** \addtogroup Core
-	*  @{
-	*/
-	/** \addtogroup Scene
-	*  @{
-	*/
-	/** A class for performing queries on a scene.
+    /** \addtogroup Core
+    *  @{
+    */
+    /** \addtogroup Scene
+    *  @{
+    */
+    /** A class for performing queries on a scene.
     @remarks
         This is an abstract class for performing a query on a scene, i.e. to retrieve
         a list of objects and/or world geometry sections which are potentially intersecting a
@@ -70,7 +65,7 @@ namespace Ogre {
         using the SceneManager interfaces for the type of query required, e.g.
         SceneManager::createSphereSceneQuery.
     */
-	class _OgreExport SceneQuery : public SceneMgtAlloc
+    class _OgreExport SceneQuery : public SceneMgtAlloc
     {
     public:
         /** This type can be used by collaborating applications & SceneManagers to 
@@ -111,7 +106,7 @@ namespace Ogre {
             /// Single intersection point, only applicable for WFT_SINGLE_INTERSECTION
             Vector3 singleIntersection;
             /// Planes bounding a convex region, only applicable for WFT_PLANE_BOUNDED_REGION
-            list<Plane>::type* planes;
+            std::vector<Plane>* planes;
             /// Custom geometry block, only applicable for WFT_CUSTOM_GEOMETRY
             void* geometry;
             /// General render operation structure, fallback if nothing else is available
@@ -121,8 +116,8 @@ namespace Ogre {
     protected:
         SceneManager* mParentSceneMgr;
         uint32 mQueryMask;
-		uint32 mQueryTypeMask;
-        set<WorldFragmentType>::type mSupportedWorldFragments;
+        uint32 mQueryTypeMask;
+        std::set<WorldFragmentType> mSupportedWorldFragments;
         WorldFragmentType mWorldFragmentType;
     
     public:
@@ -147,15 +142,15 @@ namespace Ogre {
         @remarks
             This method allows you to set a 'type mask' to limit the results of this
             query to certain types of objects. Whilst setQueryMask deals with flags
-			set per instance of object, this method deals with setting a mask on 
-			flags set per type of object. Both may exclude an object from query
-			results.
+            set per instance of object, this method deals with setting a mask on 
+            flags set per type of object. Both may exclude an object from query
+            results.
         */
         virtual void setQueryTypeMask(uint32 mask);
         /** Returns the current mask for this query. */
         virtual uint32 getQueryTypeMask(void) const;
 
-		/** Tells the query what kind of world geometry to return from queries;
+        /** Tells the query what kind of world geometry to return from queries;
             often the full renderable geometry is not what is needed. 
         @remarks
             The application receiving the world geometry is expected to know 
@@ -171,7 +166,7 @@ namespace Ogre {
         virtual WorldFragmentType getWorldFragmentType(void) const;
 
         /** Returns the types of world fragments this query supports. */
-        virtual const set<WorldFragmentType>::type* getSupportedWorldFragmentTypes(void) const
+        virtual const std::set<WorldFragmentType>* getSupportedWorldFragmentTypes(void) const
             {return &mSupportedWorldFragments;}
 
         
@@ -202,15 +197,15 @@ namespace Ogre {
 
     };
 
-    typedef list<MovableObject*>::type SceneQueryResultMovableList;
-    typedef list<SceneQuery::WorldFragment*>::type SceneQueryResultWorldFragmentList;
+    typedef std::list<MovableObject*> SceneQueryResultMovableList;
+    typedef std::list<SceneQuery::WorldFragment*> SceneQueryResultWorldFragmentList;
     /** Holds the results of a scene query. */
-	struct _OgreExport SceneQueryResult : public SceneMgtAlloc
+    struct _OgreExport SceneQueryResult : public SceneMgtAlloc
     {
         /// List of movable objects in the query (entities, particle systems etc)
         SceneQueryResultMovableList movables;
         /// List of world fragments
-		SceneQueryResultWorldFragmentList worldFragments;
+        SceneQueryResultWorldFragmentList worldFragments;
     };
 
     /** Abstract class defining a query which returns single results from a region. 
@@ -369,7 +364,7 @@ namespace Ogre {
         }
 
     };
-    typedef vector<RaySceneQueryResultEntry>::type RaySceneQueryResult;
+    typedef std::vector<RaySceneQueryResultEntry> RaySceneQueryResult;
 
     /** Specialises the SceneQuery class for querying along a ray. */
     class _OgreExport RaySceneQuery : public SceneQuery, public RaySceneQueryListener
@@ -486,10 +481,10 @@ namespace Ogre {
         
     typedef std::pair<MovableObject*, MovableObject*> SceneQueryMovableObjectPair;
     typedef std::pair<MovableObject*, SceneQuery::WorldFragment*> SceneQueryMovableObjectWorldFragmentPair;
-    typedef list<SceneQueryMovableObjectPair>::type SceneQueryMovableIntersectionList;
-    typedef list<SceneQueryMovableObjectWorldFragmentPair>::type SceneQueryMovableWorldFragmentIntersectionList;
+    typedef std::list<SceneQueryMovableObjectPair> SceneQueryMovableIntersectionList;
+    typedef std::list<SceneQueryMovableObjectWorldFragmentPair> SceneQueryMovableWorldFragmentIntersectionList;
     /** Holds the results of an intersection scene query (pair values). */
-	struct _OgreExport IntersectionSceneQueryResult : public SceneMgtAlloc
+    struct _OgreExport IntersectionSceneQueryResult : public SceneMgtAlloc
     {
         /// List of movable / movable intersections (entities, particle systems etc)
         SceneQueryMovableIntersectionList movables2movables;
@@ -554,11 +549,11 @@ namespace Ogre {
         bool queryResult(MovableObject* movable, SceneQuery::WorldFragment* fragment);
     };
     
-	/** @} */
-	/** @} */
+    /** @} */
+    /** @} */
 
 }
     
-
+#include "OgreHeaderSuffix.h"
 
 #endif
