@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@ THE SOFTWARE.
 
 #include "OgreBillboardParticleRenderer.h"
 #include "OgreParticle.h"
-#include "OgreStringConverter.h"
+#include "OgreBillboard.h"
 
 namespace Ogre {
     String rendererTypeName = "billboard";
@@ -41,7 +41,7 @@ namespace Ogre {
     BillboardParticleRenderer::CmdCommonDirection BillboardParticleRenderer::msCommonDirectionCmd;
     BillboardParticleRenderer::CmdCommonUpVector BillboardParticleRenderer::msCommonUpVectorCmd;
     BillboardParticleRenderer::CmdPointRendering BillboardParticleRenderer::msPointRenderingCmd;
-	BillboardParticleRenderer::CmdAccurateFacing BillboardParticleRenderer::msAccurateFacingCmd;
+    BillboardParticleRenderer::CmdAccurateFacing BillboardParticleRenderer::msAccurateFacingCmd;
     //-----------------------------------------------------------------------
     BillboardParticleRenderer::BillboardParticleRenderer()
     {
@@ -66,41 +66,41 @@ namespace Ogre {
 
             dict->addParameter(ParameterDef("billboard_rotation_type", 
                 "This setting controls the billboard rotation type. "
-				"'vertex' means rotate the billboard's vertices around their facing direction."
+                "'vertex' means rotate the billboard's vertices around their facing direction."
                 "'texcoord' means rotate the billboard's texture coordinates. Default value is 'texcoord'.",
                 PT_STRING),
                 &msBillboardRotationTypeCmd);
 
             dict->addParameter(ParameterDef("common_direction", 
                 "Only useful when billboard_type is oriented_common or perpendicular_common. "
-				"When billboard_type is oriented_common, this parameter sets the common orientation for "
-				"all particles in the set (e.g. raindrops may all be oriented downwards). "
-				"When billboard_type is perpendicular_common, this parameter sets the perpendicular vector for "
-				"all particles in the set (e.g. an aureola around the player and parallel to the ground).",
+                "When billboard_type is oriented_common, this parameter sets the common orientation for "
+                "all particles in the set (e.g. raindrops may all be oriented downwards). "
+                "When billboard_type is perpendicular_common, this parameter sets the perpendicular vector for "
+                "all particles in the set (e.g. an aureola around the player and parallel to the ground).",
                 PT_VECTOR3),
                 &msCommonDirectionCmd);
 
             dict->addParameter(ParameterDef("common_up_vector",
                 "Only useful when billboard_type is perpendicular_self or perpendicular_common. This "
-				"parameter sets the common up-vector for all particles in the set (e.g. an aureola around "
-				"the player and parallel to the ground).",
+                "parameter sets the common up-vector for all particles in the set (e.g. an aureola around "
+                "the player and parallel to the ground).",
                 PT_VECTOR3),
                 &msCommonUpVectorCmd);
             dict->addParameter(ParameterDef("point_rendering",
                 "Set whether or not particles will use point rendering "
-				"rather than manually generated quads. This allows for faster "
-				"rendering of point-oriented particles although introduces some "
-				"limitations too such as requiring a common particle size."
-				"Possible values are 'true' or 'false'.",
+                "rather than manually generated quads. This allows for faster "
+                "rendering of point-oriented particles although introduces some "
+                "limitations too such as requiring a common particle size."
+                "Possible values are 'true' or 'false'.",
                 PT_BOOL),
                 &msPointRenderingCmd);
-			dict->addParameter(ParameterDef("accurate_facing",
-				"Set whether or not particles will be oriented to the camera "
-				"based on the relative position to the camera rather than just "
-				"the camera direction. This is more accurate but less optimal. "
-				"Cannot be combined with point rendering.",
-				PT_BOOL),
-				&msAccurateFacingCmd);
+            dict->addParameter(ParameterDef("accurate_facing",
+                "Set whether or not particles will be oriented to the camera "
+                "based on the relative position to the camera rather than just "
+                "the camera direction. This is more accurate but less optimal. "
+                "Cannot be combined with point rendering.",
+                PT_BOOL),
+                &msAccurateFacingCmd);
         }
 
         // Create billboard set
@@ -111,10 +111,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     BillboardParticleRenderer::~BillboardParticleRenderer()
     {
-		// mBillboardSet is never actually attached to a node, we just passthrough
-		// based on the particle system's attachment. So manually notify that it's
-		// no longer attached.
-		mBillboardSet->_notifyAttached(0);
+        // mBillboardSet is never actually attached to a node, we just passthrough
+        // based on the particle system's attachment. So manually notify that it's
+        // no longer attached.
+        mBillboardSet->_notifyAttached(0);
         OGRE_DELETE  mBillboardSet;
     }
     //-----------------------------------------------------------------------
@@ -124,27 +124,45 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     void BillboardParticleRenderer::_updateRenderQueue(RenderQueue* queue, 
-        list<Particle*>::type& currentParticles, bool cullIndividually)
+        std::list<Particle*>& currentParticles, bool cullIndividually)
     {
         mBillboardSet->setCullIndividually(cullIndividually);
 
         // Update billboard set geometry
+        Vector3 bboxMin = Math::POS_INFINITY * Vector3::UNIT_SCALE;
+        Vector3 bboxMax = Math::NEG_INFINITY * Vector3::UNIT_SCALE;
+        Real radius = 0.0f;
         mBillboardSet->beginBillboards(currentParticles.size());
         Billboard bb;
-        for (list<Particle*>::type::iterator i = currentParticles.begin();
+        Affine3 invWorld;
+
+        bool invert = mBillboardSet->getBillboardsInWorldSpace() && mBillboardSet->getParentSceneNode();
+        if (invert)
+            invWorld = mBillboardSet->getParentSceneNode()->_getFullTransform().inverse();
+
+        for (std::list<Particle*>::iterator i = currentParticles.begin();
             i != currentParticles.end(); ++i)
         {
             Particle* p = *i;
-            bb.mPosition = p->position;
-			if (mBillboardSet->getBillboardType() == BBT_ORIENTED_SELF ||
-				mBillboardSet->getBillboardType() == BBT_PERPENDICULAR_SELF)
-			{
-				// Normalise direction vector
-				bb.mDirection = p->direction;
-				bb.mDirection.normalise();
-			}
-            bb.mColour = p->colour;
-            bb.mRotation = p->rotation;
+            bb.mPosition = p->mPosition;
+            Vector3 pos = p->mPosition;
+
+            if (invert)
+                pos = invWorld * pos;
+
+            bboxMin.makeFloor( pos );
+            bboxMax.makeCeil( pos );
+            radius = std::max( radius, p->mPosition.length() );
+
+            if (mBillboardSet->getBillboardType() == BBT_ORIENTED_SELF ||
+                mBillboardSet->getBillboardType() == BBT_PERPENDICULAR_SELF)
+            {
+                // Normalise direction vector
+                bb.mDirection = p->mDirection;
+                bb.mDirection.normalise();
+            }
+            bb.mColour = p->mColour;
+            bb.mRotation = p->mRotation;
             // Assign and compare at the same time
             if ((bb.mOwnDimensions = p->mOwnDimensions) == true)
             {
@@ -154,18 +172,22 @@ namespace Ogre {
             mBillboardSet->injectBillboard(bb);
 
         }
-        
+
+        // Only set bounds if there are any active particles
+        if(currentParticles.size())
+            mBillboardSet->setBounds( AxisAlignedBox( bboxMin, bboxMax ), radius );
+
         mBillboardSet->endBillboards();
 
         // Update the queue
         mBillboardSet->_updateRenderQueue(queue);
     }
-	//---------------------------------------------------------------------
-	void BillboardParticleRenderer::visitRenderables(Renderable::Visitor* visitor, 
-		bool debugRenderables)
-	{
-		mBillboardSet->visitRenderables(visitor, debugRenderables);
-	}
+    //---------------------------------------------------------------------
+    void BillboardParticleRenderer::visitRenderables(Renderable::Visitor* visitor, 
+        bool debugRenderables)
+    {
+        mBillboardSet->visitRenderables(visitor, debugRenderables);
+    }
     //-----------------------------------------------------------------------
     void BillboardParticleRenderer::_setMaterial(MaterialPtr& mat)
     {
@@ -176,16 +198,16 @@ namespace Ogre {
     {
         mBillboardSet->setBillboardType(bbt);
     }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setUseAccurateFacing(bool acc)
-	{
-		mBillboardSet->setUseAccurateFacing(acc);
-	}
-	//-----------------------------------------------------------------------
-	bool BillboardParticleRenderer::getUseAccurateFacing(void) const
-	{
-		return mBillboardSet->getUseAccurateFacing();
-	}
+    //-----------------------------------------------------------------------
+    void BillboardParticleRenderer::setUseAccurateFacing(bool acc)
+    {
+        mBillboardSet->setUseAccurateFacing(acc);
+    }
+    //-----------------------------------------------------------------------
+    bool BillboardParticleRenderer::getUseAccurateFacing(void) const
+    {
+        return mBillboardSet->getUseAccurateFacing();
+    }
     //-----------------------------------------------------------------------
     BillboardType BillboardParticleRenderer::getBillboardType(void) const
     {
@@ -251,38 +273,38 @@ namespace Ogre {
     {
         mBillboardSet->_notifyAttached(parent, isTagPoint);
     }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setRenderQueueGroup(uint8 queueID)
-	{
-		assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
-		mBillboardSet->setRenderQueueGroup(queueID);
-	}
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
-	{
-		assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
-		mBillboardSet->setRenderQueueGroupAndPriority(queueID, priority);
-	}
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setKeepParticlesInLocalSpace(bool keepLocal)
-	{
-		mBillboardSet->setBillboardsInWorldSpace(!keepLocal);
-	}
+    //-----------------------------------------------------------------------
+    void BillboardParticleRenderer::setRenderQueueGroup(uint8 queueID)
+    {
+        assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
+        mBillboardSet->setRenderQueueGroup(queueID);
+    }
+    //-----------------------------------------------------------------------
+    void BillboardParticleRenderer::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
+    {
+        assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
+        mBillboardSet->setRenderQueueGroupAndPriority(queueID, priority);
+    }
+    //-----------------------------------------------------------------------
+    void BillboardParticleRenderer::setKeepParticlesInLocalSpace(bool keepLocal)
+    {
+        mBillboardSet->setBillboardsInWorldSpace(!keepLocal);
+    }
     //-----------------------------------------------------------------------
     SortMode BillboardParticleRenderer::_getSortMode(void) const
     {
         return mBillboardSet->_getSortMode();
     }
-	//-----------------------------------------------------------------------
-	void BillboardParticleRenderer::setPointRenderingEnabled(bool enabled)
-	{
-		mBillboardSet->setPointRenderingEnabled(enabled);
-	}
-	//-----------------------------------------------------------------------
-	bool BillboardParticleRenderer::isPointRenderingEnabled(void) const
-	{
-		return mBillboardSet->isPointRenderingEnabled();
-	}
+    //-----------------------------------------------------------------------
+    void BillboardParticleRenderer::setPointRenderingEnabled(bool enabled)
+    {
+        mBillboardSet->setPointRenderingEnabled(enabled);
+    }
+    //-----------------------------------------------------------------------
+    bool BillboardParticleRenderer::isPointRenderingEnabled(void) const
+    {
+        return mBillboardSet->isPointRenderingEnabled();
+    }
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
@@ -384,7 +406,7 @@ namespace Ogre {
             return "bottom_right";
         }
         // Compiler nicety
-        return StringUtil::BLANK;
+        return BLANKSTRING;
     }
     void BillboardParticleRenderer::CmdBillboardOrigin::doSet(void* target, const String& val)
     {
@@ -428,7 +450,7 @@ namespace Ogre {
             return "texcoord";
         }
         // Compiler nicety
-        return StringUtil::BLANK;
+        return BLANKSTRING;
     }
     void BillboardParticleRenderer::CmdBillboardRotationType::doSet(void* target, const String& val)
     {
@@ -479,17 +501,17 @@ namespace Ogre {
         static_cast<BillboardParticleRenderer*>(target)->setPointRenderingEnabled(
             StringConverter::parseBool(val));
     }
-	//-----------------------------------------------------------------------
-	String BillboardParticleRenderer::CmdAccurateFacing::doGet(const void* target) const
-	{
-		return StringConverter::toString(
-			static_cast<const BillboardParticleRenderer*>(target)->getUseAccurateFacing() );
-	}
-	void BillboardParticleRenderer::CmdAccurateFacing::doSet(void* target, const String& val)
-	{
-		static_cast<BillboardParticleRenderer*>(target)->setUseAccurateFacing(
-			StringConverter::parseBool(val));
-	}
+    //-----------------------------------------------------------------------
+    String BillboardParticleRenderer::CmdAccurateFacing::doGet(const void* target) const
+    {
+        return StringConverter::toString(
+            static_cast<const BillboardParticleRenderer*>(target)->getUseAccurateFacing() );
+    }
+    void BillboardParticleRenderer::CmdAccurateFacing::doSet(void* target, const String& val)
+    {
+        static_cast<BillboardParticleRenderer*>(target)->setUseAccurateFacing(
+            StringConverter::parseBool(val));
+    }
 
 }
 
