@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,19 +28,8 @@ THE SOFTWARE.
 #include "OgreStableHeaders.h"
 
 #include "OgreBillboardSet.h"
-
 #include "OgreBillboard.h"
-#include "OgreMaterialManager.h"
-#include "OgreHardwareBufferManager.h"
-#include "OgreCamera.h"
-#include "OgreMath.h"
-#include "OgreSphere.h"
-#include "OgreRoot.h"
-#include "OgreRenderSystem.h"
-#include "OgreRenderSystemCapabilities.h"
-#include "OgreException.h"
-#include "OgreStringConverter.h"
-#include "OgreLogManager.h"
+
 #include <algorithm>
 
 namespace Ogre {
@@ -49,7 +38,7 @@ namespace Ogre {
 
     //-----------------------------------------------------------------------
     BillboardSet::BillboardSet() :
-		mBoundingRadius(0.0f), 
+        mBoundingRadius(0.0f), 
         mOriginType( BBO_CENTER ),
         mRotationType( BBR_TEXCOORD ),
         mAllDefaultSize( true ),
@@ -58,8 +47,6 @@ namespace Ogre {
         mAccurateFacing(false),
         mAllDefaultRotation(true),
         mWorldSpace(false),
-        mVertexData(0),
-        mIndexData(0),
         mCullIndividual( false ),
         mBillboardType(BBT_POINT),
         mCommonDirection(Ogre::Vector3::UNIT_Z),
@@ -67,12 +54,13 @@ namespace Ogre {
         mPointRendering(false),
         mBuffersCreated(false),
         mPoolSize(0),
-		mExternalData(false),
-		mAutoUpdate(true),
-		mBillboardDataChanged(true)
+        mExternalData(false),
+        mAutoUpdate(true),
+        mBillboardDataChanged(true)
     {
         setDefaultDimensions( 100, 100 );
-        setMaterialName( "BaseWhite" );
+        mMaterial = MaterialManager::getSingleton().getDefaultMaterial();
+        mMaterial->load();
         mCastShadows = false;
         setTextureStacksAndSlices( 1, 1 );
     }
@@ -83,7 +71,7 @@ namespace Ogre {
         unsigned int poolSize,
         bool externalData) :
         MovableObject(name),
-		mBoundingRadius(0.0f), 
+        mBoundingRadius(0.0f), 
         mOriginType( BBO_CENTER ),
         mRotationType( BBR_TEXCOORD ),
         mAllDefaultSize( true ),
@@ -92,21 +80,20 @@ namespace Ogre {
         mAccurateFacing(false),
         mAllDefaultRotation(true),
         mWorldSpace(false),
-        mVertexData(0),
-        mIndexData(0),
         mCullIndividual( false ),
         mBillboardType(BBT_POINT),
         mCommonDirection(Ogre::Vector3::UNIT_Z),
         mCommonUpVector(Vector3::UNIT_Y),
-		mPointRendering(false),
+        mPointRendering(false),
         mBuffersCreated(false),
         mPoolSize(poolSize),
         mExternalData(externalData),
-		mAutoUpdate(true),
-		mBillboardDataChanged(true)
+        mAutoUpdate(true),
+        mBillboardDataChanged(true)
     {
         setDefaultDimensions( 100, 100 );
-        setMaterialName( "BaseWhite" );
+        mMaterial = MaterialManager::getSingleton().getDefaultMaterial();
+        mMaterial->load();
         setPoolSize( poolSize );
         mCastShadows = false;
         setTextureStacksAndSlices( 1, 1 );
@@ -122,7 +109,7 @@ namespace Ogre {
         }
 
         // Delete shared buffers
-		_destroyBuffers();
+        _destroyBuffers();
     }
     //-----------------------------------------------------------------------
     Billboard* BillboardSet::createBillboard(
@@ -143,8 +130,8 @@ namespace Ogre {
 
         // Get a new billboard
         Billboard* newBill = mFreeBillboards.front();
-		mActiveBillboards.splice(
-			mActiveBillboards.end(), mFreeBillboards, mFreeBillboards.begin());
+        mActiveBillboards.splice(
+            mActiveBillboards.end(), mFreeBillboards, mFreeBillboards.begin());
         newBill->setPosition(position);
         newBill->setColour(colour);
         newBill->mDirection = Vector3::ZERO;
@@ -153,16 +140,16 @@ namespace Ogre {
         newBill->resetDimensions();
         newBill->_notifyOwner(this);
 
-		// Merge into bounds
-		Real adjust = std::max(mDefaultWidth, mDefaultHeight);
+        // Merge into bounds
+        Real adjust = std::max(mDefaultWidth, mDefaultHeight);
         Vector3 vecAdjust(adjust, adjust, adjust);
-		Vector3 newMin = position - vecAdjust;
-		Vector3 newMax = position + vecAdjust;
+        Vector3 newMin = position - vecAdjust;
+        Vector3 newMax = position + vecAdjust;
 
         mAABB.merge(newMin);
         mAABB.merge(newMax);
 
-		mBoundingRadius = Math::boundingRadiusFromAABB(mAABB);
+        mBoundingRadius = Math::boundingRadiusFromAABB(mAABB);
 
         return newBill;
     }
@@ -184,8 +171,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void BillboardSet::clear()
     {
-		// Move actives to free list
-		mFreeBillboards.splice(mFreeBillboards.end(), mActiveBillboards);
+        // Move actives to free list
+        mFreeBillboards.splice(mFreeBillboards.end(), mActiveBillboards);
     }
 
     //-----------------------------------------------------------------------
@@ -202,7 +189,7 @@ namespace Ogre {
         ActiveBillboardList::const_iterator it;
         if( index >= ( mActiveBillboards.size() >> 1 ) )
         {
-			index = static_cast<unsigned int>(mActiveBillboards.size()) - index;
+            index = static_cast<unsigned int>(mActiveBillboards.size()) - index;
             for( it = mActiveBillboards.end(); index; --index, --it );
         }
         else
@@ -229,7 +216,7 @@ namespace Ogre {
         ActiveBillboardList::iterator it;
         if( index >= ( mActiveBillboards.size() >> 1 ) )
         {
-			index = static_cast<unsigned int>(mActiveBillboards.size()) - index;
+            index = static_cast<unsigned int>(mActiveBillboards.size()) - index;
             for( it = mActiveBillboards.end(); index; --index, --it );
         }
         else
@@ -303,13 +290,11 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void BillboardSet::setMaterialName( const String& name , const String& groupName /* = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME */ )
     {
-        mMaterialName = name;
-
         mMaterial = MaterialManager::getSingleton().getByName(name, groupName);
 
-		if (mMaterial.isNull())
-			OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material " + name,
-				"BillboardSet::setMaterialName" );
+        if (!mMaterial)
+            OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find material " + name,
+                "BillboardSet::setMaterialName" );
 
         /* Ensure that the new material was loaded (will not load again if
            already loaded anyway)
@@ -320,7 +305,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     const String& BillboardSet::getMaterialName(void) const
     {
-        return mMaterialName;
+        return mMaterial->getName();
     }
 
     //-----------------------------------------------------------------------
@@ -371,7 +356,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void BillboardSet::_notifyCurrentCamera( Camera* cam )
     {
-		MovableObject::_notifyCurrentCamera(cam);
+        MovableObject::_notifyCurrentCamera(cam);
 
         mCurrentCamera = cam;
 
@@ -383,9 +368,8 @@ namespace Ogre {
             // Default behaviour is that billboards are in local node space
             // so orientation of camera (in world space) must be reverse-transformed
             // into node space
-            mCamQ = mParentNode->_getDerivedOrientation().UnitInverse() * mCamQ;
-            mCamPos = mParentNode->_getDerivedOrientation().UnitInverse() *
-                (mCamPos - mParentNode->_getDerivedPosition()) / mParentNode->_getDerivedScale();
+            mCamQ = mParentNode->convertWorldToLocalOrientation(mCamQ);
+            mCamPos = mParentNode->convertWorldToLocalPosition(mCamPos);
         }
 
         // Camera direction points down -Z
@@ -418,74 +402,74 @@ namespace Ogre {
         if(!mBuffersCreated)
             _createBuffers();
 
-		// Only calculate vertex offets et al if we're not point rendering
-		if (!mPointRendering)
-		{
+        // Only calculate vertex offets et al if we're not point rendering
+        if (!mPointRendering)
+        {
 
-			// Get offsets for origin type
-			getParametricOffsets(mLeftOff, mRightOff, mTopOff, mBottomOff);
+            // Get offsets for origin type
+            getParametricOffsets(mLeftOff, mRightOff, mTopOff, mBottomOff);
 
-			// Generate axes etc up-front if not oriented per-billboard
-			if (mBillboardType != BBT_ORIENTED_SELF &&
-				mBillboardType != BBT_PERPENDICULAR_SELF && 
-				!(mAccurateFacing && mBillboardType != BBT_PERPENDICULAR_COMMON))
-			{
-				genBillboardAxes(&mCamX, &mCamY);
+            // Generate axes etc up-front if not oriented per-billboard
+            if (mBillboardType != BBT_ORIENTED_SELF &&
+                mBillboardType != BBT_PERPENDICULAR_SELF && 
+                !(mAccurateFacing && mBillboardType != BBT_PERPENDICULAR_COMMON))
+            {
+                genBillboardAxes(&mCamX, &mCamY);
 
-				/* If all billboards are the same size we can precalculate the
-				   offsets and just use '+' instead of '*' for each billboard,
-				   and it should be faster.
-				*/
-				genVertOffsets(mLeftOff, mRightOff, mTopOff, mBottomOff,
-					mDefaultWidth, mDefaultHeight, mCamX, mCamY, mVOffset);
+                /* If all billboards are the same size we can precalculate the
+                   offsets and just use '+' instead of '*' for each billboard,
+                   and it should be faster.
+                */
+                genVertOffsets(mLeftOff, mRightOff, mTopOff, mBottomOff,
+                    mDefaultWidth, mDefaultHeight, mCamX, mCamY, mVOffset);
 
-			}
-		}
+            }
+        }
 
         // Init num visible
         mNumVisibleBillboards = 0;
 
         // Lock the buffer
-		if (numBillboards) // optimal lock
-		{
-			// clamp to max
-			numBillboards = std::min(mPoolSize, numBillboards);
+        if (numBillboards) // optimal lock
+        {
+            // clamp to max
+            numBillboards = std::min(mPoolSize, numBillboards);
 
-			size_t billboardSize;
-			if (mPointRendering)
-			{
-				// just one vertex per billboard (this also excludes texcoords)
-				billboardSize = mMainBuf->getVertexSize();
-			}
-			else
-			{
-				// 4 corners
-				billboardSize = mMainBuf->getVertexSize() * 4;
-			}
-			assert (numBillboards * billboardSize <= mMainBuf->getSizeInBytes());
+            size_t billboardSize;
+            if (mPointRendering)
+            {
+                // just one vertex per billboard (this also excludes texcoords)
+                billboardSize = mMainBuf->getVertexSize();
+            }
+            else
+            {
+                // 4 corners
+                billboardSize = mMainBuf->getVertexSize() * 4;
+            }
+            assert (numBillboards * billboardSize <= mMainBuf->getSizeInBytes());
 
-			mLockPtr = static_cast<float*>(
-				mMainBuf->lock(0, numBillboards * billboardSize, 
-				mMainBuf->getUsage() & HardwareBuffer::HBU_DYNAMIC ?
-				HardwareBuffer::HBL_DISCARD : HardwareBuffer::HBL_NORMAL) );
-		}
-		else // lock the entire thing
-			mLockPtr = static_cast<float*>(
-			mMainBuf->lock(mMainBuf->getUsage() & HardwareBuffer::HBU_DYNAMIC ?
-			HardwareBuffer::HBL_DISCARD : HardwareBuffer::HBL_NORMAL) );
+            mLockPtr = static_cast<float*>(
+                mMainBuf->lock(0, numBillboards * billboardSize, 
+                mMainBuf->getUsage() & HardwareBuffer::HBU_DYNAMIC ?
+                HardwareBuffer::HBL_DISCARD : HardwareBuffer::HBL_NORMAL) );
+        }
+        else // lock the entire thing
+            mLockPtr = static_cast<float*>(
+            mMainBuf->lock(mMainBuf->getUsage() & HardwareBuffer::HBU_DYNAMIC ?
+            HardwareBuffer::HBL_DISCARD : HardwareBuffer::HBL_NORMAL) );
 
     }
     //-----------------------------------------------------------------------
     void BillboardSet::injectBillboard(const Billboard& bb)
     {
-		// Don't accept injections beyond pool size
-		if (mNumVisibleBillboards == mPoolSize) return;
+        // Don't accept injections beyond pool size
+        if (mNumVisibleBillboards == mPoolSize) return;
 
-		// Skip if not visible (NB always true if not bounds checking individual billboards)
+        // Skip if not visible (NB always true if not bounds checking individual billboards)
         if (!billboardVisible(mCurrentCamera, bb)) return;
 
         if (!mPointRendering &&
-			(mBillboardType == BBT_ORIENTED_SELF ||
+            (mBillboardType == BBT_ORIENTED_SELF ||
             mBillboardType == BBT_PERPENDICULAR_SELF ||
             (mAccurateFacing && mBillboardType != BBT_PERPENDICULAR_COMMON)))
         {
@@ -493,7 +477,7 @@ namespace Ogre {
             genBillboardAxes(&mCamX, &mCamY, &bb);
         }
 
-		// If they're all the same size or we're point rendering
+        // If they're all the same size or we're point rendering
         if( mAllDefaultSize || mPointRendering)
         {
             /* No per-billboard checking, just blast through.
@@ -502,9 +486,9 @@ namespace Ogre {
             */
 
             if (!mPointRendering &&
-				(mBillboardType == BBT_ORIENTED_SELF ||
-           		mBillboardType == BBT_PERPENDICULAR_SELF ||
-           		(mAccurateFacing && mBillboardType != BBT_PERPENDICULAR_COMMON)))
+                (mBillboardType == BBT_ORIENTED_SELF ||
+                mBillboardType == BBT_PERPENDICULAR_SELF ||
+                (mAccurateFacing && mBillboardType != BBT_PERPENDICULAR_COMMON)))
             {
                 genVertOffsets(mLeftOff, mRightOff, mTopOff, mBottomOff,
                     mDefaultWidth, mDefaultHeight, mCamX, mCamY, mVOffset);
@@ -539,12 +523,12 @@ namespace Ogre {
     {
         mMainBuf->unlock();
     }
-	//-----------------------------------------------------------------------
-	void BillboardSet::setBounds(const AxisAlignedBox& box, Real radius)
-	{
-		mAABB = box;
-		mBoundingRadius = radius;
-	}
+    //-----------------------------------------------------------------------
+    void BillboardSet::setBounds(const AxisAlignedBox& box, Real radius)
+    {
+        mAABB = box;
+        mBoundingRadius = radius;
+    }
     //-----------------------------------------------------------------------
     void BillboardSet::_updateBounds(void)
     {
@@ -552,31 +536,32 @@ namespace Ogre {
         {
             // No billboards, null bbox
             mAABB.setNull();
-			mBoundingRadius = 0.0f;
+            mBoundingRadius = 0.0f;
         }
         else
         {
-			Real maxSqLen = -1.0f;
+            Real maxSqLen = -1.0f;
 
             Vector3 min(Math::POS_INFINITY, Math::POS_INFINITY, Math::POS_INFINITY);
             Vector3 max(Math::NEG_INFINITY, Math::NEG_INFINITY, Math::NEG_INFINITY);
             ActiveBillboardList::iterator i, iend;
 
             iend = mActiveBillboards.end();
-			Matrix4 invWorld;
-			if (mWorldSpace && getParentSceneNode())
-				invWorld = getParentSceneNode()->_getFullTransform().inverse();
+            Affine3 invWorld;
+            bool invert = mWorldSpace && getParentSceneNode();
+            if (invert)
+                invWorld = getParentSceneNode()->_getFullTransform().inverse();
 
             for (i = mActiveBillboards.begin(); i != iend; ++i)
             {
                 Vector3 pos = (*i)->getPosition();
-				// transform from world space to local space
-				if (mWorldSpace && getParentSceneNode())
-					pos = invWorld * pos;
+                // transform from world space to local space
+                if (invert)
+                    pos = invWorld * pos;
                 min.makeFloor(pos);
                 max.makeCeil(pos);
 
-				maxSqLen = std::max(maxSqLen, pos.squaredLength());
+                maxSqLen = std::max(maxSqLen, pos.squaredLength());
             }
             // Adjust for billboard size
             Real adjust = std::max(mDefaultWidth, mDefaultHeight);
@@ -585,7 +570,7 @@ namespace Ogre {
             max += vecAdjust;
 
             mAABB.setExtents(min, max);
-			mBoundingRadius = Math::Sqrt(maxSqLen);
+            mBoundingRadius = Math::Sqrt(maxSqLen);
 
         }
 
@@ -619,15 +604,15 @@ namespace Ogre {
                 injectBillboard(*(*it));
             }
             endBillboards();
-			mBillboardDataChanged = false;
+            mBillboardDataChanged = false;
         }
 
         //only set the render queue group if it has been explicitly set.
-		if (mRenderQueuePrioritySet)
-		{
-			assert(mRenderQueueIDSet == true);
-			queue->addRenderable(this, mRenderQueueID, mRenderQueuePriority);
-		}
+        if (mRenderQueuePrioritySet)
+        {
+            assert(mRenderQueueIDSet == true);
+            queue->addRenderable(this, mRenderQueueID, mRenderQueuePriority);
+        }
         else if( mRenderQueueIDSet )
         {
            queue->addRenderable(this, mRenderQueueID);
@@ -644,58 +629,48 @@ namespace Ogre {
     }
 
     void BillboardSet::setMaterial( const MaterialPtr& material )
-	{
-		mMaterial = material;
-		
-        if (mMaterial.isNull())
-        {
-			LogManager::getSingleton().logMessage("Can't assign material "  
-                                                  " to BillboardSet of " + getName() + " because this "
-                                                  "Material does not exist. Have you forgotten to define it in a "
-                                                  ".material script?", LML_CRITICAL);
-			
-			mMaterial = MaterialManager::getSingleton().getByName("BaseWhite");
-			
-            if (mMaterial.isNull())
-            {
-                OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "Can't assign default material "
-                            "to BillboardSet " + getName() + ". Did "
-                            "you forget to call MaterialManager::initialise()?",
-                            "BillboardSet::setMaterial");
-            }
-        }
-		
-		mMaterialName = mMaterial->getName();
+    {
+        mMaterial = material;
         
+        if (!mMaterial)
+        {
+            LogManager::getSingleton().logMessage("Can't assign material " + material->getName()+
+                                                  " to BillboardSet of " + getName() + " because this "
+                                                  "Material does not exist in group "+material->getGroup()+". Have you forgotten to define it in a "
+                                                  ".material script?", LML_CRITICAL);
+
+            mMaterial = MaterialManager::getSingleton().getDefaultMaterial();
+        }
+
         // Ensure new material loaded (will not load again if already loaded)
         mMaterial->load();
-	}
+    }
 
     //-----------------------------------------------------------------------
     void BillboardSet::getRenderOperation(RenderOperation& op)
     {
-        op.vertexData = mVertexData;
-       	op.vertexData->vertexStart = 0;
+        op.vertexData = mVertexData.get();
+        op.vertexData->vertexStart = 0;
 
-		if (mPointRendering)
-		{
-			op.operationType = RenderOperation::OT_POINT_LIST;
-    	    op.useIndexes = false;
+        if (mPointRendering)
+        {
+            op.operationType = RenderOperation::OT_POINT_LIST;
+            op.useIndexes = false;
             op.useGlobalInstancingVertexBufferIsAvailable = false;
-			op.indexData = 0;
-			op.vertexData->vertexCount = mNumVisibleBillboards;
-		}
-		else
-		{
-			op.operationType = RenderOperation::OT_TRIANGLE_LIST;
-    	    op.useIndexes = true;
+            op.indexData = 0;
+            op.vertexData->vertexCount = mNumVisibleBillboards;
+        }
+        else
+        {
+            op.operationType = RenderOperation::OT_TRIANGLE_LIST;
+            op.useIndexes = true;
 
-    	    op.vertexData->vertexCount = mNumVisibleBillboards * 4;
+            op.vertexData->vertexCount = mNumVisibleBillboards * 4;
 
-	        op.indexData = mIndexData;
-    	    op.indexData->indexCount = mNumVisibleBillboards * 6;
-        	op.indexData->indexStart = 0;
-		}
+            op.indexData = mIndexData.get();
+            op.indexData->indexCount = mNumVisibleBillboards * 6;
+            op.indexData->indexStart = 0;
+        }
     }
 
     //-----------------------------------------------------------------------
@@ -757,7 +732,7 @@ namespace Ogre {
 
         mPoolSize = size;
 
-		_destroyBuffers();
+        _destroyBuffers();
     }
 
     //-----------------------------------------------------------------------
@@ -774,22 +749,22 @@ namespace Ogre {
                  tex. coords ( 2D coords, 1 or 4 per billboard )
         */
 
-		// Warn if user requested an invalid setup
-		// Do it here so it only appears once
-		if (mPointRendering && mBillboardType != BBT_POINT)
-		{
+        // Warn if user requested an invalid setup
+        // Do it here so it only appears once
+        if (mPointRendering && mBillboardType != BBT_POINT)
+        {
 
-			LogManager::getSingleton().logMessage("Warning: BillboardSet " +
-				mName + " has point rendering enabled but is using a type "
-				"other than BBT_POINT, this may not give you the results you "
-				"expect.", LML_CRITICAL);
-		}
+            LogManager::getSingleton().logWarning("BillboardSet " +
+                mName + " has point rendering enabled but is using a type "
+                "other than BBT_POINT, this may not give you the results you "
+                "expect.");
+        }
 
-        mVertexData = OGRE_NEW VertexData();
-		if (mPointRendering)
-			mVertexData->vertexCount = mPoolSize;
-		else
-			mVertexData->vertexCount = mPoolSize * 4;
+        mVertexData.reset(new VertexData());
+        if (mPointRendering)
+            mVertexData->vertexCount = mPoolSize;
+        else
+            mVertexData->vertexCount = mPoolSize * 4;
 
         mVertexData->vertexStart = 0;
 
@@ -798,10 +773,8 @@ namespace Ogre {
         VertexBufferBinding* binding = mVertexData->vertexBufferBinding;
 
         size_t offset = 0;
-        decl->addElement(0, offset, VET_FLOAT3, VES_POSITION);
-        offset += VertexElement::getTypeSize(VET_FLOAT3);
-        decl->addElement(0, offset, VET_COLOUR, VES_DIFFUSE);
-        offset += VertexElement::getTypeSize(VET_COLOUR);
+        offset += decl->addElement(0, offset, VET_FLOAT3, VES_POSITION).getSize();
+        offset += decl->addElement(0, offset, VET_COLOUR, VES_DIFFUSE).getSize();
         // Texture coords irrelevant when enabled point rendering (generated
         // in point sprite mode, and unused in standard point mode)
         if (!mPointRendering)
@@ -813,80 +786,66 @@ namespace Ogre {
             HardwareBufferManager::getSingleton().createVertexBuffer(
                 decl->getVertexSize(0),
                 mVertexData->vertexCount,
-				mAutoUpdate ? HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY_DISCARDABLE : 
-				HardwareBuffer::HBU_STATIC_WRITE_ONLY);
+                mAutoUpdate ? HardwareBuffer::HBU_DYNAMIC_WRITE_ONLY_DISCARDABLE : 
+                HardwareBuffer::HBU_STATIC_WRITE_ONLY);
         // bind position and diffuses
         binding->setBinding(0, mMainBuf);
 
-		if (!mPointRendering)
-		{
-			mIndexData  = OGRE_NEW IndexData();
-			mIndexData->indexStart = 0;
-			mIndexData->indexCount = mPoolSize * 6;
+        if (!mPointRendering)
+        {
+            mIndexData.reset(new IndexData());
+            mIndexData->indexStart = 0;
+            mIndexData->indexCount = mPoolSize * 6;
 
-			mIndexData->indexBuffer = HardwareBufferManager::getSingleton().
-				createIndexBuffer(HardwareIndexBuffer::IT_16BIT,
-					mIndexData->indexCount,
-					HardwareBuffer::HBU_STATIC_WRITE_ONLY);
+            mIndexData->indexBuffer = HardwareBufferManager::getSingleton().
+                createIndexBuffer(HardwareIndexBuffer::IT_16BIT,
+                    mIndexData->indexCount,
+                    HardwareBuffer::HBU_STATIC_WRITE_ONLY);
 
-			/* Create indexes (will be the same every frame)
-			   Using indexes because it means 1/3 less vertex transforms (4 instead of 6)
+            /* Create indexes (will be the same every frame)
+               Using indexes because it means 1/3 less vertex transforms (4 instead of 6)
 
-			   Billboard layout relative to camera:
+               Billboard layout relative to camera:
 
-				0-----1
-				|    /|
-				|  /  |
-				|/    |
-				2-----3
-			*/
+                0-----1
+                |    /|
+                |  /  |
+                |/    |
+                2-----3
+            */
 
-			ushort* pIdx = static_cast<ushort*>(
-				mIndexData->indexBuffer->lock(0,
-				  mIndexData->indexBuffer->getSizeInBytes(),
-				  HardwareBuffer::HBL_DISCARD) );
+            HardwareBufferLockGuard indexLock(mIndexData->indexBuffer, HardwareBuffer::HBL_DISCARD);
+            ushort* pIdx = static_cast<ushort*>(indexLock.pData);
 
-			for(
-				size_t idx, idxOff, bboard = 0;
-				bboard < mPoolSize;
-				++bboard )
-			{
-				// Do indexes
-				idx    = bboard * 6;
-				idxOff = bboard * 4;
+            for(
+                size_t idx, idxOff, bboard = 0;
+                bboard < mPoolSize;
+                ++bboard )
+            {
+                // Do indexes
+                idx    = bboard * 6;
+                idxOff = bboard * 4;
 
-				pIdx[idx] = static_cast<unsigned short>(idxOff); // + 0;, for clarity
-				pIdx[idx+1] = static_cast<unsigned short>(idxOff + 2);
-				pIdx[idx+2] = static_cast<unsigned short>(idxOff + 1);
-				pIdx[idx+3] = static_cast<unsigned short>(idxOff + 1);
-				pIdx[idx+4] = static_cast<unsigned short>(idxOff + 2);
-				pIdx[idx+5] = static_cast<unsigned short>(idxOff + 3);
+                pIdx[idx] = static_cast<unsigned short>(idxOff); // + 0;, for clarity
+                pIdx[idx+1] = static_cast<unsigned short>(idxOff + 2);
+                pIdx[idx+2] = static_cast<unsigned short>(idxOff + 1);
+                pIdx[idx+3] = static_cast<unsigned short>(idxOff + 1);
+                pIdx[idx+4] = static_cast<unsigned short>(idxOff + 2);
+                pIdx[idx+5] = static_cast<unsigned short>(idxOff + 3);
 
-			}
-
-			mIndexData->indexBuffer->unlock();
-		}
+            }
+        }
         mBuffersCreated = true;
     }
     //-----------------------------------------------------------------------
-	void BillboardSet::_destroyBuffers(void)
-	{
-        if (mVertexData)
-        {
-            OGRE_DELETE mVertexData;
-            mVertexData = 0;
-        }
-        if (mIndexData)
-        {
-            OGRE_DELETE mIndexData;
-            mIndexData = 0;
-        }
+    void BillboardSet::_destroyBuffers(void)
+    {
+        mVertexData.reset();
+        mIndexData.reset();
+        mMainBuf.reset();
 
-        mMainBuf.setNull();
-
-		mBuffersCreated = false;
-
-	}
+        mBuffersCreated = false;
+    }
     //-----------------------------------------------------------------------
     unsigned int BillboardSet::getPoolSize(void) const
     {
@@ -997,7 +956,7 @@ namespace Ogre {
 
         getWorldTransforms(&xworld);
 
-        sph.setCenter(xworld.transformAffine(bill.mPosition));
+        sph.setCenter(xworld * bill.mPosition);
 
         if (bill.mOwnDimensions)
         {
@@ -1124,28 +1083,28 @@ namespace Ogre {
     {
         return mCommonUpVector;
     }
-	//-----------------------------------------------------------------------
-	uint32 BillboardSet::getTypeFlags(void) const
-	{
-		return SceneManager::FX_TYPE_MASK;
-	}
+    //-----------------------------------------------------------------------
+    uint32 BillboardSet::getTypeFlags(void) const
+    {
+        return SceneManager::FX_TYPE_MASK;
+    }
     //-----------------------------------------------------------------------
     void BillboardSet::genVertices(
         const Vector3* const offsets, const Billboard& bb)
     {
         RGBA colour;
         Root::getSingleton().convertColourValue(bb.mColour, &colour);
-		RGBA* pCol;
+        RGBA* pCol;
 
         // Texcoords
         assert( bb.mUseTexcoordRect || bb.mTexcoordIndex < mTextureCoords.size() );
         const Ogre::FloatRect & r =
             bb.mUseTexcoordRect ? bb.mTexcoordRect : mTextureCoords[bb.mTexcoordIndex];
 
-		if (mPointRendering)
-		{
-			// Single vertex per billboard, ignore offsets
-			// position
+        if (mPointRendering)
+        {
+            // Single vertex per billboard, ignore offsets
+            // position
             *mLockPtr++ = bb.mPosition.x;
             *mLockPtr++ = bb.mPosition.y;
             *mLockPtr++ = bb.mPosition.z;
@@ -1153,11 +1112,11 @@ namespace Ogre {
             // Convert float* to RGBA*
             pCol = static_cast<RGBA*>(static_cast<void*>(mLockPtr));
             *pCol++ = colour;
-			// Update lock pointer
-			mLockPtr = static_cast<float*>(static_cast<void*>(pCol));
+            // Update lock pointer
+            mLockPtr = static_cast<float*>(static_cast<void*>(pCol));
             // No texture coords in point rendering
-		}
-		else if (mAllDefaultRotation || bb.mRotation == Radian(0))
+        }
+        else if (mAllDefaultRotation || bb.mRotation == Radian(0))
         {
             // Left-top
             // Positions
@@ -1225,7 +1184,7 @@ namespace Ogre {
             Vector3 axis = (offsets[3] - offsets[0]).crossProduct(offsets[2] - offsets[1]).normalisedCopy();
 
             Matrix3 rotation;
-			rotation.FromAngleAxis(axis, bb.mRotation);
+            rotation.FromAngleAxis(axis, bb.mRotation);
 
             Vector3 pt;
 
@@ -1395,7 +1354,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     const String& BillboardSet::getMovableType(void) const
     {
-		return BillboardSetFactory::FACTORY_TYPE_NAME;
+        return BillboardSetFactory::FACTORY_TYPE_NAME;
     }
     //-----------------------------------------------------------------------
     Real BillboardSet::getSquaredViewDepth(const Camera* const cam) const
@@ -1404,10 +1363,10 @@ namespace Ogre {
         return mParentNode->getSquaredViewDepth(cam);
     }
     //-----------------------------------------------------------------------
-	Real BillboardSet::getBoundingRadius(void) const
-	{
-		return mBoundingRadius;
-	}
+    Real BillboardSet::getBoundingRadius(void) const
+    {
+        return mBoundingRadius;
+    }
     //-----------------------------------------------------------------------
     const LightList& BillboardSet::getLights(void) const
     {
@@ -1415,20 +1374,20 @@ namespace Ogre {
         // because most billboards are unlit, but here we go anyway
         return queryLights();
     }
-	//---------------------------------------------------------------------
-	void BillboardSet::visitRenderables(Renderable::Visitor* visitor, 
-		bool debugRenderables)
-	{
-		// only one renderable
-		visitor->visit(this, 0, false);
-	}
+    //---------------------------------------------------------------------
+    void BillboardSet::visitRenderables(Renderable::Visitor* visitor, 
+        bool debugRenderables)
+    {
+        // only one renderable
+        visitor->visit(this, 0, false);
+    }
 
 
     void BillboardSet::setTextureCoords( Ogre::FloatRect const * coords, uint16 numCoords )
     {
       if( !numCoords || !coords ) {
         setTextureStacksAndSlices( 1, 1 );
-		return;
+        return;
       }
       //  clear out any previous allocation (as vectors may not shrink)
       TextureCoordSets().swap( mTextureCoords );
@@ -1464,88 +1423,88 @@ namespace Ogre {
       }
       assert( coordIndex == (size_t)stacks * slices );
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Ogre::FloatRect const * BillboardSet::getTextureCoords( uint16 * oNumCoords )
     {
       *oNumCoords = (uint16)mTextureCoords.size();
       //  std::vector<> is guaranteed to be contiguous
       return &mTextureCoords.front();
     }
-	//-----------------------------------------------------------------------
-	void BillboardSet::setPointRenderingEnabled(bool enabled)
-	{
-		// Override point rendering if not supported
-		if (enabled && !Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_POINT_SPRITES))
-		{
-			enabled = false;
-		}
+    //-----------------------------------------------------------------------
+    void BillboardSet::setPointRenderingEnabled(bool enabled)
+    {
+        // Override point rendering if not supported
+        if (enabled && !Root::getSingleton().getRenderSystem()->getCapabilities()->hasCapability(RSC_POINT_SPRITES))
+        {
+            enabled = false;
+        }
 
-		if (enabled != mPointRendering)
-		{
-			mPointRendering = enabled;
-			// Different buffer structure (1 or 4 verts per billboard)
-			_destroyBuffers();
-		}
-	}
+        if (enabled != mPointRendering)
+        {
+            mPointRendering = enabled;
+            // Different buffer structure (1 or 4 verts per billboard)
+            _destroyBuffers();
+        }
+    }
 
-	//-----------------------------------------------------------------------
-	void BillboardSet::setAutoUpdate(bool autoUpdate)
-	{
-		// Case auto update buffers changed we have to destroy the current buffers
-		// since their usage will be different.
-		if (autoUpdate != mAutoUpdate)
-		{
-			mAutoUpdate = autoUpdate;
-			_destroyBuffers();
-		}
-	}
+    //-----------------------------------------------------------------------
+    void BillboardSet::setAutoUpdate(bool autoUpdate)
+    {
+        // Case auto update buffers changed we have to destroy the current buffers
+        // since their usage will be different.
+        if (autoUpdate != mAutoUpdate)
+        {
+            mAutoUpdate = autoUpdate;
+            _destroyBuffers();
+        }
+    }
 
-	//-----------------------------------------------------------------------
-	//-----------------------------------------------------------------------
-	String BillboardSetFactory::FACTORY_TYPE_NAME = "BillboardSet";
-	//-----------------------------------------------------------------------
-	const String& BillboardSetFactory::getType(void) const
-	{
-		return FACTORY_TYPE_NAME;
-	}
-	//-----------------------------------------------------------------------
-	MovableObject* BillboardSetFactory::createInstanceImpl( const String& name,
-		const NameValuePairList* params)
-	{
-		// may have parameters
-		bool externalData = false;
-		unsigned int poolSize = 0;
+    //-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
+    String BillboardSetFactory::FACTORY_TYPE_NAME = "BillboardSet";
+    //-----------------------------------------------------------------------
+    const String& BillboardSetFactory::getType(void) const
+    {
+        return FACTORY_TYPE_NAME;
+    }
+    //-----------------------------------------------------------------------
+    MovableObject* BillboardSetFactory::createInstanceImpl( const String& name,
+        const NameValuePairList* params)
+    {
+        // may have parameters
+        bool externalData = false;
+        unsigned int poolSize = 0;
 
-		if (params != 0)
-		{
-			NameValuePairList::const_iterator ni = params->find("poolSize");
-			if (ni != params->end())
-			{
-				poolSize = StringConverter::parseUnsignedInt(ni->second);
-			}
-			ni = params->find("externalData");
-			if (ni != params->end())
-			{
-				externalData = StringConverter::parseBool(ni->second);
-			}
+        if (params != 0)
+        {
+            NameValuePairList::const_iterator ni = params->find("poolSize");
+            if (ni != params->end())
+            {
+                poolSize = StringConverter::parseUnsignedInt(ni->second);
+            }
+            ni = params->find("externalData");
+            if (ni != params->end())
+            {
+                externalData = StringConverter::parseBool(ni->second);
+            }
 
-		}
+        }
 
-		if (poolSize > 0)
-		{
-			return OGRE_NEW BillboardSet(name, poolSize, externalData);
-		}
-		else
-		{
-			return OGRE_NEW BillboardSet(name);
-		}
+        if (poolSize > 0)
+        {
+            return OGRE_NEW BillboardSet(name, poolSize, externalData);
+        }
+        else
+        {
+            return OGRE_NEW BillboardSet(name);
+        }
 
-	}
-	//-----------------------------------------------------------------------
-	void BillboardSetFactory::destroyInstance( MovableObject* obj)
-	{
-		OGRE_DELETE obj;
-	}
+    }
+    //-----------------------------------------------------------------------
+    void BillboardSetFactory::destroyInstance( MovableObject* obj)
+    {
+        OGRE_DELETE obj;
+    }
 
 
 }
