@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,13 +27,10 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 #include "OgreCodec.h"
-#include "OgreException.h"
-#include "OgreStringConverter.h"
-
 
 namespace Ogre {
 
-    map< String, Codec * >::type Codec::msMapCodecs;
+    std::map< String, Codec * > Codec::msMapCodecs;
 
     Codec::~Codec() {
     }
@@ -53,7 +50,7 @@ namespace Ogre {
     Codec* Codec::getCodec(const String& extension)
     {
         String lwrcase = extension;
-		StringUtil::toLowerCase(lwrcase);
+        StringUtil::toLowerCase(lwrcase);
         CodecList::const_iterator i = msMapCodecs.find(lwrcase);
         if (i == msMapCodecs.end())
         {
@@ -73,27 +70,27 @@ namespace Ogre {
 
     }
 
-	Codec* Codec::getCodec(char *magicNumberPtr, size_t maxbytes)
-	{
-		for (CodecList::const_iterator i = msMapCodecs. begin(); 
-			i != msMapCodecs.end(); ++i)
-		{
-			String ext = i->second->magicNumberToFileExt(magicNumberPtr, maxbytes);
-			if (!ext.empty())
-			{
-				// check codec type matches
-				// if we have a single codec class that can handle many types, 
-				// and register many instances of it against different types, we
-				// can end up matching the wrong one here, so grab the right one
-				if (ext == i->second->getType())
-					return i->second;
-				else
-					return getCodec(ext);
-			}
-		}
+    Codec* Codec::getCodec(char *magicNumberPtr, size_t maxbytes)
+    {
+        for (CodecList::const_iterator i = msMapCodecs. begin(); 
+            i != msMapCodecs.end(); ++i)
+        {
+            String ext = i->second->magicNumberToFileExt(magicNumberPtr, maxbytes);
+            if (!ext.empty())
+            {
+                // check codec type matches
+                // if we have a single codec class that can handle many types, 
+                // and register many instances of it against different types, we
+                // can end up matching the wrong one here, so grab the right one
+                if (ext == i->second->getType())
+                    return i->second;
+                else
+                    return getCodec(ext);
+            }
+        }
 
-		return 0;
+        return 0;
 
-	}
+    }
 
 }
