@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,8 +26,6 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreColourValue.h"
-#include "OgreMath.h"
 
 namespace Ogre {
 
@@ -101,38 +99,38 @@ namespace Ogre {
 
         return val32;
     }
-	//---------------------------------------------------------------------
+    //---------------------------------------------------------------------
 #if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-	ARGB ColourValue::getAsARGB(void) const
+    ARGB ColourValue::getAsARGB(void) const
 #else
-	BGRA ColourValue::getAsBGRA(void) const
+    BGRA ColourValue::getAsBGRA(void) const
 #endif
-	{
-		uint8 val8;
-		uint32 val32 = 0;
+    {
+        uint8 val8;
+        uint32 val32 = 0;
 
-		// Convert to 32bit pattern
-		// (ARGB = 8888)
+        // Convert to 32bit pattern
+        // (ARGB = 8888)
 
-		// Blue
-		val8 = static_cast<uint8>(b * 255);
-		val32 = val8 << 24;
+        // Blue
+        val8 = static_cast<uint8>(b * 255);
+        val32 = val8 << 24;
 
-		// Green
-		val8 = static_cast<uint8>(g * 255);
-		val32 += val8 << 16;
+        // Green
+        val8 = static_cast<uint8>(g * 255);
+        val32 += val8 << 16;
 
-		// Red
-		val8 = static_cast<uint8>(r * 255);
-		val32 += val8 << 8;
+        // Red
+        val8 = static_cast<uint8>(r * 255);
+        val32 += val8 << 8;
 
-		// Alpha
-		val8 = static_cast<uint8>(a * 255);
-		val32 += val8;
+        // Alpha
+        val8 = static_cast<uint8>(a * 255);
+        val32 += val8;
 
 
-		return val32;
-	}
+        return val32;
+    }
     //---------------------------------------------------------------------
 #if OGRE_ENDIAN == OGRE_ENDIAN_BIG
     RGBA ColourValue::getAsRGBA(void) const
@@ -167,99 +165,91 @@ namespace Ogre {
     }
     //---------------------------------------------------------------------
 #if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-    void ColourValue::setAsABGR(const ABGR val)
+    void ColourValue::setAsABGR(ABGR val32)
 #else
-    void ColourValue::setAsRGBA(const RGBA val)
+    void ColourValue::setAsRGBA(RGBA val32)
 #endif
     {
-        uint32 val32 = val;
-
         // Convert from 32bit pattern
         // (RGBA = 8888)
 
         // Red
-        r = ((val32 >> 24) & 0xFF) / 255.0f;
+        r = float((val32 >> 24) & 0xFF) / 255.0f;
 
         // Green
-        g = ((val32 >> 16) & 0xFF) / 255.0f;
+        g = float((val32 >> 16) & 0xFF) / 255.0f;
 
         // Blue
-        b = ((val32 >> 8) & 0xFF) / 255.0f;
+        b = float((val32 >> 8) & 0xFF) / 255.0f;
 
         // Alpha
-        a = (val32 & 0xFF) / 255.0f;
+        a = float(val32 & 0xFF) / 255.0f;
     }
     //---------------------------------------------------------------------
 #if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-    void ColourValue::setAsBGRA(const BGRA val)
+    void ColourValue::setAsBGRA(BGRA val32)
 #else
-    void ColourValue::setAsARGB(const ARGB val)
+    void ColourValue::setAsARGB(ARGB val32)
 #endif
     {
-        uint32 val32 = val;
-
         // Convert from 32bit pattern
         // (ARGB = 8888)
 
         // Alpha
-        a = ((val32 >> 24) & 0xFF) / 255.0f;
+        a = float((val32 >> 24) & 0xFF) / 255.0f;
 
         // Red
-        r = ((val32 >> 16) & 0xFF) / 255.0f;
+        r = float((val32 >> 16) & 0xFF) / 255.0f;
 
         // Green
-        g = ((val32 >> 8) & 0xFF) / 255.0f;
+        g = float((val32 >> 8) & 0xFF) / 255.0f;
 
         // Blue
-        b = (val32 & 0xFF) / 255.0f;
+        b = float(val32 & 0xFF) / 255.0f;
     }
-	//---------------------------------------------------------------------
-#if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-	void ColourValue::setAsARGB(const ARGB val)
-#else
-	void ColourValue::setAsBGRA(const BGRA val)
-#endif
-	{
-		uint32 val32 = val;
-
-		// Convert from 32bit pattern
-		// (ARGB = 8888)
-
-		// Blue
-		b = ((val32 >> 24) & 0xFF) / 255.0f;
-
-		// Green
-		g = ((val32 >> 16) & 0xFF) / 255.0f;
-
-		// Red
-		r = ((val32 >> 8) & 0xFF) / 255.0f;
-
-		// Alpha
-		a = (val32 & 0xFF) / 255.0f;
-	}
     //---------------------------------------------------------------------
 #if OGRE_ENDIAN == OGRE_ENDIAN_BIG
-    void ColourValue::setAsRGBA(const RGBA val)
+    void ColourValue::setAsARGB(ARGB val32)
 #else
-    void ColourValue::setAsABGR(const ABGR val)
+    void ColourValue::setAsBGRA(BGRA val32)
 #endif
     {
-        uint32 val32 = val;
+        // Convert from 32bit pattern
+        // (ARGB = 8888)
 
+        // Blue
+        b = float((val32 >> 24) & 0xFF) / 255.0f;
+
+        // Green
+        g = float((val32 >> 16) & 0xFF) / 255.0f;
+
+        // Red
+        r = float((val32 >> 8) & 0xFF) / 255.0f;
+
+        // Alpha
+        a = float(val32 & 0xFF) / 255.0f;
+    }
+    //---------------------------------------------------------------------
+#if OGRE_ENDIAN == OGRE_ENDIAN_BIG
+    void ColourValue::setAsRGBA(RGBA val32)
+#else
+    void ColourValue::setAsABGR(ABGR val32)
+#endif
+    {
         // Convert from 32bit pattern
         // (ABGR = 8888)
 
         // Alpha
-        a = ((val32 >> 24) & 0xFF) / 255.0f;
+        a = float((val32 >> 24) & 0xFF) / 255.0f;
 
         // Blue
-        b = ((val32 >> 16) & 0xFF) / 255.0f;
+        b = float((val32 >> 16) & 0xFF) / 255.0f;
 
         // Green
-        g = ((val32 >> 8) & 0xFF) / 255.0f;
+        g = float((val32 >> 8) & 0xFF) / 255.0f;
 
         // Red
-        r = (val32 & 0xFF) / 255.0f;
+        r = float(val32 & 0xFF) / 255.0f;
     }
     //---------------------------------------------------------------------
     bool ColourValue::operator==(const ColourValue& rhs) const
@@ -274,133 +264,125 @@ namespace Ogre {
     {
         return !(*this == rhs);
     }
-	//---------------------------------------------------------------------
-	void ColourValue::setHSB(Real hue, Real saturation, Real brightness)
-	{
-		// wrap hue
-		if (hue > 1.0f)
-		{
-			hue -= (int)hue;
-		}
-		else if (hue < 0.0f)
-		{
-			hue += (int)hue + 1;
-		}
-		// clamp saturation / brightness
-		saturation = std::min(saturation, (Real)1.0);
-		saturation = std::max(saturation, (Real)0.0);
-		brightness = std::min(brightness, (Real)1.0);
-		brightness = std::max(brightness, (Real)0.0);
+    //---------------------------------------------------------------------
+    void ColourValue::setHSB(float hue, float saturation, float brightness)
+    {
+        // wrap hue
+        hue = std::fmod(hue, 1.0f);
 
-		if (brightness == 0.0f)
-		{   
-			// early exit, this has to be black
-			r = g = b = 0.0f;
-			return;
-		}
+        // clamp saturation / brightness
+        saturation = Math::saturate(saturation);
+        brightness = Math::saturate(brightness);
 
-		if (saturation == 0.0f)
-		{   
-			// early exit, this has to be grey
+        if (brightness == 0.0f)
+        {   
+            // early exit, this has to be black
+            r = g = b = 0.0f;
+            return;
+        }
 
-			r = g = b = brightness;
-			return;
-		}
+        if (saturation == 0.0f)
+        {   
+            // early exit, this has to be grey
+
+            r = g = b = brightness;
+            return;
+        }
 
 
-		Real hueDomain  = hue * 6.0f;
-		if (hueDomain >= 6.0f)
-		{
-			// wrap around, and allow mathematical errors
-			hueDomain = 0.0f;
-		}
-		unsigned short domain = (unsigned short)hueDomain;
-		Real f1 = brightness * (1 - saturation);
-		Real f2 = brightness * (1 - saturation * (hueDomain - domain));
-		Real f3 = brightness * (1 - saturation * (1 - (hueDomain - domain)));
+        float hueDomain  = hue * 6.0f;
+        if (hueDomain >= 6.0f)
+        {
+            // wrap around, and allow mathematical errors
+            hueDomain = 0.0f;
+        }
+        unsigned short domain = (unsigned short)hueDomain;
+        float f1 = brightness * (1 - saturation);
+        float f2 = brightness * (1 - saturation * (hueDomain - domain));
+        float f3 = brightness * (1 - saturation * (1 - (hueDomain - domain)));
 
-		switch (domain)
-		{
-		case 0:
-			// red domain; green ascends
-			r = brightness;
-			g = f3;
-			b = f1;
-			break;
-		case 1:
-			// yellow domain; red descends
-			r = f2;
-			g = brightness;
-			b = f1;
-			break;
-		case 2:
-			// green domain; blue ascends
-			r = f1;
-			g = brightness;
-			b = f3;
-			break;
-		case 3:
-			// cyan domain; green descends
-			r = f1;
-			g = f2;
-			b = brightness;
-			break;
-		case 4:
-			// blue domain; red ascends
-			r = f3;
-			g = f1;
-			b = brightness;
-			break;
-		case 5:
-			// magenta domain; blue descends
-			r = brightness;
-			g = f1;
-			b = f2;
-			break;
-		}
+        switch (domain)
+        {
+        case 0:
+            // red domain; green ascends
+            r = brightness;
+            g = f3;
+            b = f1;
+            break;
+        case 1:
+            // yellow domain; red descends
+            r = f2;
+            g = brightness;
+            b = f1;
+            break;
+        case 2:
+            // green domain; blue ascends
+            r = f1;
+            g = brightness;
+            b = f3;
+            break;
+        case 3:
+            // cyan domain; green descends
+            r = f1;
+            g = f2;
+            b = brightness;
+            break;
+        case 4:
+            // blue domain; red ascends
+            r = f3;
+            g = f1;
+            b = brightness;
+            break;
+        case 5:
+            // magenta domain; blue descends
+            r = brightness;
+            g = f1;
+            b = f2;
+            break;
+        }
 
 
-	}
-	//---------------------------------------------------------------------
-	void ColourValue::getHSB(Real* hue, Real* saturation, Real* brightness) const
-	{
+    }
+    //---------------------------------------------------------------------
+    void ColourValue::getHSB(float& hue, float& saturation, float& brightness) const
+    {
 
-		Real vMin = std::min(r, std::min(g, b));
-		Real vMax = std::max(r, std::max(g, b));
-		Real delta = vMax - vMin;
+        float vMin = std::min(r, std::min(g, b));
+        float vMax = std::max(r, std::max(g, b));
+        float delta = vMax - vMin;
 
-		*brightness = vMax;
+        brightness = vMax;
 
-		if (Math::RealEqual(delta, 0.0f, 1e-6))
-		{
-			// grey
-			*hue = 0;
-			*saturation = 0;
-		}
-		else                                    
-		{
-			// a colour
-			*saturation = delta / vMax;
+        if (Math::RealEqual(delta, 0.0f, 1e-6f))
+        {
+            // grey
+            hue = 0;
+            saturation = 0;
+        }
+        else                                    
+        {
+            // a colour
+            saturation = delta / vMax;
 
-			Real deltaR = (((vMax - r) / 6.0f) + (delta / 2.0f)) / delta;
-			Real deltaG = (((vMax - g) / 6.0f) + (delta / 2.0f)) / delta;
-			Real deltaB = (((vMax - b) / 6.0f) + (delta / 2.0f)) / delta;
+            float deltaR = (((vMax - r) / 6.0f) + (delta / 2.0f)) / delta;
+            float deltaG = (((vMax - g) / 6.0f) + (delta / 2.0f)) / delta;
+            float deltaB = (((vMax - b) / 6.0f) + (delta / 2.0f)) / delta;
 
-			if (Math::RealEqual(r, vMax))
-				*hue = deltaB - deltaG;
-			else if (Math::RealEqual(g, vMax))
-				*hue = 0.3333333f + deltaR - deltaB;
-			else if (Math::RealEqual(b, vMax)) 
-				*hue = 0.6666667f + deltaG - deltaR;
+            if (Math::RealEqual(r, vMax))
+                hue = deltaB - deltaG;
+            else if (Math::RealEqual(g, vMax))
+                hue = 0.3333333f + deltaR - deltaB;
+            else if (Math::RealEqual(b, vMax)) 
+                hue = 0.6666667f + deltaG - deltaR;
 
-			if (*hue < 0.0f) 
-				*hue += 1.0f;
-			if (*hue > 1.0f)
-				*hue -= 1.0f;
-		}
+            if (hue < 0.0f)
+                hue += 1.0f;
+            if (hue > 1.0f)
+                hue -= 1.0f;
+        }
 
-		
-	}
+        
+    }
 
 }
 
