@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,13 +27,7 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 #include "OgreHardwareVertexBuffer.h"
-#include "OgreColourValue.h"
-#include "OgreException.h"
-#include "OgreStringConverter.h"
-#include "OgreHardwareBufferManager.h"
 #include "OgreDefaultHardwareBufferManager.h"
-#include "OgreRoot.h"
-#include "OgreRenderSystem.h"
 
 namespace Ogre {
 
@@ -41,12 +35,12 @@ namespace Ogre {
     HardwareVertexBuffer::HardwareVertexBuffer(HardwareBufferManagerBase* mgr, size_t vertexSize,  
         size_t numVertices, HardwareBuffer::Usage usage, 
         bool useSystemMemory, bool useShadowBuffer) 
-        : HardwareBuffer(usage, useSystemMemory, useShadowBuffer), 
-		  mMgr(mgr),
+        : HardwareBuffer(usage, useSystemMemory, useShadowBuffer),
+          mIsInstanceData(false),
+          mMgr(mgr),
           mNumVertices(numVertices),
           mVertexSize(vertexSize),
-          mIsInstanceData(false),
-		  mInstanceDataStepRate(1)
+          mInstanceDataStepRate(1)
     {
         // Calculate the size of the vertices
         mSizeInBytes = mVertexSize * numVertices;
@@ -54,31 +48,27 @@ namespace Ogre {
         // Create a shadow buffer if required
         if (mUseShadowBuffer)
         {
-            mShadowBuffer = OGRE_NEW DefaultHardwareVertexBuffer(mMgr, mVertexSize, 
-                    mNumVertices, HardwareBuffer::HBU_DYNAMIC);
+            mShadowBuffer.reset(new DefaultHardwareVertexBuffer(mMgr, mVertexSize,
+                    mNumVertices, HardwareBuffer::HBU_DYNAMIC));
         }
 
     }
     //-----------------------------------------------------------------------------
     HardwareVertexBuffer::~HardwareVertexBuffer()
     {
-		if (mMgr)
-		{
-			mMgr->_notifyVertexBufferDestroyed(this);
-		}
-        if (mShadowBuffer)
+        if (mMgr)
         {
-            OGRE_DELETE mShadowBuffer;
+            mMgr->_notifyVertexBufferDestroyed(this);
         }
     }
     //-----------------------------------------------------------------------------
     bool HardwareVertexBuffer::checkIfVertexInstanceDataIsSupported()
     {
-    	// Use the current render system
-    	RenderSystem* rs = Root::getSingleton().getRenderSystem();
+        // Use the current render system
+        RenderSystem* rs = Root::getSingleton().getRenderSystem();
 
-    	// Check if the supported  
-    	return rs->getCapabilities()->hasCapability(RSC_VERTEX_BUFFER_INSTANCE_DATA);
+        // Check if the supported  
+        return rs->getCapabilities()->hasCapability(RSC_VERTEX_BUFFER_INSTANCE_DATA);
     }
     //-----------------------------------------------------------------------------
     void HardwareVertexBuffer::setIsInstanceData( const bool val )
@@ -91,59 +81,58 @@ namespace Ogre {
         }
         else
         {
-        	mIsInstanceData = val;  
+            mIsInstanceData = val;  
         }
     }
-	//-----------------------------------------------------------------------------
-	size_t HardwareVertexBuffer::getInstanceDataStepRate() const
-	{
-		return mInstanceDataStepRate;
-	}
-	//-----------------------------------------------------------------------------
-	void HardwareVertexBuffer::setInstanceDataStepRate( const size_t val )
-	{
+    //-----------------------------------------------------------------------------
+    size_t HardwareVertexBuffer::getInstanceDataStepRate() const
+    {
+        return mInstanceDataStepRate;
+    }
+    //-----------------------------------------------------------------------------
+    void HardwareVertexBuffer::setInstanceDataStepRate( const size_t val )
+    {
         if (val > 0)
         {
-			mInstanceDataStepRate = val;
+            mInstanceDataStepRate = val;
         }
-		else
-		{
+        else
+        {
             OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
                 "Instance data step rate must be bigger then 0.", 
                 "HardwareVertexBuffer::setInstanceDataStepRate");
-		}
-	}
-	//-----------------------------------------------------------------------------
-	// VertexElement
-	//-----------------------------------------------------------------------------
-    VertexElement::VertexElement(unsigned short source, size_t offset, 
-        VertexElementType theType, VertexElementSemantic semantic, unsigned short index)
-        : mSource(source), mOffset(offset), mType(theType), 
-        mSemantic(semantic), mIndex(index)
+        }
+    }
+    //-----------------------------------------------------------------------------
+    // VertexElement
+    //-----------------------------------------------------------------------------
+    VertexElement::VertexElement(unsigned short source, size_t offset, VertexElementType theType,
+                                 VertexElementSemantic semantic, unsigned short index)
+        : mSource(source), mIndex(index), mOffset(offset), mType(theType), mSemantic(semantic)
     {
     }
     //-----------------------------------------------------------------------------
-	size_t VertexElement::getSize(void) const
-	{
-		return getTypeSize(mType);
-	}
-	//-----------------------------------------------------------------------------
-	size_t VertexElement::getTypeSize(VertexElementType etype)
-	{
-		switch(etype)
-		{
-		case VET_COLOUR:
-		case VET_COLOUR_ABGR:
-		case VET_COLOUR_ARGB:
-			return sizeof(RGBA);
-		case VET_FLOAT1:
-			return sizeof(float);
-		case VET_FLOAT2:
-			return sizeof(float)*2;
-		case VET_FLOAT3:
-			return sizeof(float)*3;
-		case VET_FLOAT4:
-			return sizeof(float)*4;
+    size_t VertexElement::getSize(void) const
+    {
+        return getTypeSize(mType);
+    }
+    //-----------------------------------------------------------------------------
+    size_t VertexElement::getTypeSize(VertexElementType etype)
+    {
+        switch(etype)
+        {
+        case VET_COLOUR:
+        case VET_COLOUR_ABGR:
+        case VET_COLOUR_ARGB:
+            return sizeof(RGBA);
+        case VET_FLOAT1:
+            return sizeof(float);
+        case VET_FLOAT2:
+            return sizeof(float)*2;
+        case VET_FLOAT3:
+            return sizeof(float)*3;
+        case VET_FLOAT4:
+            return sizeof(float)*4;
         case VET_DOUBLE1:
             return sizeof(double);
         case VET_DOUBLE2:
@@ -152,224 +141,257 @@ namespace Ogre {
             return sizeof(double)*3;
         case VET_DOUBLE4:
             return sizeof(double)*4;
-		case VET_SHORT1:
-			return sizeof(short);
-		case VET_SHORT2:
-			return sizeof(short)*2;
-		case VET_SHORT3:
-			return sizeof(short)*3;
-		case VET_SHORT4:
-			return sizeof(short)*4;
+        case VET_SHORT1:
         case VET_USHORT1:
-            return sizeof(unsigned short);
+            return sizeof( short );
+        case VET_SHORT2:
+        case VET_SHORT2_NORM:
         case VET_USHORT2:
-            return sizeof(unsigned short)*2;
+        case VET_USHORT2_NORM:
+            return sizeof( short ) * 2;
+        case VET_SHORT3:
         case VET_USHORT3:
-            return sizeof(unsigned short)*3;
+            return sizeof( short ) * 3;
+        case VET_SHORT4:
+        case VET_SHORT4_NORM:
         case VET_USHORT4:
-            return sizeof(unsigned short)*4;
+        case VET_USHORT4_NORM:
+            return sizeof( short ) * 4;
         case VET_INT1:
-            return sizeof(int);
-        case VET_INT2:
-            return sizeof(int)*2;
-        case VET_INT3:
-            return sizeof(int)*3;
-        case VET_INT4:
-            return sizeof(int)*4;
         case VET_UINT1:
-            return sizeof(unsigned int);
+            return sizeof( int );
+        case VET_INT2:
         case VET_UINT2:
-            return sizeof(unsigned int)*2;
+            return sizeof( int ) * 2;
+        case VET_INT3:
         case VET_UINT3:
-            return sizeof(unsigned int)*3;
+            return sizeof( int ) * 3;
+        case VET_INT4:
         case VET_UINT4:
-            return sizeof(unsigned int)*4;
+            return sizeof( int ) * 4;
+        case VET_BYTE4:
+        case VET_BYTE4_NORM:
         case VET_UBYTE4:
-            return sizeof(unsigned char)*4;
-		}
-		return 0;
-	}
-	//-----------------------------------------------------------------------------
-	unsigned short VertexElement::getTypeCount(VertexElementType etype)
-	{
-		switch (etype)
-		{
-		case VET_COLOUR:
-		case VET_COLOUR_ABGR:
-		case VET_COLOUR_ARGB:
+        case VET_UBYTE4_NORM:
+            return sizeof(char)*4;
+        }
+        return 0;
+    }
+    //-----------------------------------------------------------------------------
+    unsigned short VertexElement::getTypeCount(VertexElementType etype)
+    {
+        switch (etype)
+        {
+        case VET_COLOUR:
+        case VET_COLOUR_ABGR:
+        case VET_COLOUR_ARGB:
         case VET_FLOAT1:
         case VET_SHORT1:
         case VET_USHORT1:
         case VET_UINT1:
         case VET_INT1:
         case VET_DOUBLE1:
-			return 1;
+            return 1;
         case VET_FLOAT2:
         case VET_SHORT2:
+        case VET_SHORT2_NORM:
         case VET_USHORT2:
+        case VET_USHORT2_NORM:
         case VET_UINT2:
         case VET_INT2:
         case VET_DOUBLE2:
-			return 2;
+            return 2;
         case VET_FLOAT3:
         case VET_SHORT3:
         case VET_USHORT3:
         case VET_UINT3:
         case VET_INT3:
         case VET_DOUBLE3:
-			return 3;
+            return 3;
         case VET_FLOAT4:
         case VET_SHORT4:
+        case VET_SHORT4_NORM:
         case VET_USHORT4:
+        case VET_USHORT4_NORM:
         case VET_UINT4:
         case VET_INT4:
         case VET_DOUBLE4:
+        case VET_BYTE4:
         case VET_UBYTE4:
+        case VET_BYTE4_NORM:
+        case VET_UBYTE4_NORM:
             return 4;
-		}
-		OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid type", 
-			"VertexElement::getTypeCount");
-	}
-	//-----------------------------------------------------------------------------
-	VertexElementType VertexElement::multiplyTypeCount(VertexElementType baseType, 
-		unsigned short count)
-	{
-		switch (baseType)
-		{
-		case VET_FLOAT1:
-			switch(count)
-			{
-			case 1:
-				return VET_FLOAT1;
-			case 2:
-				return VET_FLOAT2;
-			case 3:
-				return VET_FLOAT3;
-			case 4:
-				return VET_FLOAT4;
-            default:
-                break;
-			}
-			break;
-		case VET_SHORT1:
-			switch(count)
-			{
-			case 1:
-				return VET_SHORT1;
-			case 2:
-				return VET_SHORT2;
-			case 3:
-				return VET_SHORT3;
-			case 4:
-				return VET_SHORT4;
-            default:
-                break;
-			}
-			break;
+        }
+        OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid type", 
+            "VertexElement::getTypeCount");
+    }
+    //-----------------------------------------------------------------------------
+    VertexElementType VertexElement::multiplyTypeCount(VertexElementType baseType, 
+        unsigned short count)
+    {
+        OgreAssert(count > 0 && count < 5, "Count out of range");
+
+        switch (baseType)
+        {
+        case VET_FLOAT1:
+        case VET_DOUBLE1:
+        case VET_INT1:
+        case VET_UINT1:
+            // evil enumeration arithmetic
+            return static_cast<VertexElementType>( baseType + count - 1 );
+
+        case VET_SHORT1:
+        case VET_SHORT2:
+            if ( count <= 2 )
+            {
+                return VET_SHORT2;
+            }
+            return VET_SHORT4;
+
+        case VET_USHORT1:
+        case VET_USHORT2:
+            if ( count <= 2 )
+            {
+                return VET_USHORT2;
+            }
+            return VET_USHORT4;
+
+        case VET_SHORT2_NORM:
+            if ( count <= 2 )
+            {
+                return VET_SHORT2_NORM;
+            }
+            return VET_SHORT4_NORM;
+
+        case VET_USHORT2_NORM:
+            if ( count <= 2 )
+            {
+                return VET_USHORT2_NORM;
+            }
+            return VET_USHORT4_NORM;
+
+        case VET_BYTE4:
+        case VET_BYTE4_NORM:
+        case VET_UBYTE4:
+        case VET_UBYTE4_NORM:
+            return baseType;
+
         default:
             break;
-		}
-		OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid base type", 
-			"VertexElement::multiplyTypeCount");
-	}
-	//--------------------------------------------------------------------------
-	VertexElementType VertexElement::getBestColourVertexElementType(void)
-	{
-		// Use the current render system to determine if possible
-		if (Root::getSingletonPtr() && Root::getSingletonPtr()->getRenderSystem())
-		{
-			return Root::getSingleton().getRenderSystem()->getColourVertexElementType();
-		}
-		else
-		{
-			// We can't know the specific type right now, so pick a type
-			// based on platform
+        }
+        OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Invalid base type", 
+            "VertexElement::multiplyTypeCount");
+    }
+    //--------------------------------------------------------------------------
+    VertexElementType VertexElement::getBestColourVertexElementType(void)
+    {
+        // Use the current render system to determine if possible
+        if (Root::getSingletonPtr() && Root::getSingletonPtr()->getRenderSystem())
+        {
+            return Root::getSingleton().getRenderSystem()->getColourVertexElementType();
+        }
+        else
+        {
+            // We can't know the specific type right now, so pick a type
+            // based on platform
 #if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT
-			return VET_COLOUR_ARGB; // prefer D3D format on windows
+            return VET_COLOUR_ARGB; // prefer D3D format on windows
 #else
-			return VET_COLOUR_ABGR; // prefer GL format on everything else
+            return VET_COLOUR_ABGR; // prefer GL format on everything else
 #endif
 
-		}
-	}
-	//--------------------------------------------------------------------------
-	void VertexElement::convertColourValue(VertexElementType srcType, 
-		VertexElementType dstType, uint32* ptr)
-	{
-		if (srcType == dstType)
-			return;
+        }
+    }
+    //--------------------------------------------------------------------------
+    void VertexElement::convertColourValue(VertexElementType srcType, 
+        VertexElementType dstType, uint32* ptr)
+    {
+        if (srcType == dstType)
+            return;
 
-		// Conversion between ARGB and ABGR is always a case of flipping R/B
-		*ptr = 
-		   ((*ptr&0x00FF0000)>>16)|((*ptr&0x000000FF)<<16)|(*ptr&0xFF00FF00);				
-	}
-	//--------------------------------------------------------------------------
-	uint32 VertexElement::convertColourValue(const ColourValue& src, 
-		VertexElementType dst)
-	{
-		switch(dst)
-		{
+        // Conversion between ARGB and ABGR is always a case of flipping R/B
+        *ptr = 
+           ((*ptr&0x00FF0000)>>16)|((*ptr&0x000000FF)<<16)|(*ptr&0xFF00FF00);               
+    }
+    //--------------------------------------------------------------------------
+    uint32 VertexElement::convertColourValue(const ColourValue& src, 
+        VertexElementType dst)
+    {
+        switch(dst)
+        {
 #if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WINRT
         default:
 #endif
-		case VET_COLOUR_ARGB:
-			return src.getAsARGB();
+        case VET_COLOUR_ARGB:
+            return src.getAsARGB();
 #if OGRE_PLATFORM != OGRE_PLATFORM_WIN32 && OGRE_PLATFORM != OGRE_PLATFORM_WINRT
         default:
 #endif
-		case VET_COLOUR_ABGR: 
-			return src.getAsABGR();
-		};
+        case VET_COLOUR_ABGR: 
+            return src.getAsABGR();
+        };
 
-	}
-	//-----------------------------------------------------------------------------
-	VertexElementType VertexElement::getBaseType(VertexElementType multiType)
-	{
-		switch (multiType)
-		{
-			case VET_FLOAT1:
-			case VET_FLOAT2:
-			case VET_FLOAT3:
-			case VET_FLOAT4:
-				return VET_FLOAT1;
-			case VET_DOUBLE1:
-			case VET_DOUBLE2:
-			case VET_DOUBLE3:
-			case VET_DOUBLE4:
-				return VET_DOUBLE1;
+    }
+    //-----------------------------------------------------------------------------
+    VertexElementType VertexElement::getBaseType(VertexElementType multiType)
+    {
+        switch (multiType)
+        {
+            case VET_FLOAT1:
+            case VET_FLOAT2:
+            case VET_FLOAT3:
+            case VET_FLOAT4:
+                return VET_FLOAT1;
+            case VET_DOUBLE1:
+            case VET_DOUBLE2:
+            case VET_DOUBLE3:
+            case VET_DOUBLE4:
+                return VET_DOUBLE1;
             case VET_INT1:
-			case VET_INT2:
-			case VET_INT3:
-			case VET_INT4:
-				return VET_INT1;
+            case VET_INT2:
+            case VET_INT3:
+            case VET_INT4:
+                return VET_INT1;
             case VET_UINT1:
-			case VET_UINT2:
-			case VET_UINT3:
-			case VET_UINT4:
-				return VET_UINT1;
-			case VET_COLOUR:
-				return VET_COLOUR;
-			case VET_COLOUR_ABGR:
-				return VET_COLOUR_ABGR;
-			case VET_COLOUR_ARGB:
-				return VET_COLOUR_ARGB;
-			case VET_SHORT1:
-			case VET_SHORT2:
-			case VET_SHORT3:
-			case VET_SHORT4:
-				return VET_SHORT1;
-			case VET_USHORT1:
-			case VET_USHORT2:
-			case VET_USHORT3:
-			case VET_USHORT4:
-				return VET_USHORT1;
-			case VET_UBYTE4:
-				return VET_UBYTE4;
-		};
+            case VET_UINT2:
+            case VET_UINT3:
+            case VET_UINT4:
+                return VET_UINT1;
+            case VET_COLOUR:
+                return VET_COLOUR;
+            case VET_COLOUR_ABGR:
+                return VET_COLOUR_ABGR;
+            case VET_COLOUR_ARGB:
+                return VET_COLOUR_ARGB;
+            case VET_SHORT1:
+            case VET_SHORT2:
+            case VET_SHORT3:
+            case VET_SHORT4:
+                return VET_SHORT1;
+            case VET_USHORT1:
+            case VET_USHORT2:
+            case VET_USHORT3:
+            case VET_USHORT4:
+                return VET_USHORT1;
+            case VET_SHORT2_NORM:
+            case VET_SHORT4_NORM:
+                return VET_SHORT2_NORM;
+            case VET_USHORT2_NORM:
+            case VET_USHORT4_NORM:
+                return VET_USHORT2_NORM;
+            case VET_BYTE4:
+                return VET_BYTE4;
+            case VET_BYTE4_NORM:
+                return VET_BYTE4_NORM;
+            case VET_UBYTE4:
+                return VET_UBYTE4;
+            case VET_UBYTE4_NORM:
+                return VET_UBYTE4_NORM;
+        };
         // To keep compiler happy
         return VET_FLOAT1;
-	}
-	//-----------------------------------------------------------------------------
+    }
+    //-----------------------------------------------------------------------------
     VertexDeclaration::VertexDeclaration()
     {
     }
@@ -387,15 +409,16 @@ namespace Ogre {
         size_t offset, VertexElementType theType,
         VertexElementSemantic semantic, unsigned short index)
     {
-		// Refine colour type to a specific type
-		if (theType == VET_COLOUR)
-		{
-			theType = VertexElement::getBestColourVertexElementType();
-		}
+        // Refine colour type to a specific type
+        if (theType == VET_COLOUR)
+        {
+            theType = VertexElement::getBestColourVertexElementType();
+        }
         mElementList.push_back(
-            VertexElement(source, offset, theType, semantic, index)
-            );
-		return mElementList.back();
+            VertexElement(source, offset, theType, semantic, index));
+
+        notifyChanged();
+        return mElementList.back();
     }
     //-----------------------------------------------------------------------------
     const VertexElement& VertexDeclaration::insertElement(unsigned short atPosition,
@@ -413,8 +436,9 @@ namespace Ogre {
 
         i = mElementList.insert(i, 
             VertexElement(source, offset, theType, semantic, index));
-        return *i;
 
+        notifyChanged();
+        return *i;
     }
     //-----------------------------------------------------------------------------
     const VertexElement* VertexDeclaration::getElement(unsigned short index) const
@@ -436,26 +460,29 @@ namespace Ogre {
         for (unsigned short n = 0; n < elem_index; ++n)
             ++i;
         mElementList.erase(i);
+        notifyChanged();
     }
     //-----------------------------------------------------------------------------
     void VertexDeclaration::removeElement(VertexElementSemantic semantic, unsigned short index)
     {
-		VertexElementList::iterator ei, eiend;
-		eiend = mElementList.end();
-		for (ei = mElementList.begin(); ei != eiend; ++ei)
-		{
-			if (ei->getSemantic() == semantic && ei->getIndex() == index)
-			{
-				mElementList.erase(ei);
+        VertexElementList::iterator ei, eiend;
+        eiend = mElementList.end();
+        for (ei = mElementList.begin(); ei != eiend; ++ei)
+        {
+            if (ei->getSemantic() == semantic && ei->getIndex() == index)
+            {
+                mElementList.erase(ei);
+                notifyChanged();
                 break;
-			}
-		}
+            }
+        }
+    }
+    //-----------------------------------------------------------------------------
+    void VertexDeclaration::removeAllElements(void)
+    {
+        mElementList.clear();
+        notifyChanged();
     }
-	//-----------------------------------------------------------------------------
-	void VertexDeclaration::removeAllElements(void)
-	{
-		mElementList.clear();
-	}
     //-----------------------------------------------------------------------------
     void VertexDeclaration::modifyElement(unsigned short elem_index, 
         unsigned short source, size_t offset, VertexElementType theType,
@@ -465,70 +492,71 @@ namespace Ogre {
         VertexElementList::iterator i = mElementList.begin();
         std::advance(i, elem_index);
         (*i) = VertexElement(source, offset, theType, semantic, index);
+        notifyChanged();
     }
     //-----------------------------------------------------------------------------
-	const VertexElement* VertexDeclaration::findElementBySemantic(
-		VertexElementSemantic sem, unsigned short index) const
-	{
-		VertexElementList::const_iterator ei, eiend;
-		eiend = mElementList.end();
-		for (ei = mElementList.begin(); ei != eiend; ++ei)
-		{
-			if (ei->getSemantic() == sem && ei->getIndex() == index)
-			{
-				return &(*ei);
-			}
-		}
+    const VertexElement* VertexDeclaration::findElementBySemantic(
+        VertexElementSemantic sem, unsigned short index) const
+    {
+        VertexElementList::const_iterator ei, eiend;
+        eiend = mElementList.end();
+        for (ei = mElementList.begin(); ei != eiend; ++ei)
+        {
+            if (ei->getSemantic() == sem && ei->getIndex() == index)
+            {
+                return &(*ei);
+            }
+        }
 
-		return NULL;
+        return NULL;
 
 
-	}
-	//-----------------------------------------------------------------------------
-	VertexDeclaration::VertexElementList VertexDeclaration::findElementsBySource(
-		unsigned short source) const
-	{
-		VertexElementList retList;
-		VertexElementList::const_iterator ei, eiend;
-		eiend = mElementList.end();
-		for (ei = mElementList.begin(); ei != eiend; ++ei)
-		{
-			if (ei->getSource() == source)
-			{
-				retList.push_back(*ei);
-			}
-		}
-		return retList;
+    }
+    //-----------------------------------------------------------------------------
+    VertexDeclaration::VertexElementList VertexDeclaration::findElementsBySource(
+        unsigned short source) const
+    {
+        VertexElementList retList;
+        VertexElementList::const_iterator ei, eiend;
+        eiend = mElementList.end();
+        for (ei = mElementList.begin(); ei != eiend; ++ei)
+        {
+            if (ei->getSource() == source)
+            {
+                retList.push_back(*ei);
+            }
+        }
+        return retList;
 
-	}
+    }
 
-	//-----------------------------------------------------------------------------
-	size_t VertexDeclaration::getVertexSize(unsigned short source) const
-	{
-		VertexElementList::const_iterator i, iend;
-		iend = mElementList.end();
-		size_t sz = 0;
+    //-----------------------------------------------------------------------------
+    size_t VertexDeclaration::getVertexSize(unsigned short source) const
+    {
+        VertexElementList::const_iterator i, iend;
+        iend = mElementList.end();
+        size_t sz = 0;
 
-		for (i = mElementList.begin(); i != iend; ++i)
-		{
-			if (i->getSource() == source)
-			{
-				sz += i->getSize();
+        for (i = mElementList.begin(); i != iend; ++i)
+        {
+            if (i->getSource() == source)
+            {
+                sz += i->getSize();
 
-			}
-		}
-		return sz;
-	}
+            }
+        }
+        return sz;
+    }
     //-----------------------------------------------------------------------------
     VertexDeclaration* VertexDeclaration::clone(HardwareBufferManagerBase* mgr) const
     {
-		HardwareBufferManagerBase* pManager = mgr ? mgr : HardwareBufferManager::getSingletonPtr(); 
+        HardwareBufferManagerBase* pManager = mgr ? mgr : HardwareBufferManager::getSingletonPtr(); 
         VertexDeclaration* ret = pManager->createVertexDeclaration();
 
-		VertexElementList::const_iterator i, iend;
-		iend = mElementList.end();
-		for (i = mElementList.begin(); i != iend; ++i)
-		{
+        VertexElementList::const_iterator i, iend;
+        iend = mElementList.end();
+        for (i = mElementList.begin(); i != iend; ++i)
+        {
             ret->addElement(i->getSource(), i->getOffset(), i->getType(), i->getSemantic(), i->getIndex());
         }
         return ret;
@@ -597,7 +625,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     VertexDeclaration* VertexDeclaration::getAutoOrganisedDeclaration(
-		bool skeletalAnimation, bool vertexAnimation, bool vertexAnimationNormals) const
+        bool skeletalAnimation, bool vertexAnimation, bool vertexAnimationNormals) const
     {
         VertexDeclaration* newDecl = this->clone();
         // Set all sources to the same buffer (for now)
@@ -614,7 +642,7 @@ namespace Ogre {
         // Now sort out proper buffer assignments and offsets
         size_t offset = 0;
         c = 0;
-		unsigned short buffer = 0;
+        unsigned short buffer = 0;
         VertexElementSemantic prevSemantic = VES_POSITION;
         for (i = elems.begin(); i != elems.end(); ++i, ++c)
         {
@@ -626,8 +654,8 @@ namespace Ogre {
             {
             case VES_POSITION:
                 // Split positions if vertex animated with only positions
-				// group with normals otherwise
-				splitWithPrev = false;
+                // group with normals otherwise
+                splitWithPrev = false;
                 splitWithNext = vertexAnimation && !vertexAnimationNormals;
                 break;
             case VES_NORMAL:
@@ -644,15 +672,15 @@ namespace Ogre {
                 // Blend weights/indices can be sharing with their own buffer only
                 splitWithNext = true;
                 break;
-			default:
+            default:
             case VES_DIFFUSE:
             case VES_SPECULAR:
             case VES_TEXTURE_COORDINATES:
             case VES_BINORMAL:
             case VES_TANGENT:
-				// Make sure position is separate if animated & there were no normals
-				splitWithPrev = prevSemantic == VES_POSITION && 
-					(skeletalAnimation || vertexAnimation);
+                // Make sure position is separate if animated & there were no normals
+                splitWithPrev = prevSemantic == VES_POSITION && 
+                    (skeletalAnimation || vertexAnimation);
                 break;
             }
 
@@ -698,49 +726,49 @@ namespace Ogre {
         return ret;
     }
     //-----------------------------------------------------------------------------
-	unsigned short VertexDeclaration::getNextFreeTextureCoordinate() const
-	{
-		unsigned short texCoord = 0;
-		for (VertexElementList::const_iterator i = mElementList.begin(); 
-			 i != mElementList.end(); ++i)
-		{
-			const VertexElement& el = *i;
-			if (el.getSemantic() == VES_TEXTURE_COORDINATES)
-			{
-				++texCoord;
-			}
-		}
-		return texCoord;
-	}
+    unsigned short VertexDeclaration::getNextFreeTextureCoordinate() const
+    {
+        unsigned short texCoord = 0;
+        for (VertexElementList::const_iterator i = mElementList.begin(); 
+             i != mElementList.end(); ++i)
+        {
+            const VertexElement& el = *i;
+            if (el.getSemantic() == VES_TEXTURE_COORDINATES)
+            {
+                ++texCoord;
+            }
+        }
+        return texCoord;
+    }
     //-----------------------------------------------------------------------------
-	VertexBufferBinding::VertexBufferBinding() : mHighIndex(0)
-	{
-	}
+    VertexBufferBinding::VertexBufferBinding() : mHighIndex(0)
+    {
+    }
     //-----------------------------------------------------------------------------
-	VertexBufferBinding::~VertexBufferBinding()
-	{
+    VertexBufferBinding::~VertexBufferBinding()
+    {
         unsetAllBindings();
-	}
+    }
     //-----------------------------------------------------------------------------
-	void VertexBufferBinding::setBinding(unsigned short index, const HardwareVertexBufferSharedPtr& buffer)
-	{
+    void VertexBufferBinding::setBinding(unsigned short index, const HardwareVertexBufferSharedPtr& buffer)
+    {
         // NB will replace any existing buffer ptr at this index, and will thus cause
         // reference count to decrement on that buffer (possibly destroying it)
-		mBindingMap[index] = buffer;
-		mHighIndex = std::max(mHighIndex, (unsigned short)(index+1));
-	}
+        mBindingMap[index] = buffer;
+        mHighIndex = std::max(mHighIndex, (unsigned short)(index+1));
+    }
     //-----------------------------------------------------------------------------
-	void VertexBufferBinding::unsetBinding(unsigned short index)
-	{
-		VertexBufferBindingMap::iterator i = mBindingMap.find(index);
-		if (i == mBindingMap.end())
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
-				"Cannot find buffer binding for index " + StringConverter::toString(index),
-				"VertexBufferBinding::unsetBinding");
-		}
-		mBindingMap.erase(i);
-	}
+    void VertexBufferBinding::unsetBinding(unsigned short index)
+    {
+        VertexBufferBindingMap::iterator i = mBindingMap.find(index);
+        if (i == mBindingMap.end())
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+                "Cannot find buffer binding for index " + StringConverter::toString(index),
+                "VertexBufferBinding::unsetBinding");
+        }
+        mBindingMap.erase(i);
+    }
     //-----------------------------------------------------------------------------
     void VertexBufferBinding::unsetAllBindings(void)
     {
@@ -748,27 +776,27 @@ namespace Ogre {
         mHighIndex = 0;
     }
     //-----------------------------------------------------------------------------
-	const VertexBufferBinding::VertexBufferBindingMap& 
-	VertexBufferBinding::getBindings(void) const
-	{
-		return mBindingMap;
-	}
+    const VertexBufferBinding::VertexBufferBindingMap& 
+    VertexBufferBinding::getBindings(void) const
+    {
+        return mBindingMap;
+    }
     //-----------------------------------------------------------------------------
-	const HardwareVertexBufferSharedPtr& VertexBufferBinding::getBuffer(unsigned short index) const
-	{
-		VertexBufferBindingMap::const_iterator i = mBindingMap.find(index);
-		if (i == mBindingMap.end())
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No buffer is bound to that index.",
-				"VertexBufferBinding::getBuffer");
-		}
-		return i->second;
-	}
-	//-----------------------------------------------------------------------------
-	bool VertexBufferBinding::isBufferBound(unsigned short index) const
-	{
-		return mBindingMap.find(index) != mBindingMap.end();
-	}
+    const HardwareVertexBufferSharedPtr& VertexBufferBinding::getBuffer(unsigned short index) const
+    {
+        VertexBufferBindingMap::const_iterator i = mBindingMap.find(index);
+        if (i == mBindingMap.end())
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No buffer is bound to that index.",
+                "VertexBufferBinding::getBuffer");
+        }
+        return i->second;
+    }
+    //-----------------------------------------------------------------------------
+    bool VertexBufferBinding::isBufferBound(unsigned short index) const
+    {
+        return mBindingMap.find(index) != mBindingMap.end();
+    }
     //-----------------------------------------------------------------------------
     unsigned short VertexBufferBinding::getLastBoundIndex(void) const
     {
@@ -804,23 +832,17 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     bool VertexBufferBinding::hasInstanceData() const
     {
-		VertexBufferBinding::VertexBufferBindingMap::const_iterator i, iend;
-		iend = mBindingMap.end();
-		for (i = mBindingMap.begin(); i != iend; ++i)
-		{
-			if ( i->second->isInstanceData() )
+        VertexBufferBinding::VertexBufferBindingMap::const_iterator i, iend;
+        iend = mBindingMap.end();
+        for (i = mBindingMap.begin(); i != iend; ++i)
+        {
+            if ( i->second->isInstanceData() )
             {
                 return true;
             }
         }
         return false;
     }
-    //-----------------------------------------------------------------------------
-    HardwareVertexBufferSharedPtr::HardwareVertexBufferSharedPtr(HardwareVertexBuffer* buf)
-        : SharedPtr<HardwareVertexBuffer>(buf)
-    {
-
-    }
 
 
 
