@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,76 +31,71 @@ THE SOFTWARE.
 
 namespace Ogre {
 
-	String sNullLang = "null";
-	class NullProgram : public HighLevelGpuProgram
-	{
-	protected:
-		/** Internal load implementation, must be implemented by subclasses.
-		*/
-		void loadFromSource(void) {}
-		/** Internal method for creating an appropriate low-level program from this
-		high-level program, must be implemented by subclasses. */
-		void createLowLevelImpl(void) {}
-		/// Internal unload implementation, must be implemented by subclasses
-		void unloadHighLevelImpl(void) {}
-		/// Populate the passed parameters with name->index map, must be overridden
-		void populateParameterNames(GpuProgramParametersSharedPtr params)
-		{
-			// Skip the normal implementation
-			// Ensure we don't complain about missing parameter names
-			params->setIgnoreMissingParams(true);
+    String sNullLang = "null";
+    class NullProgram : public HighLevelGpuProgram
+    {
+    protected:
+        /** Internal load implementation, must be implemented by subclasses.
+        */
+        void loadFromSource(void) {}
+        /** Internal method for creating an appropriate low-level program from this
+        high-level program, must be implemented by subclasses. */
+        void createLowLevelImpl(void) {}
+        /// Internal unload implementation, must be implemented by subclasses
+        void unloadHighLevelImpl(void) {}
+        /// Populate the passed parameters with name->index map, must be overridden
+        void populateParameterNames(GpuProgramParametersSharedPtr params)
+        {
+            // Skip the normal implementation
+            // Ensure we don't complain about missing parameter names
+            params->setIgnoreMissingParams(true);
 
-		}
-		void buildConstantDefinitions() const
-		{
-			// do nothing
-		}
-	public:
-		NullProgram(ResourceManager* creator, 
-			const String& name, ResourceHandle handle, const String& group, 
-			bool isManual, ManualResourceLoader* loader)
-			: HighLevelGpuProgram(creator, name, handle, group, isManual, loader){}
-		~NullProgram() {}
-		/// Overridden from GpuProgram - never supported
-		bool isSupported(void) const { return false; }
-		/// Overridden from GpuProgram
-		const String& getLanguage(void) const { return sNullLang; }
+        }
+        void buildConstantDefinitions() const
+        {
+            // do nothing
+        }
+    public:
+        NullProgram(ResourceManager* creator, 
+            const String& name, ResourceHandle handle, const String& group, 
+            bool isManual, ManualResourceLoader* loader)
+            : HighLevelGpuProgram(creator, name, handle, group, isManual, loader){}
+        ~NullProgram() {}
+        /// Overridden from GpuProgram - never supported
+        bool isSupported(void) const { return false; }
+        /// Overridden from GpuProgram
+        const String& getLanguage(void) const { return sNullLang; }
         size_t calculateSize(void) const { return 0; }
 
-		/// Overridden from StringInterface
-		bool setParameter(const String& name, const String& value)
-		{
-			// always silently ignore all parameters so as not to report errors on
-			// unsupported platforms
-			return true;
-		}
+        /// Overridden from StringInterface
+        bool setParameter(const String& name, const String& value)
+        {
+            // always silently ignore all parameters so as not to report errors on
+            // unsupported platforms
+            return true;
+        }
 
-	};
-	class NullProgramFactory : public HighLevelGpuProgramFactory
-	{
-	public:
-		NullProgramFactory() {}
-		~NullProgramFactory() {}
-		/// Get the name of the language this factory creates programs for
-		const String& getLanguage(void) const 
-		{ 
-			return sNullLang;
-		}
-		HighLevelGpuProgram* create(ResourceManager* creator, 
-			const String& name, ResourceHandle handle,
-			const String& group, bool isManual, ManualResourceLoader* loader)
-		{
-			return OGRE_NEW NullProgram(creator, name, handle, group, isManual, loader);
-		}
-		void destroy(HighLevelGpuProgram* prog)
-		{
-			OGRE_DELETE prog;
-		}
-
-	};
-	//-----------------------------------------------------------------------
-	template<> HighLevelGpuProgramManager* 
-	Singleton<HighLevelGpuProgramManager>::msSingleton = 0;
+    };
+    class NullProgramFactory : public HighLevelGpuProgramFactory
+    {
+    public:
+        NullProgramFactory() {}
+        ~NullProgramFactory() {}
+        /// Get the name of the language this factory creates programs for
+        const String& getLanguage(void) const 
+        { 
+            return sNullLang;
+        }
+        GpuProgram* create(ResourceManager* creator,
+            const String& name, ResourceHandle handle,
+            const String& group, bool isManual, ManualResourceLoader* loader)
+        {
+            return OGRE_NEW NullProgram(creator, name, handle, group, isManual, loader);
+        }
+    };
+    //-----------------------------------------------------------------------
+    template<> HighLevelGpuProgramManager* 
+    Singleton<HighLevelGpuProgramManager>::msSingleton = 0;
     HighLevelGpuProgramManager* HighLevelGpuProgramManager::getSingletonPtr(void)
     {
         return msSingleton;
@@ -109,9 +104,9 @@ namespace Ogre {
     {  
         assert( msSingleton );  return ( *msSingleton );  
     }
-	//-----------------------------------------------------------------------
-	HighLevelGpuProgramManager::HighLevelGpuProgramManager()
-	{
+    //-----------------------------------------------------------------------
+    HighLevelGpuProgramManager::HighLevelGpuProgramManager()
+    {
         // Loading order
         mLoadOrder = 50.0f;
         // Resource type
@@ -119,24 +114,22 @@ namespace Ogre {
 
         ResourceGroupManager::getSingleton()._registerResourceManager(mResourceType, this);    
 
-		mNullFactory = OGRE_NEW NullProgramFactory();
-		addFactory(mNullFactory);
-		mUnifiedFactory = OGRE_NEW UnifiedHighLevelGpuProgramFactory();
-		addFactory(mUnifiedFactory);
-	}
-	//-----------------------------------------------------------------------
-	HighLevelGpuProgramManager::~HighLevelGpuProgramManager()
-	{
-		OGRE_DELETE mUnifiedFactory;
-		OGRE_DELETE mNullFactory;
+        mNullFactory.reset(new NullProgramFactory());
+        addFactory(mNullFactory.get());
+        mUnifiedFactory.reset(new UnifiedHighLevelGpuProgramFactory());
+        addFactory(mUnifiedFactory.get());
+    }
+    //-----------------------------------------------------------------------
+    HighLevelGpuProgramManager::~HighLevelGpuProgramManager()
+    {
         ResourceGroupManager::getSingleton()._unregisterResourceManager(mResourceType);    
-	}
+    }
     //---------------------------------------------------------------------------
-	void HighLevelGpuProgramManager::addFactory(HighLevelGpuProgramFactory* factory)
-	{
-		// deliberately allow later plugins to override earlier ones
-		mFactories[factory->getLanguage()] = factory;
-	}
+    void HighLevelGpuProgramManager::addFactory(HighLevelGpuProgramFactory* factory)
+    {
+        // deliberately allow later plugins to override earlier ones
+        mFactories[factory->getLanguage()] = factory;
+    }
     //---------------------------------------------------------------------------
     void HighLevelGpuProgramManager::removeFactory(HighLevelGpuProgramFactory* factory)
     {
@@ -149,25 +142,25 @@ namespace Ogre {
         }
     }
     //---------------------------------------------------------------------------
-	HighLevelGpuProgramFactory* HighLevelGpuProgramManager::getFactory(const String& language)
-	{
-		FactoryMap::iterator i = mFactories.find(language);
+    HighLevelGpuProgramFactory* HighLevelGpuProgramManager::getFactory(const String& language)
+    {
+        FactoryMap::iterator i = mFactories.find(language);
 
-		if (i == mFactories.end())
-		{
-			// use the null factory to create programs that will never be supported
-			i = mFactories.find(sNullLang);
-		}
-		return i->second;
-	}
-	//---------------------------------------------------------------------
-	bool HighLevelGpuProgramManager::isLanguageSupported(const String& lang)
-	{
-		FactoryMap::iterator i = mFactories.find(lang);
+        if (i == mFactories.end())
+        {
+            // use the null factory to create programs that will never be supported
+            i = mFactories.find(sNullLang);
+        }
+        return i->second;
+    }
+    //---------------------------------------------------------------------
+    bool HighLevelGpuProgramManager::isLanguageSupported(const String& lang)
+    {
+        FactoryMap::iterator i = mFactories.find(lang);
 
-		return i != mFactories.end();
+        return i != mFactories.end();
 
-	}
+    }
     //---------------------------------------------------------------------------
     Resource* HighLevelGpuProgramManager::createImpl(const String& name, ResourceHandle handle, 
         const String& group, bool isManual, ManualResourceLoader* loader,
@@ -188,25 +181,23 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     HighLevelGpuProgramPtr HighLevelGpuProgramManager::getByName(const String& name, const String& groupName)
     {
-        return getResourceByName(name, groupName).staticCast<HighLevelGpuProgram>();
+        return static_pointer_cast<HighLevelGpuProgram>(getResourceByName(name, groupName));
     }
     //---------------------------------------------------------------------------
     HighLevelGpuProgramPtr HighLevelGpuProgramManager::createProgram(
-			const String& name, const String& groupName, 
+            const String& name, const String& groupName, 
             const String& language, GpuProgramType gptype)
     {
-        ResourcePtr ret = ResourcePtr(
-            getFactory(language)->create(this, name, getNextHandle(), 
-            groupName, false, 0));
-
-        HighLevelGpuProgramPtr prg = ret.staticCast<HighLevelGpuProgram>();
+        GpuProgram* prg = getFactory(language)->create(this, name, getNextHandle(), groupName, false, 0);
         prg->setType(gptype);
         prg->setSyntaxCode(language);
 
+        ResourcePtr ret(prg);
         addImpl(ret);
         // Tell resource group manager
-        ResourceGroupManager::getSingleton()._notifyResourceCreated(ret);
-        return prg;
+        if(ret)
+            ResourceGroupManager::getSingleton()._notifyResourceCreated(ret);
+        return static_pointer_cast<HighLevelGpuProgram>(ret);
     }
     //---------------------------------------------------------------------------
     HighLevelGpuProgramFactory::~HighLevelGpuProgramFactory() 
