@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,49 +28,41 @@ THE SOFTWARE.
 #include "OgreStableHeaders.h"
 
 #include "OgreMaterial.h"
-
-#include "OgreSceneManagerEnumerator.h"
-#include "OgreMaterialManager.h"
-#include "OgreIteratorWrappers.h"
-#include "OgreTechnique.h"
-#include "OgreLogManager.h"
-#include "OgreException.h"
-#include "OgreStringConverter.h"
-#include "OgreLodStrategy.h"
 #include "OgreLodStrategyManager.h"
+#include "OgreLodStrategy.h"
 
 namespace Ogre {
 
     //-----------------------------------------------------------------------
-	Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
-		const String& group, bool isManual, ManualResourceLoader* loader)
-		:Resource(creator, name, handle, group, isManual, loader),
+    Material::Material(ResourceManager* creator, const String& name, ResourceHandle handle,
+        const String& group, bool isManual, ManualResourceLoader* loader)
+        :Resource(creator, name, handle, group, isManual, loader),
          mReceiveShadows(true),
          mTransparencyCastsShadows(false),
          mCompilationRequired(true)
     {
-		// Override isManual, not applicable for Material (we always want to call loadImpl)
-		if(isManual)
-		{
-			mIsManual = false;
-			LogManager::getSingleton().logMessage("Material " + name + 
-				" was requested with isManual=true, but this is not applicable " 
-				"for materials; the flag has been reset to false");
-		}
+        // Override isManual, not applicable for Material (we always want to call loadImpl)
+        if(isManual)
+        {
+            mIsManual = false;
+            LogManager::getSingleton().logMessage("Material " + name + 
+                " was requested with isManual=true, but this is not applicable " 
+                "for materials; the flag has been reset to false");
+        }
 
-		// Initialise to default strategy
-		mLodStrategy = LodStrategyManager::getSingleton().getDefaultStrategy();
+        // Initialise to default strategy
+        mLodStrategy = LodStrategyManager::getSingleton().getDefaultStrategy();
 
-		mLodValues.push_back(0.0f);
+        mLodValues.push_back(0.0f);
 
-		applyDefaults();
+        applyDefaults();
 
-		/* For consistency with StringInterface, but we don't add any parameters here
-		That's because the Resource implementation of StringInterface is to
-		list all the options that need to be set before loading, of which 
-		we have none as such. Full details can be set through scripts.
-		*/ 
-		createParamDictionary("Material");
+        /* For consistency with StringInterface, but we don't add any parameters here
+        That's because the Resource implementation of StringInterface is to
+        list all the options that need to be set before loading, of which 
+        we have none as such. Full details can be set through scripts.
+        */ 
+        createParamDictionary("Material");
     }
     //-----------------------------------------------------------------------
     Material::~Material()
@@ -83,18 +75,18 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Material& Material::operator=(const Material& rhs)
     {
-	    mName = rhs.mName;
-		mGroup = rhs.mGroup;
-		mCreator = rhs.mCreator;
-		mIsManual = rhs.mIsManual;
-		mLoader = rhs.mLoader;
-	    mHandle = rhs.mHandle;
+        mName = rhs.mName;
+        mGroup = rhs.mGroup;
+        mCreator = rhs.mCreator;
+        mIsManual = rhs.mIsManual;
+        mLoader = rhs.mLoader;
+        mHandle = rhs.mHandle;
         mSize = rhs.mSize;
         mReceiveShadows = rhs.mReceiveShadows;
         mTransparencyCastsShadows = rhs.mTransparencyCastsShadows;
 
-        mLoadingState = rhs.mLoadingState;
-		mIsBackgroundLoaded = rhs.mIsBackgroundLoaded;
+        mLoadingState.store(rhs.mLoadingState.load());
+        mIsBackgroundLoaded = rhs.mIsBackgroundLoaded;
 
         // Copy Techniques
         this->removeAllTechniques();
@@ -106,27 +98,27 @@ namespace Ogre {
             *t = *(*i);
             if ((*i)->isSupported())
             {
-				insertSupportedTechnique(t);
+                insertSupportedTechnique(t);
             }
         }
 
-		// Also copy LOD information
+        // Also copy LOD information
         mUserLodValues = rhs.mUserLodValues;
-		mLodValues = rhs.mLodValues;
+        mLodValues = rhs.mLodValues;
         mLodStrategy = rhs.mLodStrategy;
         mCompilationRequired = rhs.mCompilationRequired;
         // illumination passes are not compiled right away so
         // mIsLoaded state should still be the same as the original material
         assert(isLoaded() == rhs.isLoaded());
 
-	    return *this;
+        return *this;
     }
 
 
     //-----------------------------------------------------------------------
     void Material::prepareImpl(void)
     {
-		// compile if required
+        // compile if required
         if (mCompilationRequired)
             compile();
 
@@ -176,57 +168,43 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     size_t Material::calculateSize(void) const
     {
-        size_t memSize = 0;
+        size_t memSize = sizeof(*this) + Resource::calculateSize();
 
         // Tally up techniques
-        Techniques::const_iterator i, iend;
-        iend = mTechniques.end();
-        for (i = mTechniques.begin(); i != iend; ++i)
+        for (auto t : mTechniques)
         {
-            memSize += (*i)->calculateSize();
+            memSize += t->calculateSize();
         }
 
-        memSize += sizeof(bool) * 3;
         memSize += mUnsupportedReasons.size() * sizeof(char);
-        memSize += sizeof(LodStrategy);
-
-        memSize += Resource::calculateSize();
 
         return memSize;
     }
     //-----------------------------------------------------------------------
     MaterialPtr Material::clone(const String& newName, bool changeGroup, 
-		const String& newGroup) const
+        const String& newGroup) const
     {
-		MaterialPtr newMat;
-		if (changeGroup)
-		{
-			newMat = MaterialManager::getSingleton().create(newName, newGroup);
-		}
-		else
-		{
-			newMat = MaterialManager::getSingleton().create(newName, mGroup);
-		}
-        
+        MaterialPtr newMat =
+            MaterialManager::getSingleton().create(newName, changeGroup ? newGroup : mGroup);
+
+        if(!newMat) // interception by collision handler
+            return newMat;
 
         // Keep handle (see below, copy overrides everything)
         ResourceHandle newHandle = newMat->getHandle();
         // Assign values from this
         *newMat = *this;
-		// Restore new group if required, will have been overridden by operator
-		if (changeGroup)
-		{
-			newMat->mGroup = newGroup;
-		}
-		
+        // Restore new group if required, will have been overridden by operator
+        if (changeGroup)
+        {
+            newMat->mGroup = newGroup;
+        }
+        
         // Correct the name & handle, they get copied too
         newMat->mName = newName;
         newMat->mHandle = newHandle;
 
         return newMat;
-
-
-
     }
     //-----------------------------------------------------------------------
     void Material::copyDetailsTo(MaterialPtr& mat) const
@@ -235,39 +213,39 @@ namespace Ogre {
         ResourceHandle savedHandle = mat->mHandle;
         String savedName = mat->mName;
         String savedGroup = mat->mGroup;
-		ManualResourceLoader* savedLoader = mat->mLoader;
-		bool savedManual = mat->mIsManual;
+        ManualResourceLoader* savedLoader = mat->mLoader;
+        bool savedManual = mat->mIsManual;
         // Assign values from this
         *mat = *this;
         // Correct the name & handle, they get copied too
         mat->mName = savedName;
         mat->mHandle = savedHandle;
         mat->mGroup = savedGroup;
-		mat->mIsManual = savedManual;
-		mat->mLoader = savedLoader;
+        mat->mIsManual = savedManual;
+        mat->mLoader = savedLoader;
 
     }
     //-----------------------------------------------------------------------
     void Material::applyDefaults(void)
     {
-		MaterialPtr defaults = MaterialManager::getSingleton().getDefaultSettings();
+        MaterialPtr defaults = MaterialManager::getSingleton().getDefaultSettings();
 
-		if (!defaults.isNull())
-		{
+        if (defaults)
+        {
             // save name & handle
             String savedName = mName;
             String savedGroup = mGroup;
             ResourceHandle savedHandle = mHandle;
-			ManualResourceLoader *savedLoader = mLoader;
-			bool savedManual = mIsManual;
-			*this = *defaults;
+            ManualResourceLoader *savedLoader = mLoader;
+            bool savedManual = mIsManual;
+            *this = *defaults;
             // restore name & handle
             mName = savedName;
             mHandle = savedHandle;
             mGroup = savedGroup;
-			mLoader = savedLoader;
-			mIsManual = savedManual;
-		}
+            mLoader = savedLoader;
+            mIsManual = savedManual;
+        }
         mCompilationRequired = true;
 
     }
@@ -280,16 +258,16 @@ namespace Ogre {
         return t;
     }
     //-----------------------------------------------------------------------
-    Technique* Material::getTechnique(unsigned short index)
+    Technique* Material::getTechnique(unsigned short index) const
     {
         assert (index < mTechniques.size() && "Index out of bounds.");
         return mTechniques[index];
     }
     //-----------------------------------------------------------------------
-    Technique* Material::getTechnique(const String& name)
+    Technique* Material::getTechnique(const String& name) const
     {
-        Techniques::iterator i    = mTechniques.begin();
-        Techniques::iterator iend = mTechniques.end();
+        Techniques::const_iterator i    = mTechniques.begin();
+        Techniques::const_iterator iend = mTechniques.end();
         Technique* foundTechnique = 0;
 
         // iterate through techniques to find a match
@@ -305,71 +283,71 @@ namespace Ogre {
 
         return foundTechnique;
     }
-    //-----------------------------------------------------------------------	
+    //-----------------------------------------------------------------------   
     unsigned short Material::getNumTechniques(void) const
     {
-		return static_cast<unsigned short>(mTechniques.size());
+        return static_cast<unsigned short>(mTechniques.size());
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Technique* Material::getSupportedTechnique(unsigned short index)
     {
         assert (index < mSupportedTechniques.size() && "Index out of bounds.");
         return mSupportedTechniques[index];
     }
-    //-----------------------------------------------------------------------	
+    //-----------------------------------------------------------------------   
     unsigned short Material::getNumSupportedTechniques(void) const
     {
-		return static_cast<unsigned short>(mSupportedTechniques.size());
+        return static_cast<unsigned short>(mSupportedTechniques.size());
     }
-	//-----------------------------------------------------------------------
-	unsigned short Material::getNumLodLevels(unsigned short schemeIndex) const
-	{
-		// Safety check - empty list?
-		if (mBestTechniquesBySchemeList.empty())
-			return 0;
+    //-----------------------------------------------------------------------
+    unsigned short Material::getNumLodLevels(unsigned short schemeIndex) const
+    {
+        // Safety check - empty list?
+        if (mBestTechniquesBySchemeList.empty())
+            return 0;
 
-		BestTechniquesBySchemeList::const_iterator i = 
-			mBestTechniquesBySchemeList.find(schemeIndex);
-		if (i == mBestTechniquesBySchemeList.end())
-		{
-			// get the first item, will be 0 (the default) if default
-			// scheme techniques exist, otherwise the earliest defined
-			i = mBestTechniquesBySchemeList.begin();
-		}
+        BestTechniquesBySchemeList::const_iterator i = 
+            mBestTechniquesBySchemeList.find(schemeIndex);
+        if (i == mBestTechniquesBySchemeList.end())
+        {
+            // get the first item, will be 0 (the default) if default
+            // scheme techniques exist, otherwise the earliest defined
+            i = mBestTechniquesBySchemeList.begin();
+        }
 
-		return static_cast<unsigned short>(i->second->size());
-	}
-	//-----------------------------------------------------------------------
-	unsigned short Material::getNumLodLevels(const String& schemeName) const
-	{
-		return getNumLodLevels(
-			MaterialManager::getSingleton()._getSchemeIndex(schemeName));
-	}
-	//-----------------------------------------------------------------------
-	void Material::insertSupportedTechnique(Technique* t)
-	{
-		mSupportedTechniques.push_back(t);
-		// get scheme
-		unsigned short schemeIndex = t->_getSchemeIndex();
-		BestTechniquesBySchemeList::iterator i =
-			mBestTechniquesBySchemeList.find(schemeIndex);
-		LodTechniques* lodtechs = 0;
-		if (i == mBestTechniquesBySchemeList.end())
-		{
-			lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
-			mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
-		}
-		else
-		{
-			lodtechs = i->second;
-		}
+        return static_cast<unsigned short>(i->second->size());
+    }
+    //-----------------------------------------------------------------------
+    unsigned short Material::getNumLodLevels(const String& schemeName) const
+    {
+        return getNumLodLevels(
+            MaterialManager::getSingleton()._getSchemeIndex(schemeName));
+    }
+    //-----------------------------------------------------------------------
+    void Material::insertSupportedTechnique(Technique* t)
+    {
+        mSupportedTechniques.push_back(t);
+        // get scheme
+        unsigned short schemeIndex = t->_getSchemeIndex();
+        BestTechniquesBySchemeList::iterator i =
+            mBestTechniquesBySchemeList.find(schemeIndex);
+        LodTechniques* lodtechs = 0;
+        if (i == mBestTechniquesBySchemeList.end())
+        {
+            lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
+            mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
+        }
+        else
+        {
+            lodtechs = i->second;
+        }
 
-		// Insert won't replace if supported technique for this scheme/lod is
-		// already there, which is what we want
-		lodtechs->insert(LodTechniques::value_type(t->getLodIndex(), t));
+        // Insert won't replace if supported technique for this scheme/lod is
+        // already there, which is what we want
+        lodtechs->emplace(t->getLodIndex(), t);
 
-	}
-	//-----------------------------------------------------------------------------
+    }
+    //-----------------------------------------------------------------------------
     Technique* Material::getBestTechnique(unsigned short lodIndex, const Renderable* rend)
     {
         if (mSupportedTechniques.empty())
@@ -378,56 +356,56 @@ namespace Ogre {
         }
         else
         {
-			Technique* ret = 0;
-			MaterialManager& matMgr = MaterialManager::getSingleton();
-			// get scheme
-			BestTechniquesBySchemeList::iterator si = 
-				mBestTechniquesBySchemeList.find(matMgr._getActiveSchemeIndex());
-			// scheme not found?
-			if (si == mBestTechniquesBySchemeList.end())
-			{
-				// listener specified alternative technique available?
-				ret = matMgr._arbitrateMissingTechniqueForActiveScheme(this, lodIndex, rend);
-				if (ret)
-					return ret;
+            Technique* ret = 0;
+            MaterialManager& matMgr = MaterialManager::getSingleton();
+            // get scheme
+            BestTechniquesBySchemeList::iterator si = 
+                mBestTechniquesBySchemeList.find(matMgr._getActiveSchemeIndex());
+            // scheme not found?
+            if (si == mBestTechniquesBySchemeList.end())
+            {
+                // listener specified alternative technique available?
+                ret = matMgr._arbitrateMissingTechniqueForActiveScheme(this, lodIndex, rend);
+                if (ret)
+                    return ret;
 
-				// Nope, use default
-				// get the first item, will be 0 (the default) if default
-				// scheme techniques exist, otherwise the earliest defined
-				si = mBestTechniquesBySchemeList.begin();
-			}
+                // Nope, use default
+                // get the first item, will be 0 (the default) if default
+                // scheme techniques exist, otherwise the earliest defined
+                si = mBestTechniquesBySchemeList.begin();
+            }
 
-			// get LOD
-			LodTechniques::iterator li = si->second->find(lodIndex);
-			// LOD not found? 
-			if (li == si->second->end())
-			{
-				// Use the next LOD level up
-				for (LodTechniques::reverse_iterator rli = si->second->rbegin(); 
-					rli != si->second->rend(); ++rli)
-				{
-					if (rli->second->getLodIndex() < lodIndex)
-					{
-						ret = rli->second;
-						break;
-					}
+            // get LOD
+            LodTechniques::iterator li = si->second->find(lodIndex);
+            // LOD not found? 
+            if (li == si->second->end())
+            {
+                // Use the next LOD level up
+                for (LodTechniques::reverse_iterator rli = si->second->rbegin(); 
+                    rli != si->second->rend(); ++rli)
+                {
+                    if (rli->second->getLodIndex() < lodIndex)
+                    {
+                        ret = rli->second;
+                        break;
+                    }
 
-				}
-				if (!ret)
-				{
-					// shouldn't ever hit this really, unless user defines no LOD 0
-					// pick the first LOD we have (must be at least one to have a scheme entry)
-					ret = si->second->begin()->second;
-				}
+                }
+                if (!ret)
+                {
+                    // shouldn't ever hit this really, unless user defines no LOD 0
+                    // pick the first LOD we have (must be at least one to have a scheme entry)
+                    ret = si->second->begin()->second;
+                }
 
-			}
-			else
-			{
-				// LOD found
-				ret = li->second;
-			}
+            }
+            else
+            {
+                // LOD found
+                ret = li->second;
+            }
 
-			return ret;
+            return ret;
 
         }
     }
@@ -439,7 +417,7 @@ namespace Ogre {
         OGRE_DELETE(*i);
         mTechniques.erase(i);
         mSupportedTechniques.clear();
-		clearBestTechniqueList();
+        clearBestTechniqueList();
         mCompilationRequired = true;
     }
     //-----------------------------------------------------------------------
@@ -468,47 +446,47 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     bool Material::isTransparent(void) const
-	{
-		// Check each technique
-		Techniques::const_iterator i, iend;
-		iend = mTechniques.end();
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
-			if ( (*i)->isTransparent() )
-				return true;
-		}
-		return false;
-	}
+    {
+        // Check each technique
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            if ( (*i)->isTransparent() )
+                return true;
+        }
+        return false;
+    }
     //-----------------------------------------------------------------------
     void Material::compile(bool autoManageTextureUnits)
     {
         // Compile each technique, then add it to the list of supported techniques
         mSupportedTechniques.clear();
-		clearBestTechniqueList();
-		mUnsupportedReasons.clear();
+        clearBestTechniqueList();
+        mUnsupportedReasons.clear();
 
 
         Techniques::iterator i, iend;
         iend = mTechniques.end();
-		size_t techNo = 0;
+        size_t techNo = 0;
         for (i = mTechniques.begin(); i != iend; ++i, ++techNo)
         {
             String compileMessages = (*i)->_compile(autoManageTextureUnits);
             if ( (*i)->isSupported() )
             {
-				insertSupportedTechnique(*i);
+                insertSupportedTechnique(*i);
+            }
+            else
+            {
+                // Log informational
+                StringStream str;
+                str << "Material " << mName << " Technique " << techNo;
+                if (!(*i)->getName().empty())
+                    str << "(" << (*i)->getName() << ")";
+                str << " is not supported. " << compileMessages;
+                LogManager::getSingleton().logMessage(str.str(), LML_TRIVIAL);
+                mUnsupportedReasons += compileMessages;
             }
-			else
-			{
-				// Log informational
-				StringUtil::StrStreamType str;
-				str << "Material " << mName << " Technique " << techNo;
-				if (!(*i)->getName().empty())
-					str << "(" << (*i)->getName() << ")";
-				str << " is not supported. " << compileMessages;
-				LogManager::getSingleton().logMessage(str.str(), LML_TRIVIAL);
-				mUnsupportedReasons += compileMessages;
-			}
         }
 
         mCompilationRequired = false;
@@ -516,21 +494,21 @@ namespace Ogre {
         // Did we find any?
         if (mSupportedTechniques.empty())
         {
-            LogManager::getSingleton().stream(LML_CRITICAL)
-				<< "WARNING: material " << mName << " has no supportable "
-				<< "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
+            LogManager::getSingleton().stream(LML_WARNING)
+                << "Warning: material " << mName << " has no supportable "
+                << "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
         }
     }
-	//-----------------------------------------------------------------------
-	void Material::clearBestTechniqueList(void)
-	{
-		for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
-			i != mBestTechniquesBySchemeList.end(); ++i)
-		{
-			OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
-		}
-		mBestTechniquesBySchemeList.clear();
-	}
+    //-----------------------------------------------------------------------
+    void Material::clearBestTechniqueList(void)
+    {
+        for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
+            i != mBestTechniquesBySchemeList.end(); ++i)
+        {
+            OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
+        }
+        mBestTechniquesBySchemeList.clear();
+    }
     //-----------------------------------------------------------------------
     void Material::setPointSize(Real ps)
     {
@@ -543,22 +521,22 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    void Material::setAmbient(Real red, Real green, Real blue)
+    void Material::setAmbient(float red, float green, float blue)
     {
-		setAmbient(ColourValue(red, green, blue));
+        setAmbient(ColourValue(red, green, blue));
     }
     //-----------------------------------------------------------------------
     void Material::setAmbient(const ColourValue& ambient)
     {
-		Techniques::iterator i, iend;
-		iend = mTechniques.end();
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
-			(*i)->setAmbient(ambient);
-		}
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setAmbient(ambient);
+        }
     }
     //-----------------------------------------------------------------------
-    void Material::setDiffuse(Real red, Real green, Real blue, Real alpha)
+    void Material::setDiffuse(float red, float green, float blue, float alpha)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -573,7 +551,7 @@ namespace Ogre {
         setDiffuse(diffuse.r, diffuse.g, diffuse.b, diffuse.a);
     }
     //-----------------------------------------------------------------------
-    void Material::setSpecular(Real red, Real green, Real blue, Real alpha)
+    void Material::setSpecular(float red, float green, float blue, float alpha)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -598,19 +576,19 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void Material::setSelfIllumination(Real red, Real green, Real blue)
+    void Material::setSelfIllumination(float red, float green, float blue)
     {
-		setSelfIllumination(ColourValue(red, green, blue));   
+        setSelfIllumination(ColourValue(red, green, blue));   
     }
     //-----------------------------------------------------------------------
     void Material::setSelfIllumination(const ColourValue& selfIllum)
     {
-		Techniques::iterator i, iend;
-		iend = mTechniques.end();
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
-			(*i)->setSelfIllumination(selfIllum);
-		}
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setSelfIllumination(selfIllum);
+        }
     }
     //-----------------------------------------------------------------------
     void Material::setDepthCheckEnabled(bool enabled)
@@ -643,7 +621,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-	void Material::setColourWriteEnabled(bool enabled)
+    void Material::setColourWriteEnabled(bool enabled)
     {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
@@ -653,6 +631,16 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
+    void Material::setColourWriteEnabled(bool red, bool green, bool blue, bool alpha)
+    {
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setColourWriteEnabled(red, green, blue, alpha);
+        }
+    }
+    //-----------------------------------------------------------------------
     void Material::setCullingMode( CullingMode mode )
     {
         Techniques::iterator i, iend;
@@ -766,40 +754,41 @@ namespace Ogre {
     }
     // --------------------------------------------------------------------
     void Material::setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha)
-	{
+    {
         Techniques::iterator i, iend;
         iend = mTechniques.end();
         for (i = mTechniques.begin(); i != iend; ++i)
         {
             (*i)->setSeparateSceneBlending(sourceFactor, destFactor, sourceFactorAlpha, destFactorAlpha);
         }
-	}
+    }
     // --------------------------------------------------------------------
     void Material::_notifyNeedsRecompile(void)
     {
         mCompilationRequired = true;
         // Also need to unload to ensure we loaded any new items
-		if (isLoaded()) // needed to stop this being called in 'loading' state
-			unload();
+        if (isLoaded()) // needed to stop this being called in 'loading' state
+            unload();
     }
     // --------------------------------------------------------------------
     void Material::setLodLevels(const LodValueList& lodValues)
     {
         // Square the distances for the internal list
-		LodValueList::const_iterator i, iend;
-		iend = lodValues.end();
-		// First, clear and add single zero entry
-		mLodValues.clear();
+        LodValueList::const_iterator i, iend;
+        iend = lodValues.end();
+        // First, clear and add single zero entry
+        mLodValues.clear();
         mUserLodValues.clear();
         mUserLodValues.push_back(0);
-		mLodValues.push_back(mLodStrategy->getBaseValue());
-		for (i = lodValues.begin(); i != iend; ++i)
-		{
-			mUserLodValues.push_back(*i);
+        if (mLodStrategy)
+            mLodValues.push_back(mLodStrategy->getBaseValue());
+        for (i = lodValues.begin(); i != iend; ++i)
+        {
+            mUserLodValues.push_back(*i);
             if (mLodStrategy)
                 mLodValues.push_back(mLodStrategy->transformUserValue(*i));
-		}
-		
+        }
+        
     }
     // --------------------------------------------------------------------
     ushort Material::getLodIndex(Real value) const
@@ -821,15 +810,15 @@ namespace Ogre {
     bool Material::applyTextureAliases(const AliasTextureNamePairList& aliasList, const bool apply) const
     {
         // iterate through all techniques and apply texture aliases
-		Techniques::const_iterator i, iend;
-		iend = mTechniques.end();
+        Techniques::const_iterator i, iend;
+        iend = mTechniques.end();
         bool testResult = false;
 
-		for (i = mTechniques.begin(); i != iend; ++i)
-		{
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
             if ((*i)->applyTextureAliases(aliasList, apply))
                 testResult = true;
-		}
+        }
 
         return testResult;
     }
