@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,17 +27,6 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 
-#include "OgreMath.h"
-#include "asm_math.h"
-#include "OgreVector2.h"
-#include "OgreVector3.h"
-#include "OgreVector4.h"
-#include "OgreRay.h"
-#include "OgreSphere.h"
-#include "OgreAxisAlignedBox.h"
-#include "OgrePlane.h"
-
-
 namespace Ogre
 {
 
@@ -46,9 +35,9 @@ namespace Ogre
     const Real Math::PI = Real( 4.0 * atan( 1.0 ) );
     const Real Math::TWO_PI = Real( 2.0 * PI );
     const Real Math::HALF_PI = Real( 0.5 * PI );
-	const Real Math::fDeg2Rad = PI / Real(180.0);
-	const Real Math::fRad2Deg = Real(180.0) / PI;
-	const Real Math::LOG2 = log(Real(2.0));
+    const Real Math::fDeg2Rad = PI / Real(180.0);
+    const Real Math::fRad2Deg = Real(180.0) / PI;
+    const Real Math::LOG2 = std::log(Real(2.0));
 
     int Math::mTrigTableSize;
    Math::AngleUnit Math::msAngleUnit;
@@ -89,13 +78,13 @@ namespace Ogre
         Real angle;
         for (int i = 0; i < mTrigTableSize; ++i)
         {
-            angle = Math::TWO_PI * i / mTrigTableSize;
-            mSinTable[i] = sin(angle);
-            mTanTable[i] = tan(angle);
+            angle = Math::TWO_PI * i / Real(mTrigTableSize);
+            mSinTable[i] = std::sin(angle);
+            mTanTable[i] = std::tan(angle);
         }
     }
-	//-----------------------------------------------------------------------	
-	Real Math::SinTable (Real fValue)
+    //-----------------------------------------------------------------------   
+    Real Math::SinTable (Real fValue)
     {
         // Convert range to index values, wrap if required
         int idx;
@@ -110,17 +99,12 @@ namespace Ogre
 
         return mSinTable[idx];
     }
-	//-----------------------------------------------------------------------
-	Real Math::TanTable (Real fValue)
+    //-----------------------------------------------------------------------
+    Real Math::TanTable (Real fValue)
     {
         // Convert range to index values, wrap if required
-		int idx = int(fValue *= mTrigTableFactor) % mTrigTableSize;
-		return mTanTable[idx];
-    }
-    //-----------------------------------------------------------------------
-    int Math::ISign (int iValue)
-    {
-        return ( iValue > 0 ? +1 : ( iValue < 0 ? -1 : 0 ) );
+        int idx = int(fValue *= mTrigTableFactor) % mTrigTableSize;
+        return mTanTable[idx];
     }
     //-----------------------------------------------------------------------
     Radian Math::ACos (Real fValue)
@@ -128,7 +112,7 @@ namespace Ogre
         if ( -1.0 < fValue )
         {
             if ( fValue < 1.0 )
-                return Radian(acos(fValue));
+                return Radian(std::acos(fValue));
             else
                 return Radian(0.0);
         }
@@ -143,7 +127,7 @@ namespace Ogre
         if ( -1.0 < fValue )
         {
             if ( fValue < 1.0 )
-                return Radian(asin(fValue));
+                return Radian(std::asin(fValue));
             else
                 return Radian(HALF_PI);
         }
@@ -153,48 +137,19 @@ namespace Ogre
         }
     }
     //-----------------------------------------------------------------------
-    Real Math::Sign (Real fValue)
-    {
-        if ( fValue > 0.0 )
-            return 1.0;
-
-        if ( fValue < 0.0 )
-            return -1.0;
-
-        return 0.0;
-    }
-    //-----------------------------------------------------------------------
-    Real Math::InvSqrt(Real fValue)
-    {
-        return Real(asm_rsq(fValue));
-    }
-    //-----------------------------------------------------------------------
     Real Math::UnitRandom ()
     {
         if (mRandProvider)
             return mRandProvider->getRandomUnit();
-        else return asm_rand() / asm_rand_max();
+        else return Real(rand()) / RAND_MAX;
     }
     
-    //-----------------------------------------------------------------------
-    Real Math::RangeRandom (Real fLow, Real fHigh)
-    {
-        return (fHigh-fLow)*UnitRandom() + fLow;
-    }
-
-    //-----------------------------------------------------------------------
-    Real Math::SymmetricRandom ()
-    {
-        return 2.0f * UnitRandom() - 1.0f;
-    }
-
     //-----------------------------------------------------------------------
     void Math::SetRandomValueProvider(RandomValueProvider* provider)
     {
         mRandProvider = provider;
     }
 
-
    //-----------------------------------------------------------------------
     void Math::setAngleUnit(Math::AngleUnit unit)
    {
@@ -242,392 +197,312 @@ namespace Ogre
     }
 
     //-----------------------------------------------------------------------
-	bool Math::pointInTri2D(const Vector2& p, const Vector2& a, 
-		const Vector2& b, const Vector2& c)
+    bool Math::pointInTri2D(const Vector2& p, const Vector2& a, 
+        const Vector2& b, const Vector2& c)
     {
-		// Winding must be consistent from all edges for point to be inside
-		Vector2 v1, v2;
-		Real dot[3];
-		bool zeroDot[3];
-
-		v1 = b - a;
-		v2 = p - a;
-
-		// Note we don't care about normalisation here since sign is all we need
-		// It means we don't have to worry about magnitude of cross products either
-		dot[0] = v1.crossProduct(v2);
-		zeroDot[0] = Math::RealEqual(dot[0], 0.0f, 1e-3);
-
-
-		v1 = c - b;
-		v2 = p - b;
-
-		dot[1] = v1.crossProduct(v2);
-		zeroDot[1] = Math::RealEqual(dot[1], 0.0f, 1e-3);
-
-		// Compare signs (ignore colinear / coincident points)
-		if(!zeroDot[0] && !zeroDot[1] 
-		&& Math::Sign(dot[0]) != Math::Sign(dot[1]))
-		{
-			return false;
-		}
-
-		v1 = a - c;
-		v2 = p - c;
-
-		dot[2] = v1.crossProduct(v2);
-		zeroDot[2] = Math::RealEqual(dot[2], 0.0f, 1e-3);
-		// Compare signs (ignore colinear / coincident points)
-		if((!zeroDot[0] && !zeroDot[2] 
-			&& Math::Sign(dot[0]) != Math::Sign(dot[2])) ||
-			(!zeroDot[1] && !zeroDot[2] 
-			&& Math::Sign(dot[1]) != Math::Sign(dot[2])))
-		{
-			return false;
-		}
-
-
-		return true;
-    }
-	//-----------------------------------------------------------------------
-	bool Math::pointInTri3D(const Vector3& p, const Vector3& a, 
-		const Vector3& b, const Vector3& c, const Vector3& normal)
-	{
         // Winding must be consistent from all edges for point to be inside
-		Vector3 v1, v2;
-		Real dot[3];
-		bool zeroDot[3];
+        Vector2 v1, v2;
+        Real dot[3];
+        bool zeroDot[3];
 
         v1 = b - a;
         v2 = p - a;
 
-		// Note we don't care about normalisation here since sign is all we need
-		// It means we don't have to worry about magnitude of cross products either
-        dot[0] = v1.crossProduct(v2).dotProduct(normal);
-		zeroDot[0] = Math::RealEqual(dot[0], 0.0f, 1e-3);
+        // Note we don't care about normalisation here since sign is all we need
+        // It means we don't have to worry about magnitude of cross products either
+        dot[0] = v1.crossProduct(v2);
+        zeroDot[0] = Math::RealEqual(dot[0], 0.0f, 1e-3);
 
 
         v1 = c - b;
         v2 = p - b;
 
-		dot[1] = v1.crossProduct(v2).dotProduct(normal);
-		zeroDot[1] = Math::RealEqual(dot[1], 0.0f, 1e-3);
+        dot[1] = v1.crossProduct(v2);
+        zeroDot[1] = Math::RealEqual(dot[1], 0.0f, 1e-3);
 
-		// Compare signs (ignore colinear / coincident points)
-		if(!zeroDot[0] && !zeroDot[1] 
-			&& Math::Sign(dot[0]) != Math::Sign(dot[1]))
-		{
+        // Compare signs (ignore colinear / coincident points)
+        if(!zeroDot[0] && !zeroDot[1] 
+        && Math::Sign(dot[0]) != Math::Sign(dot[1]))
+        {
             return false;
-		}
+        }
 
         v1 = a - c;
         v2 = p - c;
 
-		dot[2] = v1.crossProduct(v2).dotProduct(normal);
-		zeroDot[2] = Math::RealEqual(dot[2], 0.0f, 1e-3);
-		// Compare signs (ignore colinear / coincident points)
-		if((!zeroDot[0] && !zeroDot[2] 
-			&& Math::Sign(dot[0]) != Math::Sign(dot[2])) ||
-			(!zeroDot[1] && !zeroDot[2] 
-			&& Math::Sign(dot[1]) != Math::Sign(dot[2])))
-		{
-			return false;
-		}
+        dot[2] = v1.crossProduct(v2);
+        zeroDot[2] = Math::RealEqual(dot[2], 0.0f, 1e-3);
+        // Compare signs (ignore colinear / coincident points)
+        if((!zeroDot[0] && !zeroDot[2] 
+            && Math::Sign(dot[0]) != Math::Sign(dot[2])) ||
+            (!zeroDot[1] && !zeroDot[2] 
+            && Math::Sign(dot[1]) != Math::Sign(dot[2])))
+        {
+            return false;
+        }
 
 
         return true;
-	}
+    }
     //-----------------------------------------------------------------------
-    bool Math::RealEqual( Real a, Real b, Real tolerance )
+    bool Math::pointInTri3D(const Vector3& p, const Vector3& a, 
+        const Vector3& b, const Vector3& c, const Vector3& normal)
     {
-        if (fabs(b-a) <= tolerance)
-            return true;
-        else
+        // Winding must be consistent from all edges for point to be inside
+        Vector3 v1, v2;
+        Real dot[3];
+        bool zeroDot[3];
+
+        v1 = b - a;
+        v2 = p - a;
+
+        // Note we don't care about normalisation here since sign is all we need
+        // It means we don't have to worry about magnitude of cross products either
+        dot[0] = v1.crossProduct(v2).dotProduct(normal);
+        zeroDot[0] = Math::RealEqual(dot[0], 0.0f, 1e-3);
+
+
+        v1 = c - b;
+        v2 = p - b;
+
+        dot[1] = v1.crossProduct(v2).dotProduct(normal);
+        zeroDot[1] = Math::RealEqual(dot[1], 0.0f, 1e-3);
+
+        // Compare signs (ignore colinear / coincident points)
+        if(!zeroDot[0] && !zeroDot[1] 
+            && Math::Sign(dot[0]) != Math::Sign(dot[1]))
+        {
             return false;
-    }
-
-    //-----------------------------------------------------------------------
-    std::pair<bool, Real> Math::intersects(const Ray& ray, const Plane& plane)
-    {
-
-        Real denom = plane.normal.dotProduct(ray.getDirection());
-        if (Math::Abs(denom) < std::numeric_limits<Real>::epsilon())
-        {
-            // Parallel
-            return std::pair<bool, Real>(false, (Real)0);
         }
-        else
+
+        v1 = a - c;
+        v2 = p - c;
+
+        dot[2] = v1.crossProduct(v2).dotProduct(normal);
+        zeroDot[2] = Math::RealEqual(dot[2], 0.0f, 1e-3);
+        // Compare signs (ignore colinear / coincident points)
+        if((!zeroDot[0] && !zeroDot[2] 
+            && Math::Sign(dot[0]) != Math::Sign(dot[2])) ||
+            (!zeroDot[1] && !zeroDot[2] 
+            && Math::Sign(dot[1]) != Math::Sign(dot[2])))
         {
-            Real nom = plane.normal.dotProduct(ray.getOrigin()) + plane.d;
-            Real t = -(nom/denom);
-            return std::pair<bool, Real>(t >= 0, (Real)t);
+            return false;
         }
-        
+
+
+        return true;
     }
     //-----------------------------------------------------------------------
     std::pair<bool, Real> Math::intersects(const Ray& ray, 
-        const vector<Plane>::type& planes, bool normalIsOutside)
+        const std::list<Plane>& planes, bool normalIsOutside)
     {
-		list<Plane>::type planesList;
-		for (vector<Plane>::type::const_iterator i = planes.begin(); i != planes.end(); ++i)
-		{
-			planesList.push_back(*i);
-		}
-		return intersects(ray, planesList, normalIsOutside);
+        std::vector<Plane> planesVec;
+        planesVec.reserve(planes.size());
+        for (std::list<Plane>::const_iterator i = planes.begin(); i != planes.end(); ++i)
+        {
+            planesVec.push_back(*i);
+        }
+        return intersects(ray, planesVec, normalIsOutside);
     }
     //-----------------------------------------------------------------------
     std::pair<bool, Real> Math::intersects(const Ray& ray, 
-        const list<Plane>::type& planes, bool normalIsOutside)
+        const std::vector<Plane>& planes, bool normalIsOutside)
     {
-		list<Plane>::type::const_iterator planeit, planeitend;
-		planeitend = planes.end();
-		bool allInside = true;
-		std::pair<bool, Real> ret;
-		std::pair<bool, Real> end;
-		ret.first = false;
-		ret.second = 0.0f;
-		end.first = false;
-		end.second = 0;
+        std::vector<Plane>::const_iterator planeit, planeitend;
+        planeitend = planes.end();
+        bool allInside = true;
+        std::pair<bool, Real> ret;
+        std::pair<bool, Real> end;
+        ret.first = false;
+        ret.second = 0.0f;
+        end.first = false;
+        end.second = 0;
 
 
-		// derive side
-		// NB we don't pass directly since that would require Plane::Side in 
-		// interface, which results in recursive includes since Math is so fundamental
-		Plane::Side outside = normalIsOutside ? Plane::POSITIVE_SIDE : Plane::NEGATIVE_SIDE;
+        // derive side
+        // NB we don't pass directly since that would require Plane::Side in 
+        // interface, which results in recursive includes since Math is so fundamental
+        Plane::Side outside = normalIsOutside ? Plane::POSITIVE_SIDE : Plane::NEGATIVE_SIDE;
 
-		for (planeit = planes.begin(); planeit != planeitend; ++planeit)
-		{
-			const Plane& plane = *planeit;
-			// is origin outside?
-			if (plane.getSide(ray.getOrigin()) == outside)
-			{
-				allInside = false;
-				// Test single plane
-				std::pair<bool, Real> planeRes = 
-					ray.intersects(plane);
-				if (planeRes.first)
-				{
-					// Ok, we intersected
-					ret.first = true;
-					// Use the most distant result since convex volume
-					ret.second = std::max(ret.second, planeRes.second);
-				}
-				else
-				{
-					ret.first =false;
-					ret.second=0.0f;
-					return ret;
-				}
-			}
-			else
-			{
-				std::pair<bool, Real> planeRes = 
-					ray.intersects(plane);
-				if (planeRes.first)
-				{
-					if( !end.first )
-					{
-						end.first = true;
-						end.second = planeRes.second;
-					}
-					else
-					{
-						end.second = std::min( planeRes.second, end.second );
-					}
+        for (planeit = planes.begin(); planeit != planeitend; ++planeit)
+        {
+            const Plane& plane = *planeit;
+            // is origin outside?
+            if (plane.getSide(ray.getOrigin()) == outside)
+            {
+                allInside = false;
+                // Test single plane
+                std::pair<bool, Real> planeRes = 
+                    ray.intersects(plane);
+                if (planeRes.first)
+                {
+                    // Ok, we intersected
+                    ret.first = true;
+                    // Use the most distant result since convex volume
+                    ret.second = std::max(ret.second, planeRes.second);
+                }
+                else
+                {
+                    ret.first =false;
+                    ret.second=0.0f;
+                    return ret;
+                }
+            }
+            else
+            {
+                std::pair<bool, Real> planeRes = 
+                    ray.intersects(plane);
+                if (planeRes.first)
+                {
+                    if( !end.first )
+                    {
+                        end.first = true;
+                        end.second = planeRes.second;
+                    }
+                    else
+                    {
+                        end.second = std::min( planeRes.second, end.second );
+                    }
 
-				}
+                }
 
-			}
-		}
+            }
+        }
 
-		if (allInside)
-		{
-			// Intersecting at 0 distance since inside the volume!
-			ret.first = true;
-			ret.second = 0.0f;
-			return ret;
-		}
+        if (allInside)
+        {
+            // Intersecting at 0 distance since inside the volume!
+            ret.first = true;
+            ret.second = 0.0f;
+            return ret;
+        }
 
-		if( end.first )
-		{
-			if( end.second < ret.second )
-			{
-				ret.first = false;
-				return ret;
-			}
-		}
-		return ret;
+        if( end.first )
+        {
+            if( end.second < ret.second )
+            {
+                ret.first = false;
+                return ret;
+            }
+        }
+        return ret;
     }
     //-----------------------------------------------------------------------
-    std::pair<bool, Real> Math::intersects(const Ray& ray, const Sphere& sphere, 
-        bool discardInside)
+    std::pair<bool, Real> Math::intersects(const Ray& ray, const AxisAlignedBox& box)
     {
+        if (box.isNull()) return std::pair<bool, Real>(false, (Real)0);
+        if (box.isInfinite()) return std::pair<bool, Real>(true, (Real)0);
+
+        Real lowt = 0.0f;
+        Real t;
+        bool hit = false;
+        Vector3 hitpoint;
+        const Vector3& min = box.getMinimum();
+        const Vector3& max = box.getMaximum();
+        const Vector3& rayorig = ray.getOrigin();
         const Vector3& raydir = ray.getDirection();
-        // Adjust ray origin relative to sphere center
-        const Vector3& rayorig = ray.getOrigin() - sphere.getCenter();
-        Real radius = sphere.getRadius();
 
         // Check origin inside first
-        if (rayorig.squaredLength() <= radius*radius && discardInside)
+        if ( rayorig > min && rayorig < max )
         {
             return std::pair<bool, Real>(true, (Real)0);
         }
 
-        // Mmm, quadratics
-        // Build coeffs which can be used with std quadratic solver
-        // ie t = (-b +/- sqrt(b*b + 4ac)) / 2a
-        Real a = raydir.dotProduct(raydir);
-        Real b = 2 * rayorig.dotProduct(raydir);
-        Real c = rayorig.dotProduct(rayorig) - radius*radius;
+        // Check each face in turn, only check closest 3
+        // Min x
+        if (rayorig.x <= min.x && raydir.x > 0)
+        {
+            t = (min.x - rayorig.x) / raydir.x;
 
-        // Calc determinant
-        Real d = (b*b) - (4 * a * c);
-        if (d < 0)
-        {
-            // No intersection
-            return std::pair<bool, Real>(false, (Real)0);
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.y >= min.y && hitpoint.y <= max.y &&
+                hitpoint.z >= min.z && hitpoint.z <= max.z &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
         }
-        else
+        // Max x
+        if (rayorig.x >= max.x && raydir.x < 0)
         {
-            // BTW, if d=0 there is one intersection, if d > 0 there are 2
-            // But we only want the closest one, so that's ok, just use the 
-            // '-' version of the solver
-            Real t = ( -b - Math::Sqrt(d) ) / (2 * a);
-            if (t < 0)
-                t = ( -b + Math::Sqrt(d) ) / (2 * a);
-            return std::pair<bool, Real>(true, (Real)t);
+            t = (max.x - rayorig.x) / raydir.x;
+
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.y >= min.y && hitpoint.y <= max.y &&
+                hitpoint.z >= min.z && hitpoint.z <= max.z &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
+        }
+        // Min y
+        if (rayorig.y <= min.y && raydir.y > 0)
+        {
+            t = (min.y - rayorig.y) / raydir.y;
+
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
+                hitpoint.z >= min.z && hitpoint.z <= max.z &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
+        }
+        // Max y
+        if (rayorig.y >= max.y && raydir.y < 0)
+        {
+            t = (max.y - rayorig.y) / raydir.y;
+
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
+                hitpoint.z >= min.z && hitpoint.z <= max.z &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
+        }
+        // Min z
+        if (rayorig.z <= min.z && raydir.z > 0)
+        {
+            t = (min.z - rayorig.z) / raydir.z;
+
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
+                hitpoint.y >= min.y && hitpoint.y <= max.y &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
+        }
+        // Max z
+        if (rayorig.z >= max.z && raydir.z < 0)
+        {
+            t = (max.z - rayorig.z) / raydir.z;
+
+            // Substitute t back into ray and check bounds and dist
+            hitpoint = rayorig + raydir * t;
+            if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
+                hitpoint.y >= min.y && hitpoint.y <= max.y &&
+                (!hit || t < lowt))
+            {
+                hit = true;
+                lowt = t;
+            }
         }
 
+        return std::pair<bool, Real>(hit, (Real)lowt);
 
-    }
-    //-----------------------------------------------------------------------
-	std::pair<bool, Real> Math::intersects(const Ray& ray, const AxisAlignedBox& box)
-	{
-		if (box.isNull()) return std::pair<bool, Real>(false, (Real)0);
-		if (box.isInfinite()) return std::pair<bool, Real>(true, (Real)0);
-
-		Real lowt = 0.0f;
-		Real t;
-		bool hit = false;
-		Vector3 hitpoint;
-		const Vector3& min = box.getMinimum();
-		const Vector3& max = box.getMaximum();
-		const Vector3& rayorig = ray.getOrigin();
-		const Vector3& raydir = ray.getDirection();
-
-		// Check origin inside first
-		if ( rayorig > min && rayorig < max )
-		{
-			return std::pair<bool, Real>(true, (Real)0);
-		}
-
-		// Check each face in turn, only check closest 3
-		// Min x
-		if (rayorig.x <= min.x && raydir.x > 0)
-		{
-			t = (min.x - rayorig.x) / raydir.x;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.y >= min.y && hitpoint.y <= max.y &&
-					hitpoint.z >= min.z && hitpoint.z <= max.z &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-		// Max x
-		if (rayorig.x >= max.x && raydir.x < 0)
-		{
-			t = (max.x - rayorig.x) / raydir.x;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.y >= min.y && hitpoint.y <= max.y &&
-					hitpoint.z >= min.z && hitpoint.z <= max.z &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-		// Min y
-		if (rayorig.y <= min.y && raydir.y > 0)
-		{
-			t = (min.y - rayorig.y) / raydir.y;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
-					hitpoint.z >= min.z && hitpoint.z <= max.z &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-		// Max y
-		if (rayorig.y >= max.y && raydir.y < 0)
-		{
-			t = (max.y - rayorig.y) / raydir.y;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
-					hitpoint.z >= min.z && hitpoint.z <= max.z &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-		// Min z
-		if (rayorig.z <= min.z && raydir.z > 0)
-		{
-			t = (min.z - rayorig.z) / raydir.z;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
-					hitpoint.y >= min.y && hitpoint.y <= max.y &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-		// Max z
-		if (rayorig.z >= max.z && raydir.z < 0)
-		{
-			t = (max.z - rayorig.z) / raydir.z;
-			if (t >= 0)
-			{
-				// Substitute t back into ray and check bounds and dist
-				hitpoint = rayorig + raydir * t;
-				if (hitpoint.x >= min.x && hitpoint.x <= max.x &&
-					hitpoint.y >= min.y && hitpoint.y <= max.y &&
-					(!hit || t < lowt))
-				{
-					hit = true;
-					lowt = t;
-				}
-			}
-		}
-
-		return std::pair<bool, Real>(hit, (Real)lowt);
-
-	} 
+    } 
     //-----------------------------------------------------------------------
     bool Math::intersects(const Ray& ray, const AxisAlignedBox& box,
         Real* d1, Real* d2)
@@ -829,171 +704,169 @@ namespace Ogre
         const Vector3& min = box.getMinimum();
         const Vector3& max = box.getMaximum();
 
-		// Arvo's algorithm
-		Real s, d = 0;
-		for (int i = 0; i < 3; ++i)
-		{
-			if (center.ptr()[i] < min.ptr()[i])
-			{
-				s = center.ptr()[i] - min.ptr()[i];
-				d += s * s; 
-			}
-			else if(center.ptr()[i] > max.ptr()[i])
-			{
-				s = center.ptr()[i] - max.ptr()[i];
-				d += s * s; 
-			}
-		}
-		return d <= radius * radius;
+        // Arvo's algorithm
+        Real s, d = 0;
+        for (int i = 0; i < 3; ++i)
+        {
+            if (center.ptr()[i] < min.ptr()[i])
+            {
+                s = center.ptr()[i] - min.ptr()[i];
+                d += s * s; 
+            }
+            else if(center.ptr()[i] > max.ptr()[i])
+            {
+                s = center.ptr()[i] - max.ptr()[i];
+                d += s * s; 
+            }
+        }
+        return d <= radius * radius;
 
     }
     //-----------------------------------------------------------------------
-    bool Math::intersects(const Plane& plane, const AxisAlignedBox& box)
-    {
-        return (plane.getSide(box) == Plane::BOTH_SIDE);
-    }
-    //-----------------------------------------------------------------------
-    bool Math::intersects(const Sphere& sphere, const Plane& plane)
-    {
-        return (
-            Math::Abs(plane.getDistance(sphere.getCenter()))
-            <= sphere.getRadius() );
-    }
-    //-----------------------------------------------------------------------
     Vector3 Math::calculateTangentSpaceVector(
         const Vector3& position1, const Vector3& position2, const Vector3& position3,
         Real u1, Real v1, Real u2, Real v2, Real u3, Real v3)
     {
-	    //side0 is the vector along one side of the triangle of vertices passed in, 
-	    //and side1 is the vector along another side. Taking the cross product of these returns the normal.
-	    Vector3 side0 = position1 - position2;
-	    Vector3 side1 = position3 - position1;
-	    //Calculate face normal
-	    Vector3 normal = side1.crossProduct(side0);
-	    normal.normalise();
-	    //Now we use a formula to calculate the tangent. 
-	    Real deltaV0 = v1 - v2;
-	    Real deltaV1 = v3 - v1;
-	    Vector3 tangent = deltaV1 * side0 - deltaV0 * side1;
-	    tangent.normalise();
-	    //Calculate binormal
-	    Real deltaU0 = u1 - u2;
-	    Real deltaU1 = u3 - u1;
-	    Vector3 binormal = deltaU1 * side0 - deltaU0 * side1;
-	    binormal.normalise();
-	    //Now, we take the cross product of the tangents to get a vector which 
-	    //should point in the same direction as our normal calculated above. 
-	    //If it points in the opposite direction (the dot product between the normals is less than zero), 
-	    //then we need to reverse the s and t tangents. 
-	    //This is because the triangle has been mirrored when going from tangent space to object space.
-	    //reverse tangents if necessary
-	    Vector3 tangentCross = tangent.crossProduct(binormal);
-	    if (tangentCross.dotProduct(normal) < 0.0f)
-	    {
-		    tangent = -tangent;
-		    binormal = -binormal;
-	    }
+        //side0 is the vector along one side of the triangle of vertices passed in, 
+        //and side1 is the vector along another side. Taking the cross product of these returns the normal.
+        Vector3 side0 = position1 - position2;
+        Vector3 side1 = position3 - position1;
+        //Calculate face normal
+        Vector3 normal = side1.crossProduct(side0);
+        normal.normalise();
+        //Now we use a formula to calculate the tangent. 
+        Real deltaV0 = v1 - v2;
+        Real deltaV1 = v3 - v1;
+        Vector3 tangent = deltaV1 * side0 - deltaV0 * side1;
+        tangent.normalise();
+        //Calculate binormal
+        Real deltaU0 = u1 - u2;
+        Real deltaU1 = u3 - u1;
+        Vector3 binormal = deltaU1 * side0 - deltaU0 * side1;
+        binormal.normalise();
+        //Now, we take the cross product of the tangents to get a vector which 
+        //should point in the same direction as our normal calculated above. 
+        //If it points in the opposite direction (the dot product between the normals is less than zero), 
+        //then we need to reverse the s and t tangents. 
+        //This is because the triangle has been mirrored when going from tangent space to object space.
+        //reverse tangents if necessary
+        Vector3 tangentCross = tangent.crossProduct(binormal);
+        if (tangentCross.dotProduct(normal) < 0.0f)
+        {
+            tangent = -tangent;
+            binormal = -binormal;
+        }
 
         return tangent;
 
     }
     //-----------------------------------------------------------------------
-    Matrix4 Math::buildReflectionMatrix(const Plane& p)
+    Affine3 Math::buildReflectionMatrix(const Plane& p)
     {
-        return Matrix4(
+        return Affine3(
             -2 * p.normal.x * p.normal.x + 1,   -2 * p.normal.x * p.normal.y,       -2 * p.normal.x * p.normal.z,       -2 * p.normal.x * p.d, 
             -2 * p.normal.y * p.normal.x,       -2 * p.normal.y * p.normal.y + 1,   -2 * p.normal.y * p.normal.z,       -2 * p.normal.y * p.d, 
-            -2 * p.normal.z * p.normal.x,       -2 * p.normal.z * p.normal.y,       -2 * p.normal.z * p.normal.z + 1,   -2 * p.normal.z * p.d, 
-            0,                                  0,                                  0,                                  1);
+            -2 * p.normal.z * p.normal.x,       -2 * p.normal.z * p.normal.y,       -2 * p.normal.z * p.normal.z + 1,   -2 * p.normal.z * p.d);
     }
     //-----------------------------------------------------------------------
-    Vector4 Math::calculateFaceNormal(const Vector3& v1, const Vector3& v2, const Vector3& v3)
+    Real Math::gaussianDistribution(Real x, Real offset, Real scale)
     {
-        Vector3 normal = calculateBasicFaceNormal(v1, v2, v3);
-        // Now set up the w (distance of tri from origin
-        return Vector4(normal.x, normal.y, normal.z, -(normal.dotProduct(v1)));
+        Real nom = Math::Exp(
+            -Math::Sqr(x - offset) / (2 * Math::Sqr(scale)));
+        Real denom = scale * Math::Sqrt(2 * Math::PI);
+
+        return nom / denom;
+
     }
-    //-----------------------------------------------------------------------
-    Vector3 Math::calculateBasicFaceNormal(const Vector3& v1, const Vector3& v2, const Vector3& v3)
+    //---------------------------------------------------------------------
+    Affine3 Math::makeViewMatrix(const Vector3& position, const Quaternion& orientation,
+        const Affine3* reflectMatrix)
     {
-        Vector3 normal = (v2 - v1).crossProduct(v3 - v1);
-        normal.normalise();
-        return normal;
+        // This is most efficiently done using 3x3 Matrices
+        Matrix3 rot;
+        orientation.ToRotationMatrix(rot);
+
+        // Make the translation relative to new axes
+        Matrix3 rotT = rot.Transpose();
+        Vector3 trans = -rotT * position;
+
+        // Make final matrix
+        Affine3 viewMatrix = Affine3::IDENTITY;
+        viewMatrix = rotT; // fills upper 3x3
+        viewMatrix[0][3] = trans.x;
+        viewMatrix[1][3] = trans.y;
+        viewMatrix[2][3] = trans.z;
+
+        // Deal with reflections
+        if (reflectMatrix)
+        {
+            viewMatrix = viewMatrix * (*reflectMatrix);
+        }
+
+        return viewMatrix;
+
     }
-    //-----------------------------------------------------------------------
-    Vector4 Math::calculateFaceNormalWithoutNormalize(const Vector3& v1, const Vector3& v2, const Vector3& v3)
+
+    Matrix4 Math::makePerspectiveMatrix(Real left, Real right, Real bottom, Real top, Real zNear, Real zFar)
     {
-        Vector3 normal = calculateBasicFaceNormalWithoutNormalize(v1, v2, v3);
-        // Now set up the w (distance of tri from origin)
-        return Vector4(normal.x, normal.y, normal.z, -(normal.dotProduct(v1)));
+        // The code below will dealing with general projection
+        // parameters, similar glFrustum.
+        // Doesn't optimise manually except division operator, so the
+        // code more self-explaining.
+
+        Real inv_w = 1 / (right - left);
+        Real inv_h = 1 / (top - bottom);
+        Real inv_d = 1 / (zFar - zNear);
+
+        // Calc matrix elements
+        Real A = 2 * zNear * inv_w;
+        Real B = 2 * zNear * inv_h;
+        Real C = (right + left) * inv_w;
+        Real D = (top + bottom) * inv_h;
+        Real q, qn;
+
+        if (zFar == 0)
+        {
+            // Infinite far plane
+            q = Frustum::INFINITE_FAR_PLANE_ADJUST - 1;
+            qn = zNear * (Frustum::INFINITE_FAR_PLANE_ADJUST - 2);
+        }
+        else
+        {
+            q = - (zFar + zNear) * inv_d;
+            qn = -2 * (zFar * zNear) * inv_d;
+        }
+
+        Matrix4 ret = Matrix4::ZERO;
+        ret[0][0] = A;
+        ret[0][2] = C;
+        ret[1][1] = B;
+        ret[1][2] = D;
+        ret[2][2] = q;
+        ret[2][3] = qn;
+        ret[3][2] = -1;
+
+        return ret;
     }
-    //-----------------------------------------------------------------------
-    Vector3 Math::calculateBasicFaceNormalWithoutNormalize(const Vector3& v1, const Vector3& v2, const Vector3& v3)
+    //---------------------------------------------------------------------
+    Real Math::boundingRadiusFromAABB(const AxisAlignedBox& aabb)
     {
-        Vector3 normal = (v2 - v1).crossProduct(v3 - v1);
-        return normal;
+        const Vector3& max = aabb.getMaximum();
+        const Vector3& min = aabb.getMinimum();
+
+        Vector3 magnitude = max;
+        magnitude.makeCeil(-max);
+        magnitude.makeCeil(min);
+        magnitude.makeCeil(-min);
+
+        return magnitude.length();
     }
-	//-----------------------------------------------------------------------
-	Real Math::gaussianDistribution(Real x, Real offset, Real scale)
-	{
-		Real nom = Math::Exp(
-			-Math::Sqr(x - offset) / (2 * Math::Sqr(scale)));
-		Real denom = scale * Math::Sqrt(2 * Math::PI);
 
-		return nom / denom;
-
-	}
-	//---------------------------------------------------------------------
-	Matrix4 Math::makeViewMatrix(const Vector3& position, const Quaternion& orientation, 
-		const Matrix4* reflectMatrix)
-	{
-		Matrix4 viewMatrix;
-
-		// View matrix is:
-		//
-		//  [ Lx  Uy  Dz  Tx  ]
-		//  [ Lx  Uy  Dz  Ty  ]
-		//  [ Lx  Uy  Dz  Tz  ]
-		//  [ 0   0   0   1   ]
-		//
-		// Where T = -(Transposed(Rot) * Pos)
-
-		// This is most efficiently done using 3x3 Matrices
-		Matrix3 rot;
-		orientation.ToRotationMatrix(rot);
-
-		// Make the translation relative to new axes
-		Matrix3 rotT = rot.Transpose();
-		Vector3 trans = -rotT * position;
-
-		// Make final matrix
-		viewMatrix = Matrix4::IDENTITY;
-		viewMatrix = rotT; // fills upper 3x3
-		viewMatrix[0][3] = trans.x;
-		viewMatrix[1][3] = trans.y;
-		viewMatrix[2][3] = trans.z;
-
-		// Deal with reflections
-		if (reflectMatrix)
-		{
-			viewMatrix = viewMatrix * (*reflectMatrix);
-		}
-
-		return viewMatrix;
-
-	}
-	//---------------------------------------------------------------------
-	Real Math::boundingRadiusFromAABB(const AxisAlignedBox& aabb)
-	{
-		Vector3 max = aabb.getMaximum();
-		Vector3 min = aabb.getMinimum();
-
-		Vector3 magnitude = max;
-		magnitude.makeCeil(-max);
-		magnitude.makeCeil(min);
-		magnitude.makeCeil(-min);
-
-		return magnitude.length();
-	}
+    Real Math::boundingRadiusFromAABBCentered(const AxisAlignedBox& aabb)
+    {
+        const Vector3& max = aabb.getMaximum();
+        const Vector3& min = aabb.getMinimum();
 
+        return ((min - max) * 0.5f).length();
+    }
 }
