@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,18 +27,7 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 
-#include "OgreMeshManager.h"
-
-#include "OgreMesh.h"
-#include "OgreSubMesh.h"
-#include "OgreMatrix4.h"
-#include "OgreMatrix3.h"
-#include "OgreVector3.h"
-#include "OgrePlane.h"
-#include "OgreHardwareBufferManager.h"
-#include "OgrePatchSurface.h"
-#include "OgreException.h"
-
+#include "OgrePatchMesh.h"
 #include "OgrePrefabFactory.h"
 
 namespace Ogre
@@ -57,6 +46,7 @@ namespace Ogre
     MeshManager::MeshManager():
     mBoundsPaddingFactor(0.01), mListener(0)
     {
+        mBlendWeightsBaseElementType = VET_FLOAT1;
         mPrepAllMeshesForShadowVolumes = false;
 
         mLoadOrder = 350.0f;
@@ -73,59 +63,61 @@ namespace Ogre
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::getByName(const String& name, const String& groupName)
     {
-        return getResourceByName(name, groupName).staticCast<Mesh>();
+        return static_pointer_cast<Mesh>(getResourceByName(name, groupName));
     }
     //-----------------------------------------------------------------------
     void MeshManager::_initialise(void)
     {
         // Create prefab objects
-        createPrefabPlane();
-		createPrefabCube();
-		createPrefabSphere();
+        createManual("Ogre/Debug/AxesMesh", RGN_INTERNAL, &mPrefabLoader);
+        createManual("Prefab_Sphere", RGN_INTERNAL, &mPrefabLoader);
+        createManual("Prefab_Cube", RGN_INTERNAL, &mPrefabLoader);
+        // Planes can never be manifold
+        createManual("Prefab_Plane", RGN_INTERNAL, &mPrefabLoader)->setAutoBuildEdgeLists(false);
     }
     //-----------------------------------------------------------------------
     MeshManager::ResourceCreateOrRetrieveResult MeshManager::createOrRetrieve(
         const String& name, const String& group,
         bool isManual, ManualResourceLoader* loader,
         const NameValuePairList* params,
-		HardwareBuffer::Usage vertexBufferUsage, 
-		HardwareBuffer::Usage indexBufferUsage, 
-		bool vertexBufferShadowed, bool indexBufferShadowed)
+        HardwareBuffer::Usage vertexBufferUsage, 
+        HardwareBuffer::Usage indexBufferUsage, 
+        bool vertexBufferShadowed, bool indexBufferShadowed)
     {
         ResourceCreateOrRetrieveResult res = 
             ResourceManager::createOrRetrieve(name,group,isManual,loader,params);
-		MeshPtr pMesh = res.first.staticCast<Mesh>();
-		// Was it created?
+        MeshPtr pMesh = static_pointer_cast<Mesh>(res.first);
+        // Was it created?
         if (res.second)
         {
-			pMesh->setVertexBufferPolicy(vertexBufferUsage, vertexBufferShadowed);
-			pMesh->setIndexBufferPolicy(indexBufferUsage, indexBufferShadowed);
+            pMesh->setVertexBufferPolicy(vertexBufferUsage, vertexBufferShadowed);
+            pMesh->setIndexBufferPolicy(indexBufferUsage, indexBufferShadowed);
         }
         return res;
 
     }
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::prepare( const String& filename, const String& groupName, 
-		HardwareBuffer::Usage vertexBufferUsage, 
-		HardwareBuffer::Usage indexBufferUsage, 
-		bool vertexBufferShadowed, bool indexBufferShadowed)
+        HardwareBuffer::Usage vertexBufferUsage, 
+        HardwareBuffer::Usage indexBufferUsage, 
+        bool vertexBufferShadowed, bool indexBufferShadowed)
     {
-		MeshPtr pMesh = createOrRetrieve(filename,groupName,false,0,0,
+        MeshPtr pMesh = static_pointer_cast<Mesh>(createOrRetrieve(filename,groupName,false,0,0,
                                          vertexBufferUsage,indexBufferUsage,
-                                         vertexBufferShadowed,indexBufferShadowed).first.staticCast<Mesh>();
-		pMesh->prepare();
+                                         vertexBufferShadowed,indexBufferShadowed).first);
+        pMesh->prepare();
         return pMesh;
     }
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::load( const String& filename, const String& groupName, 
-		HardwareBuffer::Usage vertexBufferUsage, 
-		HardwareBuffer::Usage indexBufferUsage, 
-		bool vertexBufferShadowed, bool indexBufferShadowed)
+        HardwareBuffer::Usage vertexBufferUsage, 
+        HardwareBuffer::Usage indexBufferUsage, 
+        bool vertexBufferShadowed, bool indexBufferShadowed)
     {
-		MeshPtr pMesh = createOrRetrieve(filename,groupName,false,0,0,
+        MeshPtr pMesh = static_pointer_cast<Mesh>(createOrRetrieve(filename,groupName,false,0,0,
                                          vertexBufferUsage,indexBufferUsage,
-                                         vertexBufferShadowed,indexBufferShadowed).first.staticCast<Mesh>();
-		pMesh->load();
+                                         vertexBufferShadowed,indexBufferShadowed).first);
+        pMesh->load();
         return pMesh;
     }
     //-----------------------------------------------------------------------
@@ -133,26 +125,26 @@ namespace Ogre
                                     bool isManual, ManualResourceLoader* loader,
                                     const NameValuePairList* createParams)
     {
-        return createResource(name,group,isManual,loader,createParams).staticCast<Mesh>();
+        return static_pointer_cast<Mesh>(createResource(name,group,isManual,loader,createParams));
     }
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::createManual( const String& name, const String& groupName, 
         ManualResourceLoader* loader)
     {
-		// Don't try to get existing, create should fail if already exists
-		return create(name, groupName, true, loader);
+        // Don't try to get existing, create should fail if already exists
+        return create(name, groupName, true, loader);
     }
     //-----------------------------------------------------------------------
     MeshPtr MeshManager::createPlane( const String& name, const String& groupName,
         const Plane& plane, Real width, Real height, int xsegments, int ysegments,
         bool normals, unsigned short numTexCoordSets, Real xTile, Real yTile, const Vector3& upVector,
-		HardwareBuffer::Usage vertexBufferUsage, HardwareBuffer::Usage indexBufferUsage,
-		bool vertexShadowBuffer, bool indexShadowBuffer)
+        HardwareBuffer::Usage vertexBufferUsage, HardwareBuffer::Usage indexBufferUsage,
+        bool vertexShadowBuffer, bool indexShadowBuffer)
     {
         // Create manual mesh which calls back self to load
-        MeshPtr pMesh = createManual(name, groupName, this);
-		// Planes can never be manifold
-		pMesh->setAutoBuildEdgeLists(false);
+        MeshPtr pMesh = createManual(name, groupName, &mPrefabLoader);
+        // Planes can never be manifold
+        pMesh->setAutoBuildEdgeLists(false);
         // store parameters
         MeshBuildParams params;
         params.type = MBT_PLANE;
@@ -170,25 +162,25 @@ namespace Ogre
         params.indexBufferUsage = indexBufferUsage;
         params.vertexShadowBuffer = vertexShadowBuffer;
         params.indexShadowBuffer = indexShadowBuffer;
-        mMeshBuildParams[pMesh.getPointer()] = params;
+        mPrefabLoader.mMeshBuildParams[pMesh.get()] = params;
 
         // to preserve previous behaviour, load immediately
         pMesh->load();
 
         return pMesh;
     }
-	
-	//-----------------------------------------------------------------------
-	MeshPtr MeshManager::createCurvedPlane( const String& name, const String& groupName, 
+    
+    //-----------------------------------------------------------------------
+    MeshPtr MeshManager::createCurvedPlane( const String& name, const String& groupName, 
         const Plane& plane, Real width, Real height, Real bow, int xsegments, int ysegments,
         bool normals, unsigned short numTexCoordSets, Real xTile, Real yTile, const Vector3& upVector,
-			HardwareBuffer::Usage vertexBufferUsage, HardwareBuffer::Usage indexBufferUsage,
-			bool vertexShadowBuffer, bool indexShadowBuffer)
+            HardwareBuffer::Usage vertexBufferUsage, HardwareBuffer::Usage indexBufferUsage,
+            bool vertexShadowBuffer, bool indexShadowBuffer)
     {
         // Create manual mesh which calls back self to load
-        MeshPtr pMesh = createManual(name, groupName, this);
-		// Planes can never be manifold
-		pMesh->setAutoBuildEdgeLists(false);
+        MeshPtr pMesh = createManual(name, groupName, &mPrefabLoader);
+        // Planes can never be manifold
+        pMesh->setAutoBuildEdgeLists(false);
         // store parameters
         MeshBuildParams params;
         params.type = MBT_CURVED_PLANE;
@@ -207,7 +199,7 @@ namespace Ogre
         params.indexBufferUsage = indexBufferUsage;
         params.vertexShadowBuffer = vertexShadowBuffer;
         params.indexShadowBuffer = indexShadowBuffer;
-        mMeshBuildParams[pMesh.getPointer()] = params;
+        mPrefabLoader.mMeshBuildParams[pMesh.get()] = params;
 
         // to preserve previous behaviour, load immediately
         pMesh->load();
@@ -216,22 +208,22 @@ namespace Ogre
 
     }
     //-----------------------------------------------------------------------
-	MeshPtr MeshManager::createCurvedIllusionPlane(
+    MeshPtr MeshManager::createCurvedIllusionPlane(
         const String& name, const String& groupName, const Plane& plane,
         Real width, Real height, Real curvature,
         int xsegments, int ysegments,
         bool normals, unsigned short numTexCoordSets,
         Real uTile, Real vTile, const Vector3& upVector,
-		const Quaternion& orientation, 
+        const Quaternion& orientation, 
         HardwareBuffer::Usage vertexBufferUsage, 
-		HardwareBuffer::Usage indexBufferUsage,
-		bool vertexShadowBuffer, bool indexShadowBuffer,
+        HardwareBuffer::Usage indexBufferUsage,
+        bool vertexShadowBuffer, bool indexShadowBuffer,
         int ySegmentsToKeep)
-	{
+    {
         // Create manual mesh which calls back self to load
-        MeshPtr pMesh = createManual(name, groupName, this);
-		// Planes can never be manifold
-		pMesh->setAutoBuildEdgeLists(false);
+        MeshPtr pMesh = createManual(name, groupName, &mPrefabLoader);
+        // Planes can never be manifold
+        pMesh->setAutoBuildEdgeLists(false);
         // store parameters
         MeshBuildParams params;
         params.type = MBT_CURVED_ILLUSION_PLANE;
@@ -252,17 +244,17 @@ namespace Ogre
         params.vertexShadowBuffer = vertexShadowBuffer;
         params.indexShadowBuffer = indexShadowBuffer;
         params.ySegmentsToKeep = ySegmentsToKeep;
-        mMeshBuildParams[pMesh.getPointer()] = params;
+        mPrefabLoader.mMeshBuildParams[pMesh.get()] = params;
 
         // to preserve previous behaviour, load immediately
         pMesh->load();
 
         return pMesh;
-	}
+    }
 
     //-----------------------------------------------------------------------
-    void MeshManager::tesselate2DMesh(SubMesh* sm, unsigned short meshWidth, unsigned short meshHeight, 
-		bool doubleSided, HardwareBuffer::Usage indexBufferUsage, bool indexShadowBuffer)
+    void MeshManager::PrefabLoader::tesselate2DMesh(SubMesh* sm, unsigned short meshWidth, unsigned short meshHeight,
+        bool doubleSided, HardwareBuffer::Usage indexBufferUsage, bool indexShadowBuffer)
     {
         // The mesh is built, just make a list of indexes to spit out the triangles
         unsigned short vInc, v, iterations;
@@ -283,16 +275,16 @@ namespace Ogre
         // Allocate memory for faces
         // Num faces, width*height*2 (2 tris per square), index count is * 3 on top
         sm->indexData->indexCount = (meshWidth-1) * (meshHeight-1) * 2 * iterations * 3;
-		sm->indexData->indexBuffer = HardwareBufferManager::getSingleton().
-			createIndexBuffer(HardwareIndexBuffer::IT_16BIT,
-			sm->indexData->indexCount, indexBufferUsage, indexShadowBuffer);
+        sm->indexData->indexBuffer = HardwareBufferManager::getSingleton().
+            createIndexBuffer(HardwareIndexBuffer::IT_16BIT,
+            sm->indexData->indexCount, indexBufferUsage, indexShadowBuffer);
 
         unsigned short v1, v2, v3;
         //bool firstTri = true;
-		HardwareIndexBufferSharedPtr ibuf = sm->indexData->indexBuffer;
-		// Lock the whole buffer
-		unsigned short* pIndexes = static_cast<unsigned short*>(
-			ibuf->lock(HardwareBuffer::HBL_DISCARD) );
+        HardwareIndexBufferSharedPtr ibuf = sm->indexData->indexBuffer;
+        // Lock the whole buffer
+        HardwareBufferLockGuard ibufLock(ibuf, HardwareBuffer::HBL_DISCARD);
+        unsigned short* pIndexes = static_cast<unsigned short*>(ibufLock.pData);
 
         while (iterations--)
         {
@@ -340,105 +332,63 @@ namespace Ogre
             vInc = -vInc;
 
         }
-		// Unlock
-		ibuf->unlock();
+    }
+    //-------------------------------------------------------------------------
+    void MeshManager::setListener(Ogre::MeshSerializerListener *listener)
+    {
+        mListener = listener;
+    }
+    //-------------------------------------------------------------------------
+    MeshSerializerListener *MeshManager::getListener()
+    {
+        return mListener;
+    }
+    //-----------------------------------------------------------------------
+    void MeshManager::PrefabLoader::loadResource(Resource* res)
+    {
+        Mesh* msh = static_cast<Mesh*>(res);
 
+        // attempt to create a prefab mesh
+        bool createdPrefab = PrefabFactory::createPrefab(msh);
+
+        // the mesh was not a prefab..
+        if(!createdPrefab)
+        {
+            // Find build parameters
+            MeshBuildParamsMap::iterator ibld = mMeshBuildParams.find(res);
+            if (ibld == mMeshBuildParams.end())
+            {
+                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+                    "Cannot find build parameters for " + res->getName(),
+                    "MeshManager::loadResource");
+            }
+            MeshBuildParams& params = ibld->second;
+
+            switch(params.type)
+            {
+            case MBT_PLANE:
+                loadManualPlane(msh, params);
+                break;
+            case MBT_CURVED_ILLUSION_PLANE:
+                loadManualCurvedIllusionPlane(msh, params);
+                break;
+            case MBT_CURVED_PLANE:
+                loadManualCurvedPlane(msh, params);
+                break;
+            default:
+                OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
+                    "Unknown build parameters for " + res->getName(),
+                    "MeshManager::loadResource");
+            }
+        }
     }
 
     //-----------------------------------------------------------------------
-    void MeshManager::createPrefabPlane(void)
+    void MeshManager::PrefabLoader::loadManualPlane(Mesh* pMesh, MeshBuildParams& params)
     {
-        MeshPtr msh = create(
-            "Prefab_Plane", 
-            ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, 
-            true, // manually loaded
-            this);
-		// Planes can never be manifold
-		msh->setAutoBuildEdgeLists(false);
-        // to preserve previous behaviour, load immediately
-        msh->load();
-    }
-	//-----------------------------------------------------------------------
-	void MeshManager::createPrefabCube(void)
-	{
-		MeshPtr msh = create(
-			"Prefab_Cube", 
-			ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, 
-			true, // manually loaded
-			this);
-
-		// to preserve previous behaviour, load immediately
-		msh->load();
-	}
-	//-------------------------------------------------------------------------
-	void MeshManager::createPrefabSphere(void)
-	{
-		MeshPtr msh = create(
-			"Prefab_Sphere", 
-			ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME, 
-			true, // manually loaded
-			this);
-
-		// to preserve previous behaviour, load immediately
-		msh->load();
-	}
-	//-------------------------------------------------------------------------
-	void MeshManager::setListener(Ogre::MeshSerializerListener *listener)
-	{
-		mListener = listener;
-	}
-	//-------------------------------------------------------------------------
-	MeshSerializerListener *MeshManager::getListener()
-	{
-		return mListener;
-	}
-    //-----------------------------------------------------------------------
-	void MeshManager::loadResource(Resource* res)
-	{
-		Mesh* msh = static_cast<Mesh*>(res);
-
-		// attempt to create a prefab mesh
-		bool createdPrefab = PrefabFactory::createPrefab(msh);
-
-		// the mesh was not a prefab..
-		if(!createdPrefab)
-		{
-			// Find build parameters
-			MeshBuildParamsMap::iterator ibld = mMeshBuildParams.find(res);
-			if (ibld == mMeshBuildParams.end())
-			{
-				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
-					"Cannot find build parameters for " + res->getName(),
-					"MeshManager::loadResource");
-			}
-			MeshBuildParams& params = ibld->second;
-
-			switch(params.type)
-			{
-			case MBT_PLANE:
-				loadManualPlane(msh, params);
-				break;
-			case MBT_CURVED_ILLUSION_PLANE:
-				loadManualCurvedIllusionPlane(msh, params);
-				break;
-			case MBT_CURVED_PLANE:
-				loadManualCurvedPlane(msh, params);
-				break;
-			default:
-				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, 
-					"Unknown build parameters for " + res->getName(),
-					"MeshManager::loadResource");
-			}
-		}
-	}
-
-    //-----------------------------------------------------------------------
-    void MeshManager::loadManualPlane(Mesh* pMesh, MeshBuildParams& params)
-    {
-		if ((params.xsegments + 1) * (params.ysegments + 1) > 65536)
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
-				"Plane tesselation is too high, must generate max 65536 vertices", 
-				__FUNCTION__);
+        if ((params.xsegments + 1) * (params.ysegments + 1) > 65536)
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
+                "Plane tessellation is too high, must generate max 65536 vertices");
         SubMesh *pSub = pMesh->createSubMesh();
 
         // Set up vertex data
@@ -449,29 +399,26 @@ namespace Ogre
         VertexDeclaration* vertexDecl = vertexData->vertexDeclaration;
         size_t currOffset = 0;
         // We always need positions
-        vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_POSITION);
-        currOffset += VertexElement::getTypeSize(VET_FLOAT3);
+        currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_POSITION).getSize();
         // Optional normals
         if(params.normals)
         {
-            vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_NORMAL);
-            currOffset += VertexElement::getTypeSize(VET_FLOAT3);
+            currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_NORMAL).getSize();
         }
 
         for (unsigned short i = 0; i < params.numTexCoordSets; ++i)
         {
             // Assumes 2D texture coords
-            vertexDecl->addElement(0, currOffset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i);
-            currOffset += VertexElement::getTypeSize(VET_FLOAT2);
+            currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i).getSize();
         }
 
         vertexData->vertexCount = (params.xsegments + 1) * (params.ysegments + 1);
 
         // Allocate vertex buffer
         HardwareVertexBufferSharedPtr vbuf = 
-            HardwareBufferManager::getSingleton().
-            createVertexBuffer(vertexDecl->getVertexSize(0), vertexData->vertexCount,
-            params.vertexBufferUsage, params.vertexShadowBuffer);
+            pMesh->getHardwareBufferManager()->createVertexBuffer(
+                vertexDecl->getVertexSize(0), vertexData->vertexCount,
+                params.vertexBufferUsage, params.vertexShadowBuffer);
 
         // Set up the binding (one source only)
         VertexBufferBinding* binding = vertexData->vertexBufferBinding;
@@ -479,9 +426,9 @@ namespace Ogre
 
         // Work out the transform required
         // Default orientation of plane is normal along +z, distance 0
-        Matrix4 xlate, xform, rot;
+        Affine3 xlate, xform, rot;
         Matrix3 rot3;
-        xlate = rot = Matrix4::IDENTITY;
+        xlate = rot = Affine3::IDENTITY;
         // Determine axes
         Vector3 zAxis, yAxis, xAxis;
         zAxis = params.plane.normal;
@@ -489,12 +436,13 @@ namespace Ogre
         yAxis = params.upVector;
         yAxis.normalise();
         xAxis = yAxis.crossProduct(zAxis);
-        if (xAxis.length() == 0)
+        if (xAxis.squaredLength() == 0)
         {
             //upVector must be wrong
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "The upVector you supplied is parallel to the plane normal, so is not valid.",
-                "MeshManager::createPlane");
+                "MeshManager::loadManualPlane");
         }
+        xAxis.normalise();
 
         rot3.FromAxes(xAxis, yAxis, zAxis);
         rot = rot3;
@@ -507,8 +455,8 @@ namespace Ogre
 
         // Generate vertex data
         // Lock the whole buffer
-        float* pReal = static_cast<float*>(
-            vbuf->lock(HardwareBuffer::HBL_DISCARD) );
+        HardwareBufferLockGuard vbufLock(vbuf, HardwareBuffer::HBL_DISCARD);
+        float* pReal = static_cast<float*>(vbufLock.pData);
         Real xSpace = params.width / params.xsegments;
         Real ySpace = params.height / params.ysegments;
         Real halfWidth = params.width / 2;
@@ -529,7 +477,7 @@ namespace Ogre
                 vec.y = (y * ySpace) - halfHeight;
                 vec.z = 0.0f;
                 // Transform by orientation and distance
-                vec = xform.transformAffine(vec);
+                vec = xform * vec;
                 // Assign to geometry
                 *pReal++ = vec.x;
                 *pReal++ = vec.y;
@@ -555,7 +503,7 @@ namespace Ogre
                     // Default normal is along unit Z
                     vec = Vector3::UNIT_Z;
                     // Rotate
-                    vec = rot.transformAffine(vec);
+                    vec = rot * vec;
 
                     *pReal++ = vec.x;
                     *pReal++ = vec.y;
@@ -573,7 +521,7 @@ namespace Ogre
         } // y
 
         // Unlock
-        vbuf->unlock();
+        vbufLock.unlock();
         // Generate face list
         pSub->useSharedVertices = true;
         tesselate2DMesh(pSub, params.xsegments + 1, params.ysegments + 1, false, 
@@ -583,12 +531,11 @@ namespace Ogre
         pMesh->_setBoundingSphereRadius(Math::Sqrt(maxSquaredLength));
     }
     //-----------------------------------------------------------------------
-    void MeshManager::loadManualCurvedPlane(Mesh* pMesh, MeshBuildParams& params)
+    void MeshManager::PrefabLoader::loadManualCurvedPlane(Mesh* pMesh, MeshBuildParams& params)
     {
-		if ((params.xsegments + 1) * (params.ysegments + 1) > 65536)
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
-				"Plane tesselation is too high, must generate max 65536 vertices", 
-				__FUNCTION__);
+        if ((params.xsegments + 1) * (params.ysegments + 1) > 65536)
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
+                "Plane tessellation is too high, must generate max 65536 vertices");
         SubMesh *pSub = pMesh->createSubMesh();
 
         // Set options
@@ -600,35 +547,30 @@ namespace Ogre
         pMesh->sharedVertexData->vertexCount = (params.xsegments + 1) * (params.ysegments + 1);
 
         size_t offset = 0;
-        decl->addElement(0, offset, VET_FLOAT3, VES_POSITION);
-        offset += VertexElement::getTypeSize(VET_FLOAT3);
+        offset += decl->addElement(0, offset, VET_FLOAT3, VES_POSITION).getSize();
         if (params.normals)
         {
-            decl->addElement(0, 0, VET_FLOAT3, VES_NORMAL);
-            offset += VertexElement::getTypeSize(VET_FLOAT3);
+            offset += decl->addElement(0, 0, VET_FLOAT3, VES_NORMAL).getSize();
         }
 
         for (unsigned short i = 0; i < params.numTexCoordSets; ++i)
         {
-            decl->addElement(0, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i);
-            offset += VertexElement::getTypeSize(VET_FLOAT2);
+            offset += decl->addElement(0, offset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i).getSize();
         }
 
 
         // Allocate memory
         HardwareVertexBufferSharedPtr vbuf = 
-            HardwareBufferManager::getSingleton().createVertexBuffer(
-            offset, 
-            pMesh->sharedVertexData->vertexCount, 
-            params.vertexBufferUsage, 
-            params.vertexShadowBuffer);
+            pMesh->getHardwareBufferManager()->createVertexBuffer(
+                offset, pMesh->sharedVertexData->vertexCount, 
+                params.vertexBufferUsage, params.vertexShadowBuffer);
         bind->setBinding(0, vbuf);
 
         // Work out the transform required
         // Default orientation of plane is normal along +z, distance 0
-        Matrix4 xlate, xform, rot;
+        Affine3 xlate, xform, rot;
         Matrix3 rot3;
-        xlate = rot = Matrix4::IDENTITY;
+        xlate = rot = Affine3::IDENTITY;
         // Determine axes
         Vector3 zAxis, yAxis, xAxis;
         zAxis = params.plane.normal;
@@ -636,12 +578,13 @@ namespace Ogre
         yAxis = params.upVector;
         yAxis.normalise();
         xAxis = yAxis.crossProduct(zAxis);
-        if (xAxis.length() == 0)
+        if (xAxis.squaredLength() == 0)
         {
             //upVector must be wrong
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "The upVector you supplied is parallel to the plane normal, so is not valid.",
                 "MeshManager::createPlane");
         }
+        xAxis.normalise();
 
         rot3.FromAxes(xAxis, yAxis, zAxis);
         rot = rot3;
@@ -653,8 +596,8 @@ namespace Ogre
         xform = xlate * rot;
 
         // Generate vertex data
-        float* pFloat = static_cast<float*>(
-            vbuf->lock(HardwareBuffer::HBL_DISCARD)); 
+        HardwareBufferLockGuard vbufLock(vbuf, HardwareBuffer::HBL_DISCARD);
+        float* pFloat = static_cast<float*>(vbufLock.pData);
         Real xSpace = params.width / params.xsegments;
         Real ySpace = params.height / params.ysegments;
         Real halfWidth = params.width / 2;
@@ -680,11 +623,11 @@ namespace Ogre
                 // Here's where curved plane is different from standard plane.  Amazing, I know.
                 diff_x = (x - ((params.xsegments) / 2)) / static_cast<Real>((params.xsegments));
                 diff_y = (y - ((params.ysegments) / 2)) / static_cast<Real>((params.ysegments));
-                dist = sqrt(diff_x*diff_x + diff_y * diff_y );
-				vec.z = (-sin((1-dist) * (Math::PI/2)) * params.curvature) + params.curvature;
+                dist = std::sqrt(diff_x*diff_x + diff_y * diff_y );
+                vec.z = (-std::sin((1-dist) * (Math::PI/2)) * params.curvature) + params.curvature;
 
                 // Transform by orientation and distance
-                Vector3 pos = xform.transformAffine(vec);
+                Vector3 pos = xform * vec;
                 // Assign to geometry
                 *pFloat++ = pos.x;
                 *pFloat++ = pos.y;
@@ -713,8 +656,8 @@ namespace Ogre
                     // Default normal is along unit Z
                     //vec = Vector3::UNIT_Z;
                     // Rotate
-                    vec = rot.transformAffine(vec);
-					vec.normalise();
+                    vec = rot * vec;
+                    vec.normalise();
 
                     *pFloat++ = vec.x;
                     *pFloat++ = vec.y;
@@ -729,7 +672,7 @@ namespace Ogre
 
             } // x
         } // y
-        vbuf->unlock();
+        vbufLock.unlock();
 
         // Generate face list
         tesselate2DMesh(pSub, params.xsegments + 1, params.ysegments + 1, 
@@ -740,14 +683,13 @@ namespace Ogre
 
     }
     //-----------------------------------------------------------------------
-    void MeshManager::loadManualCurvedIllusionPlane(Mesh* pMesh, MeshBuildParams& params)
+    void MeshManager::PrefabLoader::loadManualCurvedIllusionPlane(Mesh* pMesh, MeshBuildParams& params)
     {
-		if (params.ySegmentsToKeep == -1) params.ySegmentsToKeep = params.ysegments;
+        if (params.ySegmentsToKeep == -1) params.ySegmentsToKeep = params.ysegments;
 
-		if ((params.xsegments + 1) * (params.ySegmentsToKeep + 1) > 65536)
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
-				"Plane tesselation is too high, must generate max 65536 vertices", 
-				__FUNCTION__);
+        if ((params.xsegments + 1) * (params.ySegmentsToKeep + 1) > 65536)
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
+                "Plane tessellation is too high, must generate max 65536 vertices");
         SubMesh *pSub = pMesh->createSubMesh();
 
 
@@ -759,29 +701,26 @@ namespace Ogre
         VertexDeclaration* vertexDecl = vertexData->vertexDeclaration;
         size_t currOffset = 0;
         // We always need positions
-        vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_POSITION);
-        currOffset += VertexElement::getTypeSize(VET_FLOAT3);
+        currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_POSITION).getSize();
         // Optional normals
         if(params.normals)
         {
-            vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_NORMAL);
-            currOffset += VertexElement::getTypeSize(VET_FLOAT3);
+            currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT3, VES_NORMAL).getSize();
         }
 
         for (unsigned short i = 0; i < params.numTexCoordSets; ++i)
         {
             // Assumes 2D texture coords
-            vertexDecl->addElement(0, currOffset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i);
-            currOffset += VertexElement::getTypeSize(VET_FLOAT2);
+            currOffset += vertexDecl->addElement(0, currOffset, VET_FLOAT2, VES_TEXTURE_COORDINATES, i).getSize();
         }
 
         vertexData->vertexCount = (params.xsegments + 1) * (params.ySegmentsToKeep + 1);
 
         // Allocate vertex buffer
         HardwareVertexBufferSharedPtr vbuf = 
-            HardwareBufferManager::getSingleton().
-            createVertexBuffer(vertexDecl->getVertexSize(0), vertexData->vertexCount,
-            params.vertexBufferUsage, params.vertexShadowBuffer);
+            pMesh->getHardwareBufferManager()->createVertexBuffer(
+                vertexDecl->getVertexSize(0), vertexData->vertexCount,
+                params.vertexBufferUsage, params.vertexShadowBuffer);
 
         // Set up the binding (one source only)
         VertexBufferBinding* binding = vertexData->vertexBufferBinding;
@@ -789,9 +728,9 @@ namespace Ogre
 
         // Work out the transform required
         // Default orientation of plane is normal along +z, distance 0
-        Matrix4 xlate, xform, rot;
+        Affine3 xlate, xform, rot;
         Matrix3 rot3;
-        xlate = rot = Matrix4::IDENTITY;
+        xlate = rot = Affine3::IDENTITY;
         // Determine axes
         Vector3 zAxis, yAxis, xAxis;
         zAxis = params.plane.normal;
@@ -799,12 +738,13 @@ namespace Ogre
         yAxis = params.upVector;
         yAxis.normalise();
         xAxis = yAxis.crossProduct(zAxis);
-        if (xAxis.length() == 0)
+        if (xAxis.squaredLength() == 0)
         {
             //upVector must be wrong
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "The upVector you supplied is parallel to the plane normal, so is not valid.",
                 "MeshManager::createPlane");
         }
+        xAxis.normalise();
 
         rot3.FromAxes(xAxis, yAxis, zAxis);
         rot = rot3;
@@ -835,8 +775,8 @@ namespace Ogre
         camPos = sphereRadius - CAM_DIST;
 
         // Lock the whole buffer
-        float* pFloat = static_cast<float*>(
-            vbuf->lock(HardwareBuffer::HBL_DISCARD) );
+        HardwareBufferLockGuard vbufLock(vbuf, HardwareBuffer::HBL_DISCARD);
+        float* pFloat = static_cast<float*>(vbufLock.pData);
         Real xSpace = params.width / params.xsegments;
         Real ySpace = params.height / params.ysegments;
         Real halfWidth = params.width / 2;
@@ -855,7 +795,7 @@ namespace Ogre
                 vec.y = (y * ySpace) - halfHeight;
                 vec.z = 0.0f;
                 // Transform by orientation and distance
-                vec = xform.transformAffine(vec);
+                vec = xform * vec;
                 // Assign to geometry
                 *pFloat++ = vec.x;
                 *pFloat++ = vec.y;
@@ -913,7 +853,7 @@ namespace Ogre
         } // y
 
         // Unlock
-        vbuf->unlock();
+        vbufLock.unlock();
         // Generate face list
         pSub->useSharedVertices = true;
         tesselate2DMesh(pSub, params.xsegments + 1, params.ySegmentsToKeep + 1, false, 
@@ -923,7 +863,7 @@ namespace Ogre
         pMesh->_setBoundingSphereRadius(Math::Sqrt(maxSquaredLength));
     }
     //-----------------------------------------------------------------------
-    PatchMeshPtr MeshManager::createBezierPatch(const String& name, const String& groupName, 
+    PatchMeshPtr MeshManager::createBezierPatch(const String& name, const String& groupName,
             void* controlPointBuffer, VertexDeclaration *declaration, 
             size_t width, size_t height,
             size_t uMaxSubdivisionLevel, size_t vMaxSubdivisionLevel,
@@ -938,8 +878,8 @@ namespace Ogre
                 "MeshManager::createBezierPatch");
         }
 
-        MeshPtr pMesh = getByName(name);
-        if (!pMesh.isNull())
+        MeshPtr pMesh = getByName(name, groupName);
+        if (pMesh)
         {
             OGRE_EXCEPT(Exception::ERR_DUPLICATE_ITEM, "A mesh called " + name + 
                 " already exists!", "MeshManager::createBezierPatch");
@@ -952,7 +892,7 @@ namespace Ogre
         ResourcePtr res(pm);
         addImpl(res);
 
-        return res.staticCast<PatchMesh>();
+        return static_pointer_cast<PatchMesh>(res);
     }
     //-----------------------------------------------------------------------
     void MeshManager::setPrepareAllMeshesForShadowVolumes(bool enable)
@@ -965,7 +905,27 @@ namespace Ogre
         return mPrepAllMeshesForShadowVolumes;
     }
     //-----------------------------------------------------------------------
-    Real MeshManager::getBoundsPaddingFactor(void)
+    VertexElementType MeshManager::getBlendWeightsBaseElementType() const
+    {
+        return mBlendWeightsBaseElementType;
+    }
+    //-----------------------------------------------------------------------
+    void MeshManager::setBlendWeightsBaseElementType( VertexElementType vet )
+    {
+        switch ( vet )
+        {
+            case VET_UBYTE4_NORM:
+            case VET_USHORT2_NORM:
+            case VET_FLOAT1:
+                mBlendWeightsBaseElementType = vet;
+                break;
+            default:
+                OgreAssert(false, "Unsupported BlendWeightsBaseElementType");
+                break;
+        }
+    }
+    //-----------------------------------------------------------------------
+    Real MeshManager::getBoundsPaddingFactor( void )
     {
         return mBoundsPaddingFactor;
     }
