@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,75 +27,45 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 
-#include "OgreMovableObject.h"
-#include "OgreSceneNode.h"
 #include "OgreTagPoint.h"
 #include "OgreLight.h"
 #include "OgreEntity.h"
-#include "OgreRoot.h"
-#include "OgreSceneManager.h"
-#include "OgreCamera.h"
 #include "OgreLodListener.h"
 
 namespace Ogre {
-	//-----------------------------------------------------------------------
-	//-----------------------------------------------------------------------
-	uint32 MovableObject::msDefaultQueryFlags = 0xFFFFFFFF;
-	uint32 MovableObject::msDefaultVisibilityFlags = 0xFFFFFFFF;
     //-----------------------------------------------------------------------
-    MovableObject::MovableObject()
-        : mCreator(0)
-        , mManager(0)
-        , mParentNode(0)
-        , mParentIsTagPoint(false)
-        , mVisible(true)
-		, mDebugDisplay(false)
-        , mUpperDistance(0)
-        , mSquaredUpperDistance(0)
-		, mMinPixelSize(0)
-        , mBeyondFarDistance(false)
-        , mRenderQueueID(RENDER_QUEUE_MAIN)
-        , mRenderQueueIDSet(false)
-		, mRenderQueuePriority(100)
-		, mRenderQueuePrioritySet(false)
-		, mQueryFlags(msDefaultQueryFlags)
-        , mVisibilityFlags(msDefaultVisibilityFlags)
-        , mCastShadows(true)
-        , mRenderingDisabled(false)
-        , mListener(0)
-        , mLightListUpdated(0)
-		, mLightMask(0xFFFFFFFF)
-    {
-		if (Root::getSingletonPtr())
-			mMinPixelSize = Root::getSingleton().getDefaultMinPixelSize();
-    }
+    //-----------------------------------------------------------------------
+    uint32 MovableObject::msDefaultQueryFlags = 0xFFFFFFFF;
+    uint32 MovableObject::msDefaultVisibilityFlags = 0xFFFFFFFF;
+    //-----------------------------------------------------------------------
+    MovableObject::MovableObject() : MovableObject(BLANKSTRING) {}
     //-----------------------------------------------------------------------
     MovableObject::MovableObject(const String& name)
         : mName(name)
         , mCreator(0)
         , mManager(0)
         , mParentNode(0)
+        , mListener(0)
         , mParentIsTagPoint(false)
         , mVisible(true)
-		, mDebugDisplay(false)
+        , mDebugDisplay(false)
+        , mBeyondFarDistance(false)
+        , mCastShadows(true)
+        , mRenderQueueIDSet(false)
+        , mRenderQueuePrioritySet(false)
+        , mRenderingDisabled(false)
+        , mRenderQueueID(RENDER_QUEUE_MAIN)
+        , mRenderQueuePriority(100)
         , mUpperDistance(0)
         , mSquaredUpperDistance(0)
-		, mMinPixelSize(0)
-        , mBeyondFarDistance(false)
-        , mRenderQueueID(RENDER_QUEUE_MAIN)
-        , mRenderQueueIDSet(false)
-		, mRenderQueuePriority(100)
-		, mRenderQueuePrioritySet(false)
-		, mQueryFlags(msDefaultQueryFlags)
+        , mMinPixelSize(0)
+        , mQueryFlags(msDefaultQueryFlags)
         , mVisibilityFlags(msDefaultVisibilityFlags)
-        , mCastShadows(true)
-        , mRenderingDisabled(false)
-        , mListener(0)
         , mLightListUpdated(0)
-		, mLightMask(0xFFFFFFFF)
+        , mLightMask(0xFFFFFFFF)
     {
-		if (Root::getSingletonPtr())
-			mMinPixelSize = Root::getSingleton().getDefaultMinPixelSize();
+        if (Root::getSingletonPtr())
+            mMinPixelSize = Root::getSingleton().getDefaultMinPixelSize();
     }
     //-----------------------------------------------------------------------
     MovableObject::~MovableObject()
@@ -147,11 +117,6 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    Node* MovableObject::getParentNode(void) const
-    {
-        return mParentNode;
-    }
-    //-----------------------------------------------------------------------
     SceneNode* MovableObject::getParentSceneNode(void) const
     {
         if (mParentIsTagPoint)
@@ -164,50 +129,44 @@ namespace Ogre {
             return static_cast<SceneNode*>(mParentNode);
         }
     }
-    //-----------------------------------------------------------------------
-    bool MovableObject::isAttached(void) const
+    //---------------------------------------------------------------------
+    void MovableObject::detachFromParent(void)
     {
-        return (mParentNode != 0);
-
+        if (isAttached())
+        {
+            if (mParentIsTagPoint)
+            {
+                TagPoint* tp = static_cast<TagPoint*>(mParentNode);
+                tp->getParentEntity()->detachObjectFromBone(this);
+            }
+            else
+            {
+                SceneNode* sn = static_cast<SceneNode*>(mParentNode);
+                sn->detachObject(this);
+            }
+        }
     }
-	//---------------------------------------------------------------------
-	void MovableObject::detachFromParent(void)
-	{
-		if (isAttached())
-		{
-			if (mParentIsTagPoint)
-			{
-				TagPoint* tp = static_cast<TagPoint*>(mParentNode);
-				tp->getParentEntity()->detachObjectFromBone(this);
-			}
-			else
-			{
-				SceneNode* sn = static_cast<SceneNode*>(mParentNode);
-				sn->detachObject(this);
-			}
-		}
-	}
     //-----------------------------------------------------------------------
-	bool MovableObject::isInScene(void) const
-	{
-		if (mParentNode != 0)
-		{
-			if (mParentIsTagPoint)
-			{
-				TagPoint* tp = static_cast<TagPoint*>(mParentNode);
-				return tp->getParentEntity()->isInScene();
-			}
-			else
-			{
-				SceneNode* sn = static_cast<SceneNode*>(mParentNode);
-				return sn->isInSceneGraph();
-			}
-		}
-		else
-		{
-			return false;
-		}
-	}
+    bool MovableObject::isInScene(void) const
+    {
+        if (mParentNode != 0)
+        {
+            if (mParentIsTagPoint)
+            {
+                TagPoint* tp = static_cast<TagPoint*>(mParentNode);
+                return tp->getParentEntity()->isInScene();
+            }
+            else
+            {
+                SceneNode* sn = static_cast<SceneNode*>(mParentNode);
+                return sn->isInSceneGraph();
+            }
+        }
+        else
+        {
+            return false;
+        }
+    }
     //-----------------------------------------------------------------------
     void MovableObject::_notifyMoved(void)
     {
@@ -222,16 +181,6 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void MovableObject::setVisible(bool visible)
-    {
-        mVisible = visible;
-    }
-    //-----------------------------------------------------------------------
-    bool MovableObject::getVisible(void) const
-    {
-        return mVisible;
-    }
-    //-----------------------------------------------------------------------
     bool MovableObject::isVisible(void) const
     {
         if (!mVisible || mBeyondFarDistance || mRenderingDisabled)
@@ -243,65 +192,65 @@ namespace Ogre {
 
         return true;
     }
-	//-----------------------------------------------------------------------
-	void MovableObject::_notifyCurrentCamera(Camera* cam)
-	{
-		if (mParentNode)
-		{
-			mBeyondFarDistance = false;
+    //-----------------------------------------------------------------------
+    void MovableObject::_notifyCurrentCamera(Camera* cam)
+    {
+        if (mParentNode)
+        {
+            mBeyondFarDistance = false;
 
-			if (cam->getUseRenderingDistance() && mUpperDistance > 0)
-			{
-				Real rad = getBoundingRadius();
-				Real squaredDepth = mParentNode->getSquaredViewDepth(cam->getLodCamera());
+            if (cam->getUseRenderingDistance() && mUpperDistance > 0)
+            {
+                Real rad = getBoundingRadius();
+                Real squaredDepth = mParentNode->getSquaredViewDepth(cam->getLodCamera());
 
-				const Vector3& scl = mParentNode->_getDerivedScale();
-				Real factor = std::max(std::max(scl.x, scl.y), scl.z);
+                const Vector3& scl = mParentNode->_getDerivedScale();
+                Real factor = std::max(std::max(scl.x, scl.y), scl.z);
 
-				// Max distance to still render
-				Real maxDist = mUpperDistance + rad * factor;
-				if (squaredDepth > Math::Sqr(maxDist))
-				{
-					mBeyondFarDistance = true;
-				}
-			}
+                // Max distance to still render
+                Real maxDist = mUpperDistance + rad * factor;
+                if (squaredDepth > Math::Sqr(maxDist))
+                {
+                    mBeyondFarDistance = true;
+                }
+            }
 
-			if (!mBeyondFarDistance && cam->getUseMinPixelSize() && mMinPixelSize > 0)
-			{
+            if (!mBeyondFarDistance && cam->getUseMinPixelSize() && mMinPixelSize > 0)
+            {
 
-				Real pixelRatio = cam->getPixelDisplayRatio();
-				
-				//if ratio is relative to distance than the distance at which the object should be displayed
-				//is the size of the radius divided by the ratio
-				//get the size of the entity in the world
-				Ogre::Vector3 objBound = getBoundingBox().getSize() * 
-							getParentNode()->_getDerivedScale();
-				
-				//We object are projected from 3 dimensions to 2. The shortest displayed dimension of 
-				//as object will always be at most the second largest dimension of the 3 dimensional
-				//bounding box.
-				//The square calculation come both to get rid of minus sign and for improve speed
-				//in the final calculation
-				objBound.x = Math::Sqr(objBound.x);
-				objBound.y = Math::Sqr(objBound.y);
-				objBound.z = Math::Sqr(objBound.z);
-				float sqrObjMedianSize = std::max(std::max(
-									std::min(objBound.x,objBound.y),
-									std::min(objBound.x,objBound.z)),
-									std::min(objBound.y,objBound.z));
+                Real pixelRatio = cam->getPixelDisplayRatio();
+                
+                //if ratio is relative to distance than the distance at which the object should be displayed
+                //is the size of the radius divided by the ratio
+                //get the size of the entity in the world
+                Ogre::Vector3 objBound = getBoundingBox().getSize() * 
+                            getParentNode()->_getDerivedScale();
+                
+                //We object are projected from 3 dimensions to 2. The shortest displayed dimension of 
+                //as object will always be at most the second largest dimension of the 3 dimensional
+                //bounding box.
+                //The square calculation come both to get rid of minus sign and for improve speed
+                //in the final calculation
+                objBound.x = Math::Sqr(objBound.x);
+                objBound.y = Math::Sqr(objBound.y);
+                objBound.z = Math::Sqr(objBound.z);
+                float sqrObjMedianSize = std::max(std::max(
+                                    std::min(objBound.x,objBound.y),
+                                    std::min(objBound.x,objBound.z)),
+                                    std::min(objBound.y,objBound.z));
 
-				//If we have a perspective camera calculations are done relative to distance
-				Real sqrDistance = 1;
-				if (cam->getProjectionType() == PT_PERSPECTIVE)
-				{
-					sqrDistance = mParentNode->getSquaredViewDepth(cam->getLodCamera());
-				}
+                //If we have a perspective camera calculations are done relative to distance
+                Real sqrDistance = 1;
+                if (cam->getProjectionType() == PT_PERSPECTIVE)
+                {
+                    sqrDistance = mParentNode->getSquaredViewDepth(cam->getLodCamera());
+                }
 
-				//Final Calculation to tell whether the object is to small
-				mBeyondFarDistance =  sqrObjMedianSize < 
-							sqrDistance * Math::Sqr(pixelRatio * mMinPixelSize); 
-			}
-			
+                //Final Calculation to tell whether the object is to small
+                mBeyondFarDistance =  sqrObjMedianSize < 
+                            sqrDistance * Math::Sqr(pixelRatio * mMinPixelSize); 
+            }
+            
             // Construct event object
             MovableObjectLodChangedEvent evt;
             evt.movableObject = this;
@@ -310,68 +259,63 @@ namespace Ogre {
             // Notify LOD event listeners
             cam->getSceneManager()->_notifyMovableObjectLodChanged(evt);
 
-		}
+        }
 
         mRenderingDisabled = mListener && !mListener->objectRendering(this, cam);
-	}
+    }
     //-----------------------------------------------------------------------
     void MovableObject::setRenderQueueGroup(uint8 queueID)
     {
-		assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
+        assert(queueID <= RENDER_QUEUE_MAX && "Render queue out of range!");
         mRenderQueueID = queueID;
         mRenderQueueIDSet = true;
     }
 
-	//-----------------------------------------------------------------------
-	void MovableObject::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
-	{
-		setRenderQueueGroup(queueID);
-		mRenderQueuePriority = priority;
-		mRenderQueuePrioritySet = true;
-
-	}
-
     //-----------------------------------------------------------------------
-    uint8 MovableObject::getRenderQueueGroup(void) const
+    void MovableObject::setRenderQueueGroupAndPriority(uint8 queueID, ushort priority)
     {
-        return mRenderQueueID;
+        setRenderQueueGroup(queueID);
+        mRenderQueuePriority = priority;
+        mRenderQueuePrioritySet = true;
+
     }
+
     //-----------------------------------------------------------------------
-	const Matrix4& MovableObject::_getParentNodeFullTransform(void) const
-	{
-		
-		if(mParentNode)
-		{
-			// object attached to a sceneNode
-			return mParentNode->_getFullTransform();
-		}
+    const Affine3& MovableObject::_getParentNodeFullTransform(void) const
+    {
+        
+        if(mParentNode)
+        {
+            // object attached to a sceneNode
+            return mParentNode->_getFullTransform();
+        }
         // fallback
-        return Matrix4::IDENTITY;
-	}
+        return Affine3::IDENTITY;
+    }
     //-----------------------------------------------------------------------
     const AxisAlignedBox& MovableObject::getWorldBoundingBox(bool derive) const
     {
         if (derive)
         {
             mWorldAABB = this->getBoundingBox();
-            mWorldAABB.transformAffine(_getParentNodeFullTransform());
+            mWorldAABB.transform(_getParentNodeFullTransform());
         }
 
         return mWorldAABB;
 
     }
     //-----------------------------------------------------------------------
-	const Sphere& MovableObject::getWorldBoundingSphere(bool derive) const
-	{
-		if (derive)
-		{
-			const Vector3& scl = mParentNode->_getDerivedScale();
-			Real factor = std::max(std::max(scl.x, scl.y), scl.z);
-			mWorldBoundingSphere.setRadius(getBoundingRadius() * factor);
-			mWorldBoundingSphere.setCenter(mParentNode->_getDerivedPosition());
-		}
-		return mWorldBoundingSphere;
-	}
+    const Sphere& MovableObject::getWorldBoundingSphere(bool derive) const
+    {
+        if (derive)
+        {
+            const Vector3& scl = mParentNode->_getDerivedScale();
+            Real factor = std::max(std::max(scl.x, scl.y), scl.z);
+            mWorldBoundingSphere.setRadius(getBoundingRadius() * factor);
+            mWorldBoundingSphere.setCenter(mParentNode->_getDerivedPosition());
+        }
+        return mWorldBoundingSphere;
+    }
     //-----------------------------------------------------------------------
     const LightList& MovableObject::queryLights(void) const
     {
@@ -403,8 +347,8 @@ namespace Ogre {
             {
                 mLightListUpdated = frame;
 
-				const Vector3& scl = mParentNode->_getDerivedScale();
-				Real factor = std::max(std::max(scl.x, scl.y), scl.z);
+                const Vector3& scl = mParentNode->_getDerivedScale();
+                Real factor = std::max(std::max(scl.x, scl.y), scl.z);
 
                 sn->findLights(mLightList, this->getBoundingRadius() * factor, this->getLightMask());
             }
@@ -446,69 +390,83 @@ namespace Ogre {
     {
         if (mParentNode)
         {
-            return getExtrusionDistance(mParentNode->_getDerivedPosition(), l);
+            // exclude distance from the light to the shadow caster from the extrusion
+            Real extrusionDistance = l->getAttenuationRange() - getWorldBoundingBox().distance(l->getDerivedPosition());
+            if(extrusionDistance < 0)
+                extrusionDistance = 0;
+            
+            // Take into account that extrusion would be done in object-space,
+            // and non-uniformly scaled objects would cast non-uniformly scaled shadows.
+            Matrix3 m3 = _getParentNodeFullTransform().linear();
+            Real c0 = m3.GetColumn(0).squaredLength(), c1 = m3.GetColumn(1).squaredLength(), c2 = m3.GetColumn(2).squaredLength();
+            Real minScale = Math::Sqrt(std::min(std::min(c0, c1), c2));
+            Real maxScale = Math::Sqrt(std::max(std::max(c0, c1), c2));
+            if(minScale > 0.0)
+                extrusionDistance *= (maxScale / minScale);
+            
+            return extrusionDistance;
         }
         else
         {
             return 0;
         }
     }
-	//-----------------------------------------------------------------------
-	uint32 MovableObject::getTypeFlags(void) const
-	{
-		if (mCreator)
-		{
-			return mCreator->getTypeFlags();
-		}
-		else
-		{
-			return 0xFFFFFFFF;
-		}
-	}
-	//---------------------------------------------------------------------
-	void MovableObject::setLightMask(uint32 lightMask)
-	{
-		this->mLightMask = lightMask;
-		//make sure to request a new light list from the scene manager if mask changed
-		mLightListUpdated = 0;
-	}
-	//---------------------------------------------------------------------
-	class MORecvShadVisitor : public Renderable::Visitor
-	{
-	public:
-		bool anyReceiveShadows;
-		MORecvShadVisitor() : anyReceiveShadows(false)
-		{
+    //-----------------------------------------------------------------------
+    uint32 MovableObject::getTypeFlags(void) const
+    {
+        if (mCreator)
+        {
+            return mCreator->getTypeFlags();
+        }
+        else
+        {
+            return 0xFFFFFFFF;
+        }
+    }
+    //---------------------------------------------------------------------
+    void MovableObject::setLightMask(uint32 lightMask)
+    {
+        this->mLightMask = lightMask;
+        //make sure to request a new light list from the scene manager if mask changed
+        mLightListUpdated = 0;
+    }
+    //---------------------------------------------------------------------
+    class MORecvShadVisitor : public Renderable::Visitor
+    {
+    public:
+        bool anyReceiveShadows;
+        MORecvShadVisitor() : anyReceiveShadows(false)
+        {
 
-		}
-		void visit(Renderable* rend, ushort lodIndex, bool isDebug, 
-			Any* pAny = 0)
-		{
-			Technique* tech = rend->getTechnique();
-			bool techReceivesShadows = tech && tech->getParent()->getReceiveShadows();
-			anyReceiveShadows = anyReceiveShadows || 
-				techReceivesShadows || !tech;
-		}
-	};
-	//---------------------------------------------------------------------
-	bool MovableObject::getReceivesShadows()
-	{
-		MORecvShadVisitor visitor;
-		visitRenderables(&visitor);
-		return visitor.anyReceiveShadows;		
+        }
+        void visit(Renderable* rend, ushort lodIndex, bool isDebug, 
+            Any* pAny = 0)
+        {
+            Technique* tech = rend->getTechnique();
+            bool techReceivesShadows = tech && tech->getParent()->getReceiveShadows();
+            anyReceiveShadows = anyReceiveShadows || 
+                techReceivesShadows || !tech;
+        }
+    };
+    //---------------------------------------------------------------------
+    bool MovableObject::getReceivesShadows()
+    {
+        MORecvShadVisitor visitor;
+        visitRenderables(&visitor);
+        return visitor.anyReceiveShadows;       
 
-	}
-	//-----------------------------------------------------------------------
-	//-----------------------------------------------------------------------
-	MovableObject* MovableObjectFactory::createInstance(
-		const String& name, SceneManager* manager, 
-		const NameValuePairList* params)
-	{
-		MovableObject* m = createInstanceImpl(name, params);
-		m->_notifyCreator(this);
-		m->_notifyManager(manager);
-		return m;
-	}
+    }
+    //-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
+    MovableObject* MovableObjectFactory::createInstance(
+        const String& name, SceneManager* manager, 
+        const NameValuePairList* params)
+    {
+        MovableObject* m = createInstanceImpl(name, params);
+        m->_notifyCreator(this);
+        m->_notifyManager(manager);
+        return m;
+    }
 
 
 }
