@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,40 +29,41 @@ THE SOFTWARE.
 
 #include "OgreParticleEmitter.h"
 #include "OgreParticleEmitterFactory.h"
+#include "OgreParticleEmitterCommands.h"
 
 namespace Ogre
 {
-	// Define static members
-    EmitterCommands::CmdAngle ParticleEmitter::msAngleCmd;
-    EmitterCommands::CmdColour ParticleEmitter::msColourCmd;
-    EmitterCommands::CmdColourRangeStart ParticleEmitter::msColourRangeStartCmd;
-    EmitterCommands::CmdColourRangeEnd ParticleEmitter::msColourRangeEndCmd;
-    EmitterCommands::CmdDirection ParticleEmitter::msDirectionCmd;
-    EmitterCommands::CmdUp ParticleEmitter::msUpCmd;
-	EmitterCommands::CmdDirPositionRef ParticleEmitter::msDirPositionRefCmd;
-    EmitterCommands::CmdEmissionRate ParticleEmitter::msEmissionRateCmd;
-    EmitterCommands::CmdMaxTTL ParticleEmitter::msMaxTTLCmd;
-    EmitterCommands::CmdMaxVelocity ParticleEmitter::msMaxVelocityCmd;
-    EmitterCommands::CmdMinTTL ParticleEmitter::msMinTTLCmd;
-    EmitterCommands::CmdMinVelocity ParticleEmitter::msMinVelocityCmd;
-    EmitterCommands::CmdPosition ParticleEmitter::msPositionCmd;
-    EmitterCommands::CmdTTL ParticleEmitter::msTTLCmd;
-    EmitterCommands::CmdVelocity ParticleEmitter::msVelocityCmd;
-    EmitterCommands::CmdDuration ParticleEmitter::msDurationCmd;
-    EmitterCommands::CmdMinDuration ParticleEmitter::msMinDurationCmd;
-    EmitterCommands::CmdMaxDuration ParticleEmitter::msMaxDurationCmd;
-    EmitterCommands::CmdRepeatDelay ParticleEmitter::msRepeatDelayCmd;
-    EmitterCommands::CmdMinRepeatDelay ParticleEmitter::msMinRepeatDelayCmd;
-    EmitterCommands::CmdMaxRepeatDelay ParticleEmitter::msMaxRepeatDelayCmd;
-    EmitterCommands::CmdName ParticleEmitter::msNameCmd;
-    EmitterCommands::CmdEmittedEmitter ParticleEmitter::msEmittedEmitterCmd;
+    // Command object for setting / getting parameters
+    static EmitterCommands::CmdAngle msAngleCmd;
+    static EmitterCommands::CmdColour msColourCmd;
+    static EmitterCommands::CmdColourRangeStart msColourRangeStartCmd;
+    static EmitterCommands::CmdColourRangeEnd msColourRangeEndCmd;
+    static EmitterCommands::CmdDirection msDirectionCmd;
+    static EmitterCommands::CmdUp msUpCmd;
+    static EmitterCommands::CmdDirPositionRef msDirPositionRefCmd;
+    static EmitterCommands::CmdEmissionRate msEmissionRateCmd;
+    static EmitterCommands::CmdMaxTTL msMaxTTLCmd;
+    static EmitterCommands::CmdMaxVelocity msMaxVelocityCmd;
+    static EmitterCommands::CmdMinTTL msMinTTLCmd;
+    static EmitterCommands::CmdMinVelocity msMinVelocityCmd;
+    static EmitterCommands::CmdPosition msPositionCmd;
+    static EmitterCommands::CmdTTL msTTLCmd;
+    static EmitterCommands::CmdVelocity msVelocityCmd;
+    static EmitterCommands::CmdDuration msDurationCmd;
+    static EmitterCommands::CmdMinDuration msMinDurationCmd;
+    static EmitterCommands::CmdMaxDuration msMaxDurationCmd;
+    static EmitterCommands::CmdRepeatDelay msRepeatDelayCmd;
+    static EmitterCommands::CmdMinRepeatDelay msMinRepeatDelayCmd;
+    static EmitterCommands::CmdMaxRepeatDelay msMaxRepeatDelayCmd;
+    static EmitterCommands::CmdName msNameCmd;
+    static EmitterCommands::CmdEmittedEmitter msEmittedEmitterCmd;
 
 
     //-----------------------------------------------------------------------
     ParticleEmitter::ParticleEmitter(ParticleSystem* psys)
       : mParent(psys),
         mUseDirPositionRef(false),
-		mDirPositionRef(Vector3::ZERO),
+        mDirPositionRef(Vector3::ZERO),
         mStartTime(0),
         mDurationMin(0),
         mDurationMax(0),
@@ -82,9 +83,9 @@ namespace Ogre
         mColourRangeStart = mColourRangeEnd = ColourValue::White;
         mEnabled = true;
         mRemainder = 0;
-		mName = StringUtil::BLANK;
-		mEmittedEmitter = StringUtil::BLANK;
-		mEmitted = false;
+        mName = BLANKSTRING;
+        mEmittedEmitter = BLANKSTRING;
+        mEmitted = false;
     }
     //-----------------------------------------------------------------------
     ParticleEmitter::~ParticleEmitter() 
@@ -109,7 +110,7 @@ namespace Ogre
         mUp = mDirection.perpendicular();
         mUp.normalise();
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     const Vector3& ParticleEmitter::getDirection(void) const
     { 
         return mDirection; 
@@ -125,23 +126,23 @@ namespace Ogre
     { 
         return mUp; 
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void ParticleEmitter::setDirPositionReference( const Vector3& nposition, bool enable )
     { 
-	mUseDirPositionRef	= enable;
-        mDirPositionRef		= nposition;
+    mUseDirPositionRef  = enable;
+        mDirPositionRef     = nposition;
+    }
+    //-----------------------------------------------------------------------
+    const Vector3& ParticleEmitter::getDirPositionReference() const
+    {
+        return mDirPositionRef;
+    }
+    //-----------------------------------------------------------------------
+    bool ParticleEmitter::getDirPositionReferenceEnabled() const
+    {
+        return mUseDirPositionRef;
     }
     //-----------------------------------------------------------------------
-	const Vector3& ParticleEmitter::getDirPositionReference() const
-	{
-		return mDirPositionRef;
-	}
-	//-----------------------------------------------------------------------
-	bool ParticleEmitter::getDirPositionReferenceEnabled() const
-	{
-		return mUseDirPositionRef;
-	}
-	//-----------------------------------------------------------------------
     void ParticleEmitter::setAngle(const Radian& angle)
     {
         // Store as radians for efficiency
@@ -155,11 +156,13 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void ParticleEmitter::setParticleVelocity(Real speed)
     {
+        assert(std::isfinite(speed));
         mMinSpeed = mMaxSpeed = speed;
     }
     //-----------------------------------------------------------------------
     void ParticleEmitter::setParticleVelocity(Real min, Real max)
     {
+        assert(std::isfinite(min) && std::isfinite(max));
         mMinSpeed = min;
         mMaxSpeed = max;
     }
@@ -176,11 +179,14 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void ParticleEmitter::setTimeToLive(Real ttl)
     {
+        assert (ttl >= 0 && "Time to live can not be negative");
         mMinTTL = mMaxTTL = ttl;
     }
     //-----------------------------------------------------------------------
     void ParticleEmitter::setTimeToLive(Real minTtl, Real maxTtl)
     {
+        assert (minTtl >= 0 && "Time to live can not be negative");
+        assert (maxTtl >= 0 && "Time to live can not be negative");
         mMinTTL = minTtl;
         mMaxTTL = maxTtl;
     }
@@ -195,77 +201,77 @@ namespace Ogre
         mColourRangeStart = colourStart;
         mColourRangeEnd = colourEnd;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     const String& ParticleEmitter::getName(void) const
     {
         return mName;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void ParticleEmitter::setName(const String& newName)
     {
-		mName = newName;
+        mName = newName;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     const String& ParticleEmitter::getEmittedEmitter(void) const
     {
         return mEmittedEmitter;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void ParticleEmitter::setEmittedEmitter(const String& emittedEmitter)
     {
         mEmittedEmitter = emittedEmitter;
     }
-	//-----------------------------------------------------------------------
-	bool ParticleEmitter::isEmitted(void) const
+    //-----------------------------------------------------------------------
+    bool ParticleEmitter::isEmitted(void) const
     {
-		return mEmitted;
+        return mEmitted;
     }
-	//-----------------------------------------------------------------------
-	void ParticleEmitter::setEmitted(bool emitted)
+    //-----------------------------------------------------------------------
+    void ParticleEmitter::setEmitted(bool emitted)
     {
         mEmitted = emitted;
     }
     //-----------------------------------------------------------------------
     void ParticleEmitter::genEmissionDirection( const Vector3 &particlePos, Vector3& destVector )
     {
-		if( mUseDirPositionRef )
-		{
-			Vector3 particleDir = particlePos - mDirPositionRef;
-			particleDir.normalise();
+        if( mUseDirPositionRef )
+        {
+            Vector3 particleDir = particlePos - mDirPositionRef;
+            particleDir.normalise();
 
-			if (mAngle != Radian(0))
-			{
-				// Randomise angle
-				Radian angle = Math::UnitRandom() * mAngle;
+            if (mAngle != Radian(0))
+            {
+                // Randomise angle
+                Radian angle = Math::UnitRandom() * mAngle;
 
-				// Randomise direction
-				destVector = particleDir.randomDeviant( angle );
-			}
-			else
-			{
-				// Constant angle
-				destVector = particleDir.normalisedCopy();
-			}
-		}
-		else
-		{
-			if (mAngle != Radian(0))
-			{
-				// Randomise angle
-				Radian angle = Math::UnitRandom() * mAngle;
+                // Randomise direction
+                destVector = particleDir.randomDeviant( angle );
+            }
+            else
+            {
+                // Constant angle
+                destVector = particleDir;
+            }
+        }
+        else
+        {
+            if (mAngle != Radian(0))
+            {
+                // Randomise angle
+                Radian angle = Math::UnitRandom() * mAngle;
 
-				// Randomise direction
-				destVector = mDirection.randomDeviant(angle, mUp);
-			}
-			else
-			{
-				// Constant angle
-				destVector = mDirection;
-			}
-		}
+                // Randomise direction
+                destVector = mDirection.randomDeviant(angle, mUp);
+            }
+            else
+            {
+                // Constant angle
+                destVector = mDirection;
+            }
+        }
 
-		// Don't normalise, we can assume that it will still be a unit vector since
-		// both direction and 'up' are.
+        // Don't normalise, we can assume that it will still be a unit vector since
+        // both direction and 'up' are.
     }
     //-----------------------------------------------------------------------
     void ParticleEmitter::genEmissionVelocity(Vector3& destVector)
@@ -388,9 +394,9 @@ namespace Ogre
             "The up vector of the emitter." , PT_VECTOR3),
             &msUpCmd);
 
-		dict->addParameter(ParameterDef("direction_position_reference", 
+        dict->addParameter(ParameterDef("direction_position_reference", 
             "The reference position to calculate the direction of emitted particles "
-			"based on their position. Good for explosions and implosions (use negative velocity)" , PT_COLOURVALUE),
+            "based on their position. Good for explosions and implosions (use negative velocity)" , PT_COLOURVALUE),
             &msDirPositionRefCmd);
 
         dict->addParameter(ParameterDef("emission_rate", 
@@ -449,13 +455,13 @@ namespace Ogre
             "If set, after disabling an emitter will repeat (reenable) after this maximum number of seconds." , PT_REAL),
             &msMaxRepeatDelayCmd);
 
-		dict->addParameter(ParameterDef("name", 
-			"This is the name of the emitter" , PT_STRING),
-			&msNameCmd);
-		
-		dict->addParameter(ParameterDef("emit_emitter", 
-			"If set, this emitter will emit other emitters instead of visual particles" , PT_STRING),
-			&msEmittedEmitterCmd);
+        dict->addParameter(ParameterDef("name", 
+            "This is the name of the emitter" , PT_STRING),
+            &msNameCmd);
+        
+        dict->addParameter(ParameterDef("emit_emitter", 
+            "If set, this emitter will emit other emitters instead of visual particles" , PT_STRING),
+            &msEmittedEmitterCmd);
     }
     //-----------------------------------------------------------------------
     Real ParticleEmitter::getParticleVelocity(void) const
@@ -475,11 +481,13 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void ParticleEmitter::setMinParticleVelocity(Real min)
     {
+        assert(std::isfinite(min));
         mMinSpeed = min;
     }
     //-----------------------------------------------------------------------
     void ParticleEmitter::setMaxParticleVelocity(Real max)
     {
+        assert(std::isfinite(max));
         mMaxSpeed = max;
     }
     //-----------------------------------------------------------------------
@@ -666,7 +674,7 @@ namespace Ogre
     ParticleEmitterFactory::~ParticleEmitterFactory()
     {
         // Destroy all emitters
-        vector<ParticleEmitter*>::type::iterator i;
+        std::vector<ParticleEmitter*>::iterator i;
         for (i = mEmitters.begin(); i != mEmitters.end(); ++i)
         {
             OGRE_DELETE (*i);
@@ -678,7 +686,7 @@ namespace Ogre
     //-----------------------------------------------------------------------
     void ParticleEmitterFactory::destroyEmitter(ParticleEmitter* e)        
     {
-        vector<ParticleEmitter*>::type::iterator i;
+        std::vector<ParticleEmitter*>::iterator i;
         for (i = mEmitters.begin(); i != mEmitters.end(); ++i)
         {
             if ((*i) == e)
