@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,45 +31,43 @@ THE SOFTWARE.
 
 #include "OgreViewport.h"
 
-#include <limits>
-
 namespace Ogre {
     //-----------------------------------------------------------------------
-    PixelCountLodStrategy::PixelCountLodStrategy(const String& name)
+    PixelCountLodStrategyBase::PixelCountLodStrategyBase(const String& name)
         : LodStrategy(name)
     { }
     //---------------------------------------------------------------------
-    Real PixelCountLodStrategy::getBaseValue() const
+    Real PixelCountLodStrategyBase::getBaseValue() const
     {
         // Use the maximum possible value as base
         return std::numeric_limits<Real>::max();
     }
     //---------------------------------------------------------------------
-    Real PixelCountLodStrategy::transformBias(Real factor) const
+    Real PixelCountLodStrategyBase::transformBias(Real factor) const
     {
         // No transformation required for pixel count strategy
         return factor;
     }
     //---------------------------------------------------------------------
-    ushort PixelCountLodStrategy::getIndex(Real value, const Mesh::MeshLodUsageList& meshLodUsageList) const
+    ushort PixelCountLodStrategyBase::getIndex(Real value, const Mesh::MeshLodUsageList& meshLodUsageList) const
     {
         // Values are descending
         return getIndexDescending(value, meshLodUsageList);
     }
     //---------------------------------------------------------------------
-    ushort PixelCountLodStrategy::getIndex(Real value, const Material::LodValueList& materialLodValueList) const
+    ushort PixelCountLodStrategyBase::getIndex(Real value, const Material::LodValueList& materialLodValueList) const
     {
         // Values are descending
         return getIndexDescending(value, materialLodValueList);
     }
     //---------------------------------------------------------------------
-    void PixelCountLodStrategy::sort(Mesh::MeshLodUsageList& meshLodUsageList) const
+    void PixelCountLodStrategyBase::sort(Mesh::MeshLodUsageList& meshLodUsageList) const
     {
         // Sort descending
         sortDescending(meshLodUsageList);
     }
     //---------------------------------------------------------------------
-    bool PixelCountLodStrategy::isSorted(const Mesh::LodValueList& values) const
+    bool PixelCountLodStrategyBase::isSorted(const Mesh::LodValueList& values) const
     {
         // Check if values are sorted descending
         return isSortedDescending(values);
@@ -91,7 +89,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     AbsolutePixelCountLodStrategy::AbsolutePixelCountLodStrategy()
-        : PixelCountLodStrategy("pixel_count")
+        : PixelCountLodStrategyBase("pixel_count")
     { }
     //-----------------------------------------------------------------------
     Real AbsolutePixelCountLodStrategy::getValueImpl(const MovableObject *movableObject, const Ogre::Camera *camera) const
@@ -103,7 +101,9 @@ namespace Ogre {
         Real viewportArea = static_cast<Real>(viewport->getActualWidth() * viewport->getActualHeight());
 
         // Get area of unprojected circle with object bounding radius
-        Real boundingArea = Math::PI * Math::Sqr(movableObject->getBoundingRadius());
+        const Vector3& scl = movableObject->getParentNode()->_getDerivedScale();
+        Real factor = std::max(std::max(scl.x, scl.y), scl.z);
+        Real boundingArea = Math::PI * Math::Sqr(movableObject->getBoundingRadius() * factor);
 
         // Base computation on projection type
         switch (camera->getProjectionType())
@@ -160,7 +160,7 @@ namespace Ogre {
     }
     //-----------------------------------------------------------------------
     ScreenRatioPixelCountLodStrategy::ScreenRatioPixelCountLodStrategy()
-        : PixelCountLodStrategy("screen_ratio_pixel_count")
+        : PixelCountLodStrategyBase("screen_ratio_pixel_count")
     { }
     //-----------------------------------------------------------------------
     Real ScreenRatioPixelCountLodStrategy::getValueImpl(const MovableObject *movableObject, const Ogre::Camera *camera) const
