@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,16 +27,10 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 
-#include "OgrePlatformInformation.h"
-#include "OgreLog.h"
-#include "OgreStringConverter.h"
-
 #if OGRE_COMPILER == OGRE_COMPILER_MSVC
 #include <excpt.h>      // For SEH values
-	#if _MSC_VER >= 1400
-		#include <intrin.h>
-	#endif
-#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_NACL
+#include <intrin.h>
+#elif OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG
 #include <signal.h>
 #include <setjmp.h>
 
@@ -88,13 +82,13 @@ namespace Ogre {
     static int _isSupportCpuid(void)
     {
 #if OGRE_COMPILER == OGRE_COMPILER_MSVC
-		// Visual Studio 2005 & 64-bit compilers always supports __cpuid intrinsic
-		// note that even though this is a build rather than runtime setting, all
-		// 64-bit CPUs support this so since binary is 64-bit only we're ok
-	#if _MSC_VER >= 1400 && defined(_M_X64)
-		return true;
-	#else
-		// If we can modify flag register bit 21, the cpu is supports CPUID instruction
+        // Visual Studio 2005 & 64-bit compilers always supports __cpuid intrinsic
+        // note that even though this is a build rather than runtime setting, all
+        // 64-bit CPUs support this so since binary is 64-bit only we're ok
+    #if _MSC_VER >= 1400 && defined(_M_X64)
+        return true;
+    #else
+        // If we can modify flag register bit 21, the cpu is supports CPUID instruction
         __asm
         {
             // Read EFLAG
@@ -122,8 +116,8 @@ namespace Ogre {
 
             // Return values in eax, no return statement requirement here for VC.
         }
-	#endif
-#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_NACL
+    #endif
+#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_EMSCRIPTEN
         #if OGRE_ARCH_TYPE == OGRE_ARCHITECTURE_64
            return true;
        #else
@@ -156,28 +150,14 @@ namespace Ogre {
     static uint _performCpuid(int query, CpuidResult& result)
     {
 #if OGRE_COMPILER == OGRE_COMPILER_MSVC
-	#if _MSC_VER >= 1400 
-		int CPUInfo[4];
-		__cpuid(CPUInfo, query);
-		result._eax = CPUInfo[0];
-		result._ebx = CPUInfo[1];
-		result._ecx = CPUInfo[2];
-		result._edx = CPUInfo[3];
-		return result._eax;
-	#else
-        __asm
-        {
-            mov     edi, result
-            mov     eax, query
-            cpuid
-            mov     [edi]._eax, eax
-            mov     [edi]._ebx, ebx
-            mov     [edi]._edx, edx
-            mov     [edi]._ecx, ecx
-            // Return values in eax, no return statement requirement here for VC.
-        }
-	#endif
-#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_NACL
+        int CPUInfo[4];
+        __cpuid(CPUInfo, query);
+        result._eax = CPUInfo[0];
+        result._ebx = CPUInfo[1];
+        result._ecx = CPUInfo[2];
+        result._edx = CPUInfo[3];
+        return result._eax;
+#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_EMSCRIPTEN
         #if OGRE_ARCH_TYPE == OGRE_ARCHITECTURE_64
         __asm__
         (
@@ -208,7 +188,7 @@ namespace Ogre {
 
     //---------------------------------------------------------------------
     // Detect whether or not os support Streaming SIMD Extension.
-#if (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_NACL
+#if OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG
     #if OGRE_ARCH_TYPE == OGRE_ARCHITECTURE_32 && OGRE_CPU == OGRE_CPU_X86
     static jmp_buf sIllegalJmpBuf;
     static void _illegalHandler(int x)
@@ -235,12 +215,12 @@ namespace Ogre {
             exception using the (non-standard) __try/__except mechanism
             of Microsoft Visual C/C++.
         */
-		// Visual Studio 2005, Both AMD and Intel x64 support SSE
-		// note that even though this is a build rather than runtime setting, all
-		// 64-bit CPUs support this so since binary is 64-bit only we're ok
-	#if _MSC_VER >= 1400 && defined(_M_X64)
-			return true;
-	#else
+        // Visual Studio 2005, Both AMD and Intel x64 support SSE
+        // note that even though this is a build rather than runtime setting, all
+        // 64-bit CPUs support this so since binary is 64-bit only we're ok
+    #if _MSC_VER >= 1400 && defined(_M_X64)
+            return true;
+    #else
         __try
         {
             __asm orps  xmm0, xmm0
@@ -250,8 +230,8 @@ namespace Ogre {
         {
             return false;
         }
-	#endif
-#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_NACL
+    #endif
+#elif (OGRE_COMPILER == OGRE_COMPILER_GNUC || OGRE_COMPILER == OGRE_COMPILER_CLANG) && OGRE_PLATFORM != OGRE_PLATFORM_EMSCRIPTEN
         #if OGRE_ARCH_TYPE == OGRE_ARCHITECTURE_64 
             return true;
         #else
@@ -284,6 +264,13 @@ namespace Ogre {
 
     static uint queryCpuFeatures(void)
     {
+
+#define CPUID_FUNC_VENDOR_ID                 0x0
+#define CPUID_FUNC_STANDARD_FEATURES         0x1
+#define CPUID_FUNC_EXTENSION_QUERY           0x80000000
+#define CPUID_FUNC_EXTENDED_FEATURES         0x80000001
+#define CPUID_FUNC_ADVANCED_POWER_MANAGEMENT 0x80000007
+
 #define CPUID_STD_FPU               (1<<0)
 #define CPUID_STD_TSC               (1<<4)
 #define CPUID_STD_CMOV              (1<<15)
@@ -292,7 +279,9 @@ namespace Ogre {
 #define CPUID_STD_SSE2              (1<<26)
 #define CPUID_STD_HTT               (1<<28)     // EDX[28] - Bit 28 set indicates  Hyper-Threading Technology is supported in hardware.
 
-#define CPUID_STD_SSE3              (1<<0)      // ECX[0] - Bit 0 of standard function 1 indicate SSE3 supported
+#define CPUID_STD_SSE3              (1<<0)      // ECX[0]  - Bit 0 of standard function 1 indicate SSE3 supported
+#define CPUID_STD_SSE41             (1<<19)     // ECX[19] - Bit 0 of standard function 1 indicate SSE41 supported
+#define CPUID_STD_SSE42             (1<<20)     // ECX[20] - Bit 0 of standard function 1 indicate SSE42 supported
 
 #define CPUID_FAMILY_ID_MASK        0x0F00      // EAX[11:8] - Bit 11 thru 8 contains family  processor id
 #define CPUID_EXT_FAMILY_ID_MASK    0x0F00000   // EAX[23:20] - Bit 23 thru 20 contains extended family processor id
@@ -302,6 +291,9 @@ namespace Ogre {
 #define CPUID_EXT_AMD_3DNOWEXT      (1<<30)
 #define CPUID_EXT_AMD_MMXEXT        (1<<22)
 
+
+#define CPUID_APM_INVARIANT_TSC     (1<<8)      // EDX[8] - Bit 8 of function 0x80000007 indicates support for invariant TSC.
+
         uint features = 0;
 
         // Supports CPUID instruction ?
@@ -310,7 +302,7 @@ namespace Ogre {
             CpuidResult result;
 
             // Has standard feature ?
-            if (_performCpuid(0, result))
+            if (_performCpuid(CPUID_FUNC_VENDOR_ID, result))
             {
                 // Check vendor strings
                 if (memcmp(&result._ebx, "GenuineIntel", 12) == 0)
@@ -319,7 +311,7 @@ namespace Ogre {
                         features |= PlatformInformation::CPU_FEATURE_PRO;
 
                     // Check standard feature
-                    _performCpuid(1, result);
+                    _performCpuid(CPUID_FUNC_STANDARD_FEATURES, result);
 
                     if (result._edx & CPUID_STD_FPU)
                         features |= PlatformInformation::CPU_FEATURE_FPU;
@@ -333,9 +325,12 @@ namespace Ogre {
                         features |= PlatformInformation::CPU_FEATURE_MMXEXT | PlatformInformation::CPU_FEATURE_SSE;
                     if (result._edx & CPUID_STD_SSE2)
                         features |= PlatformInformation::CPU_FEATURE_SSE2;
-
                     if (result._ecx & CPUID_STD_SSE3)
                         features |= PlatformInformation::CPU_FEATURE_SSE3;
+                    if (result._ecx & CPUID_STD_SSE41)
+                        features |= PlatformInformation::CPU_FEATURE_SSE41;
+                    if (result._ecx & CPUID_STD_SSE42)
+                        features |= PlatformInformation::CPU_FEATURE_SSE42;
 
                     // Check to see if this is a Pentium 4 or later processor
                     if ((result._eax & CPUID_EXT_FAMILY_ID_MASK) ||
@@ -345,13 +340,23 @@ namespace Ogre {
                         if (result._edx & CPUID_STD_HTT)
                             features |= PlatformInformation::CPU_FEATURE_HTT;
                     }
+
+
+                    const uint maxExtensionFunctionSupport = _performCpuid(CPUID_FUNC_EXTENSION_QUERY, result);
+                    if (maxExtensionFunctionSupport >= CPUID_FUNC_ADVANCED_POWER_MANAGEMENT)
+                    {
+                        _performCpuid(CPUID_FUNC_ADVANCED_POWER_MANAGEMENT, result);
+
+                        if (result._edx & CPUID_APM_INVARIANT_TSC)
+                            features |= PlatformInformation::CPU_FEATURE_INVARIANT_TSC;
+                    }
                 }
                 else if (memcmp(&result._ebx, "AuthenticAMD", 12) == 0)
                 {
                     features |= PlatformInformation::CPU_FEATURE_PRO;
 
                     // Check standard feature
-                    _performCpuid(1, result);
+                    _performCpuid(CPUID_FUNC_STANDARD_FEATURES, result);
 
                     if (result._edx & CPUID_STD_FPU)
                         features |= PlatformInformation::CPU_FEATURE_FPU;
@@ -370,10 +375,11 @@ namespace Ogre {
                         features |= PlatformInformation::CPU_FEATURE_SSE3;
 
                     // Has extended feature ?
-                    if (_performCpuid(0x80000000, result) > 0x80000000)
+                    const uint maxExtensionFunctionSupport = _performCpuid(CPUID_FUNC_EXTENSION_QUERY, result);
+                    if (maxExtensionFunctionSupport >= CPUID_FUNC_EXTENDED_FEATURES)
                     {
                         // Check extended feature
-                        _performCpuid(0x80000001, result);
+                        _performCpuid(CPUID_FUNC_EXTENDED_FEATURES, result);
 
                         if (result._edx & CPUID_EXT_3DNOW)
                             features |= PlatformInformation::CPU_FEATURE_3DNOW;
@@ -382,6 +388,15 @@ namespace Ogre {
                         if (result._edx & CPUID_EXT_AMD_MMXEXT)
                             features |= PlatformInformation::CPU_FEATURE_MMXEXT;
                     }
+
+
+                    if (maxExtensionFunctionSupport >= CPUID_FUNC_ADVANCED_POWER_MANAGEMENT)
+                    {
+                        _performCpuid(CPUID_FUNC_ADVANCED_POWER_MANAGEMENT, result);
+
+                        if (result._edx & CPUID_APM_INVARIANT_TSC)
+                            features |= PlatformInformation::CPU_FEATURE_INVARIANT_TSC;
+                    }
                 }
             }
         }
@@ -393,8 +408,13 @@ namespace Ogre {
     {
         uint features = queryCpuFeatures();
 
-        const uint sse_features = PlatformInformation::CPU_FEATURE_SSE |
-            PlatformInformation::CPU_FEATURE_SSE2 | PlatformInformation::CPU_FEATURE_SSE3;
+        const uint sse_features = 0
+            | PlatformInformation::CPU_FEATURE_SSE
+            | PlatformInformation::CPU_FEATURE_SSE2
+            | PlatformInformation::CPU_FEATURE_SSE3
+            | PlatformInformation::CPU_FEATURE_SSE41
+            | PlatformInformation::CPU_FEATURE_SSE42;
+
         if ((features & sse_features) && !_checkOperatingSystemSupportSSE())
         {
             features &= ~sse_features;
@@ -405,83 +425,83 @@ namespace Ogre {
     //---------------------------------------------------------------------
     static String _detectCpuIdentifier(void)
     {
-		// Supports CPUID instruction ?
-		if (_isSupportCpuid())
-		{
-			CpuidResult result;
-			uint nExIds;
-			char CPUString[0x20];
-			char CPUBrandString[0x40];
+        // Supports CPUID instruction ?
+        if (_isSupportCpuid())
+        {
+            CpuidResult result;
+            uint nExIds;
+            char CPUString[0x20];
+            char CPUBrandString[0x40];
 
-			StringUtil::StrStreamType detailedIdentStr;
+            StringStream detailedIdentStr;
 
 
-			// Has standard feature ?
-			if (_performCpuid(0, result))
-			{
-				memset(CPUString, 0, sizeof(CPUString));
-				memset(CPUBrandString, 0, sizeof(CPUBrandString));
+            // Has standard feature ?
+            if (_performCpuid(0, result))
+            {
+                memset(CPUString, 0, sizeof(CPUString));
+                memset(CPUBrandString, 0, sizeof(CPUBrandString));
 
-				//*((int*)CPUString) = result._ebx;
-                                memcpy(CPUString, &result._ebx, sizeof(int));
-				//*((int*)(CPUString+4)) = result._edx;
-                                memcpy(CPUString+4, &result._edx, sizeof(int));
-                                //*((int*)(CPUString+8)) = result._ecx;
-                                memcpy(CPUString+8, &result._ecx, sizeof(int));
+                //*((int*)CPUString) = result._ebx;
+                //*((int*)(CPUString+4)) = result._edx;
+                //*((int*)(CPUString+8)) = result._ecx;
+                memcpy(CPUString, &result._ebx, sizeof(int));
+                memcpy(CPUString+4, &result._edx, sizeof(int));
+                memcpy(CPUString+8, &result._ecx, sizeof(int));
 
-				detailedIdentStr << CPUString;
+                detailedIdentStr << CPUString;
 
-				// Calling _performCpuid with 0x80000000 as the query argument
-				// gets the number of valid extended IDs.
-				nExIds = _performCpuid(0x80000000, result);
+                // Calling _performCpuid with 0x80000000 as the query argument
+                // gets the number of valid extended IDs.
+                nExIds = _performCpuid(0x80000000, result);
 
-				for (uint i=0x80000000; i<=nExIds; ++i)
-				{
-					_performCpuid(i, result);
+                for (uint i=0x80000000; i<=nExIds; ++i)
+                {
+                    _performCpuid(i, result);
 
-					// Interpret CPU brand string and cache information.
-					if  (i == 0x80000002)
+                    // Interpret CPU brand string and cache information.
+                    if  (i == 0x80000002)
                     {
-						memcpy(CPUBrandString + 0, &result._eax, sizeof(result._eax));
-						memcpy(CPUBrandString + 4, &result._ebx, sizeof(result._ebx));
-						memcpy(CPUBrandString + 8, &result._ecx, sizeof(result._ecx));
-						memcpy(CPUBrandString + 12, &result._edx, sizeof(result._edx));
+                        memcpy(CPUBrandString + 0, &result._eax, sizeof(result._eax));
+                        memcpy(CPUBrandString + 4, &result._ebx, sizeof(result._ebx));
+                        memcpy(CPUBrandString + 8, &result._ecx, sizeof(result._ecx));
+                        memcpy(CPUBrandString + 12, &result._edx, sizeof(result._edx));
                     }
-					else if  (i == 0x80000003)
+                    else if  (i == 0x80000003)
                     {
-						memcpy(CPUBrandString + 16 + 0, &result._eax, sizeof(result._eax));
-						memcpy(CPUBrandString + 16 + 4, &result._ebx, sizeof(result._ebx));
-						memcpy(CPUBrandString + 16 + 8, &result._ecx, sizeof(result._ecx));
-						memcpy(CPUBrandString + 16 + 12, &result._edx, sizeof(result._edx));
+                        memcpy(CPUBrandString + 16 + 0, &result._eax, sizeof(result._eax));
+                        memcpy(CPUBrandString + 16 + 4, &result._ebx, sizeof(result._ebx));
+                        memcpy(CPUBrandString + 16 + 8, &result._ecx, sizeof(result._ecx));
+                        memcpy(CPUBrandString + 16 + 12, &result._edx, sizeof(result._edx));
                     }
-					else if  (i == 0x80000004)
+                    else if  (i == 0x80000004)
                     {
-						memcpy(CPUBrandString + 32 + 0, &result._eax, sizeof(result._eax));
-						memcpy(CPUBrandString + 32 + 4, &result._ebx, sizeof(result._ebx));
-						memcpy(CPUBrandString + 32 + 8, &result._ecx, sizeof(result._ecx));
-						memcpy(CPUBrandString + 32 + 12, &result._edx, sizeof(result._edx));
+                        memcpy(CPUBrandString + 32 + 0, &result._eax, sizeof(result._eax));
+                        memcpy(CPUBrandString + 32 + 4, &result._ebx, sizeof(result._ebx));
+                        memcpy(CPUBrandString + 32 + 8, &result._ecx, sizeof(result._ecx));
+                        memcpy(CPUBrandString + 32 + 12, &result._edx, sizeof(result._edx));
                     }
-				}
+                }
 
-				String brand(CPUBrandString);
-				StringUtil::trim(brand);
-				if (!brand.empty())
-					detailedIdentStr << ": " << brand;
+                String brand(CPUBrandString);
+                StringUtil::trim(brand);
+                if (!brand.empty())
+                    detailedIdentStr << ": " << brand;
 
-				return detailedIdentStr.str();
-			}
-		}
+                return detailedIdentStr.str();
+            }
+        }
 
-		return "X86";
+        return "X86";
     }
 
 #elif OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
     static uint _detectCpuFeatures(void)
     {
         uint features = 0;
+#if OGRE_CPU == OGRE_CPU_ARM
         uint64_t cpufeatures = android_getCpuFeatures();
-        
-        if (cpufeatures & ANDROID_CPU_ARM_FEATURE_NEON) 
+        if (cpufeatures & ANDROID_CPU_ARM_FEATURE_NEON)
         {
             features |= PlatformInformation::CPU_FEATURE_NEON;
         }
@@ -490,6 +510,12 @@ namespace Ogre {
         {
             features |= PlatformInformation::CPU_FEATURE_VFP;
         }
+#elif OGRE_CPU == OGRE_CPU_X86
+        // see https://developer.android.com/ndk/guides/abis.html
+        features |= PlatformInformation::CPU_FEATURE_SSE;
+        features |= PlatformInformation::CPU_FEATURE_SSE2;
+        features |= PlatformInformation::CPU_FEATURE_SSE3;
+#endif
         return features;
     }
     //---------------------------------------------------------------------
@@ -499,6 +525,9 @@ namespace Ogre {
         AndroidCpuFamily cpuInfo = android_getCpuFamily();
         
         switch (cpuInfo) {
+            case ANDROID_CPU_FAMILY_ARM64:
+                cpuID = "ARM64";
+                break;
             case ANDROID_CPU_FAMILY_ARM:
             {
                 if (android_getCpuFeatures() & ANDROID_CPU_ARM_FEATURE_ARMv7) 
@@ -633,58 +662,64 @@ namespace Ogre {
         static const uint sFeatures = _detectCpuFeatures();
         return sFeatures;
     }
-	//---------------------------------------------------------------------
-	bool PlatformInformation::hasCpuFeature(CpuFeatures feature)
-	{
-		return (getCpuFeatures() & feature) != 0;
-	}
-	//---------------------------------------------------------------------
-	void PlatformInformation::log(Log* pLog)
-	{
-		pLog->logMessage("CPU Identifier & Features");
-		pLog->logMessage("-------------------------");
-		pLog->logMessage(
-			" *   CPU ID: " + getCpuIdentifier());
+    //---------------------------------------------------------------------
+    bool PlatformInformation::hasCpuFeature(CpuFeatures feature)
+    {
+        return (getCpuFeatures() & feature) != 0;
+    }
+    //---------------------------------------------------------------------
+    void PlatformInformation::log(Log* pLog)
+    {
+        pLog->logMessage("CPU Identifier & Features");
+        pLog->logMessage("-------------------------");
+        pLog->logMessage(
+            " *   CPU ID: " + getCpuIdentifier());
 #if OGRE_CPU == OGRE_CPU_X86
-		if(_isSupportCpuid())
-		{
-			pLog->logMessage(
-				" *      SSE: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE), true));
-			pLog->logMessage(
-				" *     SSE2: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE2), true));
-			pLog->logMessage(
-				" *     SSE3: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE3), true));
-			pLog->logMessage(
-				" *      MMX: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MMX), true));
-			pLog->logMessage(
-				" *   MMXEXT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MMXEXT), true));
-			pLog->logMessage(
-				" *    3DNOW: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_3DNOW), true));
-			pLog->logMessage(
-				" * 3DNOWEXT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_3DNOWEXT), true));
-			pLog->logMessage(
-				" *     CMOV: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_CMOV), true));
-			pLog->logMessage(
-				" *      TSC: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_TSC), true));
-			pLog->logMessage(
-				" *      FPU: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_FPU), true));
-			pLog->logMessage(
-				" *      PRO: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_PRO), true));
-			pLog->logMessage(
-				" *       HT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_HTT), true));
-		}
+        if(_isSupportCpuid())
+        {
+            pLog->logMessage(
+                " *          SSE: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE), true));
+            pLog->logMessage(
+                " *         SSE2: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE2), true));
+            pLog->logMessage(
+                " *         SSE3: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE3), true));
+            pLog->logMessage(
+                " *        SSE41: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE41), true));
+            pLog->logMessage(
+                " *        SSE42: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_SSE42), true));
+            pLog->logMessage(
+                " *          MMX: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MMX), true));
+            pLog->logMessage(
+                " *       MMXEXT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MMXEXT), true));
+            pLog->logMessage(
+                " *        3DNOW: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_3DNOW), true));
+            pLog->logMessage(
+                " *     3DNOWEXT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_3DNOWEXT), true));
+            pLog->logMessage(
+                " *         CMOV: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_CMOV), true));
+            pLog->logMessage(
+                " *          TSC: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_TSC), true));
+            pLog->logMessage(
+                " *INVARIANT TSC: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_INVARIANT_TSC), true));
+            pLog->logMessage(
+                " *          FPU: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_FPU), true));
+            pLog->logMessage(
+                " *          PRO: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_PRO), true));
+            pLog->logMessage(
+                " *           HT: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_HTT), true));
+        }
 #elif OGRE_CPU == OGRE_CPU_ARM || OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
         pLog->logMessage(
-				" *      VFP: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_VFP), true));
+                " *          VFP: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_VFP), true));
         pLog->logMessage(
-				" *     NEON: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_NEON), true));
+                " *         NEON: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_NEON), true));
 #elif OGRE_CPU == OGRE_CPU_MIPS
         pLog->logMessage(
-                " *      MSA: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MSA), true));
+                " *          MSA: " + StringConverter::toString(hasCpuFeature(CPU_FEATURE_MSA), true));
 #endif
-		pLog->logMessage("-------------------------");
+        pLog->logMessage("-------------------------");
 
-	}
+    }
 
 
 }
