@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,94 +29,43 @@ THE SOFTWARE.
 
 #include "OgreRenderSystemCapabilitiesSerializer.h"
 #include "OgreRenderSystemCapabilitiesManager.h"
-#include "OgreStringConverter.h"
-#include "OgreLogManager.h"
-#include "OgreException.h"
+#include "OgreIteratorWrappers.h"
 #include "OgreRenderSystemCapabilities.h"
 
-#include <fstream>
-
-
 namespace Ogre
 {
 
     RenderSystemCapabilitiesSerializer::RenderSystemCapabilitiesSerializer() : mCurrentLineNumber(0), mCurrentLine(0),
         mCurrentCapabilities(0)
     {
-        mCurrentStream.setNull();
-
         initialiaseDispatchTables();
     }
-	
-	//-----------------------------------------------------------------------
-	static void write(const RenderSystemCapabilities* caps, String name, std::ostream &file)
-	{
-		using namespace std;
+    
+    //-----------------------------------------------------------------------
+    void RenderSystemCapabilitiesSerializer::write(const RenderSystemCapabilities* caps, const String &name, std::ostream &file)
+    {
+        using namespace std;
 
-		file << "render_system_capabilities \"" << name << "\"" << endl;
+        file << "render_system_capabilities \"" << name << "\"" << endl;
         file << "{" << endl;
 
         file << "\t" << "render_system_name " << caps->getRenderSystemName() << endl;
         file << endl;
         
 
-		file << "\t" << "device_name " << caps->getDeviceName() << endl;
-		const DriverVersion& driverVer = caps->getDriverVersion();
-		file << "\t" << "driver_version " << driverVer.toString() << endl;
-		file << "\t" << "vendor " << caps->vendorToString(caps->getVendor());
-
-		file << endl;
+        file << "\t" << "device_name " << caps->getDeviceName() << endl;
+        const DriverVersion& driverVer = caps->getDriverVersion();
+        file << "\t" << "driver_version " << driverVer.toString() << endl;
+        file << "\t" << "vendor " << caps->vendorToString(caps->getVendor());
 
         file << endl;
-		file << "\t" << "fixed_function " << StringConverter::toString(caps->hasCapability(RSC_FIXED_FUNCTION)) << endl;
-        file << "\t" << "automipmap " << StringConverter::toString(caps->hasCapability(RSC_AUTOMIPMAP)) << endl;
-        file << "\t" << "blending " << StringConverter::toString(caps->hasCapability(RSC_BLENDING)) << endl;
-        file << "\t" << "anisotropy " << StringConverter::toString(caps->hasCapability(RSC_ANISOTROPY)) << endl;
-        file << "\t" << "dot3 " << StringConverter::toString(caps->hasCapability(RSC_DOT3)) << endl;
-        file << "\t" << "cubemapping " << StringConverter::toString(caps->hasCapability(RSC_CUBEMAPPING)) << endl;
-        file << "\t" << "hwstencil " << StringConverter::toString(caps->hasCapability(RSC_HWSTENCIL)) << endl;
-        file << "\t" << "vbo " << StringConverter::toString(caps->hasCapability(RSC_VBO)) << endl;
-        file << "\t" << "vertex_program " << StringConverter::toString(caps->hasCapability(RSC_VERTEX_PROGRAM)) << endl;
-        file << "\t" << "fragment_program " << StringConverter::toString(caps->hasCapability(RSC_FRAGMENT_PROGRAM)) << endl;
-        file << "\t" << "geometry_program " << StringConverter::toString(caps->hasCapability(RSC_GEOMETRY_PROGRAM)) << endl;
-        file << "\t" << "tesselation_hull_program " << StringConverter::toString(caps->hasCapability(RSC_TESSELATION_HULL_PROGRAM)) << endl;
-        file << "\t" << "tesselation_domain_program " << StringConverter::toString(caps->hasCapability(RSC_TESSELATION_DOMAIN_PROGRAM)) << endl;
-        file << "\t" << "compute_program " << StringConverter::toString(caps->hasCapability(RSC_COMPUTE_PROGRAM)) << endl;
-        file << "\t" << "scissor_test " << StringConverter::toString(caps->hasCapability(RSC_SCISSOR_TEST)) << endl;
-        file << "\t" << "two_sided_stencil " << StringConverter::toString(caps->hasCapability(RSC_TWO_SIDED_STENCIL)) << endl;
-        file << "\t" << "stencil_wrap " << StringConverter::toString(caps->hasCapability(RSC_STENCIL_WRAP)) << endl;
-        file << "\t" << "hwocclusion " << StringConverter::toString(caps->hasCapability(RSC_HWOCCLUSION)) << endl;
-        file << "\t" << "user_clip_planes " << StringConverter::toString(caps->hasCapability(RSC_USER_CLIP_PLANES)) << endl;
-        file << "\t" << "vertex_format_ubyte4 " << StringConverter::toString(caps->hasCapability(RSC_VERTEX_FORMAT_UBYTE4)) << endl;
-        file << "\t" << "infinite_far_plane " << StringConverter::toString(caps->hasCapability(RSC_INFINITE_FAR_PLANE)) << endl;
-        file << "\t" << "hwrender_to_texture " << StringConverter::toString(caps->hasCapability(RSC_HWRENDER_TO_TEXTURE)) << endl;
-        file << "\t" << "texture_float " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_FLOAT)) << endl;
-        file << "\t" << "non_power_of_2_textures " << StringConverter::toString(caps->hasCapability(RSC_NON_POWER_OF_2_TEXTURES)) << endl;
-        file << "\t" << "texture_3d " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_3D)) << endl;
-        file << "\t" << "texture_1d " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_1D)) << endl;
-        file << "\t" << "point_sprites " << StringConverter::toString(caps->hasCapability(RSC_POINT_SPRITES)) << endl;
-        file << "\t" << "point_extended_parameters " << StringConverter::toString(caps->hasCapability(RSC_POINT_EXTENDED_PARAMETERS)) << endl;
-        file << "\t" << "vertex_texture_fetch " << StringConverter::toString(caps->hasCapability(RSC_VERTEX_TEXTURE_FETCH)) << endl;
-        file << "\t" << "mipmap_lod_bias " << StringConverter::toString(caps->hasCapability(RSC_MIPMAP_LOD_BIAS)) << endl;
-        file << "\t" << "atomic_counters " << StringConverter::toString(caps->hasCapability(RSC_ATOMIC_COUNTERS)) << endl;
-        file << "\t" << "texture_compression " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION)) << endl;
-        file << "\t" << "texture_compression_dxt " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_DXT)) << endl;
-        file << "\t" << "texture_compression_vtc " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_VTC)) << endl;
-        file << "\t" << "texture_compression_pvrtc " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_PVRTC)) << endl;
-        file << "\t" << "texture_compression_atc " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_ATC)) << endl;
-        file << "\t" << "texture_compression_etc1 " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_ETC1)) << endl;
-        file << "\t" << "texture_compression_etc2 " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_ETC2)) << endl;
-		file << "\t" << "texture_compression_bc4_bc5 " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_BC4_BC5)) << endl;
-        file << "\t" << "texture_compression_bc6h_bc7 " << StringConverter::toString(caps->hasCapability(RSC_TEXTURE_COMPRESSION_BC6H_BC7)) << endl;
-        file << "\t" << "gl1_5_novbo " << StringConverter::toString(caps->hasCapability(RSC_GL1_5_NOVBO)) << endl;
-        file << "\t" << "fbo " << StringConverter::toString(caps->hasCapability(RSC_FBO)) << endl;
-        file << "\t" << "fbo_arb " << StringConverter::toString(caps->hasCapability(RSC_FBO_ARB)) << endl;
-        file << "\t" << "fbo_ati " << StringConverter::toString(caps->hasCapability(RSC_FBO_ATI)) << endl;
-        file << "\t" << "pbuffer " << StringConverter::toString(caps->hasCapability(RSC_PBUFFER)) << endl;
-        file << "\t" << "gl1_5_nohwocclusion " << StringConverter::toString(caps->hasCapability(RSC_GL1_5_NOHWOCCLUSION)) << endl;
-        file << "\t" << "perstageconstant " << StringConverter::toString(caps->hasCapability(RSC_PERSTAGECONSTANT)) << endl;
-        file << "\t" << "vao " << StringConverter::toString(caps->hasCapability(RSC_VAO)) << endl;
-        file << "\t" << "separate_shader_objects " << StringConverter::toString(caps->hasCapability(RSC_SEPARATE_SHADER_OBJECTS)) << endl;
+
+        file << endl;
+
+        for(CapabilitiesMap::iterator it = mCapabilitiesMap.begin(); it != mCapabilitiesMap.end(); ++it) {
+            file << "\t" << it->first << " " << StringConverter::toString(caps->hasCapability(it->second)) << endl;
+        }
+
         file << endl;
 
         RenderSystemCapabilities::ShaderProfiles profiles = caps->getSupportedShaderProfiles();
@@ -134,7 +83,6 @@ namespace Ogre
         file << "\t" << "vertex_texture_units_shared " << StringConverter::toString(caps->getVertexTextureUnitsShared())<< endl;
         
         file << endl;
-        file << "\t" << "num_world_matrices " << StringConverter::toString(caps->getNumWorldMatrices()) << endl;
         file << "\t" << "num_texture_units " << StringConverter::toString(caps->getNumTextureUnits()) << endl;
         file << "\t" << "stencil_buffer_bit_depth " << StringConverter::toString(caps->getStencilBufferBitDepth()) << endl;
         file << "\t" << "num_vertex_blend_matrices " << StringConverter::toString(caps->getNumVertexBlendMatrices()) << endl;
@@ -148,45 +96,46 @@ namespace Ogre
         file << "\t" << "geometry_program_constant_float_count " << StringConverter::toString(caps->getGeometryProgramConstantFloatCount()) << endl;
         file << "\t" << "geometry_program_constant_int_count " << StringConverter::toString(caps->getGeometryProgramConstantIntCount()) << endl;
         file << "\t" << "geometry_program_constant_bool_count " << StringConverter::toString(caps->getGeometryProgramConstantBoolCount()) << endl;
-        file << "\t" << "tesselation_hull_program_constant_float_count " << StringConverter::toString(caps->getTesselationHullProgramConstantFloatCount()) << endl;
-        file << "\t" << "tesselation_hull_program_constant_int_count " << StringConverter::toString(caps->getTesselationHullProgramConstantIntCount()) << endl;
-        file << "\t" << "tesselation_hull_program_constant_bool_count " << StringConverter::toString(caps->getTesselationHullProgramConstantBoolCount()) << endl;
-        file << "\t" << "tesselation_domain_program_constant_float_count " << StringConverter::toString(caps->getTesselationDomainProgramConstantFloatCount()) << endl;
-        file << "\t" << "tesselation_domain_program_constant_int_count " << StringConverter::toString(caps->getTesselationDomainProgramConstantIntCount()) << endl;
-        file << "\t" << "tesselation_domain_program_constant_bool_count " << StringConverter::toString(caps->getTesselationDomainProgramConstantBoolCount()) << endl;
+        file << "\t" << "tessellation_hull_program_constant_float_count " << StringConverter::toString(caps->getTessellationHullProgramConstantFloatCount()) << endl;
+        file << "\t" << "tessellation_hull_program_constant_int_count " << StringConverter::toString(caps->getTessellationHullProgramConstantIntCount()) << endl;
+        file << "\t" << "tessellation_hull_program_constant_bool_count " << StringConverter::toString(caps->getTessellationHullProgramConstantBoolCount()) << endl;
+        file << "\t" << "tessellation_domain_program_constant_float_count " << StringConverter::toString(caps->getTessellationDomainProgramConstantFloatCount()) << endl;
+        file << "\t" << "tessellation_domain_program_constant_int_count " << StringConverter::toString(caps->getTessellationDomainProgramConstantIntCount()) << endl;
+        file << "\t" << "tessellation_domain_program_constant_bool_count " << StringConverter::toString(caps->getTessellationDomainProgramConstantBoolCount()) << endl;
         file << "\t" << "compute_program_constant_float_count " << StringConverter::toString(caps->getComputeProgramConstantFloatCount()) << endl;
         file << "\t" << "compute_program_constant_int_count " << StringConverter::toString(caps->getComputeProgramConstantIntCount()) << endl;
         file << "\t" << "compute_program_constant_bool_count " << StringConverter::toString(caps->getComputeProgramConstantBoolCount()) << endl;
         file << "\t" << "num_vertex_texture_units " << StringConverter::toString(caps->getNumVertexTextureUnits()) << endl;
+        file << "\t" << "num_vertex_attributes " << StringConverter::toString(caps->getNumVertexAttributes()) << endl;
 
         file << endl;
 
         file << "}" << endl;
-	}
+    }
 
     //-----------------------------------------------------------------------
-    void RenderSystemCapabilitiesSerializer::writeScript(const RenderSystemCapabilities* caps, String name, String filename)
+    void RenderSystemCapabilitiesSerializer::writeScript(const RenderSystemCapabilities* caps, const String &name, String filename)
     {
         using namespace std;
 
-		ofstream file(filename.c_str());
+        ofstream file(filename.c_str());
 
         write(caps, name, file);
 
         file.close();
     }
-	
-	//-----------------------------------------------------------------------
-	String RenderSystemCapabilitiesSerializer::writeString(const RenderSystemCapabilities* caps, String name)
-	{
-		using namespace std;
-		
-		stringstream stream;
-		
-		write(caps, name, stream);
-		
-		return stream.str();
-	}
+    
+    //-----------------------------------------------------------------------
+    String RenderSystemCapabilitiesSerializer::writeString(const RenderSystemCapabilities* caps, const String &name)
+    {
+        using namespace std;
+        
+        stringstream stream;
+        
+        write(caps, name, stream);
+        
+        return stream.str();
+    }
 
     //-----------------------------------------------------------------------
     void RenderSystemCapabilitiesSerializer::parseScript(DataStreamPtr& stream)
@@ -194,7 +143,7 @@ namespace Ogre
         // reset parsing data to NULL
         mCurrentLineNumber = 0;
         mCurrentLine = 0;
-        mCurrentStream.setNull();
+        mCurrentStream.reset();
         mCurrentCapabilities = 0;
 
         mCurrentStream = stream;
@@ -209,17 +158,17 @@ namespace Ogre
         // comment or empty line) for further processing
         CapabilitiesLinesList capabilitiesLines;
 
-		// for reading data
-		char tmpBuf[OGRE_STREAM_TEMP_SIZE]; 
+        // for reading data
+        char tmpBuf[OGRE_STREAM_TEMP_SIZE]; 
 
 
         // TODO: build a smarter tokenizer so that "{" and "}"
         // don't need separate lines
         while (!stream->eof())
         {
-			stream->readLine(tmpBuf, OGRE_STREAM_TEMP_SIZE-1);
+            stream->readLine(tmpBuf, OGRE_STREAM_TEMP_SIZE-1);
             line = String(tmpBuf);
-			StringUtil::trim(line);
+            StringUtil::trim(line);
 
             // keep track of parse position
             mCurrentLine = &line;
@@ -329,27 +278,27 @@ namespace Ogre
 
     void RenderSystemCapabilitiesSerializer::initialiaseDispatchTables()
     {
-		// set up driver version parsing
-		addKeywordType("driver_version", SET_STRING_METHOD);
+        // set up driver version parsing
+        addKeywordType("driver_version", SET_STRING_METHOD);
         // set up the setters for driver versions
         addSetStringMethod("driver_version", &RenderSystemCapabilities::parseDriverVersionFromString);
         
         // set up device name parsing
-		addKeywordType("device_name", SET_STRING_METHOD);
+        addKeywordType("device_name", SET_STRING_METHOD);
         // set up the setters for device names
         addSetStringMethod("device_name", &RenderSystemCapabilities::setDeviceName);
         
-		// set up render system name parsing
-		addKeywordType("render_system_name", SET_STRING_METHOD);
-		// set up the setters 
-		addSetStringMethod("render_system_name", &RenderSystemCapabilities::setRenderSystemName);
+        // set up render system name parsing
+        addKeywordType("render_system_name", SET_STRING_METHOD);
+        // set up the setters 
+        addSetStringMethod("render_system_name", &RenderSystemCapabilities::setRenderSystemName);
 
-		// set up vendor parsing
-		addKeywordType("vendor", SET_STRING_METHOD);
-		// set up the setters for driver versions
-		addSetStringMethod("vendor", &RenderSystemCapabilities::parseVendorFromString);
+        // set up vendor parsing
+        addKeywordType("vendor", SET_STRING_METHOD);
+        // set up the setters for driver versions
+        addSetStringMethod("vendor", &RenderSystemCapabilities::parseVendorFromString);
 
-		// initialize int types
+        // initialize int types
         addKeywordType("num_world_matrices", SET_INT_METHOD);
         addKeywordType("num_texture_units", SET_INT_METHOD);
         addKeywordType("stencil_buffer_bit_depth", SET_INT_METHOD);
@@ -364,19 +313,18 @@ namespace Ogre
         addKeywordType("geometry_program_constant_float_count", SET_INT_METHOD);
         addKeywordType("geometry_program_constant_int_count", SET_INT_METHOD);
         addKeywordType("geometry_program_constant_bool_count", SET_INT_METHOD);
-        addKeywordType("tesselation_hull_program_constant_float_count", SET_INT_METHOD);
-        addKeywordType("tesselation_hull_program_constant_int_count", SET_INT_METHOD);
-        addKeywordType("tesselation_hull_program_constant_bool_count", SET_INT_METHOD);
-        addKeywordType("tesselation_domain_program_constant_float_count", SET_INT_METHOD);
-        addKeywordType("tesselation_domain_program_constant_int_count", SET_INT_METHOD);
-        addKeywordType("tesselation_domain_program_constant_bool_count", SET_INT_METHOD);
+        addKeywordType("tessellation_hull_program_constant_float_count", SET_INT_METHOD);
+        addKeywordType("tessellation_hull_program_constant_int_count", SET_INT_METHOD);
+        addKeywordType("tessellation_hull_program_constant_bool_count", SET_INT_METHOD);
+        addKeywordType("tessellation_domain_program_constant_float_count", SET_INT_METHOD);
+        addKeywordType("tessellation_domain_program_constant_int_count", SET_INT_METHOD);
+        addKeywordType("tessellation_domain_program_constant_bool_count", SET_INT_METHOD);
         addKeywordType("compute_program_constant_float_count", SET_INT_METHOD);
         addKeywordType("compute_program_constant_int_count", SET_INT_METHOD);
         addKeywordType("compute_program_constant_bool_count", SET_INT_METHOD);
         addKeywordType("num_vertex_texture_units", SET_INT_METHOD);
 
         // initialize int setters
-        addSetIntMethod("num_world_matrices", &RenderSystemCapabilities::setNumWorldMatrices);
         addSetIntMethod("num_texture_units", &RenderSystemCapabilities::setNumTextureUnits);
         addSetIntMethod("stencil_buffer_bit_depth", &RenderSystemCapabilities::setStencilBufferBitDepth);
         addSetIntMethod("num_vertex_blend_matrices", &RenderSystemCapabilities::setNumVertexBlendMatrices);
@@ -390,12 +338,12 @@ namespace Ogre
         addSetIntMethod("geometry_program_constant_float_count", &RenderSystemCapabilities::setGeometryProgramConstantFloatCount);
         addSetIntMethod("geometry_program_constant_int_count", &RenderSystemCapabilities::setGeometryProgramConstantIntCount);
         addSetIntMethod("geometry_program_constant_bool_count", &RenderSystemCapabilities::setGeometryProgramConstantBoolCount);
-        addSetIntMethod("tesselation_hull_program_constant_float_count", &RenderSystemCapabilities::setTesselationHullProgramConstantFloatCount);
-        addSetIntMethod("tesselation_hull_program_constant_int_count", &RenderSystemCapabilities::setTesselationHullProgramConstantIntCount);
-        addSetIntMethod("tesselation_hull_program_constant_bool_count", &RenderSystemCapabilities::setTesselationHullProgramConstantBoolCount);
-        addSetIntMethod("tesselation_domain_program_constant_float_count", &RenderSystemCapabilities::setTesselationDomainProgramConstantFloatCount);
-        addSetIntMethod("tesselation_domain_program_constant_int_count", &RenderSystemCapabilities::setTesselationDomainProgramConstantIntCount);
-        addSetIntMethod("tesselation_domain_program_constant_bool_count", &RenderSystemCapabilities::setTesselationDomainProgramConstantBoolCount);
+        addSetIntMethod("tessellation_hull_program_constant_float_count", &RenderSystemCapabilities::setTessellationHullProgramConstantFloatCount);
+        addSetIntMethod("tessellation_hull_program_constant_int_count", &RenderSystemCapabilities::setTessellationHullProgramConstantIntCount);
+        addSetIntMethod("tessellation_hull_program_constant_bool_count", &RenderSystemCapabilities::setTessellationHullProgramConstantBoolCount);
+        addSetIntMethod("tessellation_domain_program_constant_float_count", &RenderSystemCapabilities::setTessellationDomainProgramConstantFloatCount);
+        addSetIntMethod("tessellation_domain_program_constant_int_count", &RenderSystemCapabilities::setTessellationDomainProgramConstantIntCount);
+        addSetIntMethod("tessellation_domain_program_constant_bool_count", &RenderSystemCapabilities::setTessellationDomainProgramConstantBoolCount);
         addSetIntMethod("compute_program_constant_float_count", &RenderSystemCapabilities::setComputeProgramConstantFloatCount);
         addSetIntMethod("compute_program_constant_int_count", &RenderSystemCapabilities::setComputeProgramConstantIntCount);
         addSetIntMethod("compute_program_constant_bool_count", &RenderSystemCapabilities::setComputeProgramConstantBoolCount);
@@ -418,70 +366,16 @@ namespace Ogre
         // there is no dispatch table for shader profiles, just the type
         addKeywordType("shader_profile", ADD_SHADER_PROFILE_STRING);
 
-        // set up RSC_XXX style capabilities
-		addKeywordType("fixed_function", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("automipmap", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("blending", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("anisotropy", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("dot3", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("cubemapping", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("hwstencil", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("vbo", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("vertex_program", SET_CAPABILITY_ENUM_BOOL);
-		addKeywordType("geometry_program", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("fragment_program", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("tesselation_hull_program", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("tesselation_domain_program", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("compute_program", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("scissor_test", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("two_sided_stencil", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("stencil_wrap", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("hwocclusion", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("user_clip_planes", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("vertex_format_ubyte4", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("infinite_far_plane", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("hwrender_to_texture", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_float", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("non_power_of_2_textures", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_1d", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_3d", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("point_sprites", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("point_extended_parameters", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("vertex_texture_fetch", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("mipmap_lod_bias", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("atomic_counters", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_dxt", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_vtc", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_pvrtc", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_atc", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_etc1", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_etc2", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_bc4_bc5", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("texture_compression_bc6h_bc7", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("gl1_5_novbo", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("fbo", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("fbo_arb", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("fbo_ati", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("pbuffer", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("gl1_5_nohwocclusion", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("perstageconstant", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("vao", SET_CAPABILITY_ENUM_BOOL);
-        addKeywordType("separate_shader_objects", SET_CAPABILITY_ENUM_BOOL);
-
-		addCapabilitiesMapping("fixed_function", RSC_FIXED_FUNCTION);
-        addCapabilitiesMapping("automipmap", RSC_AUTOMIPMAP);
-        addCapabilitiesMapping("blending", RSC_BLENDING);
+        addCapabilitiesMapping("fixed_function", RSC_FIXED_FUNCTION);
         addCapabilitiesMapping("anisotropy", RSC_ANISOTROPY);
         addCapabilitiesMapping("dot3", RSC_DOT3);
-        addCapabilitiesMapping("cubemapping", RSC_CUBEMAPPING);
         addCapabilitiesMapping("hwstencil", RSC_HWSTENCIL);
-        addCapabilitiesMapping("vbo", RSC_VBO);
+        addCapabilitiesMapping("32bit_index", RSC_32BIT_INDEX);
         addCapabilitiesMapping("vertex_program", RSC_VERTEX_PROGRAM);
-		addCapabilitiesMapping("geometry_program", RSC_GEOMETRY_PROGRAM);
+        addCapabilitiesMapping("geometry_program", RSC_GEOMETRY_PROGRAM);
         addCapabilitiesMapping("fragment_program", RSC_FRAGMENT_PROGRAM);
-        addCapabilitiesMapping("tesselation_hull_program", RSC_TESSELATION_HULL_PROGRAM);
-        addCapabilitiesMapping("tesselation_domain_program", RSC_TESSELATION_DOMAIN_PROGRAM);
+        addCapabilitiesMapping("tessellation_hull_program", RSC_TESSELLATION_HULL_PROGRAM);
+        addCapabilitiesMapping("tessellation_domain_program", RSC_TESSELLATION_DOMAIN_PROGRAM);
         addCapabilitiesMapping("compute_program", RSC_COMPUTE_PROGRAM);
         addCapabilitiesMapping("scissor_test", RSC_SCISSOR_TEST);
         addCapabilitiesMapping("two_sided_stencil", RSC_TWO_SIDED_STENCIL);
@@ -491,11 +385,12 @@ namespace Ogre
         addCapabilitiesMapping("vertex_format_ubyte4", RSC_VERTEX_FORMAT_UBYTE4);
         addCapabilitiesMapping("infinite_far_plane", RSC_INFINITE_FAR_PLANE);
         addCapabilitiesMapping("hwrender_to_texture", RSC_HWRENDER_TO_TEXTURE);
-		addCapabilitiesMapping("texture_float", RSC_TEXTURE_FLOAT);
+        addCapabilitiesMapping("texture_float", RSC_TEXTURE_FLOAT);
         addCapabilitiesMapping("non_power_of_2_textures", RSC_NON_POWER_OF_2_TEXTURES);
         addCapabilitiesMapping("texture_3d", RSC_TEXTURE_3D);
-		addCapabilitiesMapping("texture_1d", RSC_TEXTURE_1D);
+        addCapabilitiesMapping("texture_1d", RSC_TEXTURE_1D);
         addCapabilitiesMapping("point_sprites", RSC_POINT_SPRITES);
+        addCapabilitiesMapping("wide_lines", RSC_WIDE_LINES);
         addCapabilitiesMapping("point_extended_parameters", RSC_POINT_EXTENDED_PARAMETERS);
         addCapabilitiesMapping("vertex_texture_fetch", RSC_VERTEX_TEXTURE_FETCH);
         addCapabilitiesMapping("mipmap_lod_bias", RSC_MIPMAP_LOD_BIAS);
@@ -509,17 +404,17 @@ namespace Ogre
         addCapabilitiesMapping("texture_compression_etc2", RSC_TEXTURE_COMPRESSION_ETC2);
         addCapabilitiesMapping("texture_compression_bc4_bc5", RSC_TEXTURE_COMPRESSION_BC4_BC5);
         addCapabilitiesMapping("texture_compression_bc6h_bc7", RSC_TEXTURE_COMPRESSION_BC6H_BC7);
-		addCapabilitiesMapping("hwrender_to_vertex_buffer", RSC_HWRENDER_TO_VERTEX_BUFFER);
-        addCapabilitiesMapping("gl1_5_novbo", RSC_GL1_5_NOVBO);
-        addCapabilitiesMapping("fbo", RSC_FBO);
-        addCapabilitiesMapping("fbo_arb", RSC_FBO_ARB);
-        addCapabilitiesMapping("fbo_ati", RSC_FBO_ATI);
+        addCapabilitiesMapping("texture_compression_astc", RSC_TEXTURE_COMPRESSION_ASTC);
+        addCapabilitiesMapping("hwrender_to_vertex_buffer", RSC_HWRENDER_TO_VERTEX_BUFFER);
+
         addCapabilitiesMapping("pbuffer", RSC_PBUFFER);
-        addCapabilitiesMapping("gl1_5_nohwocclusion", RSC_GL1_5_NOHWOCCLUSION);
         addCapabilitiesMapping("perstageconstant", RSC_PERSTAGECONSTANT);
         addCapabilitiesMapping("vao", RSC_VAO);
         addCapabilitiesMapping("separate_shader_objects", RSC_SEPARATE_SHADER_OBJECTS);
-
+        addCapabilitiesMapping("glsl_sso_redeclare", RSC_GLSL_SSO_REDECLARE);
+        addCapabilitiesMapping("debug", RSC_DEBUG);
+        addCapabilitiesMapping("mapbuffer", RSC_MAPBUFFER);
+        addCapabilitiesMapping("automipmap_compressed", RSC_AUTOMIPMAP_COMPRESSED);
     }
 
     void RenderSystemCapabilitiesSerializer::parseCapabilitiesLines(CapabilitiesLinesList& lines)
@@ -596,13 +491,13 @@ namespace Ogre
     void RenderSystemCapabilitiesSerializer::logParseError(const String& error) const
     {
         // log the line with error in it if the current line is available
-        if (mCurrentLine != 0 && !mCurrentStream.isNull())
+        if (mCurrentLine != 0 && mCurrentStream)
         {
             LogManager::getSingleton().logMessage(
                 "Error in .rendercaps " + mCurrentStream->getName() + ":" + StringConverter::toString(mCurrentLineNumber) +
                 " : " + error);
         }
-        else if (!mCurrentStream.isNull())
+        else if (mCurrentStream)
         {
             LogManager::getSingleton().logMessage(
                 "Error in .rendercaps " + mCurrentStream->getName() +
