@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,27 +27,27 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 #include "OgreRenderTarget.h"
-#include "OgreStringConverter.h"
 
 #include "OgreViewport.h"
-#include "OgreException.h"
-#include "OgreLogManager.h"
 #include "OgreRenderTargetListener.h"
-#include "OgreRoot.h"
-#include "OgreRenderSystem.h"
 #include "OgreDepthBuffer.h"
-#include "OgreProfiler.h"
+#include "OgreTimer.h"
 
 namespace Ogre {
 
     RenderTarget::RenderTarget()
-		: mPriority(OGRE_DEFAULT_RT_GROUP)
-		, mDepthBufferPoolId(DepthBuffer::POOL_DEFAULT)
-		, mDepthBuffer(0)
-		, mActive(true)
-		, mAutoUpdate(true)
-		, mHwGamma(false)
-		, mFSAA(0)
+        : mPriority(OGRE_DEFAULT_RT_GROUP)
+        , mDepthBufferPoolId(DepthBuffer::POOL_DEFAULT)
+        , mDepthBuffer(0)
+        , mActive(true)
+        , mAutoUpdate(true)
+        , mHwGamma(false)
+        , mFSAA(0)
+#if OGRE_NO_QUAD_BUFFER_STEREO == 0
+		, mStereoEnabled(true)
+#else
+		, mStereoEnabled(false)
+#endif
     {
         mTimer = Root::getSingleton().getTimer();
         resetStatistics();
@@ -55,24 +55,27 @@ namespace Ogre {
 
     RenderTarget::~RenderTarget()
     {
+        // make a copy of the list to avoid crashes, the viewport destructor change the list
+        ViewportList vlist = mViewportList;
+        
         // Delete viewports
-        for (ViewportList::iterator i = mViewportList.begin();
-            i != mViewportList.end(); ++i)
+        for (ViewportList::iterator i = vlist.begin();
+            i != vlist.end(); ++i)
         {
             fireViewportRemoved(i->second);
             OGRE_DELETE (*i).second;
         }
 
-		//DepthBuffer keeps track of us, avoid a dangling pointer
-		detachDepthBuffer();
+        //DepthBuffer keeps track of us, avoid a dangling pointer
+        detachDepthBuffer();
 
 
         // Write closing message
-		LogManager::getSingleton().stream(LML_TRIVIAL)
-			<< "Render Target '" << mName << "' "
-			<< "Average FPS: " << mStats.avgFPS << " "
-			<< "Best FPS: " << mStats.bestFPS << " "
-			<< "Worst FPS: " << mStats.worstFPS; 
+        LogManager::getSingleton().stream(LML_TRIVIAL)
+            << "Render Target '" << mName << "' "
+            << "Average FPS: " << mStats.avgFPS << " "
+            << "Best FPS: " << mStats.bestFPS << " "
+            << "Worst FPS: " << mStats.worstFPS; 
 
     }
 
@@ -101,157 +104,157 @@ namespace Ogre {
     {
         return mColourDepth;
     }
-	//-----------------------------------------------------------------------
-	void RenderTarget::setDepthBufferPool( uint16 poolId )
-	{
-		if( mDepthBufferPoolId != poolId )
-		{
-			mDepthBufferPoolId = poolId;
-			detachDepthBuffer();
-		}
-	}
-	//-----------------------------------------------------------------------
-	uint16 RenderTarget::getDepthBufferPool() const
-	{
-		return mDepthBufferPoolId;
-	}
-	//-----------------------------------------------------------------------
-	DepthBuffer* RenderTarget::getDepthBuffer() const
-	{
-		return mDepthBuffer;
-	}
-	//-----------------------------------------------------------------------
-	bool RenderTarget::attachDepthBuffer( DepthBuffer *depthBuffer )
-	{
-		bool retVal = false;
+    //-----------------------------------------------------------------------
+    void RenderTarget::setDepthBufferPool( uint16 poolId )
+    {
+        if( mDepthBufferPoolId != poolId )
+        {
+            mDepthBufferPoolId = poolId;
+            detachDepthBuffer();
+        }
+    }
+    //-----------------------------------------------------------------------
+    uint16 RenderTarget::getDepthBufferPool() const
+    {
+        return mDepthBufferPoolId;
+    }
+    //-----------------------------------------------------------------------
+    DepthBuffer* RenderTarget::getDepthBuffer() const
+    {
+        return mDepthBuffer;
+    }
+    //-----------------------------------------------------------------------
+    bool RenderTarget::attachDepthBuffer( DepthBuffer *depthBuffer )
+    {
+        bool retVal = false;
 
-		if( (retVal = depthBuffer->isCompatible( this )) )
-		{
-			detachDepthBuffer();
-			mDepthBuffer = depthBuffer;
-			mDepthBuffer->_notifyRenderTargetAttached( this );
-		}
+        if( (retVal = depthBuffer->isCompatible( this )) )
+        {
+            detachDepthBuffer();
+            mDepthBuffer = depthBuffer;
+            mDepthBuffer->_notifyRenderTargetAttached( this );
+        }
 
-		return retVal;
-	}
-	//-----------------------------------------------------------------------
-	void RenderTarget::detachDepthBuffer()
-	{
-		if( mDepthBuffer )
-		{
-			mDepthBuffer->_notifyRenderTargetDetached( this );
-			mDepthBuffer = 0;
-		}
-	}
-	//-----------------------------------------------------------------------
-	void RenderTarget::_detachDepthBuffer()
-	{
-		mDepthBuffer = 0;
-	}
+        return retVal;
+    }
+    //-----------------------------------------------------------------------
+    void RenderTarget::detachDepthBuffer()
+    {
+        if( mDepthBuffer )
+        {
+            mDepthBuffer->_notifyRenderTargetDetached( this );
+            mDepthBuffer = 0;
+        }
+    }
+    //-----------------------------------------------------------------------
+    void RenderTarget::_detachDepthBuffer()
+    {
+        mDepthBuffer = 0;
+    }
 
     void RenderTarget::updateImpl(void)
     {
-		_beginUpdate();
-		_updateAutoUpdatedViewports(true);
-		_endUpdate();
+        _beginUpdate();
+        _updateAutoUpdatedViewports(true);
+        _endUpdate();
     }
 
-	void RenderTarget::_beginUpdate()
-	{
-		// notify listeners (pre)
+    void RenderTarget::_beginUpdate()
+    {
+        // notify listeners (pre)
         firePreUpdate();
 
         mStats.triangleCount = 0;
         mStats.batchCount = 0;
-	}
+    }
 
-	void RenderTarget::_updateAutoUpdatedViewports(bool updateStatistics)
-	{
-		// Go through viewports in Z-order
+    void RenderTarget::_updateAutoUpdatedViewports(bool updateStatistics)
+    {
+        // Go through viewports in Z-order
         // Tell each to refresh
-		ViewportList::iterator it = mViewportList.begin();
+        ViewportList::iterator it = mViewportList.begin();
         while (it != mViewportList.end())
         {
-			Viewport* viewport = (*it).second;
-			if(viewport->isAutoUpdated())
-			{
-				_updateViewport(viewport,updateStatistics);
-			}
-			++it;
-		}
-	}
+            Viewport* viewport = (*it).second;
+            if(viewport->isAutoUpdated())
+            {
+                _updateViewport(viewport,updateStatistics);
+            }
+            ++it;
+        }
+    }
 
-	void RenderTarget::_endUpdate()
-	{
-		 // notify listeners (post)
+    void RenderTarget::_endUpdate()
+    {
+         // notify listeners (post)
         firePostUpdate();
 
         // Update statistics (always on top)
         updateStats();
-	}
+    }
 
-	void RenderTarget::_updateViewport(Viewport* viewport, bool updateStatistics)
-	{
-		assert(viewport->getTarget() == this &&
-				"RenderTarget::_updateViewport the requested viewport is "
-				"not bound to the rendertarget!");
+    void RenderTarget::_updateViewport(Viewport* viewport, bool updateStatistics)
+    {
+        assert(viewport->getTarget() == this &&
+                "RenderTarget::_updateViewport the requested viewport is "
+                "not bound to the rendertarget!");
 
-		fireViewportPreUpdate(viewport);
-		viewport->update();
-		if(updateStatistics)
-		{
-			mStats.triangleCount += viewport->_getNumRenderedFaces();
-			mStats.batchCount += viewport->_getNumRenderedBatches();
-		}
-		fireViewportPostUpdate(viewport);
-	}
+        fireViewportPreUpdate(viewport);
+        viewport->update();
+        if(updateStatistics)
+        {
+            mStats.triangleCount += viewport->_getNumRenderedFaces();
+            mStats.batchCount += viewport->_getNumRenderedBatches();
+        }
+        fireViewportPostUpdate(viewport);
+    }
 
-	void RenderTarget::_updateViewport(int zorder, bool updateStatistics)
-	{
-		ViewportList::iterator it = mViewportList.find(zorder);
+    void RenderTarget::_updateViewport(int zorder, bool updateStatistics)
+    {
+        ViewportList::iterator it = mViewportList.find(zorder);
         if (it != mViewportList.end())
         {
-			_updateViewport((*it).second,updateStatistics);
-		}
-		else
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,"No viewport with given zorder : "
-				+ StringConverter::toString(zorder), "RenderTarget::_updateViewport");
-		}
-	}
+            _updateViewport((*it).second,updateStatistics);
+        }
+        else
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,"No viewport with given zorder : "
+                + StringConverter::toString(zorder), "RenderTarget::_updateViewport");
+        }
+    }
 
     Viewport* RenderTarget::addViewport(Camera* cam, int ZOrder, float left, float top ,
         float width , float height)
-    {		
+    {       
         // Check no existing viewport with this Z-order
         ViewportList::iterator it = mViewportList.find(ZOrder);
 
         if (it != mViewportList.end())
         {
-			StringUtil::StrStreamType str;
-			str << "Can't create another viewport for "
-				<< mName << " with Z-order " << ZOrder
-				<< " because a viewport exists with this Z-order already.";
-			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, str.str(), "RenderTarget::addViewport");
+            StringStream str;
+            str << "Can't create another viewport for "
+                << mName << " with Z-order " << ZOrder
+                << " because a viewport exists with this Z-order already.";
+            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, str.str(), "RenderTarget::addViewport");
         }
         // Add viewport to list
         // Order based on Z-order
         Viewport* vp = OGRE_NEW Viewport(cam, this, left, top, width, height, ZOrder);
 
-        mViewportList.insert(ViewportList::value_type(ZOrder, vp));
+        mViewportList.emplace(ZOrder, vp);
 
-		fireViewportAdded(vp);
+        fireViewportAdded(vp);
 
         return vp;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void RenderTarget::removeViewport(int ZOrder)
     {
         ViewportList::iterator it = mViewportList.find(ZOrder);
 
         if (it != mViewportList.end())
         {
-			fireViewportRemoved((*it).second);
+            fireViewportRemoved(it->second);
             OGRE_DELETE (*it).second;
             mViewportList.erase(ZOrder);
         }
@@ -259,9 +262,10 @@ namespace Ogre {
 
     void RenderTarget::removeAllViewports(void)
     {
+        // make a copy of the list to avoid crashes, the viewport destructor change the list
+        ViewportList vlist = mViewportList;
 
-
-        for (ViewportList::iterator it = mViewportList.begin(); it != mViewportList.end(); ++it)
+        for (ViewportList::iterator it = vlist.begin(); it != vlist.end(); ++it)
         {
             fireViewportRemoved(it->second);
             OGRE_DELETE (*it).second;
@@ -271,61 +275,6 @@ namespace Ogre {
 
     }
 
-    void RenderTarget::getStatistics(float& lastFPS, float& avgFPS,
-        float& bestFPS, float& worstFPS) const
-    {
-
-        // Note - the will have been updated by the last render
-        lastFPS = mStats.lastFPS;
-        avgFPS = mStats.avgFPS;
-        bestFPS = mStats.bestFPS;
-        worstFPS = mStats.worstFPS;
-
-
-    }
-
-    const RenderTarget::FrameStats& RenderTarget::getStatistics(void) const
-    {
-        return mStats;
-    }
-
-    float RenderTarget::getLastFPS() const
-    {
-        return mStats.lastFPS;
-    }
-    float RenderTarget::getAverageFPS() const
-    {
-        return mStats.avgFPS;
-    }
-    float RenderTarget::getBestFPS() const
-    {
-        return mStats.bestFPS;
-    }
-    float RenderTarget::getWorstFPS() const
-    {
-        return mStats.worstFPS;
-    }
-
-    size_t RenderTarget::getTriangleCount(void) const
-    {
-        return mStats.triangleCount;
-    }
-
-    size_t RenderTarget::getBatchCount(void) const
-    {
-        return mStats.batchCount;
-    }
-
-    float RenderTarget::getBestFrameTime() const
-    {
-        return (float)mStats.bestFrameTime;
-    }
-
-    float RenderTarget::getWorstFrameTime() const
-    {
-        return (float)mStats.worstFrameTime;
-    }
-
     void RenderTarget::resetStatistics(void)
     {
         mStats.avgFPS = 0.0;
@@ -336,6 +285,7 @@ namespace Ogre {
         mStats.batchCount = 0;
         mStats.bestFrameTime = 999999;
         mStats.worstFrameTime = 0;
+        mStats.vBlankMissCount = -1;
 
         mLastTime = mTimer->getMilliseconds();
         mLastSecond = mLastTime;
@@ -382,21 +332,24 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void RenderTarget::addListener(RenderTargetListener* listener)
     {
-        mListeners.push_back(listener);
+        if (std::find(mListeners.begin(), mListeners.end(), listener) == mListeners.end())
+            mListeners.push_back(listener);
+    }
+    //-----------------------------------------------------------------------
+    void RenderTarget::insertListener(RenderTargetListener* listener, const unsigned int pos)
+    {
+        // if the position is larger than the list size we just set the listener at the end of the list
+        if (pos > mListeners.size())
+            mListeners.push_back(listener);
+        else
+            mListeners.insert(mListeners.begin() + pos, listener);
     }
     //-----------------------------------------------------------------------
     void RenderTarget::removeListener(RenderTargetListener* listener)
     {
-        RenderTargetListenerList::iterator i;
-        for (i = mListeners.begin(); i != mListeners.end(); ++i)
-        {
-            if (*i == listener)
-            {
-                mListeners.erase(i);
-                break;
-            }
-        }
-
+        RenderTargetListenerList::iterator i = std::find(mListeners.begin(), mListeners.end(), listener);
+        if (i != mListeners.end())
+            mListeners.erase(i);
     }
     //-----------------------------------------------------------------------
     void RenderTarget::removeAllListeners(void)
@@ -449,22 +402,22 @@ namespace Ogre {
             ++i;
         return i->second;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     Viewport* RenderTarget::getViewportByZOrder(int ZOrder)
     {
-		ViewportList::iterator i = mViewportList.find(ZOrder);
-		if(i == mViewportList.end())
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,"No viewport with given Z-order: "
-				+ StringConverter::toString(ZOrder), "RenderTarget::getViewportByZOrder");
-		}
+        ViewportList::iterator i = mViewportList.find(ZOrder);
+        if(i == mViewportList.end())
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,"No viewport with given Z-order: "
+                + StringConverter::toString(ZOrder), "RenderTarget::getViewportByZOrder");
+        }
         return i->second;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     bool RenderTarget::hasViewportWithZOrder(int ZOrder)
     {
-		ViewportList::iterator i = mViewportList.find(ZOrder);
-		return i != mViewportList.end();
+        ViewportList::iterator i = mViewportList.find(ZOrder);
+        return i != mViewportList.end();
     }
     //-----------------------------------------------------------------------
     bool RenderTarget::isActive() const
@@ -504,71 +457,69 @@ namespace Ogre {
             (*i)->postViewportUpdate(evt);
         }
     }
-	//-----------------------------------------------------------------------
-	void RenderTarget::fireViewportAdded(Viewport* vp)
-	{
-		RenderTargetViewportEvent evt;
-		evt.source = vp;
+    //-----------------------------------------------------------------------
+    void RenderTarget::fireViewportAdded(Viewport* vp)
+    {
+        RenderTargetViewportEvent evt;
+        evt.source = vp;
 
-		RenderTargetListenerList::iterator i, iend;
-		i = mListeners.begin();
-		iend = mListeners.end();
-		for(; i != iend; ++i)
-		{
-			(*i)->viewportAdded(evt);
-		}
-	}
-	//-----------------------------------------------------------------------
-	void RenderTarget::fireViewportRemoved(Viewport* vp)
-	{
-		RenderTargetViewportEvent evt;
-		evt.source = vp;
+        RenderTargetListenerList::iterator i, iend;
+        i = mListeners.begin();
+        iend = mListeners.end();
+        for(; i != iend; ++i)
+        {
+            (*i)->viewportAdded(evt);
+        }
+    }
+    //-----------------------------------------------------------------------
+    void RenderTarget::fireViewportRemoved(Viewport* vp)
+    {
+        RenderTargetViewportEvent evt;
+        evt.source = vp;
 
-		// Make a temp copy of the listeners
-		// some will want to remove themselves as listeners when they get this
-		RenderTargetListenerList tempList = mListeners;
+        // Make a temp copy of the listeners
+        // some will want to remove themselves as listeners when they get this
+        RenderTargetListenerList tempList = mListeners;
 
-		RenderTargetListenerList::iterator i, iend;
-		i = tempList.begin();
-		iend = tempList.end();
-		for(; i != iend; ++i)
-		{
-			(*i)->viewportRemoved(evt);
-		}
-	}
+        RenderTargetListenerList::iterator i, iend;
+        i = tempList.begin();
+        iend = tempList.end();
+        for(; i != iend; ++i)
+        {
+            (*i)->viewportRemoved(evt);
+        }
+    }
     //-----------------------------------------------------------------------
     String RenderTarget::writeContentsToTimestampedFile(const String& filenamePrefix, const String& filenameSuffix)
     {
         struct tm *pTime;
         time_t ctTime; time(&ctTime);
         pTime = localtime( &ctTime );
-        Ogre::StringStream oss;
-        oss	<< std::setw(2) << std::setfill('0') << (pTime->tm_mon + 1)
-            << std::setw(2) << std::setfill('0') << pTime->tm_mday
-            << std::setw(2) << std::setfill('0') << (pTime->tm_year + 1900)
-            << "_" << std::setw(2) << std::setfill('0') << pTime->tm_hour
-            << std::setw(2) << std::setfill('0') << pTime->tm_min
-            << std::setw(2) << std::setfill('0') << pTime->tm_sec
-            << std::setw(3) << std::setfill('0') << (mTimer->getMilliseconds() % 1000);
-        String filename = filenamePrefix + oss.str() + filenameSuffix;
+        // use ISO 8601 order
+        StringStream oss;
+        oss << filenamePrefix
+            << std::put_time(pTime, "%Y%m%d_%H%M%S")
+            << std::setw(3) << std::setfill('0') << (mTimer->getMilliseconds() % 1000)
+            << filenameSuffix;
+        String filename = oss.str();
         writeContentsToFile(filename);
         return filename;
 
     }
-	//-----------------------------------------------------------------------
-	void RenderTarget::writeContentsToFile(const String& filename)
-	{
-		PixelFormat pf = suggestPixelFormat();
+    //-----------------------------------------------------------------------
+    void RenderTarget::writeContentsToFile(const String& filename)
+    {
+        PixelFormat pf = suggestPixelFormat();
 
-		uchar *data = OGRE_ALLOC_T(uchar, mWidth * mHeight * PixelUtil::getNumElemBytes(pf), MEMCATEGORY_RENDERSYS);
-		PixelBox pb(mWidth, mHeight, 1, pf, data);
+        uchar *data = OGRE_ALLOC_T(uchar, mWidth * mHeight * PixelUtil::getNumElemBytes(pf), MEMCATEGORY_RENDERSYS);
+        PixelBox pb(mWidth, mHeight, 1, pf, data);
 
-		copyContentsToMemory(pb);
+        copyContentsToMemory(pb, pb);
 
-		Image().loadDynamicImage(data, mWidth, mHeight, 1, pf, false, 1, 0).save(filename);
+        Image().loadDynamicImage(data, mWidth, mHeight, 1, pf, false, 1, 0).save(filename);
 
-		OGRE_FREE(data, MEMCATEGORY_RENDERSYS);
-	}
+        OGRE_FREE(data, MEMCATEGORY_RENDERSYS);
+    }
     //-----------------------------------------------------------------------
     void RenderTarget::_notifyCameraRemoved(const Camera* cam)
     {
@@ -599,6 +550,11 @@ namespace Ogre {
     {
         // RenderWindow will override and return true for the primary window
         return false;
+    }  
+	//-----------------------------------------------------------------------
+    bool RenderTarget::isStereoEnabled(void) const
+    {
+        return mStereoEnabled;
     }
     //-----------------------------------------------------------------------
     RenderTarget::Impl *RenderTarget::_getImpl()
@@ -608,18 +564,18 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void RenderTarget::update(bool swap)
     {
-        OgreProfileBeginGPUEvent("RenderTarget: " + getName());
+        OgreProfileBeginGPUEvent(getName());
         // call implementation
         updateImpl();
 
 
-		if (swap)
-		{
-			// Swap buffers
-    	    swapBuffers();
-		}
-        OgreProfileEndGPUEvent("RenderTarget: " + getName());
+        if (swap)
+        {
+            // Swap buffers
+            swapBuffers();
+        }
+        OgreProfileEndGPUEvent(getName());
     }
-	
+    
 
 }        
