@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,11 +28,6 @@ THE SOFTWARE.
 #include "OgreStableHeaders.h"
 #include "OgreRenderWindow.h"
 
-#include "OgreRoot.h"
-#include "OgreRenderSystem.h"
-#include "OgreViewport.h"
-#include "OgreSceneManager.h"
-
 namespace Ogre {
 
     RenderWindow::RenderWindow()
@@ -43,7 +38,7 @@ namespace Ogre {
 
     //-----------------------------------------------------------------------
     void RenderWindow::getMetrics(unsigned int& width, unsigned int& height, unsigned int& colourDepth,
-		int& left, int& top)
+        int& left, int& top) const
     {
         width = mWidth;
         height = mHeight;
@@ -56,7 +51,7 @@ namespace Ogre {
     {
         return mIsFullScreen;
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     bool RenderWindow::isPrimary(void) const
     {
         return mIsPrimary;
