@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2011 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,41 +26,22 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreSceneNode.h"
-
-#include "OgreException.h"
-#include "OgreEntity.h"
-#include "OgreCamera.h"
-#include "OgreLight.h"
-#include "OgreMath.h"
-#include "OgreSceneManager.h"
-#include "OgreMovableObject.h"
-#include "OgreWireBoundingBox.h"
 
 namespace Ogre {
     //-----------------------------------------------------------------------
-    SceneNode::SceneNode(SceneManager* creator)
-        : Node()
-        , mWireBoundingBox(0)
-        , mShowBoundingBox(false)
-        , mHideBoundingBox(false)
-        , mCreator(creator)
-        , mYawFixed(false)
-        , mAutoTrackTarget(0)
-        , mIsInSceneGraph(false)
+    SceneNode::SceneNode(SceneManager* creator) : SceneNode(creator, BLANKSTRING)
     {
-        needUpdate();
     }
     //-----------------------------------------------------------------------
     SceneNode::SceneNode(SceneManager* creator, const String& name)
         : Node(name)
-        , mWireBoundingBox(0)
+        , mCreator(creator)
+        , mAutoTrackTarget(0)
+        , mGlobalIndex(-1)
+        , mYawFixed(false)
+        , mIsInSceneGraph(false)
         , mShowBoundingBox(false)
         , mHideBoundingBox(false)
-        , mCreator(creator)
-        , mYawFixed(false)
-        , mAutoTrackTarget(0)
-        , mIsInSceneGraph(false)
     {
         needUpdate();
     }
@@ -69,18 +50,11 @@ namespace Ogre {
     {
         // Detach all objects, do this manually to avoid needUpdate() call 
         // which can fail because of deleted items
-		ObjectMap::iterator itr;
-		MovableObject* ret;
-		for ( itr = mObjectsByName.begin(); itr != mObjectsByName.end(); itr++ )
-		{
-		  ret = itr->second;
-		  ret->_notifyAttached((SceneNode*)0);
-		}
+        for (ObjectMap::iterator itr = mObjectsByName.begin(); itr != mObjectsByName.end(); ++itr )
+        {
+            (*itr)->_notifyAttached((SceneNode*)0);
+        }
         mObjectsByName.clear();
-
-        if (mWireBoundingBox) {
-			OGRE_DELETE mWireBoundingBox;
-		}
     }
     //-----------------------------------------------------------------------
     void SceneNode::_update(bool updateChildren, bool parentHasChanged)
@@ -89,36 +63,41 @@ namespace Ogre {
         _updateBounds();
     }
     //-----------------------------------------------------------------------
-	void SceneNode::setParent(Node* parent)
-	{
-		Node::setParent(parent);
+    void SceneNode::setParent(Node* parent)
+    {
+        Node::setParent(parent);
 
-		if (parent)
-		{
-			SceneNode* sceneParent = static_cast<SceneNode*>(parent);
-			setInSceneGraph(sceneParent->isInSceneGraph());
-		}
-		else
-		{
-			setInSceneGraph(false);
-		}
-	}
+        if (parent)
+        {
+            SceneNode* sceneParent = static_cast<SceneNode*>(parent);
+            setInSceneGraph(sceneParent->isInSceneGraph());
+        }
+        else
+        {
+            setInSceneGraph(false);
+        }
+    }
     //-----------------------------------------------------------------------
-	void SceneNode::setInSceneGraph(bool inGraph)
-	{
-		if (inGraph != mIsInSceneGraph)
-		{
-			mIsInSceneGraph = inGraph;
-			// Tell children
-	        ChildNodeMap::iterator child;
-    	    for (child = mChildren.begin(); child != mChildren.end(); ++child)
-        	{
-            	SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
-				sceneChild->setInSceneGraph(inGraph);
-			}
-		}
-	}
+    void SceneNode::setInSceneGraph(bool inGraph)
+    {
+        if (inGraph != mIsInSceneGraph)
+        {
+            mIsInSceneGraph = inGraph;
+            // Tell children
+            for (ChildNodeMap::iterator child = mChildren.begin(); child != mChildren.end(); ++child)
+            {
+                SceneNode* sceneChild = static_cast<SceneNode*>(*child);
+                sceneChild->setInSceneGraph(inGraph);
+            }
+        }
+    }
     //-----------------------------------------------------------------------
+    struct MovableObjectNameExists {
+        const String& name;
+        bool operator()(const MovableObject* mo) {
+            return mo->getName() == name;
+        }
+    };
     void SceneNode::attachObject(MovableObject* obj)
     {
         if (obj->isAttached())
@@ -131,10 +110,11 @@ namespace Ogre {
         obj->_notifyAttached(this);
 
         // Also add to name index
-        std::pair<ObjectMap::iterator, bool> insresult = 
-            mObjectsByName.insert(ObjectMap::value_type(obj->getName(), obj));
-        assert(insresult.second && "Object was not attached because an object of the "
-            "same name was already attached to this node.");
+        MovableObjectNameExists pred = {obj->getName()};
+        ObjectMap::iterator it = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
+        OgreAssert(it == mObjectsByName.end(), "Object was not attached because an object of the "
+                                               "same name was already attached to this node.");
+        mObjectsByName.push_back(obj);
 
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
@@ -149,11 +129,7 @@ namespace Ogre {
     {
         if (index < mObjectsByName.size())
         {
-            ObjectMap::iterator i = mObjectsByName.begin();
-            // Increment (must do this one at a time)            
-            while (index--)++i;
-
-            return i->second;
+            return mObjectsByName[index];
         }
         else
         {
@@ -164,7 +140,8 @@ namespace Ogre {
     MovableObject* SceneNode::getAttachedObject(const String& name)
     {
         // Look up 
-        ObjectMap::iterator i = mObjectsByName.find(name);
+        MovableObjectNameExists pred = {name};
+        ObjectMap::iterator i = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
 
         if (i == mObjectsByName.end())
         {
@@ -172,29 +149,27 @@ namespace Ogre {
                 name + " not found.", "SceneNode::getAttachedObject");
         }
 
-        return i->second;
-
+        return *i;
     }
     //-----------------------------------------------------------------------
     MovableObject* SceneNode::detachObject(unsigned short index)
     {
-        MovableObject* ret;
         if (index < mObjectsByName.size())
         {
 
             ObjectMap::iterator i = mObjectsByName.begin();
-            // Increment (must do this one at a time)            
-            while (index--)++i;
+            i += index;
+
+            MovableObject* ret = *i;
+            std::swap(*i, mObjectsByName.back());
+            mObjectsByName.pop_back();
 
-            ret = i->second;
-            mObjectsByName.erase(i);
             ret->_notifyAttached((SceneNode*)0);
 
             // Make sure bounds get updated (must go right to the top)
             needUpdate();
 
             return ret;
-
         }
         else
         {
@@ -205,14 +180,19 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     MovableObject* SceneNode::detachObject(const String& name)
     {
-        ObjectMap::iterator it = mObjectsByName.find(name);
+        MovableObjectNameExists pred = {name};
+        ObjectMap::iterator it = std::find_if(mObjectsByName.begin(), mObjectsByName.end(), pred);
+
         if (it == mObjectsByName.end())
         {
             OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Object " + name + " is not attached "
                 "to this node.", "SceneNode::detachObject");
         }
-        MovableObject* ret = it->second;
-        mObjectsByName.erase(it);
+
+        MovableObject* ret = *it;
+        std::swap(*it, mObjectsByName.back());
+        mObjectsByName.pop_back();
+
         ret->_notifyAttached((SceneNode*)0);
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
@@ -227,9 +207,10 @@ namespace Ogre {
         iend = mObjectsByName.end();
         for (i = mObjectsByName.begin(); i != iend; ++i)
         {
-            if (i->second == obj)
+            if (*i == obj)
             {
-                mObjectsByName.erase(i);
+                std::swap(*i, mObjectsByName.back());
+                mObjectsByName.pop_back();
                 break;
             }
         }
@@ -242,13 +223,10 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void SceneNode::detachAllObjects(void)
     {
-		ObjectMap::iterator itr;
-		MovableObject* ret;
-		for ( itr = mObjectsByName.begin(); itr != mObjectsByName.end(); itr++ )
-		{
-		  ret = itr->second;
-		  ret->_notifyAttached((SceneNode*)0);
-		}
+        for (ObjectMap::iterator itr = mObjectsByName.begin(); itr != mObjectsByName.end(); ++itr )
+        {
+            (*itr)->_notifyAttached((SceneNode*)0);
+        }
         mObjectsByName.clear();
         // Make sure bounds get updated (must go right to the top)
         needUpdate();
@@ -264,22 +242,22 @@ namespace Ogre {
         for (i = mObjectsByName.begin(); i != mObjectsByName.end(); ++i)
         {
             // Merge world bounds of each object
-            mWorldAABB.merge(i->second->getWorldBoundingBox(true));
+            mWorldAABB.merge((*i)->getWorldBoundingBox(true));
         }
 
         // Merge with children
         ChildNodeMap::iterator child;
         for (child = mChildren.begin(); child != mChildren.end(); ++child)
         {
-            SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
+            SceneNode* sceneChild = static_cast<SceneNode*>(*child);
             mWorldAABB.merge(sceneChild->mWorldAABB);
         }
 
     }
     //-----------------------------------------------------------------------
     void SceneNode::_findVisibleObjects(Camera* cam, RenderQueue* queue, 
-		VisibleObjectsBoundsInfo* visibleBounds, bool includeChildren, 
-		bool displayNodes, bool onlyShadowCasters)
+        VisibleObjectsBoundsInfo* visibleBounds, bool includeChildren, 
+        bool displayNodes, bool onlyShadowCasters)
     {
         // Check self visible
         if (!cam->isVisible(mWorldAABB))
@@ -290,9 +268,9 @@ namespace Ogre {
         ObjectMap::iterator iobjend = mObjectsByName.end();
         for (iobj = mObjectsByName.begin(); iobj != iobjend; ++iobj)
         {
-			MovableObject* mo = iobj->second;
+            MovableObject* mo = *iobj;
 
-			queue->processVisibleObject(mo, cam, onlyShadowCasters, visibleBounds);
+            queue->processVisibleObject(mo, cam, onlyShadowCasters, visibleBounds);
         }
 
         if (includeChildren)
@@ -301,9 +279,9 @@ namespace Ogre {
             childend = mChildren.end();
             for (child = mChildren.begin(); child != childend; ++child)
             {
-                SceneNode* sceneChild = static_cast<SceneNode*>(child->second);
+                SceneNode* sceneChild = static_cast<SceneNode*>(*child);
                 sceneChild->_findVisibleObjects(cam, queue, visibleBounds, includeChildren, 
-					displayNodes, onlyShadowCasters);
+                    displayNodes, onlyShadowCasters);
             }
         }
 
@@ -313,47 +291,35 @@ namespace Ogre {
             queue->addRenderable(getDebugRenderable());
         }
 
-		// Check if the bounding box should be shown.
-		// See if our flag is set or if the scene manager flag is set.
-		if ( !mHideBoundingBox &&
+        // Check if the bounding box should be shown.
+        // See if our flag is set or if the scene manager flag is set.
+        if ( !mHideBoundingBox &&
              (mShowBoundingBox || (mCreator && mCreator->getShowBoundingBoxes())) )
-		{ 
-			_addBoundingBoxToQueue(queue);
-		}
+        { 
+            _addBoundingBoxToQueue(queue);
+        }
 
 
     }
 
-	Node::DebugRenderable* SceneNode::getDebugRenderable()
-	{
-		Vector3 hs = mWorldAABB.getHalfSize();
-		Real sz = std::min(hs.x, hs.y);
-		sz = std::min(sz, hs.z);
-		sz = std::max(sz, (Real)1.0);
-		return Node::getDebugRenderable(sz);
-	}
+    Node::DebugRenderable* SceneNode::getDebugRenderable()
+    {
+        Vector3 hs = mWorldAABB.getHalfSize();
+        Real sz = std::min(hs.x, hs.y);
+        sz = std::min(sz, hs.z);
+        sz = std::max(sz, (Real)1.0);
+        return Node::getDebugRenderable(sz);
+    }
 
 
-	void SceneNode::_addBoundingBoxToQueue(RenderQueue* queue) {
-		// Create a WireBoundingBox if needed.
-		if (mWireBoundingBox == NULL) {
-			mWireBoundingBox = OGRE_NEW WireBoundingBox();
-		}
-		mWireBoundingBox->setupBoundingBox(mWorldAABB);
-		queue->addRenderable(mWireBoundingBox);
-	}
-
-	void SceneNode::showBoundingBox(bool bShow) {
-		mShowBoundingBox = bShow;
-	}
-
-	bool SceneNode::getShowBoundingBox() const {
-		return mShowBoundingBox;
-	}
-
-	void SceneNode::hideBoundingBox(bool bHide) {
-		mHideBoundingBox = bHide;
-	}
+    void SceneNode::_addBoundingBoxToQueue(RenderQueue* queue) {
+        // Create a WireBoundingBox if needed.
+        if (!mWireBoundingBox) {
+            mWireBoundingBox.reset(new WireBoundingBox());
+        }
+        mWireBoundingBox->setupBoundingBox(mWorldAABB);
+        queue->addRenderable(mWireBoundingBox.get());
+    }
 
     //-----------------------------------------------------------------------
     void SceneNode::updateFromParentImpl(void) const
@@ -361,11 +327,9 @@ namespace Ogre {
         Node::updateFromParentImpl();
 
         // Notify objects that it has been moved
-        ObjectMap::const_iterator i;
-        for (i = mObjectsByName.begin(); i != mObjectsByName.end(); ++i)
+        for (ObjectMap::const_iterator i = mObjectsByName.begin(); i != mObjectsByName.end(); ++i)
         {
-            MovableObject* object = i->second;
-            object->_notifyMoved();
+            (*i)->_notifyMoved();
         }
     }
     //-----------------------------------------------------------------------
@@ -381,21 +345,6 @@ namespace Ogre {
         return mCreator->createSceneNode(name);
     }
     //-----------------------------------------------------------------------
-    const AxisAlignedBox& SceneNode::_getWorldAABB(void) const
-    {
-        return mWorldAABB;
-    }
-    //-----------------------------------------------------------------------
-    SceneNode::ObjectIterator SceneNode::getAttachedObjectIterator(void)
-    {
-        return ObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
-    }
-	//-----------------------------------------------------------------------
-	SceneNode::ConstObjectIterator SceneNode::getAttachedObjectIterator(void) const
-	{
-		return ConstObjectIterator(mObjectsByName.begin(), mObjectsByName.end());
-	}
-    //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyChild(const String& name)
     {
         SceneNode* pChild = static_cast<SceneNode*>(getChild(name));
@@ -408,41 +357,43 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyChild(unsigned short index)
     {
-        SceneNode* pChild = static_cast<SceneNode*>(getChild(index));
+        SceneNode* pChild = static_cast<SceneNode*>(mChildren[index]);
         pChild->removeAndDestroyAllChildren();
 
         removeChild(index);
-        pChild->getCreator()->destroySceneNode(pChild->getName());
+        pChild->getCreator()->destroySceneNode(pChild);
+    }
+    //-----------------------------------------------------------------------
+    void SceneNode::removeAndDestroyChild(SceneNode* child)
+    {
+        removeAndDestroyChild(std::find(mChildren.begin(), mChildren.end(), child) - mChildren.begin());
     }
     //-----------------------------------------------------------------------
     void SceneNode::removeAndDestroyAllChildren(void)
     {
-        ChildNodeMap::iterator i, iend;
-        iend = mChildren.end();
-        for (i = mChildren.begin(); i != iend;)
-        {
-            SceneNode* sn = static_cast<SceneNode*>(i->second);
-			// increment iterator before destroying (iterator invalidated by 
-			// SceneManager::destroySceneNode because it causes removal from parent)
-			++i;
+        // do not store iterators (invalidated by
+        // SceneManager::destroySceneNode because it causes removal from parent)
+        while(!mChildren.empty()) {
+            SceneNode* sn = static_cast<SceneNode*>(*mChildren.begin());
             sn->removeAndDestroyAllChildren();
-            sn->getCreator()->destroySceneNode(sn->getName());
+            sn->getCreator()->destroySceneNode(sn);
         }
-	    mChildren.clear();
+
+        mChildren.clear();
         needUpdate();
     }
     //-----------------------------------------------------------------------
-	SceneNode* SceneNode::createChildSceneNode(const Vector3& inTranslate, 
+    SceneNode* SceneNode::createChildSceneNode(const Vector3& inTranslate, 
         const Quaternion& inRotate)
-	{
-		return static_cast<SceneNode*>(this->createChild(inTranslate, inRotate));
-	}
+    {
+        return static_cast<SceneNode*>(this->createChild(inTranslate, inRotate));
+    }
     //-----------------------------------------------------------------------
     SceneNode* SceneNode::createChildSceneNode(const String& name, const Vector3& inTranslate, 
-		const Quaternion& inRotate)
-	{
-		return static_cast<SceneNode*>(this->createChild(name, inTranslate, inRotate));
-	}
+        const Quaternion& inRotate)
+    {
+        return static_cast<SceneNode*>(this->createChild(name, inTranslate, inRotate));
+    }
     //-----------------------------------------------------------------------
     void SceneNode::findLights(LightList& destList, Real radius, uint32 lightMask) const
     {
@@ -464,7 +415,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void SceneNode::setAutoTracking(bool enabled, SceneNode* target, 
+    void SceneNode::setAutoTracking(bool enabled, SceneNode* const target, 
         const Vector3& localDirectionVector,
         const Vector3& offset)
     {
@@ -488,19 +439,19 @@ namespace Ogre {
         mYawFixedAxis = fixedAxis;
     }
 
-	//-----------------------------------------------------------------------
-	void SceneNode::yaw(const Radian& angle, TransformSpace relativeTo)
-	{
-		if (mYawFixed)
-		{
-			rotate(mYawFixedAxis, angle, relativeTo);
-		}
-		else
-		{
-			rotate(Vector3::UNIT_Y, angle, relativeTo);
-		}
+    //-----------------------------------------------------------------------
+    void SceneNode::yaw(const Radian& angle, TransformSpace relativeTo)
+    {
+        if (mYawFixed)
+        {
+            rotate(mYawFixedAxis, angle, relativeTo);
+        }
+        else
+        {
+            rotate(Vector3::UNIT_Y, angle, relativeTo);
+        }
 
-	}
+    }
     //-----------------------------------------------------------------------
     void SceneNode::setDirection(Real x, Real y, Real z, TransformSpace relativeTo, 
         const Vector3& localDirectionVector)
@@ -543,7 +494,14 @@ namespace Ogre {
         if( mYawFixed )
         {
             // Calculate the quaternion for rotate local Z to target direction
-            Vector3 xVec = mYawFixedAxis.crossProduct(targetDir);
+            Vector3 yawAxis = mYawFixedAxis;
+
+            if (mInheritOrientation && mParent)
+            {
+                yawAxis = mParent->_getDerivedOrientation() * yawAxis;
+            }
+
+            Vector3 xVec = yawAxis.crossProduct(targetDir);
             xVec.normalise();
             Vector3 yVec = targetDir.crossProduct(xVec);
             yVec.normalise();
@@ -636,7 +594,7 @@ namespace Ogre {
         oiend = mObjectsByName.end();
         for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
         {
-            oi->second->setVisible(visible);
+            (*oi)->setVisible(visible);
         }
 
         if (cascade)
@@ -645,30 +603,30 @@ namespace Ogre {
             iend = mChildren.end();
             for (i = mChildren.begin(); i != iend; ++i)
             {
-                static_cast<SceneNode*>(i->second)->setVisible(visible, cascade);
+                static_cast<SceneNode*>(*i)->setVisible(visible, cascade);
             }
         }
     }
-	//-----------------------------------------------------------------------
-	void SceneNode::setDebugDisplayEnabled(bool enabled, bool cascade)
-	{
-		ObjectMap::iterator oi, oiend;
-		oiend = mObjectsByName.end();
-		for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
-		{
-			oi->second->setDebugDisplayEnabled(enabled);
-		}
+    //-----------------------------------------------------------------------
+    void SceneNode::setDebugDisplayEnabled(bool enabled, bool cascade)
+    {
+        ObjectMap::iterator oi, oiend;
+        oiend = mObjectsByName.end();
+        for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
+        {
+            (*oi)->setDebugDisplayEnabled(enabled);
+        }
 
-		if (cascade)
-		{
-			ChildNodeMap::iterator i, iend;
-			iend = mChildren.end();
-			for (i = mChildren.begin(); i != iend; ++i)
-			{
-				static_cast<SceneNode*>(i->second)->setDebugDisplayEnabled(enabled, cascade);
-			}
-		}
-	}
+        if (cascade)
+        {
+            ChildNodeMap::iterator i, iend;
+            iend = mChildren.end();
+            for (i = mChildren.begin(); i != iend; ++i)
+            {
+                static_cast<SceneNode*>(*i)->setDebugDisplayEnabled(enabled, cascade);
+            }
+        }
+    }
     //-----------------------------------------------------------------------
     void SceneNode::flipVisibility(bool cascade)
     {
@@ -676,7 +634,7 @@ namespace Ogre {
         oiend = mObjectsByName.end();
         for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
         {
-            oi->second->setVisible(!oi->second->getVisible());
+            (*oi)->setVisible(!(*oi)->getVisible());
         }
 
         if (cascade)
@@ -685,7 +643,7 @@ namespace Ogre {
             iend = mChildren.end();
             for (i = mChildren.begin(); i != iend; ++i)
             {
-                static_cast<SceneNode*>(i->second)->flipVisibility(cascade);
+                static_cast<SceneNode*>((*i))->flipVisibility(cascade);
             }
         }
     }
