@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,14 +26,9 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreSkeleton.h"
-#include "OgreBone.h"
 #include "OgreAnimationState.h"
-#include "OgreException.h"
-#include "OgreLogManager.h"
 #include "OgreSkeletonManager.h"
 #include "OgreSkeletonSerializer.h"
-#include "OgreStringConverter.h"
 // Just for logging
 #include "OgreAnimationTrack.h"
 #include "OgreKeyFrame.h"
@@ -42,14 +37,14 @@ THE SOFTWARE.
 namespace Ogre {
 
     //---------------------------------------------------------------------
-	Skeleton::Skeleton()
-		: Resource(),
+    Skeleton::Skeleton()
+        : Resource(),
         mBlendState(ANIMBLEND_AVERAGE),
-		mNextAutoHandle(0),
-		mManualBonesDirty(false)
-	{
-	}
-	//---------------------------------------------------------------------
+        mNextAutoHandle(0),
+        mManualBonesDirty(false)
+    {
+    }
+    //---------------------------------------------------------------------
     Skeleton::Skeleton(ResourceManager* creator, const String& name, ResourceHandle handle,
         const String& group, bool isManual, ManualResourceLoader* loader) 
         : Resource(creator, name, handle, group, isManual, loader), 
@@ -72,23 +67,23 @@ namespace Ogre {
     void Skeleton::loadImpl(void)
     {
         SkeletonSerializer serializer;
-		LogManager::getSingleton().stream()
-			<< "Skeleton: Loading " << mName;
+        LogManager::getSingleton().stream()
+            << "Skeleton: Loading " << mName;
 
         DataStreamPtr stream = 
             ResourceGroupManager::getSingleton().openResource(
-				mName, mGroup, true, this);
+                mName, mGroup, this);
 
         serializer.importSkeleton(stream, this);
 
-		// Load any linked skeletons
-		LinkedSkeletonAnimSourceList::iterator i;
-		for (i = mLinkedSkeletonAnimSourceList.begin(); 
-			i != mLinkedSkeletonAnimSourceList.end(); ++i)
-		{
-			i->pSkeleton = SkeletonManager::getSingleton().load(
-				i->skeletonName, mGroup).staticCast<Skeleton>();
-		}
+        // Load any linked skeletons
+        LinkedSkeletonAnimSourceList::iterator i;
+        for (i = mLinkedSkeletonAnimSourceList.begin(); 
+            i != mLinkedSkeletonAnimSourceList.end(); ++i)
+        {
+            i->pSkeleton = static_pointer_cast<Skeleton>(
+                SkeletonManager::getSingleton().load(i->skeletonName, mGroup));
+        }
 
 
     }
@@ -103,7 +98,7 @@ namespace Ogre {
         }
         mBoneList.clear();
         mBoneListByName.clear();
-		mRootBones.clear();
+        mRootBones.clear();
         mManualBones.clear();
         mManualBonesDirty = false;
 
@@ -190,18 +185,15 @@ namespace Ogre {
         return ret;
     }
 
-
-
-    //---------------------------------------------------------------------
-    Bone* Skeleton::getRootBone(void) const
-    {
+    const Skeleton::BoneList& Skeleton::getRootBones() const {
         if (mRootBones.empty())
         {
             deriveRootBone();
         }
 
-        return mRootBones[0];
+        return mRootBones;
     }
+
     //---------------------------------------------------------------------
     void Skeleton::setAnimationState(const AnimationStateSet& animSet)
     {
@@ -214,37 +206,35 @@ namespace Ogre {
         // Reset bones
         reset();
 
-		Real weightFactor = 1.0f;
-		if (mBlendState == ANIMBLEND_AVERAGE)
-		{
-			// Derive total weights so we can rebalance if > 1.0f
-			Real totalWeights = 0.0f;
-			ConstEnabledAnimationStateIterator stateIt = 
-				animSet.getEnabledAnimationStateIterator();
-			while (stateIt.hasMoreElements())
-			{
-				const AnimationState* animState = stateIt.getNext();
-				// Make sure we have an anim to match implementation
-				const LinkedSkeletonAnimationSource* linked = 0;
-				if (_getAnimationImpl(animState->getAnimationName(), &linked))
-				{
-					totalWeights += animState->getWeight();
-				}
-			}
+        Real weightFactor = 1.0f;
+        if (mBlendState == ANIMBLEND_AVERAGE)
+        {
+            // Derive total weights so we can rebalance if > 1.0f
+            Real totalWeights = 0.0f;
+            EnabledAnimationStateList::const_iterator animIt;
+            for(animIt = animSet.getEnabledAnimationStates().begin(); animIt != animSet.getEnabledAnimationStates().end(); ++animIt)
+            {
+                const AnimationState* animState = *animIt;
+                // Make sure we have an anim to match implementation
+                const LinkedSkeletonAnimationSource* linked = 0;
+                if (_getAnimationImpl(animState->getAnimationName(), &linked))
+                {
+                    totalWeights += animState->getWeight();
+                }
+            }
 
-			// Allow < 1.0f, allows fade out of all anims if required 
-			if (totalWeights > 1.0f)
-			{
-				weightFactor = 1.0f / totalWeights;
-			}
-		}
+            // Allow < 1.0f, allows fade out of all anims if required 
+            if (totalWeights > 1.0f)
+            {
+                weightFactor = 1.0f / totalWeights;
+            }
+        }
 
         // Per enabled animation state
-        ConstEnabledAnimationStateIterator stateIt = 
-            animSet.getEnabledAnimationStateIterator();
-        while (stateIt.hasMoreElements())
+        EnabledAnimationStateList::const_iterator animIt;
+        for(animIt = animSet.getEnabledAnimationStates().begin(); animIt != animSet.getEnabledAnimationStates().end(); ++animIt)
         {
-            const AnimationState* animState = stateIt.getNext();
+            const AnimationState* animState = *animIt;
             const LinkedSkeletonAnimationSource* linked = 0;
             Animation* anim = _getAnimationImpl(animState->getAnimationName(), &linked);
             // tolerate state entries for animations we're not aware of
@@ -301,7 +291,7 @@ namespace Ogre {
         }
 
         Animation* ret = OGRE_NEW Animation(name, length);
-		ret->_notifyContainer(this);
+        ret->_notifyContainer(this);
 
         // Add to list
         mAnimationsList[name] = ret;
@@ -309,62 +299,62 @@ namespace Ogre {
         return ret;
 
     }
-	//---------------------------------------------------------------------
-	Animation* Skeleton::getAnimation(const String& name, 
-		const LinkedSkeletonAnimationSource** linker) const
-	{
-		Animation* ret = _getAnimationImpl(name, linker);
-		if (!ret)
-		{
-			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No animation entry found named " + name, 
-				"Skeleton::getAnimation");
-		}
+    //---------------------------------------------------------------------
+    Animation* Skeleton::getAnimation(const String& name, 
+        const LinkedSkeletonAnimationSource** linker) const
+    {
+        Animation* ret = _getAnimationImpl(name, linker);
+        if (!ret)
+        {
+            OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "No animation entry found named " + name, 
+                "Skeleton::getAnimation");
+        }
 
-		return ret;
-	}
-	//---------------------------------------------------------------------
-	Animation* Skeleton::getAnimation(const String& name) const
-	{
-		return getAnimation(name, 0);
-	}
-	//---------------------------------------------------------------------
-	bool Skeleton::hasAnimation(const String& name) const
-	{
-		return _getAnimationImpl(name) != 0;
-	}
+        return ret;
+    }
+    //---------------------------------------------------------------------
+    Animation* Skeleton::getAnimation(const String& name) const
+    {
+        return getAnimation(name, 0);
+    }
+    //---------------------------------------------------------------------
+    bool Skeleton::hasAnimation(const String& name) const
+    {
+        return _getAnimationImpl(name) != 0;
+    }
     //---------------------------------------------------------------------
     Animation* Skeleton::_getAnimationImpl(const String& name, 
-		const LinkedSkeletonAnimationSource** linker) const
+        const LinkedSkeletonAnimationSource** linker) const
     {
-		Animation* ret = 0;
+        Animation* ret = 0;
         AnimationList::const_iterator i = mAnimationsList.find(name);
 
         if (i == mAnimationsList.end())
         {
-			LinkedSkeletonAnimSourceList::const_iterator it;
-			for (it = mLinkedSkeletonAnimSourceList.begin(); 
-				it != mLinkedSkeletonAnimSourceList.end() && !ret; ++it)
-			{
-				if (!it->pSkeleton.isNull())
-				{
-					ret = it->pSkeleton->_getAnimationImpl(name);
-					if (ret && linker)
-					{
-						*linker = &(*it);
-					}
+            LinkedSkeletonAnimSourceList::const_iterator it;
+            for (it = mLinkedSkeletonAnimSourceList.begin(); 
+                it != mLinkedSkeletonAnimSourceList.end() && !ret; ++it)
+            {
+                if (it->pSkeleton)
+                {
+                    ret = it->pSkeleton->_getAnimationImpl(name);
+                    if (ret && linker)
+                    {
+                        *linker = &(*it);
+                    }
 
-				}
-			}
+                }
+            }
 
         }
-		else
-		{
-			if (linker)
-				*linker = 0;
-			ret = i->second;
-		}
+        else
+        {
+            if (linker)
+                *linker = 0;
+            ret = i->second;
+        }
 
-		return ret;
+        return ret;
 
     }
     //---------------------------------------------------------------------
@@ -397,71 +387,71 @@ namespace Ogre {
             animSet->createAnimationState(animName, 0.0, anim->getLength());
         }
 
-		// Also iterate over linked animation
-		LinkedSkeletonAnimSourceList::iterator li;
-		for (li = mLinkedSkeletonAnimSourceList.begin(); 
-			li != mLinkedSkeletonAnimSourceList.end(); ++li)
-		{
-			if (!li->pSkeleton.isNull())
-			{
-				li->pSkeleton->_refreshAnimationState(animSet);
-			}
-		}
+        // Also iterate over linked animation
+        LinkedSkeletonAnimSourceList::iterator li;
+        for (li = mLinkedSkeletonAnimSourceList.begin(); 
+            li != mLinkedSkeletonAnimSourceList.end(); ++li)
+        {
+            if (li->pSkeleton)
+            {
+                li->pSkeleton->_refreshAnimationState(animSet);
+            }
+        }
 
     }
-	//-----------------------------------------------------------------------
-	void Skeleton::_refreshAnimationState(AnimationStateSet* animSet)
-	{
-		// Merge in any new animations
-		AnimationList::iterator i;
-		for (i = mAnimationsList.begin(); i != mAnimationsList.end(); ++i)
-		{
-			Animation* anim = i->second;
-			// Create animation at time index 0, default params mean this has weight 1 and is disabled
-			const String& animName = anim->getName();
-			if (!animSet->hasAnimationState(animName))
-			{
-				animSet->createAnimationState(animName, 0.0, anim->getLength());
-			}
-			else
-			{
-				// Update length incase changed
-				AnimationState* animState = animSet->getAnimationState(animName);
-				animState->setLength(anim->getLength());
-				animState->setTimePosition(std::min(anim->getLength(), animState->getTimePosition()));
-			}
-		}
-		// Also iterate over linked animation
-		LinkedSkeletonAnimSourceList::iterator li;
-		for (li = mLinkedSkeletonAnimSourceList.begin(); 
-			li != mLinkedSkeletonAnimSourceList.end(); ++li)
-		{
-			if (!li->pSkeleton.isNull())
-			{
-				li->pSkeleton->_refreshAnimationState(animSet);
-			}
-		}
-	}
-	//-----------------------------------------------------------------------
-	void Skeleton::_notifyManualBonesDirty(void)
-	{
-		mManualBonesDirty = true;
-	}
-	//-----------------------------------------------------------------------
-	void Skeleton::_notifyManualBoneStateChange(Bone* bone)
-	{
-		if (bone->isManuallyControlled())
-			mManualBones.insert(bone);
-		else
-			mManualBones.erase(bone);
-	}
+    //-----------------------------------------------------------------------
+    void Skeleton::_refreshAnimationState(AnimationStateSet* animSet)
+    {
+        // Merge in any new animations
+        AnimationList::iterator i;
+        for (i = mAnimationsList.begin(); i != mAnimationsList.end(); ++i)
+        {
+            Animation* anim = i->second;
+            // Create animation at time index 0, default params mean this has weight 1 and is disabled
+            const String& animName = anim->getName();
+            if (!animSet->hasAnimationState(animName))
+            {
+                animSet->createAnimationState(animName, 0.0, anim->getLength());
+            }
+            else
+            {
+                // Update length incase changed
+                AnimationState* animState = animSet->getAnimationState(animName);
+                animState->setLength(anim->getLength());
+                animState->setTimePosition(std::min(anim->getLength(), animState->getTimePosition()));
+            }
+        }
+        // Also iterate over linked animation
+        LinkedSkeletonAnimSourceList::iterator li;
+        for (li = mLinkedSkeletonAnimSourceList.begin(); 
+            li != mLinkedSkeletonAnimSourceList.end(); ++li)
+        {
+            if (li->pSkeleton)
+            {
+                li->pSkeleton->_refreshAnimationState(animSet);
+            }
+        }
+    }
+    //-----------------------------------------------------------------------
+    void Skeleton::_notifyManualBonesDirty(void)
+    {
+        mManualBonesDirty = true;
+    }
+    //-----------------------------------------------------------------------
+    void Skeleton::_notifyManualBoneStateChange(Bone* bone)
+    {
+        if (bone->isManuallyControlled())
+            mManualBones.insert(bone);
+        else
+            mManualBones.erase(bone);
+    }
     //-----------------------------------------------------------------------
     unsigned short Skeleton::getNumBones(void) const
     {
         return (unsigned short)mBoneList.size();
     }
     //-----------------------------------------------------------------------
-    void Skeleton::_getBoneMatrices(Matrix4* pMatrices)
+    void Skeleton::_getBoneMatrices(Affine3* pMatrices)
     {
         // Update derived transforms
         _updateTransforms();
@@ -495,12 +485,12 @@ namespace Ogre {
     //---------------------------------------------------------------------
     Animation* Skeleton::getAnimation(unsigned short index) const
     {
-		// If you hit this assert, then the index is out of bounds.
+        // If you hit this assert, then the index is out of bounds.
         assert( index < mAnimationsList.size() );
 
         AnimationList::const_iterator i = mAnimationsList.begin();
 
-		std::advance(i, index);
+        std::advance(i, index);
 
         return i->second;
     }
@@ -524,11 +514,11 @@ namespace Ogre {
         return i->second;
 
     }
-	//---------------------------------------------------------------------
-	bool Skeleton::hasBone(const String& name) const 
-	{	
-		return mBoneListByName.find(name) != mBoneListByName.end();
-	}
+    //---------------------------------------------------------------------
+    bool Skeleton::hasBone(const String& name) const 
+    {   
+        return mBoneListByName.find(name) != mBoneListByName.end();
+    }
     //---------------------------------------------------------------------
     void Skeleton::deriveRootBone(void) const
     {
@@ -618,15 +608,15 @@ namespace Ogre {
 
     }
     //---------------------------------------------------------------------
-	SkeletonAnimationBlendMode Skeleton::getBlendMode() const
+    SkeletonAnimationBlendMode Skeleton::getBlendMode() const
     {
-		return mBlendState;
-	}
+        return mBlendState;
+    }
     //---------------------------------------------------------------------
-	void Skeleton::setBlendMode(SkeletonAnimationBlendMode state) 
+    void Skeleton::setBlendMode(SkeletonAnimationBlendMode state) 
     {
-		mBlendState = state;
-	}
+        mBlendState = state;
+    }
     //---------------------------------------------------------------------
     Skeleton::BoneIterator Skeleton::getRootBoneIterator(void)
     {
@@ -650,11 +640,11 @@ namespace Ogre {
         {
             (*i)->_update(true, false);
         }
-		mManualBonesDirty = false;
+        mManualBonesDirty = false;
     }
     //---------------------------------------------------------------------
-	void Skeleton::optimiseAllAnimations(bool preservingIdentityNodeTracks)
-	{
+    void Skeleton::optimiseAllAnimations(bool preservingIdentityNodeTracks)
+    {
         AnimationList::iterator ai, aiend;
         aiend = mAnimationsList.end();
 
@@ -685,53 +675,53 @@ namespace Ogre {
         for (ai = mAnimationsList.begin(); ai != aiend; ++ai)
         {
             // Don't discard identity node tracks here
-			ai->second->optimise(false);
-		}
-	}
-	//---------------------------------------------------------------------
-	void Skeleton::addLinkedSkeletonAnimationSource(const String& skelName, 
-		Real scale)
-	{
-		// Check not already linked
-		LinkedSkeletonAnimSourceList::iterator i;
-		for (i = mLinkedSkeletonAnimSourceList.begin(); 
-			i != mLinkedSkeletonAnimSourceList.end(); ++i)
-		{
-			if (skelName == i->skeletonName)
-				return; // don't bother
-		}
+            ai->second->optimise(false);
+        }
+    }
+    //---------------------------------------------------------------------
+    void Skeleton::addLinkedSkeletonAnimationSource(const String& skelName, 
+        Real scale)
+    {
+        // Check not already linked
+        LinkedSkeletonAnimSourceList::iterator i;
+        for (i = mLinkedSkeletonAnimSourceList.begin(); 
+            i != mLinkedSkeletonAnimSourceList.end(); ++i)
+        {
+            if (skelName == i->skeletonName)
+                return; // don't bother
+        }
 
-		if (isLoaded())
-		{
-			// Load immediately
-			SkeletonPtr skelPtr = 
-				SkeletonManager::getSingleton().load(skelName, mGroup).staticCast<Skeleton>();
-			mLinkedSkeletonAnimSourceList.push_back(
-				LinkedSkeletonAnimationSource(skelName, scale, skelPtr));
+        if (isLoaded())
+        {
+            // Load immediately
+            SkeletonPtr skelPtr = static_pointer_cast<Skeleton>(
+                SkeletonManager::getSingleton().load(skelName, mGroup));
+            mLinkedSkeletonAnimSourceList.push_back(
+                LinkedSkeletonAnimationSource(skelName, scale, skelPtr));
 
-		}
-		else
-		{
-			// Load later
-			mLinkedSkeletonAnimSourceList.push_back(
-				LinkedSkeletonAnimationSource(skelName, scale));
-		}
+        }
+        else
+        {
+            // Load later
+            mLinkedSkeletonAnimSourceList.push_back(
+                LinkedSkeletonAnimationSource(skelName, scale));
+        }
 
-	}
-	//---------------------------------------------------------------------
-	void Skeleton::removeAllLinkedSkeletonAnimationSources(void)
-	{
-		mLinkedSkeletonAnimSourceList.clear();
-	}
-	//---------------------------------------------------------------------
-	Skeleton::LinkedSkeletonAnimSourceIterator 
-	Skeleton::getLinkedSkeletonAnimationSourceIterator(void) const
-	{
-		return LinkedSkeletonAnimSourceIterator(
-			mLinkedSkeletonAnimSourceList.begin(), 
-			mLinkedSkeletonAnimSourceList.end());
-	}
-	//---------------------------------------------------------------------
+    }
+    //---------------------------------------------------------------------
+    void Skeleton::removeAllLinkedSkeletonAnimationSources(void)
+    {
+        mLinkedSkeletonAnimSourceList.clear();
+    }
+    //---------------------------------------------------------------------
+    Skeleton::LinkedSkeletonAnimSourceIterator 
+    Skeleton::getLinkedSkeletonAnimationSourceIterator(void) const
+    {
+        return LinkedSkeletonAnimSourceIterator(
+            mLinkedSkeletonAnimSourceList.begin(), 
+            mLinkedSkeletonAnimSourceList.end());
+    }
+    //---------------------------------------------------------------------
     struct DeltaTransform
     {
         Vector3 translate;
@@ -865,7 +855,7 @@ namespace Ogre {
         //
 
         // Calculate delta-transforms for all source bones.
-        vector<DeltaTransform>::type deltaTransforms(numSrcBones);
+        std::vector<DeltaTransform> deltaTransforms(numSrcBones);
         for (handle = 0; handle < numSrcBones; ++handle)
         {
             const Bone* srcBone = src->getBone(handle);
@@ -1026,17 +1016,14 @@ namespace Ogre {
 
     size_t Skeleton::calculateSize(void) const
     {
-        size_t memSize = 0;
-        memSize += sizeof(SkeletonAnimationBlendMode);
+        size_t memSize = sizeof(*this);
         memSize += mBoneList.size() * sizeof(Bone);
         memSize += mRootBones.size() * sizeof(Bone);
         memSize += mBoneListByName.size() * (sizeof(String) + sizeof(Bone*));
         memSize += mAnimationsList.size() * (sizeof(String) + sizeof(Animation*));
         memSize += mManualBones.size() * sizeof(Bone*);
         memSize += mLinkedSkeletonAnimSourceList.size() * sizeof(LinkedSkeletonAnimationSource);
-        memSize += sizeof(bool);
 
         return memSize;
     }
 }
-
