@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,66 +29,51 @@ THE SOFTWARE.
 
 #include "OgreSkeletonFileFormat.h"
 #include "OgreSkeletonSerializer.h"
-#include "OgreSkeleton.h"
 #include "OgreAnimation.h"
 #include "OgreAnimationTrack.h"
 #include "OgreKeyFrame.h"
-#include "OgreBone.h"
-#include "OgreString.h"
-#include "OgreDataStream.h"
-#include "OgreLogManager.h"
-
-
-
 
 namespace Ogre {
     /// stream overhead = ID + size
     const long SSTREAM_OVERHEAD_SIZE = sizeof(uint16) + sizeof(uint32);
-	const uint16 HEADER_STREAM_ID_EXT = 0x1000;
-	//---------------------------------------------------------------------
+    const uint16 HEADER_STREAM_ID_EXT = 0x1000;
+    //---------------------------------------------------------------------
     SkeletonSerializer::SkeletonSerializer()
     {
         // Version number
         // NB changed to include bone names in 1.1
         mVersion = "[Unknown]";
-    }
-    //---------------------------------------------------------------------
-    SkeletonSerializer::~SkeletonSerializer()
-    {
     }
 
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::exportSkeleton(const Skeleton* pSkeleton, 
-		const String& filename, SkeletonVersion ver, Endian endianMode)
-	{
-		std::fstream *f = OGRE_NEW_T(std::fstream, MEMCATEGORY_GENERAL)();
-		f->open(filename.c_str(), std::ios::binary | std::ios::out);
-		DataStreamPtr stream(OGRE_NEW FileStreamDataStream(f));
-
-		exportSkeleton(pSkeleton, stream, ver, endianMode);
-
-		stream->close();
-	}
     //---------------------------------------------------------------------
     void SkeletonSerializer::exportSkeleton(const Skeleton* pSkeleton, 
-		DataStreamPtr stream, SkeletonVersion ver, Endian endianMode)
+        const String& filename, SkeletonVersion ver, Endian endianMode)
     {
-		setWorkingVersion(ver);
-		// Decide on endian mode
-		determineEndianness(endianMode);
+        DataStreamPtr stream = _openFileStream(filename, std::ios::binary | std::ios::out);
+        exportSkeleton(pSkeleton, stream, ver, endianMode);
+
+        stream->close();
+    }
+    //---------------------------------------------------------------------
+    void SkeletonSerializer::exportSkeleton(const Skeleton* pSkeleton, 
+        DataStreamPtr stream, SkeletonVersion ver, Endian endianMode)
+    {
+        setWorkingVersion(ver);
+        // Decide on endian mode
+        determineEndianness(endianMode);
 
-        String msg;
         mStream = stream; 
-		if (!stream->isWriteable())
-		{
-			OGRE_EXCEPT(Exception::ERR_CANNOT_WRITE_TO_FILE,
-				"Unable to write to stream " + stream->getName(),
-				"SkeletonSerializer::exportSkeleton");
-		}
+        if (!stream->isWriteable())
+        {
+            OGRE_EXCEPT(Exception::ERR_CANNOT_WRITE_TO_FILE,
+                "Unable to write to stream " + stream->getName(),
+                "SkeletonSerializer::exportSkeleton");
+        }
 
 
         writeFileHeader();
 
+        pushInnerChunk(mStream);
         // Write main skeleton data
         LogManager::getSingleton().logMessage("Exporting bones..");
         writeSkeleton(pSkeleton, ver);
@@ -97,49 +82,68 @@ namespace Ogre {
         // Write all animations
         unsigned short numAnims = pSkeleton->getNumAnimations();
         LogManager::getSingleton().stream()
-			<< "Exporting animations, count=" << numAnims;
+            << "Exporting animations, count=" << numAnims;
         for (unsigned short i = 0; i < numAnims; ++i)
         {
             Animation* pAnim = pSkeleton->getAnimation(i);
-			LogManager::getSingleton().stream()
-				<< "Exporting animation: " << pAnim->getName();
+            LogManager::getSingleton().stream()
+                << "Exporting animation: " << pAnim->getName();
             writeAnimation(pSkeleton, pAnim, ver);
             LogManager::getSingleton().logMessage("Animation exported.");
 
         }
 
-		// Write links
-		Skeleton::LinkedSkeletonAnimSourceIterator linkIt = 
-			pSkeleton->getLinkedSkeletonAnimationSourceIterator();
-		while(linkIt.hasMoreElements())
-		{
-			const LinkedSkeletonAnimationSource& link = linkIt.getNext();
-			writeSkeletonAnimationLink(pSkeleton, link);
-		}       
-
+        // Write links
+        Skeleton::LinkedSkeletonAnimSourceIterator linkIt = 
+            pSkeleton->getLinkedSkeletonAnimationSourceIterator();
+        while(linkIt.hasMoreElements())
+        {
+            const LinkedSkeletonAnimationSource& link = linkIt.getNext();
+            writeSkeletonAnimationLink(pSkeleton, link);
+        }       
+        popInnerChunk(stream);
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::importSkeleton(DataStreamPtr& stream, Skeleton* pSkel)
     {
-		// Determine endianness (must be the first thing we do!)
-		determineEndianness(stream);
+        // Determine endianness (must be the first thing we do!)
+        determineEndianness(stream);
 
-		// Check header
-        readFileHeader(stream);
+        // Check header
+        unsigned short headerID;
+        readShorts(stream, &headerID, 1);
+        if (headerID != HEADER_STREAM_ID_EXT)
+        {
+            OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "File header not found",
+                "SkeletonSerializer::importSkeleton");
+        }
+
+        // Read version
+        String ver = readString(stream);
+        if ((ver != "[Serializer_v1.10]") &&
+            (ver != "[Serializer_v1.80]"))
+        {
+            OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR,
+                "Invalid file: version incompatible, file reports " + String(ver),
+                "SkeletonSerializer::importSkeleton");
+        }
+        mVersion = ver;
+
+        pushInnerChunk(stream);
+        unsigned short streamID = readChunk(stream);
 
         while(!stream->eof())
         {
-            unsigned short streamID = readChunk(stream);
             switch (streamID)
             {
-			case SKELETON_BLENDMODE:
-			{
-				// Optional blend mode
-				uint16 blendMode;
-				readShorts(stream, &blendMode, 1);
-				pSkel->setBlendMode(static_cast<SkeletonAnimationBlendMode>(blendMode));
-				break;
-			}
+            case SKELETON_BLENDMODE:
+            {
+                // Optional blend mode
+                uint16 blendMode;
+                readShorts(stream, &blendMode, 1);
+                pSkel->setBlendMode(static_cast<SkeletonAnimationBlendMode>(blendMode));
+                break;
+            }
             case SKELETON_BONE:
                 readBone(stream, pSkel);
                 break;
@@ -148,37 +152,41 @@ namespace Ogre {
                 break;
             case SKELETON_ANIMATION:
                 readAnimation(stream, pSkel);
-				break;
-			case SKELETON_ANIMATION_LINK:
-				readSkeletonAnimationLink(stream, pSkel);
-				break;
+                break;
+            case SKELETON_ANIMATION_LINK:
+                readSkeletonAnimationLink(stream, pSkel);
+                break;
+            default:
+                break;
             }
+
+            streamID = readChunk(stream);
         }
-
-		// Assume bones are stored in the binding pose
+        // Assume bones are stored in the binding pose
         pSkel->setBindingPose();
-
+        popInnerChunk(stream);
 
     }
-	
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::setWorkingVersion(SkeletonVersion ver)
-	{
-		if (ver == SKELETON_VERSION_1_0)
-			mVersion = "[Serializer_v1.10]";
-		else mVersion = "[Serializer_v1.80]";
-	}
-	//---------------------------------------------------------------------
+    
+    //---------------------------------------------------------------------
+    void SkeletonSerializer::setWorkingVersion(SkeletonVersion ver)
+    {
+        if (ver == SKELETON_VERSION_1_0)
+            mVersion = "[Serializer_v1.10]";
+        else mVersion = "[Serializer_v1.80]";
+    }
+    //---------------------------------------------------------------------
     void SkeletonSerializer::writeSkeleton(const Skeleton* pSkel, SkeletonVersion ver)
     {
-		// Write blend mode
-		if ((int)ver > (int)SKELETON_VERSION_1_0)
-		{
-			writeChunkHeader(SKELETON_BLENDMODE, SSTREAM_OVERHEAD_SIZE + sizeof(unsigned short));
-			uint16 blendMode = static_cast<uint16>(pSkel->getBlendMode());
-			writeShorts(&blendMode, 1);
-		}
-		
+        
+        // Write blend mode
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            writeChunkHeader(SKELETON_BLENDMODE, SSTREAM_OVERHEAD_SIZE + sizeof(unsigned short));
+            uint16 blendMode = static_cast<uint16>(pSkel->getBlendMode());
+            writeShorts(&blendMode, 1);
+        }
+        
         // Write each bone
         unsigned short numBones = pSkel->getNumBones();
         unsigned short i;
@@ -207,6 +215,10 @@ namespace Ogre {
         unsigned short handle = pBone->getHandle();
         // char* name
         writeString(pBone->getName());
+#if OGRE_SERIALIZER_VALIDATE_CHUNKSIZE
+        // Hack to fix chunk size validation:
+        mChunkSizeStack.back() += calcStringSize(pBone->getName());
+#endif
         // unsigned short handle            : handle of the bone, should be contiguous & start at 0
         writeShorts(&handle, 1);
         // Vector3 position                 : position of this bone relative to parent 
@@ -235,34 +247,35 @@ namespace Ogre {
     void SkeletonSerializer::writeAnimation(const Skeleton* pSkel, 
         const Animation* anim, SkeletonVersion ver)
     {
-        writeChunkHeader(SKELETON_ANIMATION, calcAnimationSize(pSkel, anim));
+        writeChunkHeader(SKELETON_ANIMATION, calcAnimationSize(pSkel, anim, ver));
 
         // char* name                       : Name of the animation
         writeString(anim->getName());
         // float length                      : Length of the animation in seconds
         float len = anim->getLength();
         writeFloats(&len, 1);
-		
-		if ((int)ver > (int)SKELETON_VERSION_1_0)
-		{
-			if (anim->getUseBaseKeyFrame())
-			{
-				size_t size = SSTREAM_OVERHEAD_SIZE;
-				// char* baseAnimationName (including terminator)
-				size += anim->getBaseKeyFrameAnimationName().length() + 1;
-				// float baseKeyFrameTime
-				size += sizeof(float);
-				
-				writeChunkHeader(SKELETON_ANIMATION_BASEINFO, size);
-				
-				// char* baseAnimationName (blank for self)
-				writeString(anim->getBaseKeyFrameAnimationName());
-				
-				// float baseKeyFrameTime
-				float t = (float)anim->getBaseKeyFrameTime();
-				writeFloats(&t, 1);
-			}
-		}
+        pushInnerChunk(mStream);
+        {
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            if (anim->getUseBaseKeyFrame())
+            {
+                size_t size = SSTREAM_OVERHEAD_SIZE;
+                // char* baseAnimationName (including terminator)
+                    size += calcStringSize(anim->getBaseKeyFrameAnimationName());
+                // float baseKeyFrameTime
+                size += sizeof(float);
+                
+                writeChunkHeader(SKELETON_ANIMATION_BASEINFO, size);
+                
+                // char* baseAnimationName (blank for self)
+                writeString(anim->getBaseKeyFrameAnimationName());
+                
+                // float baseKeyFrameTime
+                float t = (float)anim->getBaseKeyFrameTime();
+                writeFloats(&t, 1);
+            }
+        }
 
         // Write all tracks
         Animation::NodeTrackIterator trackIt = anim->getNodeTrackIterator();
@@ -270,6 +283,8 @@ namespace Ogre {
         {
             writeAnimationTrack(pSkel, trackIt.getNext());
         }
+        }
+        popInnerChunk(mStream);
 
     }
     //---------------------------------------------------------------------
@@ -282,21 +297,20 @@ namespace Ogre {
         Bone* bone = static_cast<Bone*>(track->getAssociatedNode());
         unsigned short boneid = bone->getHandle();
         writeShorts(&boneid, 1);
-
+        pushInnerChunk(mStream);
         // Write all keyframes
         for (unsigned short i = 0; i < track->getNumKeyFrames(); ++i)
         {
             writeKeyFrame(pSkel, track->getNodeKeyFrame(i));
         }
-
+        popInnerChunk(mStream);
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::writeKeyFrame(const Skeleton* pSkel, 
         const TransformKeyFrame* key)
     {
 
-        writeChunkHeader(SKELETON_ANIMATION_TRACK_KEYFRAME, 
-            calcKeyFrameSize(pSkel, key));
+        writeChunkHeader(SKELETON_ANIMATION_TRACK_KEYFRAME, calcKeyFrameSize(pSkel, key));
 
         // float time                    : The time position (seconds)
         float time = key->getTime();
@@ -315,16 +329,7 @@ namespace Ogre {
     size_t SkeletonSerializer::calcBoneSize(const Skeleton* pSkel, 
         const Bone* pBone)
     {
-        size_t size = SSTREAM_OVERHEAD_SIZE;
-
-        // handle
-        size += sizeof(unsigned short);
-
-        // position
-        size += sizeof(float) * 3;
-
-        // orientation
-        size += sizeof(float) * 4;
+        size_t size = calcBoneSizeWithoutScale(pSkel, pBone);
 
         // scale
         if (pBone->getScale() != Vector3::UNIT_SCALE)
@@ -340,6 +345,10 @@ namespace Ogre {
     {
         size_t size = SSTREAM_OVERHEAD_SIZE;
 
+        // TODO: Add this for next skeleton format!
+        // Currently it is broken, because to determine that we have scale, it will compare chunk size.
+        //size += calcStringSize(pBone->getName());
+
         // handle
         size += sizeof(unsigned short);
 
@@ -365,20 +374,31 @@ namespace Ogre {
         return size;
     }
     //---------------------------------------------------------------------
-    size_t SkeletonSerializer::calcAnimationSize(const Skeleton* pSkel, 
-        const Animation* pAnim)
+    size_t SkeletonSerializer::calcAnimationSize(const Skeleton* pSkel, const Animation* pAnim, SkeletonVersion ver)
     {
         size_t size = SSTREAM_OVERHEAD_SIZE;
 
         // Name, including terminator
-        size += pAnim->getName().length() + 1;
+        size += calcStringSize(pAnim->getName());
         // length
         size += sizeof(float);
 
+        if ((int)ver > (int)SKELETON_VERSION_1_0)
+        {
+            if (pAnim->getUseBaseKeyFrame())
+            {
+                size += SSTREAM_OVERHEAD_SIZE;
+                // char* baseAnimationName (including terminator)
+                size += calcStringSize(pAnim->getBaseKeyFrameAnimationName());
+                // float baseKeyFrameTime
+                size += sizeof(float);
+            }
+        }
+
         // Nested animation tracks
-		Animation::NodeTrackIterator trackIt = pAnim->getNodeTrackIterator();
-		while(trackIt.hasMoreElements())
-		{
+        Animation::NodeTrackIterator trackIt = pAnim->getNodeTrackIterator();
+        while(trackIt.hasMoreElements())
+        {
             size += calcAnimationTrackSize(pSkel, trackIt.getNext());
         }
 
@@ -405,14 +425,8 @@ namespace Ogre {
     size_t SkeletonSerializer::calcKeyFrameSize(const Skeleton* pSkel, 
         const TransformKeyFrame* pKey)
     {
-        size_t size = SSTREAM_OVERHEAD_SIZE;
+        size_t size = calcKeyFrameSizeWithoutScale(pSkel, pKey);
 
-        // float time                    : The time position (seconds)
-        size += sizeof(float);
-        // Quaternion rotate            : Rotation to apply at this keyframe
-        size += sizeof(float) * 4;
-        // Vector3 translate            : Translation to apply at this keyframe
-        size += sizeof(float) * 3;
         // Vector3 scale                : Scale to apply at this keyframe
         if (pKey->getScale() != Vector3::UNIT_SCALE)
         {
@@ -436,34 +450,7 @@ namespace Ogre {
 
         return size;
     }
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::readFileHeader(DataStreamPtr& stream)
-	{
-		unsigned short headerID;
-
-		// Read header ID
-		readShorts(stream, &headerID, 1);
-
-		if (headerID == HEADER_STREAM_ID_EXT)
-		{
-			// Read version
-			String ver = readString(stream);
-			if ((ver != "[Serializer_v1.10]") &&
-				(ver != "[Serializer_v1.80]"))
-			{
-				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Invalid file: version incompatible, file reports " + String(ver),
-					"Serializer::readFileHeader");
-			}
-			mVersion = ver;
-		}
-		else
-		{
-			OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, "Invalid file: no header", 
-				"Serializer::readFileHeader");
-		}
-	}
-	//---------------------------------------------------------------------
+    //---------------------------------------------------------------------
     void SkeletonSerializer::readBone(DataStreamPtr& stream, Skeleton* pSkel)
     {
         // char* name
@@ -483,6 +470,13 @@ namespace Ogre {
         Quaternion q;
         readObject(stream, q);
         pBone->setOrientation(q);
+
+#if OGRE_SERIALIZER_VALIDATE_CHUNKSIZE
+        // Hack to fix chunk size validation:
+        mChunkSizeStack.back() += calcStringSize(name);
+#endif
+        // TODO: don't depend on mCurrentstreamLen in next skeleton format!
+        // Currently we use wrong chunk sizes, but we can't fix it, because we depend on mCurrentstreamLen
         // Do we have scale?
         if (mCurrentstreamLen > calcBoneSizeWithoutScale(pSkel, pBone))
         {
@@ -490,6 +484,7 @@ namespace Ogre {
             readObject(stream, scale);
             pBone->setScale(scale);
         }
+        
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::readBoneParent(DataStreamPtr& stream, Skeleton* pSkel)
@@ -522,29 +517,29 @@ namespace Ogre {
         readFloats(stream, &len, 1);
 
         Animation *pAnim = pSkel->createAnimation(name, len);
-		
         // Read all tracks
         if (!stream->eof())
         {
+            pushInnerChunk(stream);
             unsigned short streamID = readChunk(stream);
-			// Optional base info is possible
-			if (streamID == SKELETON_ANIMATION_BASEINFO)
-			{
-				// char baseAnimationName
-				String baseAnimName = readString(stream);
-				// float baseKeyFrameTime
-				float baseKeyTime;
-				readFloats(stream, &baseKeyTime, 1);
-				
-				pAnim->setUseBaseKeyFrame(true, baseKeyTime, baseAnimName);
-				
+            // Optional base info is possible
+            if (streamID == SKELETON_ANIMATION_BASEINFO)
+            {
+                // char baseAnimationName
+                String baseAnimName = readString(stream);
+                // float baseKeyFrameTime
+                float baseKeyTime;
+                readFloats(stream, &baseKeyTime, 1);
+                
+                pAnim->setUseBaseKeyFrame(true, baseKeyTime, baseAnimName);
+                
                 if (!stream->eof())
                 {
                     // Get next stream
                     streamID = readChunk(stream);
                 }
-			}
-			
+            }
+            
             while(streamID == SKELETON_ANIMATION_TRACK && !stream->eof())
             {
                 readAnimationTrack(stream, pAnim, pSkel);
@@ -558,13 +553,10 @@ namespace Ogre {
             if (!stream->eof())
             {
                 // Backpedal back to start of this stream if we've found a non-track
-                stream->skip(-SSTREAM_OVERHEAD_SIZE);
+                backpedalChunkHeader(stream);
             }
-
+            popInnerChunk(stream);
         }
-
-
-
     }
     //---------------------------------------------------------------------
     void SkeletonSerializer::readAnimationTrack(DataStreamPtr& stream, Animation* anim, 
@@ -583,6 +575,7 @@ namespace Ogre {
         // Keep looking for nested keyframes
         if (!stream->eof())
         {
+            pushInnerChunk(stream);
             unsigned short streamID = readChunk(stream);
             while(streamID == SKELETON_ANIMATION_TRACK_KEYFRAME && !stream->eof())
             {
@@ -597,9 +590,9 @@ namespace Ogre {
             if (!stream->eof())
             {
                 // Backpedal back to start of this stream if we've found a non-keyframe
-                stream->skip(-SSTREAM_OVERHEAD_SIZE);
+                backpedalChunkHeader(stream);
             }
-
+            popInnerChunk(stream);
         }
 
 
@@ -630,46 +623,46 @@ namespace Ogre {
             kf->setScale(scale);
         }
     }
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::writeSkeletonAnimationLink(const Skeleton* pSkel, 
-		const LinkedSkeletonAnimationSource& link)
-	{
-		writeChunkHeader(SKELETON_ANIMATION_LINK, 
-			calcSkeletonAnimationLinkSize(pSkel, link));
-
-		// char* skeletonName
-		writeString(link.skeletonName);
-		// float scale
-		writeFloats(&(link.scale), 1);
-
-	}
     //---------------------------------------------------------------------
-	size_t SkeletonSerializer::calcSkeletonAnimationLinkSize(const Skeleton* pSkel, 
-		const LinkedSkeletonAnimationSource& link)
-	{
-		size_t size = SSTREAM_OVERHEAD_SIZE;
+    void SkeletonSerializer::writeSkeletonAnimationLink(const Skeleton* pSkel, 
+        const LinkedSkeletonAnimationSource& link)
+    {
+        writeChunkHeader(SKELETON_ANIMATION_LINK, 
+            calcSkeletonAnimationLinkSize(pSkel, link));
 
-		// char* skeletonName
-		size += link.skeletonName.length() + 1;
-		// float scale
-		size += sizeof(float);
+        // char* skeletonName
+        writeString(link.skeletonName);
+        // float scale
+        writeFloats(&(link.scale), 1);
 
-		return size;
+    }
+    //---------------------------------------------------------------------
+    size_t SkeletonSerializer::calcSkeletonAnimationLinkSize(const Skeleton* pSkel, 
+        const LinkedSkeletonAnimationSource& link)
+    {
+        size_t size = SSTREAM_OVERHEAD_SIZE;
 
-	}
-	//---------------------------------------------------------------------
-	void SkeletonSerializer::readSkeletonAnimationLink(DataStreamPtr& stream, 
-		Skeleton* pSkel)
-	{
-		// char* skeletonName
-		String skelName = readString(stream);
-		// float scale
-		float scale;
-		readFloats(stream, &scale, 1);
+        // char* skeletonName
+        size += link.skeletonName.length() + 1;
+        // float scale
+        size += sizeof(float);
 
-		pSkel->addLinkedSkeletonAnimationSource(skelName, scale);
+        return size;
 
-	}
+    }
+    //---------------------------------------------------------------------
+    void SkeletonSerializer::readSkeletonAnimationLink(DataStreamPtr& stream, 
+        Skeleton* pSkel)
+    {
+        // char* skeletonName
+        String skelName = readString(stream);
+        // float scale
+        float scale;
+        readFloats(stream, &scale, 1);
+
+        pSkel->addLinkedSkeletonAnimationSource(skelName, scale);
+
+    }
 
 
 
