@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,13 +26,6 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreSubMesh.h"
-
-#include "OgreMesh.h"
-#include "OgreException.h"
-#include "OgreMeshManager.h"
-#include "OgreMaterialManager.h"
-#include "OgreStringConverter.h"
 
 namespace Ogre {
     //-----------------------------------------------------------------------
@@ -41,56 +34,46 @@ namespace Ogre {
         , operationType(RenderOperation::OT_TRIANGLE_LIST)
         , vertexData(0)
         , parent(0)
-        , mMatInitialised(false)
         , mBoneAssignmentsOutOfDate(false)
-		, mVertexAnimationType(VAT_NONE)
-		, mVertexAnimationIncludesNormals(false)
-		, mBuildEdgesEnabled(true)
+        , mVertexAnimationType(VAT_NONE)
+        , mVertexAnimationIncludesNormals(false)
+        , mBuildEdgesEnabled(true)
     {
-		indexData = OGRE_NEW IndexData();
+        indexData = OGRE_NEW IndexData();
     }
     //-----------------------------------------------------------------------
     SubMesh::~SubMesh()
     {
+        removeLodLevels();
         OGRE_DELETE vertexData;
-		OGRE_DELETE indexData;
-
-		removeLodLevels();
+        OGRE_DELETE indexData;
     }
 
     //-----------------------------------------------------------------------
     void SubMesh::setMaterialName( const String& name, const String& groupName /* = ResourceGroupManager::AUTODETECT_RESOURCE_GROUP_NAME */)
     {
-        mMaterialName = name;
-        mMatInitialised = true;
+        mMaterial = MaterialManager::getSingleton().getByName(name, groupName);
     }
     //-----------------------------------------------------------------------
     const String& SubMesh::getMaterialName() const
     {
-        return mMaterialName;
-    }
-    //-----------------------------------------------------------------------
-    bool SubMesh::isMatInitialised(void) const
-    {
-        return mMatInitialised;
-
+        return mMaterial ? mMaterial->getName() : BLANKSTRING;
     }
     //-----------------------------------------------------------------------
     void SubMesh::_getRenderOperation(RenderOperation& ro, ushort lodIndex)
     {
-
-        ro.useIndexes = indexData->indexCount != 0;
-		if (lodIndex > 0 && static_cast< size_t >( lodIndex - 1 ) < mLodFaceList.size())
-		{
-			// lodIndex - 1 because we don't store full detail version in mLodFaceList
-			ro.indexData = mLodFaceList[lodIndex-1];
+        if (lodIndex > 0 && static_cast< size_t >( lodIndex - 1 ) < mLodFaceList.size())
+        {
+            // lodIndex - 1 because we don't store full detail version in mLodFaceList
+            ro.indexData = mLodFaceList[lodIndex-1];
         }
         else
         {
-    		ro.indexData = indexData;
+            ro.indexData = indexData;
         }
-		ro.operationType = operationType;
-		ro.vertexData = useSharedVertices? parent->sharedVertexData : vertexData;
+        ro.useIndexes = ro.indexData->indexCount != 0;
+        ro.operationType = operationType;
+        ro.vertexData = useSharedVertices? parent->sharedVertexData : vertexData;
 
     }
     //-----------------------------------------------------------------------
@@ -101,8 +84,7 @@ namespace Ogre {
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "This SubMesh uses shared geometry,  you "
                 "must assign bones to the Mesh, not the SubMesh", "SubMesh.addBoneAssignment");
         }
-        mBoneAssignments.insert(
-            VertexBoneAssignmentList::value_type(vertBoneAssign.vertexIndex, vertBoneAssign));
+        mBoneAssignments.emplace(vertBoneAssign.vertexIndex, vertBoneAssign);
         mBoneAssignmentsOutOfDate = true;
     }
     //-----------------------------------------------------------------------
@@ -158,52 +140,50 @@ namespace Ogre {
     {
         bool newMaterialCreated = false;
         // if submesh has texture aliases
-        // ask the material manager if the current summesh material exists
-        if (hasTextureAliases() && MaterialManager::getSingleton().resourceExists(mMaterialName))
+        // ask the material manager if the current submesh material exists
+        if (hasTextureAliases() && mMaterial)
         {
             // get the current submesh material
-            MaterialPtr material = MaterialManager::getSingleton().getByName( mMaterialName );
+            const String& materialName = mMaterial->getName();
             // get test result for if change will occur when the texture aliases are applied
-            if (material->applyTextureAliases(mTextureAliases, false))
+            if (mMaterial->applyTextureAliases(mTextureAliases, false))
             {
-				Ogre::String newMaterialName;
+                Ogre::String newMaterialName;
 
-				// If this material was already derived from another material
-				// due to aliasing, let's strip off the aliasing suffix and
-				// generate a new one using our current aliasing table.
+                // If this material was already derived from another material
+                // due to aliasing, let's strip off the aliasing suffix and
+                // generate a new one using our current aliasing table.
 
-				Ogre::String::size_type pos = mMaterialName.find("?TexAlias(", 0);
-				if( pos != Ogre::String::npos )
-					newMaterialName = mMaterialName.substr(0, pos);
-				else
-					newMaterialName = mMaterialName;
+                Ogre::String::size_type pos = materialName.find("?TexAlias(", 0);
+                if( pos != Ogre::String::npos )
+                    newMaterialName = materialName.substr(0, pos);
+                else
+                    newMaterialName = materialName;
 
-				newMaterialName += "?TexAlias(";
-				// Iterate deterministically over the aliases (always in the same
-				// order via std::map's sorted iteration nature).
-				AliasTextureIterator aliasIter = getAliasTextureIterator();
-				while( aliasIter.hasMoreElements() )
-				{
-					newMaterialName += aliasIter.peekNextKey();
-					newMaterialName += "=";
-					newMaterialName += aliasIter.getNext();
-					newMaterialName += " ";
-				}
-				newMaterialName += ")";
-					
-				// Reuse the material if it's already been created. This decreases batch
-				// count and keeps material explosion under control.
-				if(!MaterialManager::getSingleton().resourceExists(newMaterialName))
-				{
-					Ogre::MaterialPtr newMaterial = Ogre::MaterialManager::getSingleton().create(
-						newMaterialName, material->getGroup());
-					// copy parent material details to new material
-					material->copyDetailsTo(newMaterial);
-					// apply texture aliases to new material
-					newMaterial->applyTextureAliases(mTextureAliases);
-				}
+                newMaterialName += "?TexAlias(";
+                // Iterate deterministically over the aliases (always in the same
+                // order via std::map's sorted iteration nature).
+                AliasTextureIterator aliasIter = getAliasTextureIterator();
+                while( aliasIter.hasMoreElements() )
+                {
+                    newMaterialName += aliasIter.peekNextKey();
+                    newMaterialName += "=";
+                    newMaterialName += aliasIter.getNext();
+                    newMaterialName += " ";
+                }
+                newMaterialName += ")";
+                    
+                // Reuse the material if it's already been created. This decreases batch
+                // count and keeps material explosion under control.
+                MaterialPtr newMaterial = MaterialManager::getSingleton().getByName(newMaterialName, mMaterial->getGroup());
+                if(!newMaterial)
+                {
+                    newMaterial = mMaterial->clone(newMaterialName);
+                    // apply texture aliases to new material
+                    newMaterial->applyTextureAliases(mTextureAliases);
+                }
                 // place new material name in submesh
-                setMaterialName(newMaterialName);
+                mMaterial = newMaterial;
                 newMaterialCreated = true;
             }
         }
@@ -214,25 +194,25 @@ namespace Ogre {
     void SubMesh::removeLodLevels(void)
     {
         LODFaceList::iterator lodi, lodend;
-		lodend = mLodFaceList.end();
-		for (lodi = mLodFaceList.begin(); lodi != lodend; ++lodi)
-		{
-			OGRE_DELETE *lodi;
-		}
+        lodend = mLodFaceList.end();
+        for (lodi = mLodFaceList.begin(); lodi != lodend; ++lodi)
+        {
+            OGRE_DELETE *lodi;
+        }
 
         mLodFaceList.clear();
 
     }
-	//---------------------------------------------------------------------
-	VertexAnimationType SubMesh::getVertexAnimationType(void) const
-	{
-		if(parent->_getAnimationTypesDirty())
-		{
-			parent->_determineAnimationTypes();
-		}
-		return mVertexAnimationType;
-	}
-	//---------------------------------------------------------------------
+    //---------------------------------------------------------------------
+    VertexAnimationType SubMesh::getVertexAnimationType(void) const
+    {
+        if(parent->_getAnimationTypesDirty())
+        {
+            parent->_determineAnimationTypes();
+        }
+        return mVertexAnimationType;
+    }
+    //---------------------------------------------------------------------
     /* To find as many points from different domains as we need,
      * such that those domains are from different parts of the mesh,
      * we implement a simplified Heckbert quantization algorithm.
@@ -243,7 +223,7 @@ namespace Ogre {
     struct Cluster
     {
         Vector3 mMin, mMax;
-        set<uint32>::type mIndices;
+        std::set<uint32> mIndices;
 
         Cluster ()
         { }
@@ -277,7 +257,7 @@ namespace Ogre {
             mMin.x = mMin.y = mMin.z = Math::POS_INFINITY;
             mMax.x = mMax.y = mMax.z = Math::NEG_INFINITY;
 
-            for (set<uint32>::type::const_iterator i = mIndices.begin ();
+            for (std::set<uint32>::const_iterator i = mIndices.begin ();
                  i != mIndices.end (); ++i)
             {
                 float *v;
@@ -293,7 +273,7 @@ namespace Ogre {
             Cluster newbox;
 
             // Separate all points that are inside the new bbox
-            for (set<uint32>::type::iterator i = mIndices.begin ();
+            for (std::set<uint32>::iterator i = mIndices.begin ();
                  i != mIndices.end (); )
             {
                 float *v;
@@ -301,7 +281,7 @@ namespace Ogre {
                 if (v [split_axis] > r)
                 {
                     newbox.mIndices.insert (*i);
-                    set<uint32>::type::iterator x = i++;
+                    std::set<uint32>::iterator x = i++;
                     mIndices.erase(x);
                 }
                 else
@@ -319,6 +299,9 @@ namespace Ogre {
     {
         extremityPoints.clear();
 
+		if (count == 0)
+			return;
+
         /* Currently this uses just one criteria: the points must be
          * as far as possible from each other. This at least ensures
          * that the extreme points characterise the submesh as
@@ -334,38 +317,38 @@ namespace Ogre {
         uint8 *vdata = (uint8 *)vbuf->lock (HardwareBuffer::HBL_READ_ONLY);
         size_t vsz = vbuf->getVertexSize ();
 
-        vector<Cluster>::type boxes;
+        std::vector<Cluster> boxes;
         boxes.reserve (count);
 
-		// First of all, find min and max bounding box of the submesh
-		boxes.push_back (Cluster ());
+        // First of all, find min and max bounding box of the submesh
+        boxes.push_back (Cluster ());
 
-		if (indexData->indexCount > 0)
-		{
+        if (indexData->indexCount > 0)
+        {
 
-			uint elsz = indexData->indexBuffer->getType () == HardwareIndexBuffer::IT_32BIT ?
-				4 : 2;
-			uint8 *idata = (uint8 *)indexData->indexBuffer->lock (
-				indexData->indexStart * elsz, indexData->indexCount * elsz,
-				HardwareIndexBuffer::HBL_READ_ONLY);
+            uint elsz = indexData->indexBuffer->getType () == HardwareIndexBuffer::IT_32BIT ?
+                4 : 2;
+            uint8 *idata = (uint8 *)indexData->indexBuffer->lock (
+                indexData->indexStart * elsz, indexData->indexCount * elsz,
+                HardwareIndexBuffer::HBL_READ_ONLY);
 
-			for (size_t i = 0; i < indexData->indexCount; i++)
-			{
-				int idx = (elsz == 2) ? ((uint16 *)idata) [i] : ((uint32 *)idata) [i];
-				boxes [0].mIndices.insert (idx);
-			}
-			indexData->indexBuffer->unlock ();
+            for (size_t i = 0; i < indexData->indexCount; i++)
+            {
+                int idx = (elsz == 2) ? ((uint16 *)idata) [i] : ((uint32 *)idata) [i];
+                boxes [0].mIndices.insert (idx);
+            }
+            indexData->indexBuffer->unlock ();
 
-		}
-		else
-		{
-			// just insert all indexes
-			for (size_t i = vertexData->vertexStart; i < vertexData->vertexCount; i++)
-			{
-				boxes [0].mIndices.insert (static_cast<int>(i));
-			}
+        }
+        else
+        {
+            // just insert all indexes
+            for (size_t i = vertexData->vertexStart; i < vertexData->vertexCount; i++)
+            {
+                boxes [0].mIndices.insert (static_cast<int>(i));
+            }
 
-		}
+        }
 
         boxes [0].computeBBox (poselem, vdata, vsz);
 
@@ -378,7 +361,7 @@ namespace Ogre {
             // Find the largest box with more than one vertex :)
             Cluster *split_box = NULL;
             Real split_volume = -1;
-            for (vector<Cluster>::type::iterator b = boxes.begin ();
+            for (std::vector<Cluster>::iterator b = boxes.begin ();
                  b != boxes.end (); ++b)
             {
                 if (b->empty ())
@@ -414,13 +397,13 @@ namespace Ogre {
 
         // Fine, now from every cluster choose the vertex that is most
         // distant from the geometrical center and from other extremes.
-        for (vector<Cluster>::type::const_iterator b = boxes.begin ();
+        for (std::vector<Cluster>::const_iterator b = boxes.begin ();
              b != boxes.end (); ++b)
         {
             Real rating = 0;
             Vector3 best_vertex;
 
-            for (set<uint32>::type::const_iterator i = b->mIndices.begin ();
+            for (std::set<uint32>::const_iterator i = b->mIndices.begin ();
                  i != b->mIndices.end (); ++i)
             {
                 float *v;
@@ -429,7 +412,7 @@ namespace Ogre {
                 Vector3 vv (v [0], v [1], v [2]);
                 Real r = (vv - center).squaredLength ();
 
-                for (vector<Vector3>::type::const_iterator e = extremityPoints.begin ();
+                for (std::vector<Vector3>::const_iterator e = extremityPoints.begin ();
                      e != extremityPoints.end (); ++e)
                     r += (*e - vv).squaredLength ();
                 if (r > rating)
@@ -446,29 +429,28 @@ namespace Ogre {
         vbuf->unlock ();
     }
     //---------------------------------------------------------------------
-	void SubMesh::setBuildEdgesEnabled(bool b)
-	{
-		mBuildEdgesEnabled = b;
-		if(parent)
-		{
-			parent->freeEdgeList();
-			parent->setAutoBuildEdgeLists(true);
-		}
-	}
+    void SubMesh::setBuildEdgesEnabled(bool b)
+    {
+        mBuildEdgesEnabled = b;
+        if(parent)
+        {
+            parent->freeEdgeList();
+            parent->setAutoBuildEdgeLists(true);
+        }
+    }
     //---------------------------------------------------------------------
     SubMesh * SubMesh::clone(const String& newName, Mesh *parentMesh)
     {
         // This is a bit like a copy constructor, but with the additional aspect of registering the clone with
         //  the MeshManager
 
-        SubMesh* newSub;
-        if(!parentMesh)
-            newSub = parent->createSubMesh(newName);
-        else
-            newSub = parentMesh->createSubMesh(newName);
+        if(parentMesh == NULL)
+            parentMesh = parent;
 
-        newSub->mMaterialName = this->mMaterialName;
-        newSub->mMatInitialised = this->mMatInitialised;
+        HardwareBufferManagerBase* bufferManager = parentMesh->getHardwareBufferManager();
+        SubMesh* newSub = parentMesh->createSubMesh(newName);
+
+        newSub->mMaterial = this->mMaterial;
         newSub->operationType = this->operationType;
         newSub->useSharedVertices = this->useSharedVertices;
         newSub->extremityPoints = this->extremityPoints;
@@ -476,14 +458,14 @@ namespace Ogre {
         if (!this->useSharedVertices)
         {
             // Copy unique vertex data
-            newSub->vertexData = this->vertexData->clone();
+            newSub->vertexData = this->vertexData->clone(true, bufferManager);
             // Copy unique index map
             newSub->blendIndexToBoneIndexMap = this->blendIndexToBoneIndexMap;
         }
 
         // Copy index data
         OGRE_DELETE newSub->indexData;
-        newSub->indexData = this->indexData->clone();
+        newSub->indexData = this->indexData->clone(true, bufferManager);
         // Copy any bone assignments
         newSub->mBoneAssignments = this->mBoneAssignments;
         newSub->mBoneAssignmentsOutOfDate = this->mBoneAssignmentsOutOfDate;
@@ -494,7 +476,7 @@ namespace Ogre {
         newSub->mLodFaceList.reserve(this->mLodFaceList.size());
         SubMesh::LODFaceList::const_iterator facei;
         for (facei = this->mLodFaceList.begin(); facei != this->mLodFaceList.end(); ++facei) {
-            IndexData* newIndexData = (*facei)->clone();
+            IndexData* newIndexData = (*facei)->clone(true, bufferManager);
             newSub->mLodFaceList.push_back(newIndexData);
         }
         return newSub;
