@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -26,11 +26,7 @@ THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
 #include "OgreStableHeaders.h"
-#include "OgreTextureManager.h"
-#include "OgreException.h"
 #include "OgrePixelFormat.h"
-#include "OgreRoot.h"
-#include "OgreRenderSystem.h"
 
 namespace Ogre {
     //-----------------------------------------------------------------------
@@ -63,14 +59,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     TexturePtr TextureManager::getByName(const String& name, const String& groupName)
     {
-        return getResourceByName(name, groupName).staticCast<Texture>();
+        return static_pointer_cast<Texture>(getResourceByName(name, groupName));
     }
     //-----------------------------------------------------------------------
     TexturePtr TextureManager::create (const String& name, const String& group,
                                     bool isManual, ManualResourceLoader* loader,
                                     const NameValuePairList* createParams)
     {
-        return createResource(name,group,isManual,loader,createParams).staticCast<Texture>();
+        return static_pointer_cast<Texture>(createResource(name,group,isManual,loader,createParams));
     }
     //-----------------------------------------------------------------------
     TextureManager::ResourceCreateOrRetrieveResult TextureManager::createOrRetrieve(
@@ -78,19 +74,19 @@ namespace Ogre {
             const NameValuePairList* createParams, TextureType texType, int numMipmaps, Real gamma,
             bool isAlpha, PixelFormat desiredFormat, bool hwGamma)
     {
-		ResourceCreateOrRetrieveResult res =
+        ResourceCreateOrRetrieveResult res =
             Ogre::ResourceManager::createOrRetrieve(name, group, isManual, loader, createParams);
-		// Was it created?
-		if(res.second)
+        // Was it created?
+        if(res.second)
         {
-            TexturePtr tex = res.first.staticCast<Texture>();
+            TexturePtr tex = static_pointer_cast<Texture>(res.first);
             tex->setTextureType(texType);
             tex->setNumMipmaps((numMipmaps == MIP_DEFAULT)? mDefaultNumMipmaps :
-				static_cast<size_t>(numMipmaps));
+                static_cast<uint32>(numMipmaps));
             tex->setGamma(gamma);
             tex->setTreatLuminanceAsAlpha(isAlpha);
             tex->setFormat(desiredFormat);
-			tex->setHardwareGammaEnabled(hwGamma);
+            tex->setHardwareGammaEnabled(hwGamma);
         }
         return res;
     }
@@ -99,10 +95,10 @@ namespace Ogre {
                                        int numMipmaps, Real gamma, bool isAlpha,
                                        PixelFormat desiredFormat, bool hwGamma)
     {
-		ResourceCreateOrRetrieveResult res =
+        ResourceCreateOrRetrieveResult res =
             createOrRetrieve(name,group,false,0,0,texType,numMipmaps,gamma,isAlpha,desiredFormat,hwGamma);
-        TexturePtr tex = res.first.staticCast<Texture>();
-		tex->prepare();
+        TexturePtr tex = static_pointer_cast<Texture>(res.first);
+        tex->prepare();
         return tex;
     }
     //-----------------------------------------------------------------------
@@ -110,27 +106,27 @@ namespace Ogre {
                                     int numMipmaps, Real gamma, bool isAlpha, PixelFormat desiredFormat,
                                     bool hwGamma)
     {
-		ResourceCreateOrRetrieveResult res =
+        ResourceCreateOrRetrieveResult res =
             createOrRetrieve(name,group,false,0,0,texType,numMipmaps,gamma,isAlpha,desiredFormat,hwGamma);
-        TexturePtr tex = res.first.staticCast<Texture>();
-		tex->load();
+        TexturePtr tex = static_pointer_cast<Texture>(res.first);
+        tex->load();
         return tex;
     }
 
     //-----------------------------------------------------------------------
     TexturePtr TextureManager::loadImage( const String &name, const String& group,
         const Image &img, TextureType texType, int numMipmaps, Real gamma, bool isAlpha, 
-		PixelFormat desiredFormat, bool hwGamma)
+        PixelFormat desiredFormat, bool hwGamma)
     {
-        TexturePtr tex = createResource(name, group, true).staticCast<Texture>();
+        TexturePtr tex = create(name, group, true);
 
         tex->setTextureType(texType);
         tex->setNumMipmaps((numMipmaps == MIP_DEFAULT)? mDefaultNumMipmaps :
-			static_cast<size_t>(numMipmaps));
+            static_cast<uint32>(numMipmaps));
         tex->setGamma(gamma);
         tex->setTreatLuminanceAsAlpha(isAlpha);
         tex->setFormat(desiredFormat);
-		tex->setHardwareGammaEnabled(hwGamma);
+        tex->setHardwareGammaEnabled(hwGamma);
         tex->loadImage(img);
 
         return tex;
@@ -140,29 +136,30 @@ namespace Ogre {
         DataStreamPtr& stream, ushort uWidth, ushort uHeight, 
         PixelFormat format, TextureType texType, 
         int numMipmaps, Real gamma, bool hwGamma)
-	{
-		TexturePtr tex = createResource(name, group, true).staticCast<Texture>();
+    {
+        TexturePtr tex = create(name, group, true);
 
         tex->setTextureType(texType);
         tex->setNumMipmaps((numMipmaps == MIP_DEFAULT)? mDefaultNumMipmaps :
-			static_cast<size_t>(numMipmaps));
+            static_cast<uint32>(numMipmaps));
         tex->setGamma(gamma);
-		tex->setHardwareGammaEnabled(hwGamma);
-		tex->loadRawData(stream, uWidth, uHeight, format);
-		
+        tex->setHardwareGammaEnabled(hwGamma);
+        tex->loadRawData(stream, uWidth, uHeight, format);
+        
         return tex;
-	}
+    }
     //-----------------------------------------------------------------------
     TexturePtr TextureManager::createManual(const String & name, const String& group,
         TextureType texType, uint width, uint height, uint depth, int numMipmaps,
         PixelFormat format, int usage, ManualResourceLoader* loader, bool hwGamma, 
-		uint fsaa, const String& fsaaHint)
+        uint fsaa, const String& fsaaHint)
     {
         TexturePtr ret;
-        ret.setNull();
+
+        OgreAssert(width && height && depth, "total size of texture must not be zero");
 
         // Check for 3D texture support
-		const RenderSystemCapabilities* caps =
+        const RenderSystemCapabilities* caps =
             Root::getSingleton().getRenderSystem()->getCapabilities();
         if (((texType == TEX_TYPE_3D) || (texType == TEX_TYPE_2D_ARRAY)) &&
             !caps->hasCapability(RSC_TEXTURE_3D))
@@ -172,19 +169,23 @@ namespace Ogre {
         {
             usage = (usage & ~(int)TU_STATIC) | (int)TU_DYNAMIC;
         }
-        ret = createResource(name, group, true, loader).staticCast<Texture>();
+        ret = create(name, group, true, loader);
+
+        if(!ret)
+            return ret;
+
         ret->setTextureType(texType);
         ret->setWidth(width);
         ret->setHeight(height);
-		ret->setDepth(depth);
+        ret->setDepth(depth);
         ret->setNumMipmaps((numMipmaps == MIP_DEFAULT)? mDefaultNumMipmaps :
-			static_cast<size_t>(numMipmaps));
+            static_cast<uint32>(numMipmaps));
         ret->setFormat(format);
         ret->setUsage(usage);
-		ret->setHardwareGammaEnabled(hwGamma);
-		ret->setFSAA(fsaa, fsaaHint);
-		ret->createInternalResources();
-		return ret;
+        ret->setHardwareGammaEnabled(hwGamma);
+        ret->setFSAA(fsaa, fsaaHint);
+        ret->createInternalResources();
+        return ret;
     }
     //-----------------------------------------------------------------------
     void TextureManager::setPreferredIntegerBitDepth(ushort bits, bool reloadTextures)
@@ -273,22 +274,69 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void TextureManager::setDefaultNumMipmaps( size_t num )
+    void TextureManager::setDefaultNumMipmaps( uint32 num )
     {
         mDefaultNumMipmaps = num;
     }
     //-----------------------------------------------------------------------
-	bool TextureManager::isFormatSupported(TextureType ttype, PixelFormat format, int usage)
-	{
-		return getNativeFormat(ttype, format, usage) == format;
-	}
+    bool TextureManager::isFormatSupported(TextureType ttype, PixelFormat format, int usage)
+    {
+        return getNativeFormat(ttype, format, usage) == format;
+    }
     //-----------------------------------------------------------------------
-	bool TextureManager::isEquivalentFormatSupported(TextureType ttype, PixelFormat format, int usage)
-	{
-		PixelFormat supportedFormat = getNativeFormat(ttype, format, usage);
+    bool TextureManager::isEquivalentFormatSupported(TextureType ttype, PixelFormat format, int usage)
+    {
+        PixelFormat supportedFormat = getNativeFormat(ttype, format, usage);
 
-		// Assume that same or greater number of bits means quality not degraded
-		return PixelUtil::getNumElemBits(supportedFormat) >= PixelUtil::getNumElemBits(format);
-		
-	}
+        // Assume that same or greater number of bits means quality not degraded
+        return PixelUtil::getNumElemBits(supportedFormat) >= PixelUtil::getNumElemBits(format);
+        
+    }
+
+    bool TextureManager::isHardwareFilteringSupported(TextureType ttype, PixelFormat format,
+                                                      int usage, bool preciseFormatOnly)
+    {
+        if (format == PF_UNKNOWN)
+            return false;
+
+        // Check native format
+        if (preciseFormatOnly && !isFormatSupported(ttype, format, usage))
+            return false;
+
+        return true;
+    }
+
+    const TexturePtr& TextureManager::_getWarningTexture()
+    {
+        if(mWarningTexture)
+            return mWarningTexture;
+
+        // Generate warning texture
+        PixelBox pixels(8, 8, 1, PF_R5G6B5);
+        DataStreamPtr data(new MemoryDataStream(pixels.getConsecutiveSize()));
+        pixels.data = static_pointer_cast<MemoryDataStream>(data)->getPtr();
+
+        // Yellow/black stripes
+        const ColourValue black(0, 0, 0), yellow(1, 1, 0);
+        for (size_t y = 0; y < pixels.getHeight(); ++y)
+        {
+            for (size_t x = 0; x < pixels.getWidth(); ++x)
+            {
+                pixels.setColourAt((((x + y) % 8) < 4) ? black : yellow, x, y, 0);
+            }
+        }
+
+        mWarningTexture = loadRawData("Warning", ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME,
+                                      data, pixels.getWidth(), pixels.getHeight(), pixels.format);
+
+        return mWarningTexture;
+    }
+
+    const SamplerPtr& TextureManager::getDefaultSampler()
+    {
+        if(!mDefaultSampler)
+            mDefaultSampler = createSampler();
+
+        return mDefaultSampler;
+    }
 }
