@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2012 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -27,15 +27,7 @@ THE SOFTWARE.
 */
 #include "OgreStableHeaders.h"
 #include "OgreViewport.h"
-
-#include "OgreLogManager.h"
 #include "OgreRenderTarget.h"
-#include "OgreCamera.h"
-#include "OgreMath.h"
-#include "OgreRoot.h"
-#include "OgreMaterialManager.h"
-#include "OgreRenderSystem.h"
-#include "OgreRenderWindow.h"
 
 namespace Ogre {
     OrientationMode Viewport::mDefaultOrientationMode = OR_DEGREE_0;
@@ -50,30 +42,31 @@ namespace Ogre {
         // Actual dimensions will update later
         , mZOrder(ZOrder)
         , mBackColour(ColourValue::Black)
-		, mDepthClearValue(1)
+        , mDepthClearValue(1)
         , mClearEveryFrame(true)
-		, mClearBuffers(FBT_COLOUR | FBT_DEPTH)
+        , mClearBuffers(FBT_COLOUR | FBT_DEPTH)
         , mUpdated(false)
         , mShowOverlays(true)
         , mShowSkies(true)
-		, mShowShadows(true)
-		, mVisibilityMask(0xFFFFFFFF)
-		, mRQSequence(0)
-		, mMaterialSchemeName(MaterialManager::DEFAULT_SCHEME_NAME)
-		, mIsAutoUpdated(true)
-    {			
+        , mShowShadows(true)
+        , mVisibilityMask(0xFFFFFFFF)
+        , mRQSequence(0)
+        , mMaterialSchemeName(MaterialManager::DEFAULT_SCHEME_NAME)
+        , mIsAutoUpdated(true)
+		, mColourBuffer(CBT_BACK)
+    {           
 #if OGRE_COMPILER != OGRE_COMPILER_GCCE && OGRE_PLATFORM != OGRE_PLATFORM_ANDROID
-		LogManager::getSingleton().stream(LML_TRIVIAL)
-			<< "Creating viewport on target '" << target->getName() << "'"
-			<< ", rendering from camera '" << (cam != 0 ? cam->getName() : "NULL") << "'"
-			<< ", relative dimensions "	<< std::ios::fixed << std::setprecision(2) 
-			<< "L: " << left << " T: " << top << " W: " << width << " H: " << height
-			<< " ZOrder: " << ZOrder;
+        LogManager::getSingleton().stream(LML_TRIVIAL)
+            << "Creating viewport on target '" << target->getName() << "'"
+            << ", rendering from camera '" << (cam != 0 ? cam->getName() : "NULL") << "'"
+            << ", relative dimensions " << std::ios::fixed << std::setprecision(2) 
+            << "L: " << left << " T: " << top << " W: " << width << " H: " << height
+            << " Z-order: " << ZOrder;
 #endif
 
         // Set the default orientation mode
         mOrientationMode = mDefaultOrientationMode;
-			
+            
         // Set the default material scheme
         RenderSystem* rs = Root::getSingleton().getRenderSystem();
         mMaterialSchemeName = rs->_getDefaultViewportMaterialScheme();
@@ -87,11 +80,18 @@ namespace Ogre {
     //---------------------------------------------------------------------
     Viewport::~Viewport()
     {
-		RenderSystem* rs = Root::getSingleton().getRenderSystem();
-		if ((rs) && (rs->_getViewport() == this))
-		{
-			rs->_setViewport(NULL);
-		}
+        ListenerList listenersCopy;
+        std::swap(mListeners, listenersCopy);
+        for (ListenerList::iterator i = listenersCopy.begin(); i != listenersCopy.end(); ++i)
+        {
+            (*i)->viewportDestroyed(this);
+        }
+
+        RenderSystem* rs = Root::getSingleton().getRenderSystem();
+        if ((rs) && (rs->_getViewport() == this))
+        {
+            rs->_setViewport(NULL);
+        }
     }
     //---------------------------------------------------------------------
     bool Viewport::_isUpdated(void) const
@@ -131,25 +131,25 @@ namespace Ogre {
         }
 
 #if OGRE_COMPILER != OGRE_COMPILER_GCCE
-		LogManager::getSingleton().stream(LML_TRIVIAL)
-			<< "Viewport for camera '" << (mCamera != 0 ? mCamera->getName() : "NULL") << "'"
-			<< ", actual dimensions "	<< std::ios::fixed << std::setprecision(2) 
-			<< "L: " << mActLeft << " T: " << mActTop << " W: " << mActWidth << " H: " << mActHeight;
+        LogManager::getSingleton().stream(LML_TRIVIAL)
+            << "Viewport for camera '" << (mCamera != 0 ? mCamera->getName() : "NULL") << "'"
+            << ", actual dimensions "   << std::ios::fixed << std::setprecision(2) 
+            << "L: " << mActLeft << " T: " << mActTop << " W: " << mActWidth << " H: " << mActHeight;
 #endif
 
-		 mUpdated = true;
+         mUpdated = true;
 
-		for (ListenerList::iterator i = mListeners.begin(); i != mListeners.end(); ++i)
-		{
-			(*i)->viewportDimensionsChanged(this);
-		}
-	}
-	//---------------------------------------------------------------------
-	int Viewport::getZOrder(void) const
-	{
-		return mZOrder;
-	}
-	//---------------------------------------------------------------------
+        for (ListenerList::iterator i = mListeners.begin(); i != mListeners.end(); ++i)
+        {
+            (*i)->viewportDimensionsChanged(this);
+        }
+    }
+    //---------------------------------------------------------------------
+    int Viewport::getZOrder(void) const
+    {
+        return mZOrder;
+    }
+    //---------------------------------------------------------------------
     RenderTarget* Viewport::getTarget(void) const
     {
         return mTarget;
@@ -213,6 +213,9 @@ namespace Ogre {
     {
         if (mCamera)
         {
+            if (mCamera->getViewport() != this)
+                mCamera->_notifyViewport(this);
+
             // Tell Camera to render into me
             mCamera->_renderScene(this, mShowOverlays);
         }
@@ -222,8 +225,7 @@ namespace Ogre {
     {
 #if OGRE_NO_VIEWPORT_ORIENTATIONMODE != 0
         OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
-                    "Setting Viewport orientation mode is not supported",
-                    __FUNCTION__);
+                    "Setting Viewport orientation mode is not supported");
 #endif
         mOrientationMode = orientationMode;
 
@@ -237,7 +239,7 @@ namespace Ogre {
             mCamera->setOrientationMode(mOrientationMode);
         }
 
-	// Update the render system config
+    // Update the render system config
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
         RenderSystem* rs = Root::getSingleton().getRenderSystem();
         if(mOrientationMode == OR_LANDSCAPELEFT)
@@ -253,8 +255,7 @@ namespace Ogre {
     {
 #if OGRE_NO_VIEWPORT_ORIENTATIONMODE != 0
         OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
-                    "Getting Viewport orientation mode is not supported",
-                    __FUNCTION__);
+                    "Getting Viewport orientation mode is not supported");
 #endif
         return mOrientationMode;
     }
@@ -263,8 +264,7 @@ namespace Ogre {
     {
 #if OGRE_NO_VIEWPORT_ORIENTATIONMODE != 0
         OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
-                    "Setting default Viewport orientation mode is not supported",
-                    __FUNCTION__);
+                    "Setting default Viewport orientation mode is not supported");
 #endif
         mDefaultOrientationMode = orientationMode;
     }
@@ -273,8 +273,7 @@ namespace Ogre {
     {
 #if OGRE_NO_VIEWPORT_ORIENTATIONMODE != 0
         OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED,
-                    "Getting default Viewport orientation mode is not supported",
-                    __FUNCTION__);
+                    "Getting default Viewport orientation mode is not supported");
 #endif
         return mDefaultOrientationMode;
     }
@@ -289,7 +288,7 @@ namespace Ogre {
         return mBackColour;
     }
     //---------------------------------------------------------------------
-	void Viewport::setDepthClear( Real depth )
+    void Viewport::setDepthClear( Real depth )
     {
         mDepthClearValue = depth;
     }
@@ -298,11 +297,11 @@ namespace Ogre {
     {
         return mDepthClearValue;
     }
-	//---------------------------------------------------------------------
+    //---------------------------------------------------------------------
     void Viewport::setClearEveryFrame(bool inClear, unsigned int inBuffers)
     {
         mClearEveryFrame = inClear;
-		mClearBuffers = inBuffers;
+        mClearBuffers = inBuffers;
     }
     //---------------------------------------------------------------------
     bool Viewport::getClearEveryFrame(void) const
@@ -315,23 +314,23 @@ namespace Ogre {
         return mClearBuffers;
     }
     //---------------------------------------------------------------------
-	void Viewport::clear(unsigned int buffers, const ColourValue& col,  
-						 Real depth, unsigned short stencil)
-	{
-		RenderSystem* rs = Root::getSingleton().getRenderSystem();
-		if (rs)
-		{
-			Viewport* currentvp = rs->_getViewport();
-			if (currentvp && currentvp == this)
-				rs->clearFrameBuffer(buffers, col, depth, stencil);
-			else if (currentvp)
-			{
-				rs->_setViewport(this);
-				rs->clearFrameBuffer(buffers, col, depth, stencil);
-				rs->_setViewport(currentvp);
-			}
-		}
-	}
+    void Viewport::clear(unsigned int buffers, const ColourValue& col,  
+                         Real depth, unsigned short stencil)
+    {
+        RenderSystem* rs = Root::getSingleton().getRenderSystem();
+        if (rs)
+        {
+            Viewport* currentvp = rs->_getViewport();
+            if (currentvp && currentvp == this)
+                rs->clearFrameBuffer(buffers, col, depth, stencil);
+            else
+            {
+                rs->_setViewport(this);
+                rs->clearFrameBuffer(buffers, col, depth, stencil);
+                rs->_setViewport(currentvp);
+            }
+        }
+    }
     //---------------------------------------------------------------------
     void Viewport::getActualDimensions(int &left, int&top, int &width, int &height) const
     {
@@ -344,54 +343,51 @@ namespace Ogre {
     //---------------------------------------------------------------------
     unsigned int Viewport::_getNumRenderedFaces(void) const
     {
-		return mCamera ? mCamera->_getNumRenderedFaces() : 0;
+        return mCamera ? mCamera->_getNumRenderedFaces() : 0;
     }
     //---------------------------------------------------------------------
     unsigned int Viewport::_getNumRenderedBatches(void) const
     {
-		return mCamera ? mCamera->_getNumRenderedBatches() : 0;
+        return mCamera ? mCamera->_getNumRenderedBatches() : 0;
     }
-	//---------------------------------------------------------------------
-	void Viewport::setCamera(Camera* cam)
-	{
-		if(mCamera)
-		{
-			if(mCamera->getViewport() == this)
-			{
-				mCamera->_notifyViewport(0);
-			}
-		}
+    //---------------------------------------------------------------------
+    void Viewport::setCamera(Camera* cam)
+    {
+        if (cam != NULL && mCamera != NULL && mCamera->getViewport() == this)
+        {
+                mCamera->_notifyViewport(NULL);
+        }
 
-		mCamera = cam;
-		if (cam)
-		{
-			// update aspect ratio of new camera if needed.
-			if (cam->getAutoAspectRatio())
-			{
-				cam->setAspectRatio((Real) mActWidth / (Real) mActHeight);
-			}
+        mCamera = cam;
+        if (cam)
+        {
+            // update aspect ratio of new camera if needed.
+            if (cam->getAutoAspectRatio())
+            {
+                cam->setAspectRatio((Real) mActWidth / (Real) mActHeight);
+            }
 #if OGRE_NO_VIEWPORT_ORIENTATIONMODE == 0
-			cam->setOrientationMode(mOrientationMode);
+            cam->setOrientationMode(mOrientationMode);
 #endif
-			cam->_notifyViewport(this);
-		}
+            cam->_notifyViewport(this);
+        }
 
-		for (ListenerList::iterator i = mListeners.begin(); i != mListeners.end(); ++i)
-		{
-			(*i)->viewportCameraChanged(this);
-		}
+        for (ListenerList::iterator i = mListeners.begin(); i != mListeners.end(); ++i)
+        {
+            (*i)->viewportCameraChanged(this);
+        }
+    }
+    //---------------------------------------------------------------------
+    void Viewport::setAutoUpdated(bool inAutoUpdated)
+    {
+        mIsAutoUpdated = inAutoUpdated;
+    }
+    //---------------------------------------------------------------------
+    bool Viewport::isAutoUpdated() const
+    {
+        return mIsAutoUpdated;
     }
     //---------------------------------------------------------------------
-	void Viewport::setAutoUpdated(bool inAutoUpdated)
-	{
-		mIsAutoUpdated = inAutoUpdated;
-	}
-	//---------------------------------------------------------------------
-	bool Viewport::isAutoUpdated() const
-	{
-		return mIsAutoUpdated;
-	}
-	//---------------------------------------------------------------------
     void Viewport::setOverlaysEnabled(bool enabled)
     {
         mShowOverlays = enabled;
@@ -421,36 +417,36 @@ namespace Ogre {
     {
         return mShowShadows;
     }
-	//-----------------------------------------------------------------------
-	void Viewport::setRenderQueueInvocationSequenceName(const String& sequenceName)
-	{
-		mRQSequenceName = sequenceName;
-		if (mRQSequenceName.empty())
-		{
-			mRQSequence = 0;
-		}
-		else
-		{
-			mRQSequence =
-				Root::getSingleton().getRenderQueueInvocationSequence(mRQSequenceName);
-		}
-	}
-	//-----------------------------------------------------------------------
-	const String& Viewport::getRenderQueueInvocationSequenceName(void) const
-	{
-		return mRQSequenceName;
-	}
-	//-----------------------------------------------------------------------
-	RenderQueueInvocationSequence* Viewport::_getRenderQueueInvocationSequence(void)
-	{
-		return mRQSequence;
-	}
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
+    void Viewport::setRenderQueueInvocationSequenceName(const String& sequenceName)
+    {
+        mRQSequenceName = sequenceName;
+        if (mRQSequenceName.empty())
+        {
+            mRQSequence = 0;
+        }
+        else
+        {
+            mRQSequence =
+                Root::getSingleton().getRenderQueueInvocationSequence(mRQSequenceName);
+        }
+    }
+    //-----------------------------------------------------------------------
+    const String& Viewport::getRenderQueueInvocationSequenceName(void) const
+    {
+        return mRQSequenceName;
+    }
+    //-----------------------------------------------------------------------
+    RenderQueueInvocationSequence* Viewport::_getRenderQueueInvocationSequence(void)
+    {
+        return mRQSequence;
+    }
+    //-----------------------------------------------------------------------
     void Viewport::pointOrientedToScreen(const Vector2 &v, int orientationMode, Vector2 &outv)
     {
         pointOrientedToScreen(v.x, v.y, orientationMode, outv.x, outv.y);
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void Viewport::pointOrientedToScreen(Real orientedX, Real orientedY, int orientationMode,
                                          Real &screenX, Real &screenY)
     {
@@ -475,19 +471,44 @@ namespace Ogre {
             screenY = orY;
             break;
         }
+    }
+    //-----------------------------------------------------------------------
+    void Viewport::addListener(Listener* l)
+    {
+        if (std::find(mListeners.begin(), mListeners.end(), l) == mListeners.end())
+            mListeners.push_back(l);
+    }
+    //-----------------------------------------------------------------------
+    void Viewport::removeListener(Listener* l)
+    {
+        ListenerList::iterator i = std::find(mListeners.begin(), mListeners.end(), l);
+        if (i != mListeners.end())
+            mListeners.erase(i);
     }
 	//-----------------------------------------------------------------------
-	void Viewport::addListener(Listener* l)
+	void Viewport::setDrawBuffer(ColourBufferType colourBuffer) 
 	{
-		if (std::find(mListeners.begin(), mListeners.end(), l) == mListeners.end())
-			mListeners.push_back(l);
+		mColourBuffer = colourBuffer;
 	}
 	//-----------------------------------------------------------------------
-	void Viewport::removeListener(Listener* l)
+	ColourBufferType Viewport::getDrawBuffer() const
 	{
-		ListenerList::iterator i = std::find(mListeners.begin(), mListeners.end(), l);
-		if (i != mListeners.end())
-			mListeners.erase(i);
+		return mColourBuffer;
 	}
+	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
+    void Viewport::Listener::viewportCameraChanged(Viewport*)
+    {
+    }
+
+    //-----------------------------------------------------------------------
+    void Viewport::Listener::viewportDimensionsChanged(Viewport*)
+    {
+    }
+
+    //-----------------------------------------------------------------------
+    void Viewport::Listener::viewportDestroyed(Viewport*)
+    {
+    }
 
 }
