@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,9 +29,13 @@ THE SOFTWARE.
 #include "OgreRoot.h"
 #include "OgreBspSceneManagerPlugin.h"
 
-Ogre::BspSceneManagerPlugin* bspPlugin;
 //-----------------------------------------------------------------------
 #ifndef OGRE_STATIC_LIB
+static Ogre::BspSceneManagerPlugin* bspPlugin;
+
+extern "C" void _OgreBspPluginExport dllStartPlugin(void);
+extern "C" void _OgreBspPluginExport dllStopPlugin(void);
+
 extern "C" void _OgreBspPluginExport dllStartPlugin(void)
 {
     // Create new scene manager
@@ -44,7 +48,7 @@ extern "C" void _OgreBspPluginExport dllStartPlugin(void)
 }
 extern "C" void _OgreBspPluginExport dllStopPlugin(void)
 {
-	Ogre::Root::getSingleton().uninstallPlugin(bspPlugin);
+    Ogre::Root::getSingleton().uninstallPlugin(bspPlugin);
     OGRE_DELETE bspPlugin;
 }
 #endif
