@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -31,28 +31,32 @@ THE SOFTWARE.
 #include "OgreImageCodec.h"
 
 namespace Ogre {
-
-    /** 
-     * Codec specialized in loading OpenEXR high dynamic range images.
-     */
+    /** \addtogroup Plugins Plugins
+    *  @{
+    */
+    /** \defgroup EXRCodec EXRCodec
+    * %Codec specialized in loading OpenEXR high dynamic range images.
+    *  @{
+    */
     class EXRCodec : public ImageCodec
     {
     public:
         EXRCodec();
         virtual ~EXRCodec();
 
-        /// @copydoc Codec::code
-        DataStreamPtr code(MemoryDataStreamPtr& input, CodecDataPtr& pData) const;
-        /// @copydoc Codec::codeToFile
-        void codeToFile(MemoryDataStreamPtr& input, const String& outFileName, CodecDataPtr& pData) const;
+        /// @copydoc Codec::encode
+        DataStreamPtr encode(const MemoryDataStreamPtr& input, const CodecDataPtr& pData) const;
+        /// @copydoc Codec::encodeToFile
+        void encodeToFile(const MemoryDataStreamPtr& input, const String& outFileName, const CodecDataPtr& pData) const;
         /// @copydoc Codec::decode
-        DecodeResult decode(DataStreamPtr& input) const;
+        DecodeResult decode(const DataStreamPtr& input) const;
         /// @copydoc Codec::magicNumberToFileExt
         String magicNumberToFileExt(const char* magicNumberPtr, size_t maxbytes) const;
 
         String getType() const;
     };
-
+    /** @} */
+    /** @} */
 } // namespace
 
 #endif
