@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,20 +34,20 @@ THE SOFTWARE.
 namespace Ogre {
     
     // init statics
-    ColourInterpolatorAffector::CmdColourAdjust		ColourInterpolatorAffector::msColourCmd[MAX_STAGES];
-    ColourInterpolatorAffector::CmdTimeAdjust		ColourInterpolatorAffector::msTimeCmd[MAX_STAGES];
+    ColourInterpolatorAffector::CmdColourAdjust     ColourInterpolatorAffector::msColourCmd[MAX_STAGES];
+    ColourInterpolatorAffector::CmdTimeAdjust       ColourInterpolatorAffector::msTimeCmd[MAX_STAGES];
 
     //-----------------------------------------------------------------------
     ColourInterpolatorAffector::ColourInterpolatorAffector(ParticleSystem* psys)
         : ParticleAffector(psys)
     {
-		for (int i=0;i<MAX_STAGES;i++)
-		{
-			// set default colour to transparent grey, transparent since we might not want to display the particle here
-			// grey because when a colour component is 0.5f the maximum difference to another colour component is 0.5f
-			mColourAdj[i]	= ColourValue(0.5f, 0.5f, 0.5f, 0.0f);
-			mTimeAdj[i]		= 1.0f;
-		}
+        for (int i=0;i<MAX_STAGES;i++)
+        {
+            // set default colour to transparent grey, transparent since we might not want to display the particle here
+            // grey because when a colour component is 0.5f the maximum difference to another colour component is 0.5f
+            mColourAdj[i]   = ColourValue(0.5f, 0.5f, 0.5f, 0.0f);
+            mTimeAdj[i]     = 1.0f;
+        }
 
         mType = "ColourInterpolator";
 
@@ -56,63 +56,63 @@ namespace Ogre {
         {
             ParamDictionary* dict = getParamDictionary();
 
-			for (int i=0;i<MAX_STAGES;i++)
-			{
-				msColourCmd[i].mIndex	= i;
-				msTimeCmd[i].mIndex		= i;
+            for (int i=0;i<MAX_STAGES;i++)
+            {
+                msColourCmd[i].mIndex   = i;
+                msTimeCmd[i].mIndex     = i;
 
-				StringUtil::StrStreamType stage;
-				stage << i;
-				String	colour_title	= String("colour") + stage.str();
-				String	time_title		= String("time") + stage.str();
-				String	colour_descr	= String("Stage ") + stage.str() + String(" colour.");
-				String	time_descr		= String("Stage ") + stage.str() + String(" time.");
+                StringStream stage;
+                stage << i;
+                String  colour_title    = String("colour") + stage.str();
+                String  time_title      = String("time") + stage.str();
+                String  colour_descr    = String("Stage ") + stage.str() + String(" colour.");
+                String  time_descr      = String("Stage ") + stage.str() + String(" time.");
 
-				dict->addParameter(ParameterDef(colour_title, colour_descr, PT_COLOURVALUE), &msColourCmd[i]);
-				dict->addParameter(ParameterDef(time_title,   time_descr,   PT_REAL),		 &msTimeCmd[i]);
-			}
+                dict->addParameter(ParameterDef(colour_title, colour_descr, PT_COLOURVALUE), &msColourCmd[i]);
+                dict->addParameter(ParameterDef(time_title,   time_descr,   PT_REAL),        &msTimeCmd[i]);
+            }
         }
     }
     //-----------------------------------------------------------------------
     void ColourInterpolatorAffector::_affectParticles(ParticleSystem* pSystem, Real timeElapsed)
     {
-        Particle*			p;
-		ParticleIterator	pi				= pSystem->_getIterator();
+        Particle*           p;
+        ParticleIterator    pi              = pSystem->_getIterator();
 
 
-		while (!pi.end())
+        while (!pi.end())
         {
             p = pi.getNext();
-			const Real		life_time		= p->totalTimeToLive;
-			Real			particle_time	= 1.0f - (p->timeToLive / life_time); 
+            const Real      life_time       = p->mTotalTimeToLive;
+            Real            particle_time   = 1.0f - (p->mTimeToLive / life_time);
 
-			if (particle_time <= mTimeAdj[0])
-			{
-				p->colour = mColourAdj[0];
-			} else
-			if (particle_time >= mTimeAdj[MAX_STAGES - 1])
-			{
-				p->colour = mColourAdj[MAX_STAGES-1];
-			} else
-			{
-				for (int i=0;i<MAX_STAGES-1;i++)
-				{
-					if (particle_time >= mTimeAdj[i] && particle_time < mTimeAdj[i + 1])
-					{
-						particle_time -= mTimeAdj[i];
-						particle_time /= (mTimeAdj[i+1]-mTimeAdj[i]);
-						p->colour.r = ((mColourAdj[i+1].r * particle_time) + (mColourAdj[i].r * (1.0f - particle_time)));
-						p->colour.g = ((mColourAdj[i+1].g * particle_time) + (mColourAdj[i].g * (1.0f - particle_time)));
-						p->colour.b = ((mColourAdj[i+1].b * particle_time) + (mColourAdj[i].b * (1.0f - particle_time)));
-						p->colour.a = ((mColourAdj[i+1].a * particle_time) + (mColourAdj[i].a * (1.0f - particle_time)));
-						break;
-					}
-				}
-			}
-		}
+            if (particle_time <= mTimeAdj[0])
+            {
+                p->mColour = mColourAdj[0];
+            } else
+            if (particle_time >= mTimeAdj[MAX_STAGES - 1])
+            {
+                p->mColour = mColourAdj[MAX_STAGES-1];
+            } else
+            {
+                for (int i=0;i<MAX_STAGES-1;i++)
+                {
+                    if (particle_time >= mTimeAdj[i] && particle_time < mTimeAdj[i + 1])
+                    {
+                        particle_time -= mTimeAdj[i];
+                        particle_time /= (mTimeAdj[i+1]-mTimeAdj[i]);
+                        p->mColour.r = ((mColourAdj[i+1].r * particle_time) + (mColourAdj[i].r * (1.0f - particle_time)));
+                        p->mColour.g = ((mColourAdj[i+1].g * particle_time) + (mColourAdj[i].g * (1.0f - particle_time)));
+                        p->mColour.b = ((mColourAdj[i+1].b * particle_time) + (mColourAdj[i].b * (1.0f - particle_time)));
+                        p->mColour.a = ((mColourAdj[i+1].a * particle_time) + (mColourAdj[i].a * (1.0f - particle_time)));
+                        break;
+                    }
+                }
+            }
+        }
     }
     
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     void ColourInterpolatorAffector::setColourAdjust(size_t index, ColourValue colour)
     {
         mColourAdj[index] = colour;
@@ -135,8 +135,8 @@ namespace Ogre {
         return mTimeAdj[index];
     }
     
-	
-	//-----------------------------------------------------------------------
+    
+    //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     //-----------------------------------------------------------------------
     // Command objects
@@ -152,7 +152,7 @@ namespace Ogre {
         static_cast<ColourInterpolatorAffector*>(target)->setColourAdjust(mIndex,
             StringConverter::parseColourValue(val));
     }
-	//-----------------------------------------------------------------------
+    //-----------------------------------------------------------------------
     String ColourInterpolatorAffector::CmdTimeAdjust::doGet(const void* target) const
     {
         return StringConverter::toString(
