@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,45 +34,33 @@ THE SOFTWARE.
 
 namespace Ogre { 
 
+    class _OgreD3D11Export D3D11HardwareIndexBuffer : public HardwareIndexBuffer
+    {
+    protected:
+        D3D11HardwareBuffer* mBufferImpl;
+        // have to implement these, but do nothing as overridden lock/unlock
+        void* lockImpl(size_t offset, size_t length, LockOptions options) {return 0;}
+        void unlockImpl(void) {}
 
-	class D3D11HardwareIndexBuffer : public HardwareIndexBuffer
-	{
-	protected:
-		D3D11HardwareBuffer* mBufferImpl;
-		// have to implement these, but do nothing as overridden lock/unlock
-		void* lockImpl(size_t offset, size_t length, LockOptions options) {return 0;}
-		void unlockImpl(void) {}
+    public:
+        D3D11HardwareIndexBuffer(HardwareBufferManagerBase* mgr, IndexType idxType, size_t numIndexes, 
+            HardwareBuffer::Usage usage, D3D11Device & device, bool useSystemMem, bool useShadowBuffer);
+        ~D3D11HardwareIndexBuffer();
 
-	public:
-		D3D11HardwareIndexBuffer(HardwareBufferManagerBase* mgr, IndexType idxType, size_t numIndexes, 
-			HardwareBuffer::Usage usage, D3D11Device & device, bool useSystemMem, bool useShadowBuffer);
-		~D3D11HardwareIndexBuffer();
+        // override all data-gathering methods
+        void* lock(size_t offset, size_t length, LockOptions options);
+        void unlock(void);
+        void readData(size_t offset, size_t length, void* pDest);
+        void writeData(size_t offset, size_t length, const void* pSource,
+            bool discardWholeBuffer = false);
 
-		// override all data-gathering methods
-		void* lock(size_t offset, size_t length, LockOptions options);
-		void unlock(void);
-		void readData(size_t offset, size_t length, void* pDest);
-		void writeData(size_t offset, size_t length, const void* pSource,
-			bool discardWholeBuffer = false);
-
-		void copyData(HardwareBuffer& srcBuffer, size_t srcOffset, 
-			size_t dstOffset, size_t length, bool discardWholeBuffer = false);
-		bool isLocked(void) const;
-
-		/// For dealing with lost devices - release the resource if in the default pool
-		bool releaseIfDefaultPool(void);
-		/// For dealing with lost devices - recreate the resource if in the default pool
-		bool recreateIfDefaultPool(D3D11Device & device);
-
-		/// Get the D3D-specific index buffer
-		ID3D11Buffer * getD3DIndexBuffer(void) const;
-
-
-
-
-
-	};
+        void copyData(HardwareBuffer& srcBuffer, size_t srcOffset, 
+            size_t dstOffset, size_t length, bool discardWholeBuffer = false);
+        bool isLocked(void) const;
 
+        /// Get the D3D-specific index buffer
+        ID3D11Buffer * getD3DIndexBuffer(void) const;
+    };
 
 }
 
