@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -48,31 +48,6 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------------
-    Resource* D3D9GpuProgramManager::createImpl(const String& name, ResourceHandle handle, 
-        const String& group, bool isManual, ManualResourceLoader* loader,
-        const NameValuePairList* params)
-    {
-        NameValuePairList::const_iterator paramIt;
-
-        if (!params || (paramIt = params->find("type")) == params->end())
-        {
-            OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, 
-                "You must supply a 'type' parameter",
-                "D3D9GpuProgramManager::createImpl");
-        }
-
-        if (paramIt->second == "vertex_program")
-        {
-            return OGRE_NEW D3D9GpuVertexProgram(this, name, handle, group, 
-                isManual, loader);
-        }
-        else
-        {
-            return OGRE_NEW D3D9GpuFragmentProgram(this, name, handle, group, 
-                isManual, loader);
-        }
-    }
-    //-----------------------------------------------------------------------------
     Resource* D3D9GpuProgramManager::createImpl(const String& name, ResourceHandle handle, 
         const String& group, bool isManual, ManualResourceLoader* loader,
         GpuProgramType gptype, const String& syntaxCode)
