@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -30,35 +30,11 @@ THE SOFTWARE.
 #define __GLES2HardwarePixelBuffer_H__
 
 #include "OgreGLES2Prerequisites.h"
-#include "OgreHardwarePixelBuffer.h"
+#include "OgreGLHardwarePixelBufferCommon.h"
 
 namespace Ogre {
-    class _OgreGLES2Export GLES2HardwarePixelBuffer: public HardwarePixelBuffer
+    class _OgreGLES2Export GLES2HardwarePixelBuffer: public GLHardwarePixelBufferCommon
     {
-        protected:
-            /// Lock a box
-            PixelBox lockImpl(const Image::Box lockBox,  LockOptions options);
-
-            /// Unlock a box
-            void unlockImpl(void);
-
-            // Internal buffer; either on-card or in system memory, freed/allocated on demand
-            // depending on buffer usage
-            PixelBox mBuffer;
-            GLenum mGLInternalFormat; // GL internal format
-            LockOptions mCurrentLockOptions;
-
-            // Buffer allocation/freeage
-            void allocateBuffer();
-
-            void freeBuffer();
-
-            // Upload a box of pixels to this buffer on the card
-            virtual void upload(const PixelBox &data, const Image::Box &dest);
-
-            // Download a box of pixels from the card
-            virtual void download(const PixelBox &data);
-        
         public:
             /// Should be called by HardwareBufferManager
             GLES2HardwarePixelBuffer(uint32 mWidth, uint32 mHeight, uint32 mDepth,
@@ -66,17 +42,10 @@ namespace Ogre {
                                   HardwareBuffer::Usage usage);
 
             /// @copydoc HardwarePixelBuffer::blitFromMemory
-            void blitFromMemory(const PixelBox &src, const Image::Box &dstBox);
+            void blitFromMemory(const PixelBox &src, const Box &dstBox);
 
             /// @copydoc HardwarePixelBuffer::blitToMemory
-            void blitToMemory(const Image::Box &srcBox, const PixelBox &dst);
-
-            virtual ~GLES2HardwarePixelBuffer();
-
-            /** Bind surface to frame buffer. Needs FBO extension.
-            */
-            virtual void bindToFramebuffer(GLenum attachment, size_t zoffset);
-            GLenum getGLFormat() { return mGLInternalFormat; }
+            void blitToMemory(const Box &srcBox, const PixelBox &dst);
     };
 
     /** Texture surface.
@@ -85,24 +54,24 @@ namespace Ogre {
             {
         public:
             /** Texture constructor */
-            GLES2TextureBuffer(const String &baseName, GLenum target, GLuint id, GLint width, GLint height, GLint depth, GLint internalFormat,
-                               GLint format, GLint face, GLint level, Usage usage, bool softwareMipmap, bool writeGamma, uint fsaa);
+            GLES2TextureBuffer(GLES2Texture* parent, GLint face, GLint level, GLint width,
+                               GLint height, GLint depth);
             virtual ~GLES2TextureBuffer();
 
-            /// @copydoc HardwarePixelBuffer::bindToFramebuffer
-            virtual void bindToFramebuffer(GLenum attachment, size_t zoffset);
+            /// @copydoc GLES2HardwarePixelBuffer::bindToFramebuffer
+            virtual void bindToFramebuffer(uint32 attachment, uint32 zoffset);
 
             /// @copydoc HardwarePixelBuffer::getRenderTarget
             RenderTexture* getRenderTarget(size_t slice);
 
             /// Upload a box of pixels to this buffer on the card
-            virtual void upload(const PixelBox &data, const Image::Box &dest);
+            virtual void upload(const PixelBox &data, const Box &dest);
 
             /// Download a box of pixels from the card
             virtual void download(const PixelBox &data);
 
             /// Hardware implementation of blitFromMemory
-            virtual void blitFromMemory(const PixelBox &src_orig, const Image::Box &dstBox);
+            virtual void blitFromMemory(const PixelBox &src_orig, const Box &dstBox);
 
             /// Notify TextureBuffer of destruction of render target
             void _clearSliceRTT(size_t zoffset)
@@ -114,11 +83,11 @@ namespace Ogre {
             void copyFromFramebuffer(size_t zoffset);
 
             /// @copydoc HardwarePixelBuffer::blit
-            void blit(const HardwarePixelBufferSharedPtr &src, const Image::Box &srcBox, const Image::Box &dstBox);
+            void blit(const HardwarePixelBufferSharedPtr &src, const Box &srcBox, const Box &dstBox);
             // Blitting implementation
-            void blitFromTexture(GLES2TextureBuffer *src, const Image::Box &srcBox, const Image::Box &dstBox);
+            void blitFromTexture(GLES2TextureBuffer *src, const Box &srcBox, const Box &dstBox);
             
-#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
+#if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID || OGRE_PLATFORM == OGRE_PLATFORM_EMSCRIPTEN
         // Friends.
         protected:
             friend class GLES2Texture;
@@ -131,12 +100,10 @@ namespace Ogre {
             GLenum mTarget;
             GLenum mFaceTarget; // same as mTarget in case of GL_TEXTURE_xD, but cubemap face for cubemaps
             GLuint mTextureID;
-            GLuint mBufferId;
             GLint mFace;
             GLint mLevel;
-            bool mSoftwareMipmap;
                 
-            typedef vector<RenderTexture*>::type SliceTRT;
+            typedef std::vector<RenderTexture*> SliceTRT;
             SliceTRT mSliceTRT;
 
             void buildMipmaps(const PixelBox &data);
@@ -147,11 +114,11 @@ namespace Ogre {
     class _OgreGLES2Export GLES2RenderBuffer: public GLES2HardwarePixelBuffer
     {
         public:
-            GLES2RenderBuffer(GLenum format, size_t width, size_t height, GLsizei numSamples);
+            GLES2RenderBuffer(GLenum format, uint32 width, uint32 height, GLsizei numSamples);
             virtual ~GLES2RenderBuffer();
 
             /// @copydoc GLES2HardwarePixelBuffer::bindToFramebuffer
-            virtual void bindToFramebuffer(GLenum attachment, size_t zoffset);
+            virtual void bindToFramebuffer(uint32 attachment, uint32 zoffset);
 
         protected:
             // In case this is a render buffer
