@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -25,34 +25,30 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
-#include <cppunit/TestFixture.h>
-#include <cppunit/extensions/HelperMacros.h>
+
+#ifndef __PixelFormatTests_H__
+#define __PixelFormatTests_H__
+
+#include <gtest/gtest.h>
 #include "OgrePixelFormat.h"
 
 using namespace Ogre;
 
-class PixelFormatTests : public CppUnit::TestFixture
+class PixelFormatTests : public ::testing::Test
 {
-    // CppUnit macros for setting up the test suite
-    CPPUNIT_TEST_SUITE( PixelFormatTests );
-    CPPUNIT_TEST( testIntegerPackUnpack );
-    CPPUNIT_TEST( testFloatPackUnpack );
-    CPPUNIT_TEST( testBulkConversion );
-    CPPUNIT_TEST_SUITE_END();
-public:
-    void setUp();
-    void tearDown();
 
-    void testIntegerPackUnpack();
-    void testFloatPackUnpack();
-    void testBulkConversion();
+public:
+    void SetUp();
+    void TearDown();
 
     // Utils
     void setupBoxes(PixelFormat srcFormat, PixelFormat dstFormat);
     void testCase(PixelFormat srcFormat, PixelFormat dstFormat);
-private:
-    int size;
-    uint8 *randomData;
-    uint8 *temp, *temp2;
-    PixelBox src, dst1, dst2;
+
+    int mSize;
+    uint8 *mRandomData;
+    uint8 *mTemp, *mTemp2;
+    PixelBox mSrc, mDst1, mDst2;
 };
+
+#endif
