@@ -4,7 +4,7 @@ This source file is part of OGRE
 (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -25,32 +25,20 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 -----------------------------------------------------------------------------
 */
-
-#include "OgreBuildSettings.h"
-
-#if defined(OGRE_BUILD_RENDERSYSTEM_GLES2) || defined(OGRE_BUILD_RENDERSYSTEM_GL3PLUS) || defined(OGRE_BUILD_RENDERSYSTEM_D3D11)
-#  define INCLUDE_RTSHADER_SYSTEM
-#endif
-
-//#define _RTSS_WRITE_SHADERS_TO_DISK
-
 #include "TestContext.h"
 #include "SamplePlugin.h"
 #include "TestResultWriter.h"
 #include "HTMLWriter.h"
-#include "SimpleResultWriter.h"
+#include "CppUnitResultWriter.h"
 #include "OgreConfigFile.h"
 #include "OgrePlatform.h"
+#include "OgreBitesConfigDialog.h"
 
+#include <iostream>
 
-#ifdef WIN32
+#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
 #define WIN32_LEAN_AND_MEAN
-#include "windows.h"
-#endif
-
-#if (OGRE_PLATFORM == OGRE_PLATFORM_APPLE) && __LP64__
-#include <AppKit/AppKit.h>
-static id mAppDelegate;
+#include <windows.h>
 #endif
 
 #ifdef OGRE_STATIC_LIB
@@ -58,12 +46,12 @@ static id mAppDelegate;
 #include "PlayPenTestPlugin.h"
 #endif
 
-TestContext::TestContext(int argc, char** argv) : mTimestep(0.01f), mBatch(0), mCurrentTest(0)
+TestContext::TestContext(int argc, char** argv) : OgreBites::SampleContext(), mSuccess(true), mTimestep(0.01f), mCurrentTest(0), mBatch(0)
 {
     Ogre::UnaryOptionList unOpt;
     Ogre::BinaryOptionList binOpt;
 
-    // prepopulate expected options
+    // Prepopulate expected options.
     unOpt["-r"] = false;        // generate reference set
     unOpt["--no-html"] = false; // whether or not to generate HTML
     unOpt["-d"] = false;        // force config dialog
@@ -71,13 +59,15 @@ TestContext::TestContext(int argc, char** argv) : mTimestep(0.01f), mBatch(0), m
     unOpt["-h"] = false;        // help, give usage details
     unOpt["--help"] = false;    // help, give usage details
     binOpt["-m"] = "";          // optional comment
+    binOpt["-rp"] = "";         // optional specified reference set location
+    binOpt["-od"] = "";         // directory to write output to
     binOpt["-ts"] = "VTests";   // name of the test set to use
     binOpt["-c"] = "Reference"; // name of batch to compare against
     binOpt["-n"] = "AUTO";      // name for this batch
     binOpt["-rs"] = "SAVED";    // rendersystem to use (default: use name from the config file/dialog)
     binOpt["-o"] = "NONE";      // path to output a summary file to (default: don't output a file)
 
-    // parse
+    // Parse.
     Ogre::findCommandLineOpts(argc, argv, unOpt, binOpt);
 
     mReferenceSet = unOpt["-r"];
@@ -88,9 +78,14 @@ TestContext::TestContext(int argc, char** argv) : mTimestep(0.01f), mBatch(0), m
     mCompareWith = binOpt["-c"];
     mForceConfig = unOpt["-d"];
     mNoGrabMouse = unOpt["--nograb"];
+    mOutputDir = binOpt["-od"];
     mRenderSystemName = binOpt["-rs"];
+    mReferenceSetPath = binOpt["-rp"];
     mSummaryOutputDir = binOpt["-o"];
     mHelp = unOpt["-h"] || unOpt["--help"];
+
+    if(mReferenceSetPath.empty())
+        mReferenceSetPath = mOutputDir;
 }
 //-----------------------------------------------------------------------
 
@@ -103,7 +98,10 @@ TestContext::~TestContext()
 
 void TestContext::setup()
 {
-    // standard setup
+    NameValuePairList miscParams;
+    mRoot->initialise(false, "OGRE Sample Browser");
+
+    // Standard setup.
 #if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
     CGSize modeSize = [[UIScreen mainScreen] currentMode].size;
     uint w = modeSize.width / [UIScreen mainScreen].scale;
@@ -114,52 +112,65 @@ void TestContext::setup()
     if (UIInterfaceOrientationIsPortrait(orientation))
         std::swap(w, h);
 
-    mRoot->initialise(false, "OGRE Sample Browser");
-    NameValuePairList miscParams;
+
     miscParams["retainedBacking"] = StringConverter::toString(true);
     mWindow = mRoot->createRenderWindow("OGRE Sample Browser", w, h, true, &miscParams);
 #else
-    mWindow = createWindow();
+    Ogre::ConfigOptionMap ropts = mRoot->getRenderSystem()->getConfigOptions();
+
+    size_t w, h;
+
+    std::istringstream mode(ropts["Video Mode"].currentValue);
+    Ogre::String token;
+    mode >> w; // width
+    mode >> token; // 'x' as seperator between width and height
+    mode >> h; // height
+
+    miscParams["FSAA"] = ropts["FSAA"].currentValue;
+    miscParams["vsync"] = ropts["VSync"].currentValue;
+
+    mWindow = mRoot->createRenderWindow("OGRE Sample Browser", w, h, false, &miscParams);
 #endif
 
     mWindow->setDeactivateOnFocusChange(false);
-
-    // grab input, since moving the window seemed to change the results (in Linux anyways)
-    setupInput(mNoGrabMouse);
-
+    
     locateResources();
-    createDummyScene();
-#ifdef INCLUDE_RTSHADER_SYSTEM
-    if (mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
-    {
-        Ogre::RTShader::ShaderGenerator::getSingletonPtr()->addSceneManager(mRoot->getSceneManager("DummyScene"));
-    }
-#endif // INCLUDE_RTSHADER_SYSTEM
+    initialiseRTShaderSystem();
 
     loadResources();
     Ogre::TextureManager::getSingleton().setDefaultNumMipmaps(5);
     mRoot->addFrameListener(this);
 
-#if OGRE_PLATFORM != OGRE_PLATFORM_ANDROID
-    Ogre::WindowEventUtilities::addWindowEventListener(mWindow, this);
-#endif
-
-    // get the path and list of test plugins from the config file
+    // Get the path and list of test plugins from the config file.
     Ogre::ConfigFile testConfig;
     testConfig.load(mFSLayer->getConfigFilePath("tests.cfg"));
-    mPluginDirectory = testConfig.getSetting("TestFolder");
+    mPluginDirectory = Ogre::FileSystemLayer::resolveBundlePath(testConfig.getSetting("TestFolder"));
 
-    Ogre::ConfigFile::SectionIterator sections = testConfig.getSectionIterator();
+#if OGRE_PLATFORM != OGRE_PLATFORM_APPLE && OGRE_PLATFORM != OGRE_PLATFORM_APPLE_IOS
+    if (mPluginDirectory.empty()) mPluginDirectory = ".";   // user didn't specify plugins folder, try current one
+#endif
 
-    // parse for the test sets and plugins that they're made up of
-    for (; sections.hasMoreElements(); sections.moveNext())
+    // add slash or backslash based on platform
+    char lastChar = mPluginDirectory[mPluginDirectory.length() - 1];
+    if (lastChar != '/' && lastChar != '\\')
     {
-        Ogre::String setName = sections.peekNextKey();
+#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32 || (OGRE_PLATFORM == OGRE_PLATFORM_WINRT)
+        mPluginDirectory += "\\";
+#elif OGRE_PLATFORM == OGRE_PLATFORM_LINUX
+        mPluginDirectory += "/";
+#endif
+    }
+
+    // Parse for the test sets and plugins that they're made up of.
+    ConfigFile::SettingsBySection_::const_iterator seci;
+    for(seci = testConfig.getSettingsBySection().begin(); seci != testConfig.getSettingsBySection().end(); ++seci) {
+        const ConfigFile::SettingsMultiMap& settings = seci->second;
+        Ogre::String setName = seci->first;
         if (setName != "")
         {
             mTestSets[setName] = Ogre::StringVector();
-            Ogre::ConfigFile::SettingsMultiMap::iterator it = sections.peekNextValue()->begin();
-            for (; it != sections.peekNextValue()->end(); ++it)
+            Ogre::ConfigFile::SettingsMultiMap::const_iterator it = settings.begin();
+            for (; it != settings.end(); ++it)
                 mTestSets[setName].push_back(it->second);
         }
     }
@@ -169,27 +180,33 @@ void TestContext::setup()
     mPluginNameMap["PlayPenTests"] = (OgreBites::SamplePlugin *) OGRE_NEW PlaypenTestPlugin();
 #endif
 
+    Ogre::String batchName = BLANKSTRING;
+    time_t raw = time(0);
+
     // timestamp for the filename
     char temp[25];
-    time_t raw = time(0);
     strftime(temp, 19, "%Y_%m_%d_%H%M_%S", gmtime(&raw));
-    Ogre::String filestamp = Ogre::String(temp);
-    // name for this batch (used for naming the directory, and uniquely identifying this batch)
-    Ogre::String batchName = mTestSetName + "_" + filestamp;
 
-    // a nicer formatted version for display
+    // A nicer formatted version for display.
     strftime(temp, 20, "%Y-%m-%d %H:%M:%S", gmtime(&raw));
     Ogre::String timestamp = Ogre::String(temp);
 
-    if (mReferenceSet)
-        batchName = "Reference";
-    else if (mBatchName != "AUTO")
-        batchName = mBatchName;
+    if(mOutputDir.empty())
+    {
+        Ogre::String filestamp = Ogre::String(temp);
+        // name for this batch (used for naming the directory, and uniquely identifying this batch)
+        batchName = mTestSetName + "_" + filestamp;
 
-    // set up output directories
+        if (mReferenceSet)
+            batchName = "Reference";
+        else if (mBatchName != "AUTO")
+            batchName = mBatchName;
+    }
+
+    // Set up output directories.
     setupDirectories(batchName);
 
-    // an object storing info about this set
+    // An object storing info about this set.
     mBatch = new TestBatch(batchName, mTestSetName, timestamp,
                            mWindow->getWidth(), mWindow->getHeight(), mOutputDir + batchName + "/");
     mBatch->comment = mComment;
@@ -230,11 +247,11 @@ OgreBites::Sample* TestContext::loadTests(Ogre::String set)
                 mRoot->installPlugin(pluginInstance);
             }
 #else
-            mRoot->loadPlugin(mPluginDirectory + "/" + plugin);
+            mRoot->loadPlugin(mPluginDirectory + plugin);
 #endif
         }
         // if it fails, just return right away
-        catch (Ogre::Exception e)
+        catch (Ogre::Exception&)
         {
             return 0;
         }
@@ -251,17 +268,12 @@ OgreBites::Sample* TestContext::loadTests(Ogre::String set)
         OgreBites::SampleSet newSamples = sp->getSamples();
         for (OgreBites::SampleSet::iterator j = newSamples.begin(); j != newSamples.end(); j++)
         {
-            // skip it if using wrong rendersystem
-            Ogre::String rs = (*j)->getRequiredRenderSystem();
-            if(!rs.empty() && rs != mRoot->getRenderSystem()->getName())
-                continue;
-
             // capability check
             try
             {
                 (*j)->testCapabilities(mRoot->getRenderSystem()->getCapabilities());
             }
-            catch(Ogre::Exception e)
+            catch(Ogre::Exception&)
             {
                 continue;
             }
@@ -293,7 +305,7 @@ OgreBites::Sample* TestContext::loadTests(Ogre::String set)
 
 bool TestContext::frameStarted(const Ogre::FrameEvent& evt)
 {
-    captureInputDevices();
+    pollEvents();
 
     // pass a fixed timestep along to the tests
     Ogre::FrameEvent fixed_evt = Ogre::FrameEvent();
@@ -344,12 +356,6 @@ bool TestContext::frameEnded(const Ogre::FrameEvent& evt)
 
         if (mCurrentTest->isDone())
         {
-#ifdef INCLUDE_RTSHADER_SYSTEM
-            mShaderGenerator->removeAllShaderBasedTechniques(); // clear techniques from the RTSS
-#endif
-
-            createDummyScene();
-
             // continue onto the next test
             runSample(0);
 
@@ -378,11 +384,11 @@ void TestContext::runSample(OgreBites::Sample* s)
     Ogre::ControllerManager::getSingleton().setFrameDelay(0);
     Ogre::ControllerManager::getSingleton().setTimeFactor(1.f);
     mCurrentFrame = 0;
-    destroyDummyScene();
 
     OgreBites::Sample* sampleToRun = s;
 
-    // if a valid test is passed, then run it, if null, grab the next one from the deque
+    // If a valid test is passed, then run it
+    // If null, grab the next one from the deque
     if (!sampleToRun && !mTests.empty())
     {
         mTests.pop_front();
@@ -390,10 +396,10 @@ void TestContext::runSample(OgreBites::Sample* s)
             sampleToRun = mTests.front();
     }
 
-    // check if this is a VisualTest
+    // Check if this is a VisualTest
     mCurrentTest = static_cast<VisualTest*>(sampleToRun);
 
-    // set things up to be deterministic
+    // Set things up to be deterministic
     if (mCurrentTest)
     {
         // Seed rand with a predictable value
@@ -401,10 +407,8 @@ void TestContext::runSample(OgreBites::Sample* s)
 
         // Give a fixed timestep for particles and other time-dependent things in OGRE
         Ogre::ControllerManager::getSingleton().setFrameDelay(mTimestep);
-    }
-
-    if (mCurrentTest)
         LogManager::getSingleton().logMessage("----- Running Visual Test " + mCurrentTest->getInfo()["Title"] + " -----");
+    }
 
 #ifdef INCLUDE_RTSHADER_SYSTEM
     if (sampleToRun) {
@@ -422,7 +426,7 @@ void TestContext::createRoot()
 #if OGRE_PLATFORM == OGRE_PLATFORM_ANDROID
     mRoot = Ogre::Root::getSingletonPtr();
 #else
-    Ogre::String pluginsPath = Ogre::StringUtil::BLANK;
+    Ogre::String pluginsPath = Ogre::BLANKSTRING;
 #ifndef OGRE_STATIC_LIB
     pluginsPath = mFSLayer->getConfigFilePath("plugins.cfg");
 #endif
@@ -471,53 +475,75 @@ bool TestContext::oneTimeConfig()
     // if forced, just do it and return
     if(mForceConfig)
     {
-        bool temp = mRoot->showConfigDialog();
+        bool temp = mRoot->showConfigDialog(OgreBites::getNativeConfigDialog());
         if(!temp)
-            mRoot->setRenderSystem(0);
+            mRoot->setRenderSystem(NULL);
         return temp;
     }
 
     // try restore
-    bool success = mRoot->restoreConfig();
-
-    // if restoring failed, show the dialog
-    if(!success)
-        success = mRoot->showConfigDialog();
+    bool restore = mRoot->restoreConfig();
 
     // set render system if user-defined
-    if(success && mRenderSystemName != "SAVED" && mRoot->getRenderSystemByName(mRenderSystemName))
+    if(restore && mRenderSystemName != "SAVED" && mRoot->getRenderSystemByName(mRenderSystemName)) {
         mRoot->setRenderSystem(mRoot->getRenderSystemByName(mRenderSystemName));
-    else if(!success)
-        mRoot->setRenderSystem(0);
+    }
+    else if(!restore) {
+        // just select the first available render system
+        const RenderSystemList lstRend = Root::getSingleton().getAvailableRenderers();
+        RenderSystemList::const_iterator pRend = lstRend.begin();
+
+        mRoot->setRenderSystem(pRend != lstRend.end() ? *pRend : NULL);
+
+        RenderSystem* rs = mRoot->getRenderSystem();
+
+        if(rs) {
+            // set sane defaults
+            rs->setConfigOption("Full Screen", "No");
+            rs->setConfigOption("Video Mode", "640x 480");
+
+            // test alpha to coverage and MSAA resolve
+            rs->setConfigOption("FSAA", "2");
+
+            try {
+                rs->setConfigOption("Fixed Pipeline Enabled", "No");
+            } catch(...) {}
+            try {
+                rs->setConfigOption("VSync", "No");
+            } catch(...) {}
+        }
+    }
 
     mRenderSystemName = mRoot->getRenderSystem() ? mRoot->getRenderSystem()->getName() : "";
 
-    return success;
+    return mRoot->getRenderSystem() != NULL;
 }
 //-----------------------------------------------------------------------
 
 void TestContext::setupDirectories(Ogre::String batchName)
 {
     // ensure there's a root directory for visual tests
-    mOutputDir = mFSLayer->getWritablePath("VisualTests/");
-    static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
+    if(mOutputDir.empty())
+    {
+        mOutputDir = mFSLayer->getWritablePath("VisualTests/");
+        static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
 
-    // make sure there's a directory for the test set
-    mOutputDir += mTestSetName + "/";
-    static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
+        // make sure there's a directory for the test set
+        mOutputDir += mTestSetName + "/";
+        static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
 
-    // add a directory for the render system
-    Ogre::String rsysName = Ogre::Root::getSingleton().getRenderSystem()->getName();
-    // strip spaces from render system name
-    for (unsigned int i = 0;i < rsysName.size(); ++i)
-        if (rsysName[i] != ' ')
-            mOutputDir += rsysName[i];
-    mOutputDir += "/";
-    static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
+        // add a directory for the render system
+        Ogre::String rsysName = Ogre::Root::getSingleton().getRenderSystem()->getName();
+        // strip spaces from render system name
+        for (unsigned int i = 0;i < rsysName.size(); ++i)
+            if (rsysName[i] != ' ')
+                mOutputDir += rsysName[i];
+        mOutputDir += "/";
+        static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mOutputDir);
+    }
 
     if(mSummaryOutputDir != "NONE")
     {
-        mSummaryOutputDir = mFSLayer->getWritablePath(mSummaryOutputDir);
         static_cast<Ogre::FileSystemLayer*>(mFSLayer)->createDirectory(mSummaryOutputDir);
     }
 
@@ -532,8 +558,7 @@ void TestContext::finishedTests()
     if ((mGenerateHtml || mSummaryOutputDir != "NONE") && !mReferenceSet)
     {
         const TestBatch* compareTo = 0;
-        TestBatchSetPtr batches;
-        TestBatchSet::iterator i;
+        TestBatchSet batches;
 
         Ogre::ConfigFile info;
         bool foundReference = true;
@@ -542,15 +567,16 @@ void TestContext::finishedTests()
         // look for a reference set first (either "Reference" or a user-specified image set)
         try
         {
-            info.load(mOutputDir + mCompareWith + "/info.cfg");
+            info.load(mReferenceSetPath + mCompareWith + "/info.cfg");
         }
-        catch (Ogre::FileNotFoundException e)
+        catch (Ogre::FileNotFoundException&)
         {
             // if no luck, just grab the most recent compatible set
             foundReference = false;
             batches = TestBatch::loadTestBatches(mOutputDir);
             
-            for (i = batches->begin(); i != batches->end(); ++i)
+            TestBatchSet::iterator i;
+            for (i = batches.begin(); i != batches.end(); ++i)
             {
                 if (mBatch->canCompareWith((*i)))
                 {
@@ -562,14 +588,14 @@ void TestContext::finishedTests()
 
         if (foundReference)
         {
-            ref = OGRE_NEW TestBatch(info, mOutputDir + mCompareWith);
+            ref = OGRE_NEW TestBatch(info, mReferenceSetPath + mCompareWith);
             if (mBatch->canCompareWith(*ref))
                 compareTo = ref;
         }
 
         if (compareTo)
         {
-            ComparisonResultVectorPtr results = mBatch->compare(*compareTo);
+            ComparisonResultVector results = mBatch->compare(*compareTo);
 
             if(mGenerateHtml)
             {
@@ -585,12 +611,16 @@ void TestContext::finishedTests()
             if(mSummaryOutputDir != "NONE")
             {
                 Ogre::String rs;
-                for(size_t i = 0; i < mRenderSystemName.size(); ++i)
-                    if(mRenderSystemName[i]!=' ')
-                        rs += mRenderSystemName[i];
+                for(size_t j = 0; j < mRenderSystemName.size(); ++j)
+                    if(mRenderSystemName[j]!=' ')
+                        rs += mRenderSystemName[j];
 
-                SimpleResultWriter simpleWriter(*compareTo, *mBatch, results);
-                simpleWriter.writeToFile(mSummaryOutputDir + "/TestResults_" + rs + ".txt");
+                CppUnitResultWriter cppunitWriter(*compareTo, *mBatch, results);
+                cppunitWriter.writeToFile(mSummaryOutputDir + "/TestResults_" + rs + ".xml");
+            }
+
+            for(size_t i = 0; i < results.size(); i++) {
+                mSuccess = mSuccess && results[i].passed;
             }
         }
 
@@ -613,170 +643,6 @@ void TestContext::setTimestep(Ogre::Real timestep)
     // ensure we're getting a positive value
     mTimestep = timestep >= 0.f ? timestep : mTimestep;
 }
-//-----------------------------------------------------------------------
-
-void TestContext::createDummyScene()
-{
-    mWindow->removeAllViewports();
-    Ogre::SceneManager* sm = mRoot->createSceneManager(Ogre::ST_GENERIC, "DummyScene");
-    sm->addRenderQueueListener(mOverlaySystem);
-    Ogre::Camera* cam = sm->createCamera("DummyCamera");
-    mWindow->addViewport(cam);
-#ifdef INCLUDE_RTSHADER_SYSTEM
-    // Initialize shader generator.
-    // Must be before resource loading in order to allow parsing extended material attributes.
-    bool success = initialiseRTShaderSystem(sm);
-    if (!success)
-    {
-        OGRE_EXCEPT(Ogre::Exception::ERR_FILE_NOT_FOUND,
-                    "Shader Generator Initialization failed - Core shader libs path not found",
-                    "SampleBrowser::createDummyScene");
-    }
-    if(mRoot->getRenderSystem()->getCapabilities()->hasCapability(Ogre::RSC_FIXED_FUNCTION) == false)
-    {
-        //newViewport->setMaterialScheme(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-
-        // creates shaders for base material BaseWhite using the RTSS
-        Ogre::MaterialPtr baseWhite = Ogre::MaterialManager::getSingleton().getByName("BaseWhite", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME).staticCast<Material>();
-        baseWhite->setLightingEnabled(false);
-        mShaderGenerator->createShaderBasedTechnique(
-            "BaseWhite",
-            Ogre::MaterialManager::DEFAULT_SCHEME_NAME,
-            Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-        mShaderGenerator->validateMaterial(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME,
-                                           "BaseWhite");
-        if(baseWhite->getNumTechniques() > 1)
-        {
-            baseWhite->getTechnique(0)->getPass(0)->setVertexProgram(
-                baseWhite->getTechnique(1)->getPass(0)->getVertexProgram()->getName());
-            baseWhite->getTechnique(0)->getPass(0)->setFragmentProgram(
-                baseWhite->getTechnique(1)->getPass(0)->getFragmentProgram()->getName());
-        }
-
-        // creates shaders for base material BaseWhiteNoLighting using the RTSS
-        mShaderGenerator->createShaderBasedTechnique(
-            "BaseWhiteNoLighting",
-            Ogre::MaterialManager::DEFAULT_SCHEME_NAME,
-            Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME);
-        mShaderGenerator->validateMaterial(Ogre::RTShader::ShaderGenerator::DEFAULT_SCHEME_NAME,
-                                           "BaseWhiteNoLighting");
-        Ogre::MaterialPtr baseWhiteNoLighting = Ogre::MaterialManager::getSingleton().getByName("BaseWhiteNoLighting", Ogre::ResourceGroupManager::INTERNAL_RESOURCE_GROUP_NAME).staticCast<Material>();
-        if(baseWhite->getNumTechniques() > 1)
-        {
-            baseWhiteNoLighting->getTechnique(0)->getPass(0)->setVertexProgram(
-                baseWhiteNoLighting->getTechnique(1)->getPass(0)->getVertexProgram()->getName());
-            baseWhiteNoLighting->getTechnique(0)->getPass(0)->setFragmentProgram(
-                baseWhiteNoLighting->getTechnique(1)->getPass(0)->getFragmentProgram()->getName());
-        }
-    }
-#endif // INCLUDE_RTSHADER_SYSTEM
-}
-
-void TestContext::destroyDummyScene()
-{
-    Ogre::SceneManager*  dummyScene = mRoot->getSceneManager("DummyScene");
-#ifdef INCLUDE_RTSHADER_SYSTEM
-    mShaderGenerator->removeSceneManager(dummyScene);
-#endif
-    dummyScene->removeRenderQueueListener(mOverlaySystem);
-    mWindow->removeAllViewports();
-    mRoot->destroySceneManager(dummyScene);
-}
-
-#ifdef INCLUDE_RTSHADER_SYSTEM
-
-/*-----------------------------------------------------------------------------
-  | Initialize the RT Shader system.
-  -----------------------------------------------------------------------------*/
-bool TestContext::initialiseRTShaderSystem(Ogre::SceneManager* sceneMgr)
-{
-    if (Ogre::RTShader::ShaderGenerator::initialize())
-    {
-        mShaderGenerator = Ogre::RTShader::ShaderGenerator::getSingletonPtr();
-
-        mShaderGenerator->addSceneManager(sceneMgr);
-
-#if OGRE_PLATFORM != OGRE_PLATFORM_ANDROID && OGRE_PLATFORM != OGRE_PLATFORM_NACL && OGRE_PLATFORM != OGRE_PLATFORM_WINRT
-        // Setup core libraries and shader cache path.
-        Ogre::StringVector groupVector = Ogre::ResourceGroupManager::getSingleton().getResourceGroups();
-        Ogre::StringVector::iterator itGroup = groupVector.begin();
-        Ogre::StringVector::iterator itGroupEnd = groupVector.end();
-        Ogre::String shaderCoreLibsPath;
-        Ogre::String shaderCachePath;
-
-        for (; itGroup != itGroupEnd; ++itGroup)
-        {
-            Ogre::ResourceGroupManager::LocationList resLocationsList = Ogre::ResourceGroupManager::getSingleton().getResourceLocationList(*itGroup);
-            Ogre::ResourceGroupManager::LocationList::iterator it = resLocationsList.begin();
-            Ogre::ResourceGroupManager::LocationList::iterator itEnd = resLocationsList.end();
-            bool coreLibsFound = false;
-
-            // Try to find the location of the core shader lib functions and use it
-            // as shader cache path as well - this will reduce the number of generated files
-            // when running from different directories.
-            for (; it != itEnd; ++it)
-            {
-                if ((*it)->archive->getName().find("RTShaderLib") != Ogre::String::npos)
-                {
-                    shaderCoreLibsPath = (*it)->archive->getName() + "/cache/";
-                    shaderCachePath = shaderCoreLibsPath;
-                    coreLibsFound = true;
-                    break;
-                }
-            }
-            // Core libs path found in the current group.
-            if (coreLibsFound)
-                break;
-        }
-
-        // Core shader libs not found -> shader generating will fail.
-        if (shaderCoreLibsPath.empty())
-            return false;
-
-#ifdef _RTSS_WRITE_SHADERS_TO_DISK
-        // Set shader cache path.
-#if OGRE_PLATFORM == OGRE_PLATFORM_APPLE_IOS
-        shaderCachePath = Ogre::macCachePath();
-#elif OGRE_PLATFORM == OGRE_PLATFORM_APPLE
-        shaderCachePath = Ogre::macCachePath() + "/org.ogre3d.RTShaderCache";
-#endif
-        mShaderGenerator->setShaderCachePath(shaderCachePath);
-#endif
-#endif
-        // Create and register the material manager listener if it doesn't exist yet.
-        if (mMaterialMgrListener == NULL) {
-            mMaterialMgrListener = new ShaderGeneratorTechniqueResolverListener(mShaderGenerator);
-            Ogre::MaterialManager::getSingleton().addListener(mMaterialMgrListener);
-        }
-    }
-
-    return true;
-}
-
-/*-----------------------------------------------------------------------------
-  | Destroy the RT Shader system.
-  -----------------------------------------------------------------------------*/
-void TestContext::finaliseRTShaderSystem()
-{
-    // Restore default scheme.
-    Ogre::MaterialManager::getSingleton().setActiveScheme(Ogre::MaterialManager::DEFAULT_SCHEME_NAME);
-
-    // Unregister the material manager listener.
-    if (mMaterialMgrListener != NULL)
-    {
-        Ogre::MaterialManager::getSingleton().removeListener(mMaterialMgrListener);
-        delete mMaterialMgrListener;
-        mMaterialMgrListener = NULL;
-    }
-
-    // Destroy RTShader system.
-    if (mShaderGenerator != NULL)
-    {
-        Ogre::RTShader::ShaderGenerator::destroy();
-        mShaderGenerator = NULL;
-    }
-}
-#endif // INCLUDE_RTSHADER_SYSTEM
 
 // main, platform-specific stuff is copied from SampleBrowser and not guaranteed to work...
 
@@ -791,22 +657,12 @@ int main(int argc, char *argv[])
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
     int retVal = UIApplicationMain(argc, argv, @"UIApplication", @"AppDelegate");
     [pool release];
-    return retVal;
-#elif (OGRE_PLATFORM == OGRE_PLATFORM_APPLE) && __LP64__
-    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
-
-    mAppDelegate = [[AppDelegate alloc] init];
-    [[NSApplication sharedApplication] setDelegate:mAppDelegate];
-    int retVal = NSApplicationMain(argc, (const char **) argv);
-
-    [pool release];
-
     return retVal;
 #else
+    TestContext tc(argc, argv);
 
     try
     {
-        TestContext tc = TestContext(argc, argv);
         tc.go();
     }
     catch (Ogre::Exception& e)
@@ -815,9 +671,11 @@ int main(int argc, char *argv[])
         MessageBoxA(NULL, e.getFullDescription().c_str(), "An exception has occurred!", MB_ICONERROR | MB_TASKMODAL);
 #else
         std::cerr << "An exception has occurred: " << e.getFullDescription().c_str() << std::endl;
+
+        return -1;
 #endif
     }
 
+    return !tc.wasSuccessful();
 #endif
-    return 0;
 }
