@@ -4,7 +4,7 @@ This source file is part of OGRE
     (Object-oriented Graphics Rendering Engine)
 For the latest info, see http://www.ogre3d.org/
 
-Copyright (c) 2000-2013 Torus Knot Software Ltd
+Copyright (c) 2000-2014 Torus Knot Software Ltd
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@ THE SOFTWARE.
 
 using namespace Ogre;
 
+#ifdef OGRE_BUILD_COMPONENT_MESHLODGENERATOR
 //---------------------------------------------------------------------------
 /** Test of manual LOD */
 class _OgreSampleClassExport PlayPen_ManualLOD : public VisualTest
@@ -65,7 +66,7 @@ protected:
     String getLODMesh();
 
 };
-
+#endif
 //---------------------------------------------------------------------------
 /** Test of Manual Blending */
 class _OgreSampleClassExport PlayPen_ManualBlend : public VisualTest
@@ -420,13 +421,10 @@ class _OgreSampleClassExport PlayPen_ClearScene : public VisualTest
 public:
 
     PlayPen_ClearScene();
-    bool frameStarted(const Ogre::FrameEvent& evt);
+    bool isScreenshotFrame(unsigned int frame);
 
 protected:
-
-    size_t mFramesElapsed;
     void setupContent();
-
 };
 
 //------------------------------------------------------------------------------
@@ -550,38 +548,6 @@ protected:
 
 };
 
-//------------------------------------------------------------------------------
-/** Tests intersection scene query */
-class _OgreSampleClassExport PlayPen_IntersectionSceneQuery : public VisualTest
-{
-public:
-
-    PlayPen_IntersectionSceneQuery();
-    bool frameStarted(const FrameEvent& evt);
-
-protected:
-
-    void setupContent();
-    IntersectionSceneQuery* intersectionQuery;
-
-};
-
-//------------------------------------------------------------------------------
-/** Tests ray scene query */
-class _OgreSampleClassExport PlayPen_RaySceneQuery : public VisualTest
-{
-public:
-
-    PlayPen_RaySceneQuery();
-    bool frameStarted(const FrameEvent& evt);
-
-protected:
-
-    void setupContent();
-    RaySceneQuery* rayQuery;
-
-};
-
 //------------------------------------------------------------------------------
 /** Tests creation and rendering of a whole lot of entities */
 class _OgreSampleClassExport PlayPen_LotsAndLotsOfEntities : public VisualTest
@@ -806,6 +772,18 @@ protected:
 
 };
 
+class _OgreSampleClassExport PlayPen_AlphaTex : public VisualTest
+{
+public:
+
+    PlayPen_AlphaTex();
+
+protected:
+
+    void setupContent();
+
+};
+
 //------------------------------------------------------------------------------
 /** Tests DXT1 loading */
 class _OgreSampleClassExport PlayPen_Dxt1 : public VisualTest
@@ -1110,9 +1088,10 @@ public:
 
     PlayPen_VertexTexture();
 
-    virtual Ogre::String getRequiredRenderSystem()
+    virtual void testCapabilities(const RenderSystemCapabilities* caps)
     {
-        return "Direct3D9 Rendering Subsystem";
+        if(!caps->isShaderProfileSupported("hlsl"))
+            OGRE_EXCEPT(Exception::ERR_NOT_IMPLEMENTED, "HLSL only for now");
     }
 
 protected:
@@ -1157,6 +1136,12 @@ public:
 
     PlayPen_TextureShadowsTransparentCaster();
 
+    void testCapabilities(const Ogre::RenderSystemCapabilities* caps)
+    {
+        if(!caps->hasCapability(RSC_FIXED_FUNCTION))
+            throw Ogre::Exception(999, "API doesn't support fixed function.", "testCapabilities");
+    }
+
 protected:
 
     void setupContent();
@@ -1256,6 +1241,7 @@ class _OgreSampleClassExport PlayPen_16Textures : public VisualTest
 public:
 
     PlayPen_16Textures();
+    void testCapabilities(const Ogre::RenderSystemCapabilities* caps);
 
 protected:
 
@@ -1438,7 +1424,19 @@ protected:
     void setupContent();
 
 };
+//------------------------------------------------------------------------------
+/** Tests blitting to texture with hardware scaling */
+class _OgreSampleClassExport PlayPen_HardwareScaleBlit : public VisualTest
+{
+public:
 
+    PlayPen_HardwareScaleBlit();
+
+protected:
+
+    void setupContent();
+
+};
 //------------------------------------------------------------------------------
 /** Tests depth shadow mapping */
 class _OgreSampleClassExport PlayPen_DepthShadowMap : public VisualTest
@@ -1448,7 +1446,7 @@ public:
     PlayPen_DepthShadowMap();
 
 protected:
-
+    std::unique_ptr<MovablePlane> movablePlane;
     void setupContent();
 
 };
@@ -1617,7 +1615,5 @@ void addTextureDebugOverlay(const Ogre::String& texname, size_t i);
 void addTextureDebugOverlay(Ogre::TexturePtr tex, size_t i);
 void addTextureShadowDebugOverlay(size_t num, Ogre::SceneManager* mgr);
 void clearDebugTextureOverlays();
-void createRandomEntityClones(Entity* ent, size_t cloneCount, 
-    const Vector3& min, const Vector3& max, SceneManager* mgr);
 
 #endif
